@@ -5,7 +5,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -342,8 +342,6 @@ def run_minimal_debug(path, args=None,
     command lines do not allow passing GDB options or command files.
     """
 
-    print("ici", debugger_ui)
-
     # Start building debugger command options
 
     cmd = [debugger]
@@ -764,6 +762,7 @@ def main(argv, pkg=None):
 
     if 'mpiexec' in cmds.keys():
         cmd_mpi = cmds['mpiexec']
+        cmd_mpi.append(sys.argv[0])
         for k in ['debugger', 'valgrind', 'program']:
             if k in cmds:
                 cmd_mpi += cmds[k]
