@@ -5,7 +5,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2015 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -241,6 +241,18 @@ def update_command_single_value(args, options, value):
                 else:
                     j = j+1
 
+    # Special cases at end
+
+    i = 0
+    args_tail = []
+    while i < len(args):
+        if args[i][0] == '$' or args[i] == '&':
+             a = args.pop(i)
+             args_tail.append(a)
+        else:
+             i = i+1
+    args += args_tail
+
     # Return updated list
 
     return args
@@ -360,13 +372,18 @@ def write_export_env(fd, var, value):
     Write the correct command so as to export environment variables.
     """
 
+    if not value:
+        value = ''
     if sys.platform.startswith('win'):
         export_cmd = 'set ' + var + '=' + value
     else:
         if get_shell_type()[-3:] == 'csh': # handle C-type shells
             export_cmd = 'setenv ' + var + ' ' + value
         else:                              # handle Bourne-type shells
-            export_cmd = 'export ' + var + '=' + value
+            if value:
+                export_cmd = 'export ' + var + '=' + value
+            else:
+                export_cmd = 'unset ' + var
     export_cmd = export_cmd + '\n'
     fd.write(export_cmd)
 
@@ -672,21 +689,44 @@ def source_syrthes_env(pkg):
     # Determine SYRTHES home
 
     syrthes_home = None
-    env_syrthes_home = os.getenv('SYRTHES4_HOME')
 
+    # Home based on configuration info
+
+    cfg_syrthes_home = None
     config = configparser.ConfigParser()
     config.read(pkg.get_configfiles())
     if config.has_option('install', 'syrthes'):
-        syrthes_home = os.path.join(config.get('install', 'syrthes'))
-        if not os.path.isdir(syrthes_home):
+        cfg_syrthes_home = os.path.join(config.get('install', 'syrthes'))
+        if not os.path.isdir(cfg_syrthes_home):
             sys.stderr.write("\nIncorrect install/syrthes entry "
                          + "specified in one of:\n")
             for f in pkg.get_configfiles():
                 sys.stderr.write("  " + f + "\n")
             sys.stderr.write("Directory: '"
-                             + syrthes_home + "' does not exist.\n")
+                             + cfg_syrthes_home + "' does not exist.\n")
             sys.exit(1)
-    else:
+            cfg_syrthes_home = None
+        else:
+            syrthes_home = cfg_syrthes_home
+
+    # Home based on current environment
+
+    # Try to base Syrthes version on path used to import the syrthes module,
+    # for consistency with case creation parameters; it this is not enough,
+    # use existing environment or load one based on current configuration.
+
+    try:
+        for p in sys.path:
+            if p[-14:] == '/share/syrthes' or p[-14:] == '\share\syrthes':
+                syr_profile = os.path.join(p[:,-14], 'bin', 'syrthes.profile')
+                if os.path.isfile(syr_profile):
+                    source_shell_script(syr_profile)
+    except Exception:
+        pass
+
+    env_syrthes_home = os.getenv('SYRTHES4_HOME')
+
+    if not syrthes_home:
         syrthes_home = env_syrthes_home
 
     if not syrthes_home:
@@ -772,6 +812,14 @@ class batch_info:
                 self.job_id = os.getenv('PBS_JOBID')
                 self.queue = os.getenv('PBS_QUEUE')
 
+        if self.batch_type == None:
+            s = os.getenv('OAR_JOBID') # OAR
+            if s != None:
+                self.batch_type = 'OAR'
+                self.submit_dir = os.getenv('OAR_WORKING_DIRECTORY')
+                self.job_name = os.getenv('OAR_JOBNAME')
+                self.job_id = os.getenv('OAR_JOBID')
+
         if self.batch_type == None:
             s = os.getenv('LOADL_JOB_NAME') # LoadLeveler
             if s != None:
@@ -838,6 +886,8 @@ class resource_info(batch_info):
         self.n_procs = None
         self.n_nodes = None
         self.n_threads = None
+        self.n_procs_ri = None
+        self.n_threads_ri = None
 
         # Pre-set the number of threads if OMP_NUM_THREADS given
 
@@ -889,7 +939,7 @@ class resource_info(batch_info):
             if s != None:
                 self.nthreads = int(s)
 
-        # Test for Platform LSF.
+        # Test for Platform LSF or OpenLava.
 
         if self.manager == None and self.batch_type == 'LSF':
             self.manager = 'LSF'
@@ -951,7 +1001,15 @@ class resource_info(batch_info):
                     if s != None:
                         self.n_procs = int(s)*self.n_nodes
 
-        # Test for Oracle Grid Engine.
+        # Test for OAR
+
+        if self.manager == None and self.batch_type == 'OAR':
+            s = os.getenv('OAR_NODEFILE')
+            if s != None:
+                self.manager = 'OAR'
+                self.hosts_file = '$OAR_NODEFILE'
+
+        # Test for Sun (/Oracle/Whowever keeps this zombie going) Grid Engine
 
         if self.manager == None and self.batch_type == 'SGE':
             s = os.getenv('NSLOTS')
@@ -969,6 +1027,7 @@ class resource_info(batch_info):
             else:
                 s = os.getenv('PE_HOSTFILE')
                 if s != None:
+                    self.manager = 'SGE'
                     self.hosts_file = '$PE_HOSTFILE'
 
         # Check hosts file presence
@@ -991,16 +1050,21 @@ class resource_info(batch_info):
             elif self.hosts_list != None:
                 self.n_procs_from_hosts_list(self.hosts_list)
 
+        # keep track of automatically determined values
+
+        self.n_procs_ri = self.n_procs
+        self.n_threads_ri = self.n_threads
+
         # Check and possibly set number of processes
 
         if n_procs != None:
             if self.n_procs != None:
                 if self.n_procs != n_procs:
                     sys.stderr.write('Warning:\n'
-                                     +'   Will use ' + str(self.n_procs)
+                                     +'   Will try to use ' + str(n_procs)
                                      + ' processes while resource manager ('
                                      + self.manager + ')\n   allows for '
-                                     + str(n_procs) + '.\n\n')
+                                     + str(self.n_procs) + '.\n\n')
             self.n_procs = n_procs
 
         if self.n_procs == None:
@@ -1011,7 +1075,7 @@ class resource_info(batch_info):
         if n_threads != None:
             if self.n_threads != None and n_threads != self.n_threads:
                 sys.stderr.write('Warning:\n'
-                                 +'   Will use ' + str(self.n_threads)
+                                 +'   Will try to use ' + str(self.n_threads)
                                  + ' threads per task while resource manager ('
                                  + self.manager + ')\n   allows for '
                                  + str(n_threads) + '.\n\n')
@@ -1096,58 +1160,8 @@ class resource_info(batch_info):
         # Check for resource manager and eventual hostsfile
 
         elif self.manager == 'SLURM':
-            s = os.getenv('SLURM_TASKS_PER_NODE')
-            hosts_count = None
-            if s != None:
-                # Syntax may be similar to SLURM_TASKS_PER_NODE=2(x3),1"
-                # indicating three nodes will each execute 2 tasks and
-                # the  fourth node will execute 1 task.
-                hosts_count = []
-                for s0 in s.split(','):
-                    i = s0.find('(')
-                    if i > -1:
-                        for j in range(int(s0[i+2:-1])):
-                            hosts_count.append(int(s0[0:i]))
-                    else:
-                        hosts_count.append(int(s0))
-            s = os.getenv('SLURM_NODELIST')
-            if s != None:
-                hosts_list = []
-                l = 0
-                # List uses a compact representation
-                for s0 in s.split(','):
-                    i = s0.find('[')
-                    if i > -1:
-                        basename = s0[0:i]
-                        for s1 in s0[i+1:-1].split(','):
-                            s2 = s1.split('-')
-                            if len(s2) > 1:
-                                fmt_s = basename + '%0' + str(len(s2[0])) + 'd'
-                                for j in range(int(s2[0]), int(s2[1])+1):
-                                    host_name = fmt_s % j
-                                    if hosts_count:
-                                        for k in range(hosts_count[l]):
-                                            hosts_list.append(host_name)
-                                        l += 1
-                                    else:
-                                        hosts_list.append(host_name)
-                            else:
-                                if hosts_count:
-                                    for k in range(hosts_count[l]):
-                                        hosts_list.append(s2[0])
-                                    l += 1
-                                else:
-                                    hosts_list.append(s2[0])
-                    else:
-                        if hosts_count:
-                            for k in range(hosts_count[l]):
-                                hosts_list.append(s0)
-                            l += 1
-                        else:
-                            hosts_list.append(s0)
-            else:
-                hosts_list = get_command_output('srun hostname -s').split()
-                hosts_list.sort()
+            hosts_list = get_command_output('srun hostname -s').split()
+            hosts_list.sort()
 
         elif self.manager == 'LSF':
             s = os.getenv('LSB_MCPU_HOSTS')
@@ -1276,9 +1290,13 @@ class mpi_environment:
         # We may have a specific syntax for tasks per node if
         # a launcher from a resource manager is used:
 
-        if resource_info and self.mpiexec and not self.mpiexec_n_per_node:
+        if resource_info and self.mpiexec:
             if os.path.basename(self.mpiexec)[:4] == 'srun':
-                self.mpiexec_n_per_node = ' --ntasks-per-node='
+                if resource_info.n_procs_ri != None \
+                  and resource_info.n_procs_ri != resource_info.n_procs:
+                    self.mpiexec_n = ' --ntasks='
+                elif not self.mpiexec_n_per_node:
+                    self.mpiexec_n_per_node = ' --ntasks-per-node='
 
         # Initialize based on known MPI types, or default.
 
@@ -1646,6 +1664,10 @@ class mpi_environment:
                 info = get_command_output(info_name)
                 if info.find(rc_mca_by_type[resource_info.manager]) > -1:
                     known_manager = True
+            elif resource_info.manager == 'OAR':
+                self.mpiexec += ' -machinefile $OAR_FILE_NODES'
+                self.mpiexec += ' -mca pls_rsh_agent "oarsh"'
+                known_manager = True
             if known_manager == False:
                 hostsfile = resource_info.get_hosts_file(wdir)
                 if hostsfile != None:
