@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -83,8 +83,8 @@ export CFDSTUDY_ROOT_DIR PYTHONPATH
         sys.stderr.write("SALOME is not available in this installation.\n")
         sys.exit(1)
 
-    # Skipped modules (version 6.3.0): YACS,JOBMANAGER,HOMARD,OPENTURNS
-    default_modules = "GEOM,SMESH,MED,CFDSTUDY,PARAVIS,VISU"
+    # Skipped modules: YACS,JOBMANAGER,HOMARD,OPENTURNS
+    default_modules = "GEOM,SMESH,MED,CFDSTUDY,PARAVIS"
 
     run_cmd = pkg.config.salome_run
     if not run_cmd:
@@ -93,9 +93,7 @@ export CFDSTUDY_ROOT_DIR PYTHONPATH
     path = pkg.get_dir('pkgpythondir')
     path = path+":"+pkg.get_dir('pythondir')
     if pkg.name == 'neptune_cfd':
-        cspath = os.path.join(pkg.get_dir('csdir'), "lib",
-                                                   "python" + sys.version[:3],
-                                                   "site-packages", "code_saturne")
+        cspath = os.path.join(pkg.get_dir('pythondir'), 'code_saturne')
         path = path+":"+cspath
 
         # Test if EOS modules could be imported
@@ -110,12 +108,8 @@ export CFDSTUDY_ROOT_DIR PYTHONPATH
                 if os.path.isdir(eospath) and not eospath in sys.path:
                     path = path+":"+eospath
 
-    if pkg.name == 'neptune_cfd':
-        prefix = pkg.get_dir('csdir')
-        pythondir = pkg.get_dir('cspythondir')
-    else:
-        prefix = pkg.get_dir('prefix')
-        pythondir = pkg.get_dir('pythondir')
+    prefix = pkg.get_dir('prefix')
+    pythondir = pkg.get_dir('pythondir')
     cmd = template % {'salomeenv': pkg.config.salome_env,
                       'prefix': prefix,
                       'pythondir': pythondir,
