@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -31,7 +31,7 @@ This module defines the following classes:
 - NewCaseDialogView
 - MainView
 
-    @copyright: 1998-2016 EDF S.A., France
+    @copyright: 1998-2017 EDF S.A., France
     @author: U{EDF<mailto:saturne-support@edf.fr>}
     @license: GNU GPL v2, see COPYING for details.
 """
@@ -40,7 +40,7 @@ This module defines the following classes:
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import os, sys, string, shutil, signal, logging
+import os, sys, shutil, signal, logging
 import subprocess, platform
 
 try:
@@ -76,7 +76,7 @@ except:
 
 from code_saturne.Base.IdView import IdView
 from code_saturne.Base.BrowserView import BrowserView
-from code_saturne.Base import XMLengine
+from code_saturne.Base import XMLengine, QtCase
 from code_saturne.Base.XMLinitialize import *
 from code_saturne.Base.XMLmodel import *
 from code_saturne.Base.Toolbox import GuiParam, displaySelectedPage
@@ -90,6 +90,7 @@ except:
 from code_saturne.Pages.WelcomeView import WelcomeView
 from code_saturne.Pages.IdentityAndPathesModel import IdentityAndPathesModel
 from code_saturne.Pages.XMLEditorView import XMLEditorView
+from code_saturne.Pages.ScriptRunningModel import ScriptRunningModel
 from code_saturne.Base.QtPage import getexistingdirectory
 from code_saturne.Base.QtPage import from_qvariant, to_text_string, getopenfilename, getsavefilename
 
@@ -213,17 +214,17 @@ class NewCaseDialogView(QDialog, Ui_NewCaseDialogForm):
     @pyqtSlot()
     def slotPostStatus(self):
         if self.checkBoxPost.isChecked():
-            self.createMesh = True
+            self.createPost = True
         else:
-            self.createMesh = False
+            self.createPost = False
 
 
     @pyqtSlot()
     def slotMeshStatus(self):
         if self.checkBoxMesh.isChecked():
-            self.createPost = True
+            self.createMesh = True
         else:
-            self.createPost = False
+            self.createMesh = False
 
 
     @pyqtSlot()
@@ -647,7 +648,7 @@ class MainView(object):
         create new Code_Saturne case
         """
         if not hasattr(self, 'case'):
-            self.case = XMLengine.Case(package=self.package)
+            self.case = QtCase.QtCase(package=self.package)
             self.case.root()['version'] = self.XML_DOC_VERSION
             self.initCase()
             title = self.tr("New parameters set") + \
@@ -678,7 +679,7 @@ class MainView(object):
         if not hasattr(self, 'case') or not self.case['xmlfile']:
             dialog = NewCaseDialogView(self, self.package)
             if dialog.exec_():
-                self.case = XMLengine.Case(package=self.package)
+                self.case = QtCase.QtCase(package=self.package)
                 self.case.root()['version'] = self.XML_DOC_VERSION
                 self.initCase()
                 title = self.tr("New parameters set") + \
@@ -693,6 +694,8 @@ class MainView(object):
                 self.scrollArea.setWidget(self.displayFisrtPage())
                 self.case['saved'] = "yes"
 
+                self.actionPrepro.setEnabled(True)
+                self.actionCalculation.setEnabled(True)
                 self.case.undo_signal.connect(self.slotUndoRedoView)
         else:
             MainView(cmd_package=self.package, cmd_case="new case").show()
@@ -783,7 +786,7 @@ class MainView(object):
         # Instantiate a new case
 
         try:
-            self.case = XMLengine.Case(package=self.package, file_name=file_name)
+            self.case = QtCase.QtCase(package=self.package, file_name=file_name)
         except:
             msg = self.tr("This file is not in accordance with XML specifications.")
             self.loadingAborted(msg, fn)
@@ -808,8 +811,9 @@ class MainView(object):
         # Cleaning the '\n' and '\t' from file_name (except in formula)
         self.case.xmlCleanAllBlank(self.case.xmlRootNode())
 
-        # we consider we are in calculation mode when we open an xml file
-        self.case['prepro'] = False
+        # we determine if we are in calculation mode when we open an xml file
+        mdl = ScriptRunningModel(self.case)
+        self.case['prepro'] = mdl.getPreproMode()
 
         msg = self.initCase()
         if msg:
@@ -837,6 +841,7 @@ class MainView(object):
 
         self.case['saved'] = "yes"
 
+
         # Update the Tree Navigator layout
 
         self.case.undo_signal.connect(self.slotUndoRedoView)
@@ -944,8 +949,13 @@ class MainView(object):
         file_name = self.case['xmlfile']
         log.debug("fileSave(): %s" % file_name)
         if not file_name:
-            self.fileSaveAs()
-            return
+            default_name = os.path.join(os.getcwd(), "setup.xml")
+            if os.path.exists(default_name) or os.path.islink(default_name):
+                self.fileSaveAs()
+                return
+            else:
+                self.case['xmlfile'] = default_name
+                file_name = default_name
 
         log.debug("fileSave(): %s" % os.path.dirname(file_name))
         log.debug("fileSave(): %s" % os.access(os.path.dirname(file_name), os.W_OK))
@@ -1113,7 +1123,8 @@ class MainView(object):
                                os.path.join(os.path.abspath(os.path.split(file_dir)[0],
                                                             'MESH')))
         self.case['runcase'] = cs_runcase.runcase(os.path.join(self.case['scripts_path'],
-                                                               self.batch_file), package=self.package)
+                                                               self.batch_file),
+                                                  package=self.package)
         del IdentityAndPathesModel
 
         self.updateStudyId()
@@ -1131,26 +1142,17 @@ class MainView(object):
         if not hasattr(self, 'case'):
             return
 
-        if self.case['runcase']:
+        if not self.case['runcase']:
+            self.case['runcase'] \
+                = cs_runcase.runcase(os.path.join(self.case['scripts_path'],
+                                                  self.batch_file),
+                                     package=self.package)
 
-            parameters = os.path.basename(self.case['xmlfile'])
 
-            self.case['runcase'].set_parameters(parameters)
-            self.case['runcase'].save()
+        parameters = os.path.basename(self.case['xmlfile'])
 
-        else:
-
-            try:
-                node_models = self.case.xmlGetNode('thermophysical_models')
-                node_cht = node_models.xmlGetNode('conjugate_heat_transfer')
-                node_syr = node_cht.xmlGetNode('external_coupling')
-                if len(node_syr.xmlGetNodeList('syrthes')) > 0:
-                    msg = "The Calculation management/Prepare batch page should be " \
-                        + "visited before saving so that the script file is updated " \
-                        + "relative to the XML file."
-                    QMessageBox.warning(self, self.package.code_name + ' Interface', msg)
-            except Exception:
-                pass
+        self.case['runcase'].set_parameters(parameters)
+        self.case['runcase'].save()
 
 
     def displayNewPage(self, index):
@@ -1319,7 +1321,7 @@ class MainViewSaturne(QMainWindow, Ui_MainForm, MainView):
         """
         Initializes a Main Window for a new document:
           1. finish the Main Window layout
-          2. connection betwenn signal and slot
+          2. connection between signal and slot
           3. Ctrl+C signal handler
           4. create some instance variables
           5. restore system settings
@@ -1346,7 +1348,8 @@ class MainViewSaturne(QMainWindow, Ui_MainForm, MainView):
 
         self.displayCSManualAction.triggered.connect(self.displayCSManual)
         self.displayCSTutorialAction.triggered.connect(self.displayCSTutorial)
-        self.displayCSKernelAction.triggered.connect(self.displayCSKernel)
+        self.displayCSTheoryAction.triggered.connect(self.displayCSTheory)
+        self.displayCSSmgrAction.triggered.connect(self.displayCSSmgr)
         self.displayCSRefcardAction.triggered.connect(self.displayCSRefcard)
         self.displayCSDoxygenAction.triggered.connect(self.displayCSDoxygen)
         self.actionUndo.triggered.connect(self.slotUndo)
@@ -1363,7 +1366,9 @@ class MainViewSaturne(QMainWindow, Ui_MainForm, MainView):
         if 'user.pdf' not in liste:
             self.displayCSManualAction.setEnabled(False)
         if 'theory.pdf' not in liste:
-            self.displayCSKernelAction.setEnabled(False)
+            self.displayCSTheoryAction.setEnabled(False)
+        if 'studymanager.pdf' not in liste:
+            self.displayCSSmgrAction.setEnabled(False)
         if 'refcard.pdf' not in liste:
             self.displayCSRefcardAction.setEnabled(False)
         if 'doxygen' not in liste:
@@ -1420,7 +1425,7 @@ class MainViewSaturne(QMainWindow, Ui_MainForm, MainView):
         QMessageBox.about(self, self.package.name + ' Interface', msg)
 
 
-    def displayCSKernel(self):
+    def displayCSTheory(self):
         """
         public slot
 
@@ -1428,6 +1433,13 @@ class MainViewSaturne(QMainWindow, Ui_MainForm, MainView):
         """
         self.displayManual(self.package, 'theory')
 
+    def displayCSSmgr(self):
+        """
+        public slot
+
+        open the studymanager guide
+        """
+        self.displayManual(self.package, 'studymanager')
 
     def displayCSRefcard(self):
         """
@@ -1521,18 +1533,39 @@ class MainViewSaturne(QMainWindow, Ui_MainForm, MainView):
         """
         mode prepro slot
         """
+        mdl = ScriptRunningModel(self.case)
+        rt = mdl.getRunType(self.case['prepro'])
         self.case['prepro'] = True
         self.initCase()
+        mdl.setRunType(rt)
         self.Browser.configureTree(self.case)
+        if self.case['current_page'] == 'Prepare batch calculation':
+            p = displaySelectedPage(self.case['current_page'],
+                                    self,
+                                    self.case,
+                                    stbar=self.statusbar,
+                                    study=self.Id,
+                                    tree=self.Browser)
+            self.scrollArea.setWidget(p)
 
 
     def slotCalculationMode(self):
         """
         mode calculation slot
         """
+        mdl = ScriptRunningModel(self.case)
         self.case['prepro'] = False
         self.initCase()
+        mdl.setRunType('standard')
         self.Browser.configureTree(self.case)
+        if self.case['current_page'] == 'Prepare batch calculation':
+            p = displaySelectedPage(self.case['current_page'],
+                                    self,
+                                    self.case,
+                                    stbar=self.statusbar,
+                                    study=self.Id,
+                                    tree=self.Browser)
+            self.scrollArea.setWidget(p)
 
 
     def slotUndoRedoView(self):
