@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2015 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -85,6 +85,10 @@ def displaySelectedPage(page_name, root, case, stbar=None, study=None, tree=None
         import code_saturne.Pages.SolutionDomainView as Page
         thisPage = Page.SolutionDomainView(root, case, stbar)
 
+    elif page_name == tr("Notebook"):
+        import code_saturne.Pages.NotebookView as Page
+        thisPage = Page.NotebookView(root, case)
+
     elif page_name == tr("Volume regions definition"):
         import code_saturne.Pages.LocalizationView as Page
         thisPage = Page.VolumeLocalizationView(root, case, tree)
@@ -145,9 +149,9 @@ def displaySelectedPage(page_name, root, case, stbar=None, study=None, tree=None
         import code_saturne.Pages.CoriolisSourceTermsView as Page
         thisPage = Page.CoriolisSourceTermsView(root, case)
 
-    elif page_name == tr("Darcy laws"):
-        import code_saturne.Pages.DarcyLawView as Page
-        thisPage = Page.DarcyLawView(root, case)
+    elif page_name == tr("Groundwater laws"):
+        import code_saturne.Pages.GroundwaterLawView as Page
+        thisPage = Page.GroundwaterLawView(root, case)
 
     elif page_name == tr("Reference values"):
         import code_saturne.Pages.ReferenceValuesView as Page
@@ -163,15 +167,19 @@ def displaySelectedPage(page_name, root, case, stbar=None, study=None, tree=None
 
     elif page_name == tr("Species transport"):
         import code_saturne.Pages.DefineUserScalarsView as Page
-        thisPage = Page.DefineUserScalarsView(root, case, stbar)
+        thisPage = Page.DefineUserScalarsView(root, case, stbar, tree)
 
     elif page_name == tr("Turbomachinery"):
         import code_saturne.Pages.TurboMachineryView as Page
         thisPage = Page.TurboMachineryView(root, case)
 
-    elif page_name == tr("Darcy"):
-        import code_saturne.Pages.DarcyView as Page
-        thisPage = Page.DarcyView(root, case)
+    elif page_name == tr("Fans"):
+        import code_saturne.Pages.FansView as Page
+        thisPage = Page.FansView(root, case)
+
+    elif page_name == tr("Groundwater flows"):
+        import code_saturne.Pages.GroundwaterView as Page
+        thisPage = Page.GroundwaterView(root, case)
 
     elif page_name == tr("Global settings"):
         import code_saturne.Pages.LagrangianView as Page
@@ -229,6 +237,10 @@ def displaySelectedPage(page_name, root, case, stbar=None, study=None, tree=None
         import code_saturne.Pages.ProfilesView as Page
         thisPage = Page.ProfilesView(root, case, stbar)
 
+    elif page_name == tr("Balance by zone"):
+        import code_saturne.Pages.BalanceView as Page
+        thisPage = Page.BalanceView(root, case)
+
     elif page_name == tr("Equation parameters"):
         import code_saturne.Pages.NumericalParamEquationView as Page
         thisPage = Page.NumericalParamEquationView(root, case)
@@ -287,6 +299,7 @@ def dicoLabel(name):
     """
     for (n, labF, labE) in [('velocity',                      "Vitesse",   "Velocity"),
                             ('pressure',                      "Pression",   "Pressure"),
+                            ('hydraulic_head',                "Charge hydraulique", "Hydraulic head"),
                             ('k',                             "EnerTurb",   "TurbEner"),
                             ('epsilon',                       "Dissip",     "Dissip"),
                             ('turbulent_viscosity',           "ViscTurb",   "TurbVisc"),
@@ -296,6 +309,7 @@ def dicoLabel(name):
                             ('r12',                           "R12",        "R12"),
                             ('r13',                           "R13",        "R13"),
                             ('r23',                           "R23",        "R23"),
+                            ('rij',                           "Rij",        "Rij"),
                             ('phi',                           "phi",        "phi"),
                             ('alpha',                         "alpha",      "alpha"),
                             ('omega',                         "omega",      "omega"),
@@ -318,7 +332,15 @@ def dicoLabel(name):
                             ('fourier_number',                "NbFourier",  "FourierNb"),
                             ('weight_matrix_X',               "VPsolve1",   "VPsolve1"),
                             ('weight_matrix_Y',               "VPsolve2",   "VPsolve2"),
-                            ('weight_matrix_Z',               "VPsolve3",   "VPsolve3")]:
+                            ('weight_matrix_Z',               "VPsolve3",   "VPsolve3"),
+                            ('est_error_cor_1',               "EsCor1",     "EsCor1"),
+                            ('est_error_der_1',               "EsDer1",     "EsDer1"),
+                            ('est_error_pre_1',               "EsPre1",     "EsPre1"),
+                            ('est_error_tot_1',               "EsTot1",     "EsTot1"),
+                            ('est_error_cor_2',               "EsCor2",     "EsCor2"),
+                            ('est_error_der_2',               "EsDer2",     "EsDer2"),
+                            ('est_error_pre_2',               "EsPre2",     "EsPre2"),
+                            ('est_error_tot_2',               "EsTot2",     "EsTot2")]:
 
         if n == name:
             if GuiParam.lang == 'fr':
