@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -69,8 +69,7 @@ class CoalCombustionModel(Variables, Model):
         self.node_turb   = self.node_models.xmlGetNode('turbulence',   'model')
         self.node_fuel   = self.node_models.xmlInitNode('solid_fuels', 'model')
 
-        self.coalCombustionModel = ('off', 'homogeneous_fuel', 'homogeneous_fuel_moisture',
-                                    'homogeneous_fuel_moisture_lagr')
+        self.coalCombustionModel = ('off', 'homogeneous_fuel', 'homogeneous_fuel_moisture')
 
 
     def defaultValues(self):
@@ -146,9 +145,6 @@ class CoalCombustionModel(Variables, Model):
         """
         coalCombustionList = self.coalCombustionModel
 
-        if self.node_lagr and self.node_lagr['model'] != 'off':
-            coalCombustionList = ('off', 'homogeneous_fuel_moisture_lagr')
-
         if self.node_turb != None:
             if self.node_turb['model'] not in ('k-epsilon',
                                                'k-epsilon-PL',
@@ -171,7 +167,7 @@ class CoalCombustionModel(Variables, Model):
         Create list of variables for a class
         """
         modelVariables =  ["n_p_", "x_p_coal_", "x_p_char_", "x_p_h_"]
-        if self.getCoalCombustionModel() == 'homogeneous_fuel_moisture' or self.getCoalCombustionModel() == 'homogeneous_fuel_moisture_lagr':
+        if self.getCoalCombustionModel() == 'homogeneous_fuel_moisture':
             modelVariables.append("x_p_wt_")
 
         return modelVariables
@@ -184,7 +180,7 @@ class CoalCombustionModel(Variables, Model):
         """
         modelProperties = ["t_p_", "x_p_", "rho_p_", "diam_p_", "dissapear_rate_p_",
                            "m_transfer_v1_p_",  "m_transfer_v2_p_", "het_ts_o2_p_"]
-        if self.getCoalCombustionModel() == 'homogeneous_fuel_moisture' or self.getCoalCombustionModel() == 'homogeneous_fuel_moisture_lagr':
+        if self.getCoalCombustionModel() == 'homogeneous_fuel_moisture':
             modelProperties.append("dry_ts_p")
 
         if self.getCO2KineticsStatus() == 'on':
@@ -235,7 +231,7 @@ class CoalCombustionModel(Variables, Model):
             lst.append("x_c_h_ox")
             lst.append("x_c_nh3")
 
-        if self.getCoalCombustionModel() == 'homogeneous_fuel_moisture' or self.getCoalCombustionModel() == 'homogeneous_fuel_moisture_lagr':
+        if self.getCoalCombustionModel() == 'homogeneous_fuel_moisture':
             lst.append("fr_h2o")
 
         if self.getOxidantNumber() >= 2:
@@ -367,9 +363,15 @@ class CoalCombustionModel(Variables, Model):
         Private method
         Create scalars and properties when coal combustion is selected
         """
-        self.node_fuel.xmlInitNode('CO2_kinetics', status = self.defaultValues()['CO2_Kinetics'])
-        self.node_fuel.xmlInitNode('H2O_kinetics', status = self.defaultValues()['H2O_Kinetics'])
-        self.node_fuel.xmlInitNode('NOx_formation', status = self.defaultValues()['NOx_formation'])
+        if not self.node_fuel.xmlGetNode('CO2_kinetics'):
+            self.node_fuel.xmlInitNode('CO2_kinetics',
+                                       status = self.defaultValues()['CO2_Kinetics'])
+        if not self.node_fuel.xmlGetNode('H2O_kinetics'):
+            self.node_fuel.xmlInitNode('H2O_kinetics',
+                                       status = self.defaultValues()['H2O_Kinetics'])
+        if not self.node_fuel.xmlGetNode('NOx_formation'):
+            self.node_fuel.xmlInitNode('NOx_formation',
+                                       status = self.defaultValues()['NOx_formation'])
         self.__createModelScalars()
         self.__createModelProperties()
 
@@ -381,7 +383,7 @@ class CoalCombustionModel(Variables, Model):
         if model isn't 'homogeneous_fuel_moisture'
         """
         # TODO a supprimer doit etre appele si on change le modele uniquement
-        if model != 'homogeneous_fuel_moisture' and self.getCoalCombustionModel() != 'homogeneous_fuel_moisture_lagr':
+        if model != 'homogeneous_fuel_moisture':
             nod = self.node_fuel.xmlGetNode('variable', type="model", name="fr_h2o")
             if nod:
                 nod.xmlRemoveNode()
@@ -500,7 +502,11 @@ class CoalCombustionModel(Variables, Model):
         """
         self.isInList(model, self.__coalCombustionModelsList())
 
+        if self.node_fuel['model'] == model:
+            return
+
         self.node_fuel['model']  = model
+
         self.__updateScalarAndProperty(model)
 
         if model == 'off':
@@ -539,7 +545,7 @@ class CoalCombustionModel(Variables, Model):
             self.createOxidant()
             self.createCoalModelScalarsAndProperties()
 
-            if model != 'homogeneous_fuel_moisture' and self.getCoalCombustionModel() != 'homogeneous_fuel_moisture_lagr':
+            if model != 'homogeneous_fuel_moisture':
                 self.__updateWetScalarsAndProperty(model)
 
 
