@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -44,8 +44,9 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
@@ -60,6 +61,7 @@ from code_saturne.Pages.DefineUserScalarsForm import Ui_DefineUserScalarsForm
 from code_saturne.Pages.LocalizationModel import LocalizationModel
 from code_saturne.Pages.DefineUserScalarsModel import DefineUserScalarsModel
 from code_saturne.Pages.TurbulenceModel import TurbulenceModel
+from code_saturne.Pages.GroundwaterModel import GroundwaterModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -149,13 +151,20 @@ class GGDHDelegate(QItemDelegate):
 
     def setEditorData(self, editor, index):
 
-        self.modelCombo.addItem(self.tr("SGDH"), "SGDH")
-        if TurbulenceModel(self.case).getTurbulenceModel() == "Rij-epsilon" or \
-           TurbulenceModel(self.case).getTurbulenceModel() == "Rij-SSG" or \
-           TurbulenceModel(self.case).getTurbulenceModel() == "Rij-EBRSM":
-            self.modelCombo.addItem(self.tr("GGDH"), "GGDH")
-            self.modelCombo.addItem(self.tr("AFM"), "AFM")
-            self.modelCombo.addItem(self.tr("DFM"), "DFM")
+        if GroundwaterModel(self.case).getGroundwaterModel() == "groundwater":
+            self.modelCombo.addItem(self.tr("OFF"), "OFF")
+        else:
+            self.modelCombo.addItem(self.tr("SGDH"), "SGDH")
+            if TurbulenceModel(self.case).getTurbulenceModel() == "Rij-epsilon" or \
+               TurbulenceModel(self.case).getTurbulenceModel() == "Rij-SSG" or \
+               TurbulenceModel(self.case).getTurbulenceModel() == "Rij-EBRSM":
+                self.modelCombo.addItem(self.tr("GGDH"), "GGDH")
+                self.modelCombo.addItem(self.tr("AFM"), "AFM")
+                self.modelCombo.addItem(self.tr("DFM"), "DFM")
+            if TurbulenceModel(self.case).getTurbulenceModel() == "Rij-EBRSM":
+                self.modelCombo.addItem(self.tr("EB-GGDH"), "EB-GGDH")
+                self.modelCombo.addItem(self.tr("EB-AFM"),  "EB-AFM")
+                self.modelCombo.addItem(self.tr("EB-DFM"),  "EB-DFM")
 
 
     def setModelData(self, comboBox, model, index):
@@ -332,7 +341,7 @@ class StandardItemModelScalars(QStandardItemModel):
         row = index.row()
         col = index.column()
 
-        # Label
+        # Name
         if col == 0:
             old_pname = self._data[row][col]
             new_pname = str(from_qvariant(value, to_text_string))
@@ -343,10 +352,10 @@ class StandardItemModelScalars(QStandardItemModel):
         elif col == 1:
             turbFlux = str(from_qvariant(value, to_text_string))
             self._data[row][col] = turbFlux
-            [name, var] = self._data[row]
+            name = self._data[row][0]
             self.mdl.setTurbulentFluxModel(name, turbFlux)
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -405,6 +414,7 @@ class StandardItemModelVariance(QStandardItemModel):
 
         self.setColumnCount(len(self.headers))
 
+        # FIXME
         self.toolTipRole = [self.tr("Code_Saturne keyword: ???"),
                             self.tr("Code_Saturne keyword: ???")]
 
@@ -448,7 +458,7 @@ class StandardItemModelVariance(QStandardItemModel):
         row = index.row()
         col = index.column()
 
-        # Label
+        # name
         if col == 0:
             old_pname = self._data[row][col]
             new_pname = str(from_qvariant(value, to_text_string))
@@ -456,14 +466,14 @@ class StandardItemModelVariance(QStandardItemModel):
             self.mdl.renameScalarLabel(old_pname, new_pname)
 
 
-        # Variance
+        # Variance (associated scalar name)
         elif col == 1:
             variance = str(from_qvariant(value, to_text_string))
             self._data[row][col] = variance
-            [name, var] = self._data[row]
-            self.mdl.setScalarVariance(name,var)
+            name = self._data[row][0]
+            self.mdl.setScalarVariance(name, variance)
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -522,7 +532,7 @@ class StandardItemModelVariance(QStandardItemModel):
 class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
     """
     """
-    def __init__(self, parent, case, stbar):
+    def __init__(self, parent, case, stbar, tree):
         """
         Constructor
         """
@@ -536,18 +546,23 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         self.case.undoStopGlobal()
 
         self.mdl = DefineUserScalarsModel(self.case)
+        self.browser = tree
 
         # tableView
         self.modelScalars = StandardItemModelScalars(self, self.mdl)
         self.modelVariance = StandardItemModelVariance(self, self.mdl)
-        self.tableScalars.horizontalHeader().setResizeMode(QHeaderView.Stretch)
-        self.tableVariance.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        if QT_API == "PYQT4":
+            self.tableScalars.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+            self.tableVariance.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        elif QT_API == "PYQT5":
+            self.tableScalars.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
+            self.tableVariance.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
 
         # Delegates
         delegateLabel        = NameDelegate(self.tableScalars)
         delegateGGDH         = GGDHDelegate(self.tableScalars, self.case)
         delegateVarianceName = VarianceNameDelegate(self.tableVariance)
-        delegateVariance     = VarianceDelegate(self.tableVariance)
+        delegateVariance     = VarianceDelegate(self.tableScalars)
 
         self.tableScalars.setItemDelegateForColumn(0, delegateLabel)
         self.tableScalars.setItemDelegateForColumn(1, delegateGGDH)
@@ -555,12 +570,12 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         self.tableVariance.setItemDelegateForColumn(1, delegateVariance)
 
         # Connections
-        self.connect(self.pushButtonNew,       SIGNAL("clicked()"), self.slotAddScalar)
-        self.connect(self.pushButtonDelete,    SIGNAL("clicked()"), self.slotDeleteScalar)
-        self.connect(self.modelScalars,        SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), self.dataChanged)
-        self.connect(self.pushButtonVarNew,    SIGNAL("clicked()"), self.slotAddVariance)
-        self.connect(self.pushButtonVarDelete, SIGNAL("clicked()"), self.slotDeleteVariance)
-        self.connect(self.modelVariance,       SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), self.dataChanged)
+        self.pushButtonNew.clicked.connect(self.slotAddScalar)
+        self.pushButtonDelete.clicked.connect(self.slotDeleteScalar)
+        self.modelScalars.dataChanged.connect(self.dataChanged)
+        self.pushButtonVarNew.clicked.connect(self.slotAddVariance)
+        self.pushButtonVarDelete.clicked.connect(self.slotDeleteVariance)
+        self.modelVariance.dataChanged.connect(self.dataChanged)
 
         # widget initialization
         self.tableScalars.reset()
@@ -581,10 +596,12 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         for name in self.mdl.getScalarsVarianceList():
             self.modelVariance.newItem(name)
 
+        if GroundwaterModel(self.case).getGroundwaterModel() != "off":
+            self.groupBox_3.hide()
         self.case.undoStartGlobal()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotAddScalar(self):
         """
         Add a new item in the table when the 'Create' button is pushed.
@@ -593,7 +610,7 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         self.modelScalars.newItem()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotDeleteScalar(self):
         """
         Just delete the current selected entries from the table and
@@ -627,9 +644,10 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
                             self.modelVariance.deleteItem(rr)
 
         self.tableScalars.clearSelection()
+        self.browser.configureTree(self.case)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotAddVariance(self):
         """
         Add a new item in the table when the 'Create' button is pushed.
@@ -638,7 +656,7 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         self.modelVariance.newItem()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotDeleteVariance(self):
         """
         Just delete the current selected entries from the table and
@@ -660,7 +678,7 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         self.tableVariance.clearSelection()
 
 
-    @pyqtSignature("const QModelIndex &, const QModelIndex &")
+    @pyqtSlot("QModelIndex, QModelIndex")
     def dataChanged(self, topLeft, bottomRight):
         for row in range(topLeft.row(), bottomRight.row()+1):
             self.tableView.resizeRowToContents(row)
