@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -29,7 +29,7 @@
 # Library odules import
 #-------------------------------------------------------------------------------
 
-import string, unittest
+import unittest
 
 #-------------------------------------------------------------------------------
 # Application modules import
@@ -46,7 +46,7 @@ from code_saturne.Pages.ThermalRadiationModel import ThermalRadiationModel
 # Model class
 #-------------------------------------------------------------------------------
 
-class OutputVolumicVariablesModel(Model):
+class OutputVolumicVariablesModel(Variables, Model):
 
     def __init__(self, case):
         """
@@ -61,9 +61,31 @@ class OutputVolumicVariablesModel(Model):
         self.node_output    = self.analysis_ctrl.xmlInitNode('output')
         self.node_probe     = self.node_output.xmlGetNodeList('probe','name')
         self.node_means     = self.analysis_ctrl.xmlInitNode('time_averages')
+        self.node_error     = self.analysis_ctrl.xmlInitNode('error_estimator')
 
         model = XMLmodel(self.case)
 
+        self.updateList()
+
+
+# Following private methods: (to see for gathering eventually)
+
+    def _defaultValues(self):
+        """
+        Return in a dictionnary which contains default values
+        """
+        default = {}
+        default['status']    = "on"
+        default['estimator'] = "0"
+
+        return default
+
+
+    def updateList(self):
+        """
+        """
+        model = XMLmodel(self.case)
+
         self.listNodeVolum = (self._getListOfVelocityPressureVariables(),
                               model.getTurbNodeList(),
                               self.getThermalScalar(),
@@ -78,38 +100,62 @@ class OutputVolumicVariablesModel(Model):
                               self._getWeightMatrixProperty(),
                               self.getListOfTimeAverage(),
                               self._getListOfAleMethod(),
-                              self._getThermalRadiativeProperties())
+                              self._getThermalRadiativeProperties(),
+                              self._getListOfEstimator())
+
+        self.listNode = []
+        for part in self._getListOfVelocityPressureVariables():
+            self.listNode.append([part, 'base'])
+        for part in model.getTurbNodeList():
+            self.listNode.append([part, 'turbulence'])
+        for part in self.getThermalScalar():
+            self.listNode.append([part, 'thermal'])
+        for part in self._getThermalRadiativeProperties():
+            self.listNode.append([part, 'thermal'])
+        for part in self.getPuCoalScalProper():
+            self.listNode.append([part, 'coal'])
+        for part in self.getGasCombScalProper():
+            self.listNode.append([part, 'gas'])
+        for part in self.getMeteoScalProper():
+            self.listNode.append([part, 'atmospheric'])
+        for part in self.getElecScalProper():
+            self.listNode.append([part, 'electric'])
+        for part in self.getAdditionalScalar():
+            self.listNode.append([part, 'other'])
+        for part in self.getAdditionalScalarProperty():
+            self.listNode.append([part, 'other'])
+        for part in self.getFluidProperty():
+            self.listNode.append([part, 'physical_properties'])
+        for part in self.getTimeProperty():
+            self.listNode.append([part, 'other'])
+        for part in self.getListOfTimeAverage():
+            self.listNode.append([part, 'other'])
+        for part in self._getListOfAleMethod():
+            self.listNode.append([part, 'other'])
+        for part in self._getListOfEstimator():
+            self.listNode.append([part, 'estimator'])
 
         self.dicoLabelName = {}
         self.list_name = []
-        self._updateDicoLabelName()
+        self._updateDictLabelName()
 
 
-# Following private methods: (to see for gathering eventually)
-
-    def _defaultValues(self):
-        """
-        Return in a dictionnary which contains default values
-        """
-        default = {}
-        default['status']    = "on"
-
-        return default
-
-
-    def _updateDicoLabelName(self):
+    def _updateDictLabelName(self):
         """
         Update dictionaries of labels for all variables, properties .....
         """
-        for nodeList in self.listNodeVolum:
-            for node in nodeList:
-                name = node['name']
-                if not name: name = node['label']
-                if not node['label']:
-                    msg = "xml node named "+ name +" has no label"
-                    raise ValueError(msg)
-                self.dicoLabelName[name] = node['label']
-                self.list_name.append(name)
+        for nodeList in self.listNode:
+            node = nodeList[0]
+            tpe  = nodeList[1]
+
+            name = node['name']
+            if not name:
+                name = node['label']
+            if not node['label']:
+                msg = "xml node named "+ name +" has no label"
+                raise ValueError(msg)
+            self.dicoLabelName[name] = node['label']
+            self.list_name.append([name, tpe])
 
 
     def _getListOfVelocityPressureVariables(self):
@@ -124,6 +170,17 @@ class OutputVolumicVariablesModel(Model):
         return nodeList
 
 
+    def _getListOfEstimator(self):
+        """
+        Private method: return node of properties of weight matrix
+        """
+        nodeList = []
+        for node in self.node_error.xmlGetNodeList('property'):
+            if not node['support']:
+                nodeList.append(node)
+        return nodeList
+
+
     def _getWeightMatrixProperty(self):
         """
         Private method: return node of properties of weight matrix
@@ -196,7 +253,7 @@ class OutputVolumicVariablesModel(Model):
     @Variables.noUndo
     def getGasCombScalProper(self):
         """
-        Return list fo nodes of gas combustion.
+        Return list of nodes of gas combustion.
         Also called by ProfilesModel and TimeAveragesModel
         """
         nodList = []
@@ -249,6 +306,23 @@ class OutputVolumicVariablesModel(Model):
         return varList
 
 
+    @Variables.noUndo
+    def getModelVariables(self, model_name):
+        """
+        Return list of variable nodes for a given model.
+        """
+        nodList = []
+        node = self.node_models.xmlGetNode(model_name, 'model')
+        if not node: return []
+        model = node['model']
+        varList = []
+        if model != 'off':
+            nodList = node.xmlGetNodeList('variable')
+            for nodvar in nodList:
+                varList.append(nodvar)
+        return varList
+
+
     @Variables.noUndo
     def getAdditionalScalar(self):
         """
@@ -346,7 +420,19 @@ class OutputVolumicVariablesModel(Model):
 
 
     @Variables.noUndo
-    def getVariableProbeList(self):
+    def getNamesList(self):
+        """
+        Return list of names for all variables, properties .....Only for the View
+        """
+        lst = []
+        for nodeList in self.listNodeVolum:
+            for node in nodeList:
+                lst.append(node['name'])
+        return lst
+
+
+    @Variables.noUndo
+    def getProbeList(self):
         """ Return list of node for probes """
         probeList = []
         for node in self.node_probe:
@@ -355,37 +441,15 @@ class OutputVolumicVariablesModel(Model):
 
 
     @Variables.noUndo
-    def getProbesList(self, label):
+    def getPrintingStatus(self, name):
         """
-        Return list of probes if it exists for node['name'] = name. Only for the View
+        Return status of markup printing from node with name. Only for the View
         """
-        self.isInList(label, self.getLabelsList())
-        lst = self.getVariableProbeList()
-        for nodeList in self.listNodeVolum:
-            for node in nodeList:
-                if node['label'] == label:
-                    node_probes = node.xmlGetChildNode('probes')
-                    if node_probes:
-                        nb_probes = node_probes['choice']
-                        if nb_probes == '0':
-                            lst = []
-                        elif nb_probes > '0':
-                            lst = []
-                            for n in node_probes.xmlGetChildNodeList('probe_recording'):
-                                lst.append(n['name'])
-        return lst
-
-
-    @Variables.noUndo
-    def getPrintingStatus(self, label):
-        """
-        Return status of markup printing from node with label. Only for the View
-        """
-        self.isInList(label, self.getLabelsList())
+        self.isInList(name, self.getNamesList())
         status = self._defaultValues()['status']
         for nodeList in self.listNodeVolum:
             for node in nodeList:
-                if node['label'] == label:
+                if node['name'] == name:
                     node_printing = node.xmlGetChildNode('listing_printing', 'status')
                     if node_printing:
                         status = node_printing['status']
@@ -393,21 +457,37 @@ class OutputVolumicVariablesModel(Model):
 
 
     @Variables.noUndo
-    def getPostStatus(self, label):
+    def getPostStatus(self, name):
         """
-        Return status of markup  post processing from node with label. Only for the View
+        Return status of markup  post processing from node with name. Only for the View
         """
-        self.isInList(label, self.getLabelsList())
+        self.isInList(name, self.getNamesList())
         status = self._defaultValues()['status']
         for nodeList in self.listNodeVolum:
             for node in nodeList:
-                if node['label'] == label:
+                if node['name'] == name:
                     node_post = node.xmlGetChildNode('postprocessing_recording', 'status')
                     if node_post:
                         status = node_post['status']
         return status
 
 
+    @Variables.noUndo
+    def getMonitorStatus(self, name):
+        """
+        Return status of markup monitoring from node with name. Only for the View
+        """
+        self.isInList(name, self.getNamesList())
+        status = self._defaultValues()['status']
+        for nodeList in self.listNodeVolum:
+            for node in nodeList:
+                if node['name'] == name:
+                    node_post = node.xmlGetChildNode('probes_recording', 'status')
+                    if node_post:
+                        status = node_post['status']
+        return status
+
+
     @Variables.undoLocal
     def setVariableLabel(self, old_label, new_label):
         """
@@ -424,7 +504,7 @@ class OutputVolumicVariablesModel(Model):
                 if node['label'] == old_label:
                     node['label'] = new_label
 
-        self._updateDicoLabelName()
+        self._updateDictLabelName()
         self._updateBoundariesNodes(old_label, new_label)
 
         for node in self.case.xmlGetNodeList('formula'):
@@ -447,15 +527,15 @@ class OutputVolumicVariablesModel(Model):
 
 
     @Variables.undoLocal
-    def setPrintingStatus(self, label, status):
+    def setPrintingStatus(self, name, status):
         """
-        Put status for balise printing from node with name and label
+        Put status for printing from node with name and label
         """
         self.isOnOff(status)
-        self.isInList(label, self.getLabelsList())
+        self.isInList(name, self.getNamesList())
         for nodeList in self.listNodeVolum:
             for node in nodeList:
-                if node['label'] == label:
+                if node['name'] == name:
                     if status == 'off':
                         node.xmlInitChildNode('listing_printing')['status'] = status
                     else:
@@ -482,15 +562,15 @@ class OutputVolumicVariablesModel(Model):
 
 
     @Variables.undoLocal
-    def setPostStatus(self, label, status):
+    def setPostStatus(self, name, status):
         """
-        Put status for balise postprocessing from node with name and label
+        Put status for postprocessing from node with name and label
         """
         self.isOnOff(status)
-        self.isInList(label, self.getLabelsList())
+        self.isInList(name, self.getNamesList())
         for nodeList in self.listNodeVolum:
             for node in nodeList:
-                if node['label'] == label:
+                if node['name'] == name:
                     if status == 'off':
                         node.xmlInitChildNode('postprocessing_recording')['status'] = status
                     else:
@@ -498,32 +578,69 @@ class OutputVolumicVariablesModel(Model):
                             node.xmlRemoveChild('postprocessing_recording')
 
 
-    def updateProbes(self, label, lst):
+    @Variables.undoLocal
+    def setMonitorStatus(self, name, status):
         """
-        Update probe_recording markups if it exists
+        Put status for monitoring from node with name and label
         """
-        self.isInList(label, self.getLabelsList())
-        nb = len(lst.split())
-        if nb == len(self.getVariableProbeList()):
-            for nodeList in self.listNodeVolum:
-                for node in nodeList:
-                    if node['label'] == label:
-                        try:
-                            node.xmlRemoveChild('probes')
-                        except:
-                            pass
+        self.isOnOff(status)
+        self.isInList(name, self.getNamesList())
+        for nodeList in self.listNodeVolum:
+            for node in nodeList:
+                if node['name'] == name:
+                    if status == 'off':
+                        node.xmlInitChildNode('probes_recording')['status'] = status
+                    else:
+                        if node.xmlGetChildNode('probes_recording'):
+                            node.xmlRemoveChild('probes_recording')
+
+
+    @Variables.noUndo
+    def getEstimatorModel(self, name):
+        """
+        Return model for an error estimator
+        """
+        self.isInList(name, ["Correction", "Drift", "Prediction", "Total"])
+        status = self._defaultValues()['estimator']
+
+        nn = self.node_error.xmlGetChildNode(name, 'model')
+        if nn:
+            status = nn['model']
+
+        return status
+
+
+    @Variables.undoLocal
+    def setEstimatorModel(self, name, model):
+        """
+        Put model for an error estimator
+        """
+        self.isInList(model, ['0', '1', '2'])
+        self.isInList(name, ["Correction", "Drift", "Prediction", "Total"])
+        status = self._defaultValues()['estimator']
+
+        if model != status:
+            if self.node_error.xmlGetChildNode(name):
+                self.node_error.xmlRemoveChild(name)
+            self.node_error.xmlInitChildNode(name)['model'] = model
         else:
-            for nodeList in self.listNodeVolum:
-                for node in nodeList:
-                    if node['label'] == label:
-                        try:
-                            node.xmlRemoveChild('probes')
-                        except:
-                            pass
-                        n = node.xmlInitNode('probes', choice=str(nb))
-                        if nb > 0:
-                            for i in lst.split():
-                                n.xmlInitChildNodeList('probe_recording',name=i)
+            if self.node_error.xmlGetChildNode(name):
+                self.node_error.xmlRemoveChild(name)
+
+        # add and remove field associated
+        if model != status:
+            nn = self.node_error.xmlGetNode(name)
+            if name == "Correction":
+                self.setNewProperty(nn, "est_error_cor_" + model)
+            elif name == "Drift":
+                self.setNewProperty(nn, "est_error_der_" + model)
+            elif name == "Prediction":
+                self.setNewProperty(nn, "est_error_pre_" + model)
+            elif name == "Total":
+                self.setNewProperty(nn, "est_error_tot_" + model)
+
+        self.updateList()
+
 
 #-------------------------------------------------------------------------------
 # OutputVolumicVariablesModel Test Class
