@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -130,10 +130,10 @@ class StandardItemModelRotor(QStandardItemModel):
 
         self.mdl = mdl
 
-        self.headers = [self.tr("Rotation velocity (rad)"),
+        self.headers = [self.tr("Rotation velocity (rad/s)"),
                         self.tr("Selection criteria")]
 
-        self.tooltip = [self.tr("Rotation velocity"),
+        self.tooltip = [self.tr("Rotation velocity (rad/s)"),
                         self.tr("Selection criteria string")]
 
         self.setColumnCount(len(self.headers))
@@ -237,7 +237,9 @@ class TurboMachineryView(QWidget, Ui_TurboMachineryForm):
         self.modelTurboMachineryType = ComboModel(self.comboBoxTurboMachineryType, 3, 1)
         self.modelTurboMachineryType.addItem(self.tr("None"), "off")
         self.modelTurboMachineryType.addItem(self.tr("Full transient simulation"), "transient")
+        self.modelTurboMachineryType.addItem(self.tr("Transient with explicit coupling"), "transient_coupled")
         self.modelTurboMachineryType.addItem(self.tr("Frozen rotor model"), "frozen")
+        self.modelTurboMachineryType.addItem(self.tr("Frozen rotor with explicit coupling"), "frozen_coupled")
 
         # Set up validators
         self.lineEditDX.setValidator(DoubleValidator(self.lineEditDX))
@@ -336,9 +338,19 @@ class TurboMachineryView(QWidget, Ui_TurboMachineryForm):
                 self.groupBoxRotation.show()
                 self.__setValuesRotation()
             if mdl == "frozen":
-                self.groupBoxJoin.setTitle("Face joining (optionnal)")
+                self.groupBoxJoin.setTitle("Face joining (optional)")
             elif mdl == "transient":
                 self.groupBoxJoin.setTitle("Face joining")
+            elif mdl == "frozen_coupled" or mdl == "transient_coupled":
+                self.groupBoxJoin.setTitle("Face mapping")
+
+            if mdl == "frozen_coupled" or mdl == "transient_coupled":
+                self.widgetFacesJoin.tableView.hideColumn(1)
+                self.widgetFacesJoin.tableView.hideColumn(3)
+            else:
+                self.widgetFacesJoin.tableView.showColumn(1)
+                self.widgetFacesJoin.tableView.showColumn(3)
+
         else:
             self.groupBoxDefineTurboMachinery.hide()
             self.groupBoxRotation.hide()
