@@ -4,7 +4,7 @@
 
 /* This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -380,6 +380,7 @@ cs_matrix_wrapper_scalar_conv_diff(int               iconvp,
 void
 cs_matrix_wrapper_vector(int                  iconvp,
                          int                  idiffp,
+                         int                  tensorial_diffusion,
                          int                  ndircp,
                          int                  isym,
                          double               thetap,
@@ -402,33 +403,69 @@ cs_matrix_wrapper_vector(int                  iconvp,
               _("invalid value of isym"));
   }
 
-  /* Symmetric matrix */
-  if (isym == 1) {
-    cs_sym_matrix_vector(m,
-                         idiffp,
-                         thetap,
-                         cofbfu,
-                         fimp,
-                         i_visc,
-                         b_visc,
-                         da,
-                         xa);
+  /* scalar diffusion or right anisotropic diffusion */
+  if (tensorial_diffusion == 1) {
+    /* Symmetric matrix */
+    if (isym == 1) {
+      cs_sym_matrix_vector(m,
+                           idiffp,
+                           thetap,
+                           cofbfu,
+                           fimp,
+                           i_visc,
+                           b_visc,
+                           da,
+                           xa);
+
+    /* Non-symmetric matrix */
+    } else {
+      cs_matrix_vector(m,
+                       iconvp,
+                       idiffp,
+                       thetap,
+                       coefbu,
+                       cofbfu,
+                       fimp,
+                       i_massflux,
+                       b_massflux,
+                       i_visc,
+                       b_visc,
+                       da,
+                       (cs_real_2_t*) xa);
+    }
+  }
+  /* left tensor diffusion */
+  else {
+
+    /* Symmetric matrix */
+    if (isym == 1) {
+      cs_sym_matrix_anisotropic_diffusion(m,
+                                          idiffp,
+                                          thetap,
+                                          cofbfu,
+                                          fimp,
+                                          (const cs_real_33_t *)i_visc,
+                                          b_visc,
+                                          da,
+                                          (cs_real_33_t *) xa);
+
+    /* Non-symmetric matrix */
+    } else {
+      cs_matrix_anisotropic_diffusion(m,
+                                      iconvp,
+                                      idiffp,
+                                      thetap,
+                                      coefbu,
+                                      cofbfu,
+                                      fimp,
+                                      i_massflux,
+                                      b_massflux,
+                                      (const cs_real_33_t *)i_visc,
+                                      b_visc,
+                                      da,
+                                      (cs_real_332_t *) xa);
+    }
 
-  /* Non-symmetric matrix */
-  } else {
-    cs_matrix_vector(m,
-                     iconvp,
-                     idiffp,
-                     thetap,
-                     coefbu,
-                     cofbfu,
-                     fimp,
-                     i_massflux,
-                     b_massflux,
-                     i_visc,
-                     b_visc,
-                     da,
-                     (cs_real_2_t*) xa);
   }
 
   /* Penalization if non invertible matrix */
@@ -463,6 +500,7 @@ cs_matrix_wrapper_vector(int                  iconvp,
 void
 cs_matrix_wrapper_tensor(int                  iconvp,
                          int                  idiffp,
+                         int                  tensorial_diffusion,
                          int                  ndircp,
                          int                  isym,
                          double               thetap,
@@ -485,203 +523,69 @@ cs_matrix_wrapper_tensor(int                  iconvp,
               _("invalid value of isym"));
   }
 
-  /* Symmetric matrix */
-  if (isym == 1) {
-    cs_sym_matrix_tensor(m,
-                         idiffp,
-                         thetap,
-                         cofbfts,
-                         fimp,
-                         i_visc,
-                         b_visc,
-                         da,
-                         xa);
+  /* scalar diffusion or right anisotropic diffusion */
+  if (tensorial_diffusion == 1) {
+    /* Symmetric matrix */
+    if (isym == 1) {
+      cs_sym_matrix_tensor(m,
+                           idiffp,
+                           thetap,
+                           cofbfts,
+                           fimp,
+                           i_visc,
+                           b_visc,
+                           da,
+                           xa);
 
-  /* Non-symmetric matrix */
-  } else {
-    cs_matrix_tensor(m,
-                     iconvp,
-                     idiffp,
-                     thetap,
-                     coefbts,
-                     cofbfts,
-                     fimp,
-                     i_massflux,
-                     b_massflux,
-                     i_visc,
-                     b_visc,
-                     da,
-                     (cs_real_2_t*) xa);
-  }
-
-  /* Penalization if non invertible matrix */
-
-  /* If no Dirichlet condition, the diagonal is slightly increased in order
-     to shift the eigenvalues spectrum (if IDIRCL=0, we force NDIRCP to be at
-     least 1 in order not to shift the diagonal). */
-
-  if (ndircp <= 0) {
-    const double epsi = 1.e-7;
-
-    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-      for (int isou = 0; isou < 6; isou++) {
-        da[cell_id][isou][isou] = (1.+epsi)*da[cell_id][isou][isou];
-      }
+    /* Non-symmetric matrix */
+    } else {
+      cs_matrix_tensor(m,
+                       iconvp,
+                       idiffp,
+                       thetap,
+                       coefbts,
+                       cofbfts,
+                       fimp,
+                       i_massflux,
+                       b_massflux,
+                       i_visc,
+                       b_visc,
+                       da,
+                       (cs_real_2_t*) xa);
     }
   }
+  /* left tensor diffusion */
+  else {
+    /* Symmetric matrix */
+    if (isym == 1) {
+      cs_sym_matrix_anisotropic_diffusion_tensor(m,
+                                                 idiffp,
+                                                 thetap,
+                                                 cofbfts,
+                                                 fimp,
+                                                 (cs_real_66_t *)i_visc,
+                                                 b_visc,
+                                                 da,
+                                                 (cs_real_66_t *)xa);
 
-  /* If a whole line of the matrix is 0, the diagonal is set to 1 */
-  for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-    cs_lnum_t corr_cell_id = CS_MIN(cs_glob_porous_model, 1)*cell_id;
-    for (int isou = 0; isou < 6; isou++) {
-      da[cell_id][isou][isou] += mq->c_solid_flag[corr_cell_id];
+    /* Non-symmetric matrix */
+    } else {
+      cs_matrix_anisotropic_diffusion_tensor(m,
+                                             iconvp,
+                                             idiffp,
+                                             thetap,
+                                             coefbts,
+                                             cofbfts,
+                                             fimp,
+                                             i_massflux,
+                                             b_massflux,
+                                             (cs_real_66_t *)i_visc,
+                                             b_visc,
+                                             da,
+                                             (cs_real_662_t *)xa);
     }
   }
 
-}
-
-/*----------------------------------------------------------------------------
- * Wrapper to cs_matrix_anisotropic_diffusion (or its counterpart for
- * symmetric matrices)
- *----------------------------------------------------------------------------*/
-
-void
-cs_matrix_anisotropic_diffusion_wrapper(int                  iconvp,
-                                        int                  idiffp,
-                                        int                  ndircp,
-                                        int                  isym,
-                                        double               thetap,
-                                        const cs_real_33_t   coefbu[],
-                                        const cs_real_33_t   cofbfu[],
-                                        const cs_real_33_t   fimp[],
-                                        const cs_real_t      i_massflux[],
-                                        const cs_real_t      b_massflux[],
-                                        const cs_real_33_t   i_visc[],
-                                        const cs_real_t      b_visc[],
-                                        cs_real_33_t         da[],
-                                        cs_real_t            xa[])
-{
-  const cs_mesh_t  *m = cs_glob_mesh;
-  const cs_mesh_quantities_t *mq = cs_glob_mesh_quantities;
-  const cs_lnum_t n_cells = m->n_cells;
-
-
-  if (isym != 1 && isym != 2) {
-    bft_error(__FILE__, __LINE__, 0,
-              _("invalid value of isym"));
-  }
-
-  /* Symmetric matrix */
-  if (isym == 1) {
-    cs_sym_matrix_anisotropic_diffusion(m,
-                                        idiffp,
-                                        thetap,
-                                        cofbfu,
-                                        fimp,
-                                        i_visc,
-                                        b_visc,
-                                        da,
-                                        (cs_real_33_t *) xa);
-
-  /* Non-symmetric matrix */
-  } else {
-    cs_matrix_anisotropic_diffusion(m,
-                                    iconvp,
-                                    idiffp,
-                                    thetap,
-                                    coefbu,
-                                    cofbfu,
-                                    fimp,
-                                    i_massflux,
-                                    b_massflux,
-                                    i_visc,
-                                    b_visc,
-                                    da,
-                                    (cs_real_332_t *) xa);
-  }
-
-  /* Penalization if non invertible matrix */
-
-  /* If no Dirichlet condition, the diagonal is slightly increased in order
-     to shift the eigenvalues spectrum. */
-
-  if (ndircp <= 0) {
-    const double epsi = 1.e-7;
-    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-      for (int isou = 0; isou < 3; isou++) {
-        da[cell_id][isou][isou] = (1.+epsi)*da[cell_id][isou][isou];
-      }
-    }
-  }
-
-  /* If a whole line of the matrix is 0, the diagonal is set to 1 */
-# pragma omp parallel for
-  for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-    for (int isou = 0; isou < 3; isou++)
-      da[cell_id][isou][isou] += mq->c_solid_flag[CS_MIN(cs_glob_porous_model, 1)*cell_id];
-  }
-
-}
-
-/*----------------------------------------------------------------------------
- * Wrapper to cs_matrix_anisotropic_diffusion_tensor (or its counterpart for
- * symmetric matrices)
- *----------------------------------------------------------------------------*/
-
-void
-cs_matrix_anisotropic_diffusion_wrapper_tensor(int                  iconvp,
-                                               int                  idiffp,
-                                               int                  ndircp,
-                                               int                  isym,
-                                               double               thetap,
-                                               const cs_real_66_t   coefbts[],
-                                               const cs_real_66_t   cofbfts[],
-                                               const cs_real_66_t   fimp[],
-                                               const cs_real_t      i_massflux[],
-                                               const cs_real_t      b_massflux[],
-                                               cs_real_t            i_visc[],
-                                               const cs_real_t      b_visc[],
-                                               cs_real_66_t         da[],
-                                               cs_real_662_t        xa[])
-{
-  const cs_mesh_t *m = cs_glob_mesh;
-  const cs_mesh_quantities_t *mq = cs_glob_mesh_quantities;
-  const cs_lnum_t n_cells = m->n_cells;
-
-  if (isym != 1 && isym != 2) {
-    bft_error(__FILE__, __LINE__, 0,
-              _("invalid value of isym"));
-  }
-
-  /* Symmetric matrix */
-  if (isym == 1) {
-    cs_sym_matrix_anisotropic_diffusion_tensor(m,
-                                               idiffp,
-                                               thetap,
-                                               cofbfts,
-                                               fimp,
-                                               (const cs_real_66_t *)i_visc,
-                                               b_visc,
-                                               da,
-                                               (const cs_real_66_t *)xa);
-
-  /* Non-symmetric matrix */
-  } else {
-    cs_matrix_anisotropic_diffusion_tensor(m,
-                                           iconvp,
-                                           idiffp,
-                                           thetap,
-                                           coefbts,
-                                           cofbfts,
-                                           fimp,
-                                           i_massflux,
-                                           b_massflux,
-                                           (const cs_real_66_t *)i_visc,
-                                           b_visc,
-                                           da,
-                                           (const cs_real_662_t *)xa);
-  }
-
   /* Penalization if non invertible matrix */
 
   /* If no Dirichlet condition, the diagonal is slightly increased in order
@@ -1137,7 +1041,7 @@ cs_sym_matrix_vector(const cs_mesh_t          *m,
 
   }
 
-  if(n_cells_ext > n_cells) {
+  if (n_cells_ext > n_cells) {
     for (cs_lnum_t cell_id = n_cells; cell_id < n_cells_ext; cell_id++) {
 
       for (int isou = 0; isou < 3; isou++) {
@@ -1208,7 +1112,7 @@ cs_sym_matrix_vector(const cs_mesh_t          *m,
  *                               - 0 otherwise
  * \param[in]     thetap        weighting coefficient for the theta-scheme,
  *                               - thetap = 0: explicit scheme
- *                               - thetap = 0.5: time-centred
+ *                               - thetap = 0.5: time-centered
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
@@ -1390,7 +1294,7 @@ cs_matrix_vector(const cs_mesh_t          *m,
 
   }
 
-  if(n_cells_ext > n_cells) {
+  if (n_cells_ext > n_cells) {
     for (cs_lnum_t cell_id = n_cells; cell_id < n_cells_ext; cell_id++) {
 
       for (int isou = 0; isou < 3; isou++) {
@@ -1421,7 +1325,7 @@ cs_matrix_vector(const cs_mesh_t          *m,
 
   /* 3. Contribution of the extra-diagonal terms to the diagonal */
 
-  for (cs_lnum_t face_id = 0; face_id <n_i_faces; face_id++) {
+  for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
 
     cs_lnum_t ii = i_face_cells[face_id][0];
     cs_lnum_t jj = i_face_cells[face_id][1];
@@ -1442,7 +1346,7 @@ cs_matrix_vector(const cs_mesh_t          *m,
 
   /* 4. Contribution of border faces to the diagonal */
 
-  for (cs_lnum_t face_id = 0; face_id <n_b_faces; face_id++) {
+  for (cs_lnum_t face_id = 0; face_id < n_b_faces; face_id++) {
 
     cs_lnum_t ii = b_face_cells[face_id];
     double flui = 0.5*(b_massflux[face_id] -fabs(b_massflux[face_id]));
@@ -1491,7 +1395,7 @@ cs_matrix_vector(const cs_mesh_t          *m,
  *                               - 0 otherwise
  * \param[in]     thetap        weighting coefficient for the theta-scheme,
  *                               - thetap = 0: explicit scheme
- *                               - thetap = 0.5: time-centred
+ *                               - thetap = 0.5: time-centered
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
@@ -1540,9 +1444,9 @@ cs_matrix_tensor(const cs_mesh_t          *m,
 
   for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
 
-    for (int isou = 0; isou < 6; isou++) {
-      for (int jsou = 0; jsou < 6; jsou++) {
-        da[cell_id][jsou][isou] = fimp[cell_id][jsou][isou];
+    for (int i = 0; i < 6; i++) {
+      for (int j = 0; j < 6; j++) {
+        da[cell_id][i][j] = fimp[cell_id][i][j];
       }
     }
 
@@ -1551,9 +1455,9 @@ cs_matrix_tensor(const cs_mesh_t          *m,
   if(n_cells_ext > n_cells) {
     for (cs_lnum_t cell_id = n_cells; cell_id < n_cells_ext; cell_id++) {
 
-      for (int isou = 0; isou < 6; isou++) {
-        for (int jsou = 0; jsou < 6; jsou++) {
-          da[cell_id][jsou][isou] = 0.;
+      for (int i = 0; i < 6; i++) {
+        for (int j = 0; j < 6; j++) {
+          da[cell_id][i][j] = 0.;
         }
       }
 
@@ -1802,9 +1706,9 @@ cs_matrix_time_step(const cs_mesh_t          *m,
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
- * \param[in]     coefbu        boundary condition array for the variable
+ * \param[in]     coefbp        boundary condition array for the variable
  *                               (implicit part - 3x3 tensor array)
- * \param[in]     cofbfu        boundary condition array for the variable flux
+ * \param[in]     cofbfp        boundary condition array for the variable flux
  *                               (implicit part - 3x3 tensor array)
  * \param[in]     fimp          part of the diagonal
  * \param[in]     i_massflux    mass flux at interior faces
@@ -1823,8 +1727,8 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
                                 int                       iconvp,
                                 int                       idiffp,
                                 double                    thetap,
-                                const cs_real_33_t        coefbu[],
-                                const cs_real_33_t        cofbfu[],
+                                const cs_real_33_t        coefbp[],
+                                const cs_real_33_t        cofbfp[],
                                 const cs_real_33_t        fimp[],
                                 const cs_real_t           i_massflux[],
                                 const cs_real_t           b_massflux[],
@@ -1852,8 +1756,8 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
       }
     }
   }
-  if(n_cells_ext > n_cells) {
-    for (cs_lnum_t cell_id = n_cells+0; cell_id < n_cells_ext; cell_id++) {
+  if (n_cells_ext > n_cells) {
+    for (cs_lnum_t cell_id = n_cells; cell_id < n_cells_ext; cell_id++) {
       for (int isou = 0; isou < 3; isou++) {
         for (int jsou = 0; jsou < 3; jsou++) {
           da[cell_id][jsou][isou] = 0.;
@@ -1875,17 +1779,17 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
 
   for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
 
-    double flui = 0.5*( i_massflux[face_id] -fabs(i_massflux[face_id]) );
-    double fluj =-0.5*( i_massflux[face_id] +fabs(i_massflux[face_id]) );
+    double flui = 0.5*(i_massflux[face_id] -fabs(i_massflux[face_id]));
+    double fluj =-0.5*(i_massflux[face_id] +fabs(i_massflux[face_id]));
 
-    for (int isou = 0; isou < 3; isou++) {
-      xa[face_id][0][isou][isou] = iconvp*flui;
-      xa[face_id][1][isou][isou] = iconvp*fluj;
-      for (int jsou = 0; jsou < 3; jsou++) {
-        xa[face_id][0][jsou][isou] = thetap*( xa[face_id][0][jsou][isou]
-                                         - idiffp*i_visc[face_id][jsou][isou]);
-        xa[face_id][1][jsou][isou] = thetap*( xa[face_id][1][jsou][isou]
-                                         - idiffp*i_visc[face_id][jsou][isou]);
+    for (int i = 0; i < 3; i++) {
+      xa[face_id][0][i][i] = iconvp*flui;
+      xa[face_id][1][i][i] = iconvp*fluj;
+      for (int j = 0; j < 3; j++) {
+        xa[face_id][0][i][j] = thetap*( xa[face_id][0][i][j]
+                                      - idiffp*i_visc[face_id][i][j]);
+        xa[face_id][1][i][j] = thetap*( xa[face_id][1][i][j]
+                                      - idiffp*i_visc[face_id][i][j]);
       }
     }
 
@@ -1903,12 +1807,12 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
      * D_jj = -theta (m_ij)^- + m_ij
      *      = -X_ji + (1-theta)*m_ij
      */
-    for (int isou = 0; isou < 3; isou++) {
-      da[ii][isou][isou] -= iconvp*(1. - thetap)*i_massflux[face_id];
-      da[jj][isou][isou] += iconvp*(1. - thetap)*i_massflux[face_id];
-      for (int jsou = 0; jsou < 3; jsou++) {
-        da[ii][jsou][isou] -= xa[face_id][0][jsou][isou];
-        da[jj][jsou][isou] -= xa[face_id][1][jsou][isou];
+    for (int i = 0; i < 3; i++) {
+      da[ii][i][i] -= iconvp*(1. - thetap)*i_massflux[face_id];
+      da[jj][i][i] += iconvp*(1. - thetap)*i_massflux[face_id];
+      for (int j = 0; j < 3; j++) {
+        da[ii][i][j] -= xa[face_id][0][i][j];
+        da[jj][i][j] -= xa[face_id][1][i][j];
       }
     }
   }
@@ -1920,22 +1824,22 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
     cs_lnum_t ii = b_face_cells[face_id];
     double flui = 0.5*( b_massflux[face_id] -fabs(b_massflux[face_id]) );
 
-    for (int isou = 0; isou < 3; isou++) {
-      for (int jsou = 0; jsou < 3; jsou++) {
+    for (int i = 0; i < 3; i++) {
+      for (int j = 0; j < 3; j++) {
         /* D_ii = theta (m_f)^+ + theta B (m_f)^- - m_f
          *      = (theta B -1)*(m_f)^- - (1-theta)*(m_f)^+
          *      = theta*(B -1)*(m_f)^- - (1-theta)*m_f
          */
-        if(isou == jsou) {
-          da[ii][jsou][isou] += iconvp*( thetap*flui
-                                        *(coefbu[face_id][jsou][isou]-1.)
+        if(j == i) {
+          da[ii][i][j] += iconvp*( thetap*flui
+                                        *(coefbp[face_id][i][j]-1.)
                                        - (1. - thetap)*b_massflux[face_id])
                               + idiffp*thetap*b_visc[face_id]
-                                      *cofbfu[face_id][jsou][isou];
+                                      *cofbfp[face_id][i][j];
         } else {
-          da[ii][jsou][isou] += thetap*( iconvp*flui*coefbu[face_id][jsou][isou]
+          da[ii][i][j] += thetap*( iconvp*flui*coefbp[face_id][i][j]
                                         +idiffp*b_visc[face_id]
-                                         *cofbfu[face_id][jsou][isou] );
+                                         *cofbfp[face_id][i][j] );
         }
       }
     }
@@ -2011,27 +1915,27 @@ cs_matrix_anisotropic_diffusion_tensor(const cs_mesh_t          *m,
   /* 1. Initialization */
 
   for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-    for (int isou = 0; isou < 6; isou++) {
-      for (int jsou = 0; jsou < 6; jsou++) {
-        da[cell_id][jsou][isou] = fimp[cell_id][jsou][isou];
+    for (int j = 0; j < 6; j++) {
+      for (int i = 0; i < 6; i++) {
+        da[cell_id][i][j] = fimp[cell_id][i][j];
       }
     }
   }
   if(n_cells_ext > n_cells) {
     for (cs_lnum_t cell_id = n_cells+0; cell_id < n_cells_ext; cell_id++) {
-      for (int isou = 0; isou < 6; isou++) {
-        for (int jsou = 0; jsou < 6; jsou++) {
-          da[cell_id][jsou][isou] = 0.;
+      for (int j = 0; j < 6; j++) {
+        for (int i = 0; i < 6; i++) {
+          da[cell_id][i][j] = 0.;
         }
       }
     }
   }
 
   for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
-    for (int isou = 0; isou < 6; isou++) {
-      for (int jsou = 0; jsou < 6; jsou++) {
-        xa[face_id][0][jsou][isou] = 0.;
-        xa[face_id][1][jsou][isou] = 0.;
+    for (int j = 0; j < 6; j++) {
+      for (int i = 0; i < 6; i++) {
+        xa[face_id][0][j][i] = 0.;
+        xa[face_id][1][j][i] = 0.;
       }
     }
   }
@@ -2043,14 +1947,14 @@ cs_matrix_anisotropic_diffusion_tensor(const cs_mesh_t          *m,
     double flui = 0.5*( i_massflux[face_id] -fabs(i_massflux[face_id]) );
     double fluj =-0.5*( i_massflux[face_id] +fabs(i_massflux[face_id]) );
 
-    for (int isou = 0; isou < 6; isou++) {
-      xa[face_id][0][isou][isou] = iconvp*flui;
-      xa[face_id][1][isou][isou] = iconvp*fluj;
-      for (int jsou = 0; jsou < 6; jsou++) {
-        xa[face_id][0][jsou][isou] = thetap*( xa[face_id][0][jsou][isou]
-                                         - idiffp*i_visc[face_id][jsou][isou]);
-        xa[face_id][1][jsou][isou] = thetap*( xa[face_id][1][jsou][isou]
-                                         - idiffp*i_visc[face_id][jsou][isou]);
+    for (int i = 0; i < 6; i++) {
+      xa[face_id][0][i][i] = iconvp*flui;
+      xa[face_id][1][i][i] = iconvp*fluj;
+      for (int j = 0; j < 6; j++) {
+        xa[face_id][0][i][j] = thetap*( xa[face_id][0][i][j]
+                                         - idiffp*i_visc[face_id][i][j]);
+        xa[face_id][1][i][j] = thetap*( xa[face_id][1][i][j]
+                                         - idiffp*i_visc[face_id][i][j]);
       }
     }
 
@@ -2068,13 +1972,13 @@ cs_matrix_anisotropic_diffusion_tensor(const cs_mesh_t          *m,
      * D_jj = -theta (m_ij)^- + m_ij
      *      = -X_ji + (1-theta)*m_ij
      */
-    for (int isou = 0; isou < 6; isou++) {
-      da[ii][isou][isou] -= iconvp*(1. - thetap)*i_massflux[face_id];
-      da[jj][isou][isou] += iconvp*(1. - thetap)*i_massflux[face_id];
+    for (int i = 0; i < 6; i++) {
+      da[ii][i][i] -= iconvp*(1. - thetap)*i_massflux[face_id];
+      da[jj][i][i] += iconvp*(1. - thetap)*i_massflux[face_id];
 
-      for (int jsou = 0; jsou < 6; jsou++) {
-        da[ii][jsou][isou] -= xa[face_id][0][jsou][isou];
-        da[jj][jsou][isou] -= xa[face_id][1][jsou][isou];
+      for (int j = 0; j < 6; j++) {
+        da[ii][i][j] -= xa[face_id][0][i][j];
+        da[jj][i][j] -= xa[face_id][1][i][j];
       }
     }
   }
@@ -2086,22 +1990,22 @@ cs_matrix_anisotropic_diffusion_tensor(const cs_mesh_t          *m,
     cs_lnum_t ii = b_face_cells[face_id];
     double flui = 0.5*( b_massflux[face_id] -fabs(b_massflux[face_id]) );
 
-    for (int isou = 0; isou < 6; isou++) {
-      for (int jsou = 0; jsou < 6; jsou++) {
+    for (int i = 0; i < 6; i++) {
+      for (int j = 0; j < 6; j++) {
         /* D_ii = theta (m_f)^+ + theta B (m_f)^- - m_f
          *      = (theta B -1)*(m_f)^- - (1-theta)*(m_f)^+
          *      = theta*(B -1)*(m_f)^- - (1-theta)*m_f
          */
-        if(isou == jsou) {
-          da[ii][jsou][isou] += iconvp*( thetap*flui
-                                        *(coefbu[face_id][jsou][isou]-1.)
+        if(j == i) {
+          da[ii][i][j] += iconvp*( thetap*flui
+                                        *(coefbu[face_id][i][j]-1.)
                                        - (1. - thetap)*b_massflux[face_id])
                               + idiffp*thetap*b_visc[face_id]
-                                      *cofbfu[face_id][jsou][isou];
+                                      *cofbfu[face_id][i][j];
         } else {
-          da[ii][jsou][isou] += thetap*( iconvp*flui*coefbu[face_id][jsou][isou]
+          da[ii][i][j] += thetap*( iconvp*flui*coefbu[face_id][i][j]
                                         +idiffp*b_visc[face_id]
-                                         *cofbfu[face_id][jsou][isou] );
+                                         *cofbfu[face_id][i][j] );
         }
       }
     }
@@ -2241,7 +2145,7 @@ cs_sym_matrix_anisotropic_diffusion(const cs_mesh_t           *m,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Build the diffusion matrix for a vector field with a
+ * \brief Build the diffusion matrix for a tensor field with a
  * tensorial diffusivity (symmetric matrix).
  *
  * The diffusion is not reconstructed.
