@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -157,6 +157,7 @@ cs_matrix_wrapper_scalar_conv_diff(int               iconvp,
 void
 cs_matrix_wrapper_vector(int                  iconvp,
                          int                  idiffp,
+                         int                  tensorial_diffusion,
                          int                  ndircp,
                          int                  isym,
                          double               thetap,
@@ -178,6 +179,7 @@ cs_matrix_wrapper_vector(int                  iconvp,
 void
 cs_matrix_wrapper_tensor(int                  iconvp,
                          int                  idiffp,
+                         int                  tensorial_diffusion,
                          int                  ndircp,
                          int                  isym,
                          double               thetap,
@@ -191,48 +193,6 @@ cs_matrix_wrapper_tensor(int                  iconvp,
                          cs_real_66_t         da[],
                          cs_real_t            xa[]);
 
-/*----------------------------------------------------------------------------
- * Wrapper to cs_matrix_anisotropic_diffusion (or its counterpart for
- * symmetric matrices)
- *----------------------------------------------------------------------------*/
-
-void
-cs_matrix_anisotropic_diffusion_wrapper(int                  iconvp,
-                                        int                  idiffp,
-                                        int                  ndircp,
-                                        int                  isym,
-                                        double               thetap,
-                                        const cs_real_33_t   coefbu[],
-                                        const cs_real_33_t   cofbfu[],
-                                        const cs_real_33_t   fimp[],
-                                        const cs_real_t      i_massflux[],
-                                        const cs_real_t      b_massflux[],
-                                        const cs_real_33_t   i_visc[],
-                                        const cs_real_t      b_visc[],
-                                        cs_real_33_t         da[],
-                                        cs_real_t            xa[]);
-
-/*----------------------------------------------------------------------------
- * Wrapper to cs_matrix_anisotropic_diffusion_tensor (or its counterpart for
- * symmetric matrices)
- *----------------------------------------------------------------------------*/
-
-void
-cs_matrix_anisotropic_diffusion_wrapper_tensor(int                  iconvp,
-                                               int                  idiffp,
-                                               int                  ndircp,
-                                               int                  isym,
-                                               double               thetap,
-                                               const cs_real_66_t   coefbts[],
-                                               const cs_real_66_t   cofbfts[],
-                                               const cs_real_66_t   fimp[],
-                                               const cs_real_t      i_massflux[],
-                                               const cs_real_t      b_massflux[],
-                                               cs_real_t            i_visc[],
-                                               const cs_real_t      b_visc[],
-                                               cs_real_66_t         da[],
-                                               cs_real_662_t        xa[]);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Build the diffusion matrix for a scalar field.
@@ -291,7 +251,7 @@ cs_sym_matrix_scalar(const cs_mesh_t          *m,
  *                               - 0 otherwise
  * \param[in]     thetap        weighting coefficient for the theta-scheme,
  *                               - thetap = 0: explicit scheme
- *                               - thetap = 0.5: time-centred
+ *                               - thetap = 0.5: time-centered
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
@@ -337,7 +297,7 @@ cs_sym_matrix_tensor(const cs_mesh_t          *m,
  *                               - 0 otherwise
  * \param[in]     thetap        weighting coefficient for the theta-scheme,
  *                               - thetap = 0: explicit scheme
- *                               - thetap = 0.5: time-centred
+ *                               - thetap = 0.5: time-centered
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
