@@ -216,7 +216,7 @@ if (ifilechemistry.ge.1) then
     call add_model_scalar_field('species_hono',  'HONO',  isca_chem(11))
     call add_model_scalar_field('species_hno3',  'HNO3',  isca_chem(12))
     call add_model_scalar_field('species_co',    'CO',    isca_chem(13))
-    call add_model_scalar_field('species_hco',   'HCO',   isca_chem(14))
+    call add_model_scalar_field('species_hcho',  'HCHO',  isca_chem(14))
     call add_model_scalar_field('species_ald2',  'ALD2',  isca_chem(15))
     call add_model_scalar_field('species_c2o3',  'C2O3',  isca_chem(16))
     call add_model_scalar_field('species_pan',   'PAN',   isca_chem(17))
@@ -272,7 +272,7 @@ if (ifilechemistry.ge.1) then
     call add_model_scalar_field('species_hno3',  'HNO3',  isca_chem(12))
     call add_model_scalar_field('species_hno4',  'HNO4',  isca_chem(13))
     call add_model_scalar_field('species_co',    'CO',    isca_chem(14))
-    call add_model_scalar_field('species_hco',   'HCO',   isca_chem(15))
+    call add_model_scalar_field('species_hcho',  'HCHO',  isca_chem(15))
     call add_model_scalar_field('species_ald2',  'ALD2',  isca_chem(16))
     call add_model_scalar_field('species_c2o3',  'C2O3',  isca_chem(17))
     call add_model_scalar_field('species_pan',   'PAN',   isca_chem(18))
@@ -296,6 +296,7 @@ if (ifilechemistry.ge.1) then
     call add_model_scalar_field('species_ror',   'ROR',   isca_chem(36))
     call add_model_scalar_field('species_eth',   'ETH',   isca_chem(37))
     call add_model_scalar_field('species_ole',   'OLE',   isca_chem(38))
+    call add_model_scalar_field('species_iole',  'IOLE',  isca_chem(39))
     call add_model_scalar_field('species_isop',  'ISOP',  isca_chem(40))
     call add_model_scalar_field('species_ispd',  'ISPD',  isca_chem(41))
     call add_model_scalar_field('species_terp',  'TERP',  isca_chem(42))
