@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -29,6 +29,10 @@
 !> \brief Symmetry boundary conditions for vectors and tensors.
 !>
 !> Correspond to the code icodcl(ivar) = 4.
+!>
+!> Please refer to the
+!> <a href="../../theory.pdf#clsyvt"><b>clsyvt</b></a> section of the
+!> theory guide for more informations.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -85,11 +89,13 @@ use numvar
 use optcal
 use cstphy
 use cstnum
+use dimens, only: nvar
 use pointe
 use entsor
 use albase
 use field
 use mesh
+use cs_c_bindings
 
 !===============================================================================
 
@@ -99,15 +105,15 @@ implicit none
 
 integer          nscal
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 double precision velipb(nfabor,ndim), rijipb(nfabor,6)
 
 ! Local variables
 
 integer          ifac, ii, isou
-integer          iel
+integer          iel, f_dim
 integer          iscal , ivar
 
 double precision rnx, rny, rnz, rxnn
@@ -140,7 +146,10 @@ double precision, dimension(:,:), pointer :: coefa_rij, coefaf_rij, coefad_rij
 double precision, dimension(:,:,:), pointer :: coefb_rij, coefbf_rij, coefbd_rij
 
 double precision, dimension(:), pointer :: viscl, visct
-double precision, dimension(:), pointer :: cpro_visma1, cpro_visma2, cpro_visma3
+double precision, dimension(:), pointer :: cpro_visma_s
+double precision, dimension(:,:), pointer :: cpro_visma_v
+
+type(var_cal_opt) :: vcopt_rij
 
 !===============================================================================
 
@@ -152,8 +161,8 @@ cpp = 0.d0
 
 if (itytur.eq.3 .and. idirsm.eq.1) call field_get_val_v(ivsten, visten)
 
-call field_get_val_s(iprpfl(iviscl), viscl)
-call field_get_val_s(iprpfl(ivisct), visct)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 
 ! Boundary Conditions
 
@@ -163,6 +172,7 @@ call field_get_coefaf_v(ivarfl(iu), cofafu)
 call field_get_coefbf_v(ivarfl(iu), cofbfu)
 
 if (itytur.eq.3) then
+  call field_get_key_struct_var_cal_opt(ivarfl(ir11), vcopt_rij)
   if (irijco.eq.1) then
     call field_get_coefa_v(ivarfl(irij), coefa_rij)
     call field_get_coefb_v(ivarfl(irij), coefb_rij)
@@ -464,6 +474,57 @@ do ifac = 1, nfabor
 
     if (itytur.eq.3) then
 
+      ! Symmetric tensor diffusivity (Daly Harlow -- GGDH)
+      if (iand(vcopt_rij%idften, ANISOTROPIC_RIGHT_DIFFUSION).ne.0) then
+
+        visci(1,1) = visclc + visten(1,iel)
+        visci(2,2) = visclc + visten(2,iel)
+        visci(3,3) = visclc + visten(3,iel)
+        visci(1,2) =          visten(4,iel)
+        visci(2,1) =          visten(4,iel)
+        visci(2,3) =          visten(5,iel)
+        visci(3,2) =          visten(5,iel)
+        visci(1,3) =          visten(6,iel)
+        visci(3,1) =          visten(6,iel)
+
+        ! ||Ki.S||^2
+        viscis = ( visci(1,1)*surfbo(1,ifac)       &
+                 + visci(1,2)*surfbo(2,ifac)       &
+                 + visci(1,3)*surfbo(3,ifac))**2   &
+               + ( visci(2,1)*surfbo(1,ifac)       &
+                 + visci(2,2)*surfbo(2,ifac)       &
+                 + visci(2,3)*surfbo(3,ifac))**2   &
+               + ( visci(3,1)*surfbo(1,ifac)       &
+                 + visci(3,2)*surfbo(2,ifac)       &
+                 + visci(3,3)*surfbo(3,ifac))**2
+
+        ! IF.Ki.S
+        fikis = ( (cdgfbo(1,ifac)-xyzcen(1,iel))*visci(1,1)   &
+                + (cdgfbo(2,ifac)-xyzcen(2,iel))*visci(2,1)   &
+                + (cdgfbo(3,ifac)-xyzcen(3,iel))*visci(3,1)   &
+                )*surfbo(1,ifac)                              &
+              + ( (cdgfbo(1,ifac)-xyzcen(1,iel))*visci(1,2)   &
+                + (cdgfbo(2,ifac)-xyzcen(2,iel))*visci(2,2)   &
+                + (cdgfbo(3,ifac)-xyzcen(3,iel))*visci(3,2)   &
+                )*surfbo(2,ifac)                              &
+              + ( (cdgfbo(1,ifac)-xyzcen(1,iel))*visci(1,3)   &
+                + (cdgfbo(2,ifac)-xyzcen(2,iel))*visci(2,3)   &
+                + (cdgfbo(3,ifac)-xyzcen(3,iel))*visci(3,3)   &
+                )*surfbo(3,ifac)
+
+        distfi = distb(ifac)
+
+        ! Take I" so that I"F= eps*||FI||*Ki.n when J" is in cell rji
+        ! NB: eps =1.d-1 must be consistent with vitens.f90
+        fikis = max(fikis, 1.d-1*sqrt(viscis)*distfi)
+
+        hint = viscis/surfbn(ifac)/fikis
+
+      ! Scalar diffusivity
+      else
+        hint = (visclc+visctc*csrij/cmu)/distbf
+      endif
+
       do isou = 1, 6
         fcoefa(isou) = 0.d0
         fcofad(isou) = 0.d0
@@ -487,8 +548,22 @@ do ifac = 1, nfabor
           ivar = ir13
         endif
 
-        ! IMPLICITATION PARTIELLE EVENTUELLE DES CL
-        if (iclsyr.eq.1) then
+        ! Partial (or total if coupled) implicitation
+        if (irijco.eq.1) then
+          coefa_rij(isou, ifac)  = 0.d0
+          coefaf_rij(isou, ifac) = 0.d0
+          coefad_rij(isou, ifac) = 0.d0
+          do ii = 1, 6
+            coefb_rij(isou,ii, ifac)  = alpha(ii, isou)
+            if (ii.eq.isou) then
+              coefbf_rij(isou,ii, ifac) = hint * (1.d0 - coefb_rij(isou,ii, ifac))
+            else
+              coefbf_rij(isou,ii, ifac) = - hint * coefb_rij(isou,ii, ifac)
+            endif
+            coefbd_rij(isou,ii, ifac) = coefb_rij(isou,ii, ifac)
+          enddo
+
+        else if (iclsyr.eq.1) then
           do ii = 1, 6
             if (ii.ne.isou) then
               fcoefa(isou) = fcoefa(isou) + alpha(isou,ii) * rijipb(ifac,ii)
@@ -506,70 +581,12 @@ do ifac = 1, nfabor
         fcofad(isou) = fcoefa(isou)
         fcofbd(isou) = fcoefb(isou)
 
-        ! Symmetric tensor diffusivity (Daly Harlow -- GGDH)
-        if (idften(ivar).eq.6) then
-
-          visci(1,1) = visclc + visten(1,iel)
-          visci(2,2) = visclc + visten(2,iel)
-          visci(3,3) = visclc + visten(3,iel)
-          visci(1,2) =          visten(4,iel)
-          visci(2,1) =          visten(4,iel)
-          visci(2,3) =          visten(5,iel)
-          visci(3,2) =          visten(5,iel)
-          visci(1,3) =          visten(6,iel)
-          visci(3,1) =          visten(6,iel)
-
-          ! ||Ki.S||^2
-          viscis = ( visci(1,1)*surfbo(1,ifac)       &
-                   + visci(1,2)*surfbo(2,ifac)       &
-                   + visci(1,3)*surfbo(3,ifac))**2   &
-                 + ( visci(2,1)*surfbo(1,ifac)       &
-                   + visci(2,2)*surfbo(2,ifac)       &
-                   + visci(2,3)*surfbo(3,ifac))**2   &
-                 + ( visci(3,1)*surfbo(1,ifac)       &
-                   + visci(3,2)*surfbo(2,ifac)       &
-                   + visci(3,3)*surfbo(3,ifac))**2
-
-          ! IF.Ki.S
-          fikis = ( (cdgfbo(1,ifac)-xyzcen(1,iel))*visci(1,1)   &
-                  + (cdgfbo(2,ifac)-xyzcen(2,iel))*visci(2,1)   &
-                  + (cdgfbo(3,ifac)-xyzcen(3,iel))*visci(3,1)   &
-                  )*surfbo(1,ifac)                              &
-                + ( (cdgfbo(1,ifac)-xyzcen(1,iel))*visci(1,2)   &
-                  + (cdgfbo(2,ifac)-xyzcen(2,iel))*visci(2,2)   &
-                  + (cdgfbo(3,ifac)-xyzcen(3,iel))*visci(3,2)   &
-                  )*surfbo(2,ifac)                              &
-                + ( (cdgfbo(1,ifac)-xyzcen(1,iel))*visci(1,3)   &
-                  + (cdgfbo(2,ifac)-xyzcen(2,iel))*visci(2,3)   &
-                  + (cdgfbo(3,ifac)-xyzcen(3,iel))*visci(3,3)   &
-                  )*surfbo(3,ifac)
-
-          distfi = distb(ifac)
-
-          ! Take I" so that I"F= eps*||FI||*Ki.n when J" is in cell rji
-          ! NB: eps =1.d-1 must be consistent with vitens.f90
-          fikis = max(fikis, 1.d-1*sqrt(viscis)*distfi)
-
-          hint = viscis/surfbn(ifac)/fikis
-
-        ! Scalar diffusivity
-        else
-          hint = (visclc+visctc*csrij/cmu)/distbf
-        endif
-
         ! Translate into Diffusive flux BCs
         fcofaf(isou) = -hint*fcoefa(isou)
         fcofbf(isou) = hint*(1.d0-fcoefb(isou))
       enddo
-      do isou = 1, 6
-        if (irijco.eq.1) then
-          coefa_rij(isou, ifac)       = fcoefa(isou)
-          coefb_rij(isou,isou, ifac)  = fcoefb(isou)
-          coefaf_rij(isou, ifac)      = fcofaf(isou)
-          coefbf_rij(isou,isou, ifac) = fcofbf(isou)
-          coefad_rij(isou, ifac)      = fcofad(isou)
-          coefbd_rij(isou,isou, ifac) = fcofbd(isou)
-        else
+      if (irijco.ne.1) then
+        do isou = 1, 6
           if (isou.eq.1) then
             coefa_r11(ifac) = fcoefa(isou)
             coefb_r11(ifac) = fcoefb(isou)
@@ -613,8 +630,8 @@ do ifac = 1, nfabor
             coefad_r13(ifac) = fcofad(isou)
             coefbd_r13(ifac) = fcofbd(isou)
           endif
-        endif
-      enddo
+        enddo
+      endif
 
     endif
 
@@ -625,18 +642,20 @@ enddo
 ! ---  End of loop over boundary faces
 
 !===========================================================================
-! 3.bis Boundary conditions on u'T'
+! 3.bis Boundary conditions on transported vectors
 !===========================================================================
 
 do iscal = 1, nscal
-
+  ! u'T'
   if (ityturt(iscal).eq.3) then
-
     call clsyvt_scalar(iscal, icodcl)
-    !=================
-
   endif
 
+  ! additional transported vectors
+  call field_get_dim(ivarfl(isca(iscal)), f_dim)
+  if (f_dim.gt.1) then
+    call clsyvt_vector(iscal, icodcl)
+  endif
 enddo
 
 !===============================================================================
@@ -650,9 +669,11 @@ if (iale.eq.1) then
   call field_get_coefaf_v(ivarfl(iuma), cfaale)
   call field_get_coefbf_v(ivarfl(iuma), cfbale)
 
-  call field_get_val_s(iprpfl(ivisma(1)), cpro_visma1)
-  call field_get_val_s(iprpfl(ivisma(2)), cpro_visma2)
-  call field_get_val_s(iprpfl(ivisma(3)), cpro_visma3)
+  if (iortvm.eq.0) then
+    call field_get_val_s(ivisma, cpro_visma_s)
+  else
+    call field_get_val_v(ivisma, cpro_visma_v)
+  endif
 
   do ifac = 1, nfabor
     if (icodcl(ifac,iuma).eq.4) then
@@ -668,9 +689,15 @@ if (iale.eq.1) then
       srfbnf = surfbn(ifac)
 
       ! --- Physical properties
-      vismsh(1) = cpro_visma1(iel)
-      vismsh(2) = cpro_visma2(iel)
-      vismsh(3) = cpro_visma3(iel)
+      if (iortvm.eq.0) then
+        vismsh(1) = cpro_visma_s(iel)
+        vismsh(2) = cpro_visma_s(iel)
+        vismsh(3) = cpro_visma_s(iel)
+      else
+        vismsh(1) = cpro_visma_v(1,iel)
+        vismsh(2) = cpro_visma_v(2,iel)
+        vismsh(3) = cpro_visma_v(3,iel)
+      endif
 
       hintv(1) = vismsh(1)/distbf
       hintv(2) = vismsh(2)/distbf
@@ -778,6 +805,7 @@ use numvar
 use optcal
 use cstphy
 use cstnum
+use dimens, only: nvar
 use pointe
 use entsor
 use albase
@@ -797,18 +825,19 @@ implicit none
 
 integer          iscal
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 
 ! Local variables
 
 integer          ivar, f_id
 integer          ifac, iel, isou, jsou
-integer          ipccv, ifcvsl
+integer          ifcvsl
 
 double precision cpp, rkl, visclc
 double precision distbf, srfbnf
 double precision rnx, rny, rnz
 double precision hintt(6)
+double precision hint, qimp
 
 character(len=80) :: fname
 
@@ -816,6 +845,7 @@ double precision, dimension(:), pointer :: val_s, crom
 double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
 double precision, dimension(:,:), pointer :: coefaut, cofafut, cofarut, visten
 double precision, dimension(:,:,:), pointer :: coefbut, cofbfut, cofbrut
+double precision, dimension(:), pointer :: a_al, b_al, af_al, bf_al
 
 double precision, dimension(:), pointer :: viscl, viscls, cpro_cp
 
@@ -828,7 +858,7 @@ f_id = ivarfl(ivar)
 
 call field_get_val_s(ivarfl(ivar), val_s)
 call field_get_val_v(ivsten, visten)
-call field_get_val_s(iprpfl(iviscl), viscl)
+call field_get_val_s(iviscl, viscl)
 
 call field_get_coefa_s(f_id, coefap)
 call field_get_coefb_s(f_id, coefbp)
@@ -837,8 +867,8 @@ call field_get_coefbf_s(f_id, cofbfp)
 
 call field_get_val_s(icrom, crom)
 
-if (icp.gt.0) then
-  call field_get_val_s(iprpfl(icp), cpro_cp)
+if (icp.ge.0) then
+  call field_get_val_s(icp, cpro_cp)
 endif
 
 call field_get_key_int (f_id, kivisl, ifcvsl)
@@ -846,15 +876,6 @@ if (ifcvsl .ge. 0) then
   call field_get_val_s(ifcvsl, viscls)
 endif
 
-ipccv = 0
-if (ippmod(icompf) .ge. 0) then
-  if (icv.gt.0) then
-    ipccv  = ipproc(icv)
-  else
-    ipccv = 0
-  endif
-endif
-
 ! Turbulent diffusive flux of the scalar T
 ! (blending factor so that the component v'T' have only
 !  mu_T/(mu+mu_T)* Phi_T)
@@ -872,6 +893,21 @@ call field_get_coefbf_v(f_id,cofbfut)
 call field_get_coefad_v(f_id,cofarut)
 call field_get_coefbd_v(f_id,cofbrut)
 
+! EB-GGDH/AFM/DFM alpha boundary conditions
+if (iturt(iscal).eq.11 .or. iturt(iscal).eq.21 .or. iturt(iscal).eq.31) then
+
+  ! Name of the scalar ivar
+  call field_get_name(ivarfl(ivar), fname)
+
+  ! Index of the corresponding turbulent flux
+  call field_get_id(trim(fname)//'_alpha', f_id)
+
+  call field_get_coefa_s (f_id, a_al)
+  call field_get_coefb_s (f_id, b_al)
+  call field_get_coefaf_s(f_id, af_al)
+  call field_get_coefbf_s(f_id, bf_al)
+endif
+
 ! --- Loop on boundary faces
 do ifac = 1, nfabor
 
@@ -883,7 +919,7 @@ do ifac = 1, nfabor
     visclc = viscl(iel)
     cpp = 1.d0
     if (iscacp(iscal).eq.1) then
-      if (icp.gt.0) then
+      if (icp.ge.0) then
         cpp = cpro_cp(iel)
       else
         cpp = cp0
@@ -904,15 +940,15 @@ do ifac = 1, nfabor
       rkl = viscls(iel)/cpp
     endif
 
-    hintt(1) = 0.5d0*(visclc+rkl)/distbf                        &
-             + visten(1,iel)*ctheta(iscal)/distbf
-    hintt(2) = 0.5d0*(visclc+rkl)/distbf                        &
-             + visten(2,iel)*ctheta(iscal)/distbf
-    hintt(3) = 0.5d0*(visclc+rkl)/distbf                        &
-             + visten(3,iel)*ctheta(iscal)/distbf
-    hintt(4) = visten(4,iel)*ctheta(iscal)/distbf
-    hintt(5) = visten(5,iel)*ctheta(iscal)/distbf
-    hintt(6) = visten(6,iel)*ctheta(iscal)/distbf
+    !FIXME for EB DFM
+    do isou = 1, 6
+      if (isou.le.3) then
+        hintt(isou) = (0.5d0*(visclc + rkl)    &
+          + ctheta(iscal)*visten(isou,iel)/csrij) / distbf
+      else
+        hintt(isou) = ctheta(iscal)*visten(isou,iel) / csrij / distbf
+      endif
+    enddo
 
     ! Gradient BCs
     coefaut(1,ifac) = 0.d0
@@ -940,11 +976,11 @@ do ifac = 1, nfabor
     cofbfut(3,3,ifac) = hintt(6)*rnx*rnz + hintt(5)*rny*rnz + hintt(3)*rnz**2
 
     cofbfut(1,2,ifac) = hintt(1)*rnx*rny + hintt(4)*rny**2  + hintt(6)*rny*rnz
-    cofbfut(2,1,ifac) = hintt(1)*rnx*rny + hintt(4)*rny**2  + hintt(6)*rny*rnz
+    cofbfut(2,1,ifac) = hintt(4)*rnx**2  + hintt(2)*rny*rnx + hintt(5)*rnx*rnz
     cofbfut(1,3,ifac) = hintt(1)*rnx*rnz + hintt(4)*rny*rnz + hintt(6)*rnz**2
-    cofbfut(3,1,ifac) = hintt(1)*rnx*rnz + hintt(4)*rny*rnz + hintt(6)*rnz**2
+    cofbfut(3,1,ifac) = hintt(6)*rnx**2  + hintt(5)*rny*rnx + hintt(3)*rnz*rnx
     cofbfut(2,3,ifac) = hintt(4)*rnx*rnz + hintt(2)*rny*rnz + hintt(5)*rnz**2
-    cofbfut(3,2,ifac) = hintt(4)*rnx*rnz + hintt(2)*rny*rnz + hintt(5)*rnz**2
+    cofbfut(3,2,ifac) = hintt(6)*rnx*rny + hintt(5)*rny**2  + hintt(3)*rnz*rny
 
     ! Boundary conditions for thermal transport equation
     do isou = 1, 3
@@ -954,6 +990,224 @@ do ifac = 1, nfabor
       enddo
     enddo
 
+    ! EB-GGDH/AFM/DFM alpha boundary conditions
+    if (iturt(iscal).eq.11 .or. iturt(iscal).eq.21 .or. iturt(iscal).eq.31) then
+
+      ! Dirichlet Boundary Condition
+      !-----------------------------
+
+      qimp = 0.d0
+
+      hint = 1.d0/distbf
+
+      call set_neumann_scalar &
+        !====================
+      ( a_al(ifac), af_al(ifac),             &
+        b_al(ifac), bf_al(ifac),             &
+        qimp      , hint       )
+
+    endif
+
+
+  endif
+  ! Test on velocity symmetry condition: end
+
+enddo
+
+!----
+! End
+!----
+
+return
+end subroutine
+
+!===============================================================================
+! Local functions
+!===============================================================================
+
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]     iscal         scalar id
+!> \param[in,out] icodcl        face boundary condition code:
+!>                               - 1 Dirichlet
+!>                               - 3 Neumann
+!>                               - 4 sliding and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 5 smooth wall and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 6 rough wall and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 9 free inlet/outlet
+!>                                 (input mass flux blocked to 0)
+!_______________________________________________________________________________
+
+subroutine clsyvt_vector &
+ ( iscal  , icodcl )
+
+!===============================================================================
+! Module files
+!===============================================================================
+
+use paramx
+use numvar
+use optcal
+use cstphy
+use cstnum
+use dimens, only: nvar
+use pointe
+use entsor
+use albase
+use parall
+use ppppar
+use ppthch
+use ppincl
+use cplsat
+use mesh
+use field
+use cs_c_bindings
+
+!===============================================================================
+
+implicit none
+
+! Arguments
+
+integer          iscal
+
+integer          icodcl(nfabor,nvar)
+
+! Local variables
+
+integer          ivar, f_id
+integer          ifac, iel
+integer          ifcvsl
+
+double precision rkl, visclc
+double precision distbf, srfbnf
+double precision rnx, rny, rnz, temp
+double precision hintt(6)
+double precision turb_schmidt
+
+double precision, dimension(:), pointer :: crom
+double precision, dimension(:,:), pointer :: coefav, cofafv
+double precision, dimension(:,:,:), pointer :: coefbv, cofbfv
+double precision, dimension(:,:), pointer :: visten
+
+double precision, dimension(:), pointer :: viscl, visct, viscls
+
+type(var_cal_opt) :: vcopt
+
+!===============================================================================
+
+ivar = isca(iscal)
+f_id = ivarfl(ivar)
+
+call field_get_key_struct_var_cal_opt(f_id, vcopt)
+
+if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
+  if (iturb.ne.32) then
+    call field_get_val_v(ivsten, visten)
+  else ! EBRSM and (GGDH or AFM)
+    call field_get_val_v(ivstes, visten)
+  endif
+endif
+
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
+
+call field_get_coefa_v(f_id, coefav)
+call field_get_coefb_v(f_id, coefbv)
+call field_get_coefaf_v(f_id, cofafv)
+call field_get_coefbf_v(f_id, cofbfv)
+
+call field_get_val_s(icrom, crom)
+
+call field_get_key_int (f_id, kivisl, ifcvsl)
+if (ifcvsl .ge. 0) then
+  call field_get_val_s(ifcvsl, viscls)
+endif
+
+! retrieve turbulent Schmidt value for current scalar
+call field_get_key_double(ivarfl(ivar), ksigmas, turb_schmidt)
+
+! --- Loop on boundary faces
+do ifac = 1, nfabor
+
+  ! Test on symmetry boundary condition: start
+  if (icodcl(ifac,ivar).eq.4) then
+
+    iel = ifabor(ifac)
+    ! --- Physical Propreties
+    visclc = viscl(iel)
+
+    ! --- Geometrical quantities
+    distbf = distb(ifac)
+    srfbnf = surfbn(ifac)
+
+    rnx = surfbo(1,ifac)/srfbnf
+    rny = surfbo(2,ifac)/srfbnf
+    rnz = surfbo(3,ifac)/srfbnf
+
+    if (ifcvsl .lt. 0) then
+      rkl = visls0(iscal)
+    else
+      rkl = viscls(iel)
+    endif
+
+    ! Isotropic diffusivity
+    if (iand(vcopt%idften, ISOTROPIC_DIFFUSION).ne.0) then
+      hintt(1) = (vcopt%idifft*max(visct(iel),zero)/turb_schmidt + rkl)/distbf
+      hintt(2) = hintt(1)
+      hintt(3) = hintt(1)
+      hintt(4) = hintt(1)
+      hintt(5) = hintt(1)
+      hintt(6) = hintt(1)
+    ! Symmetric tensor diffusivity
+    elseif (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
+      temp = vcopt%idifft*ctheta(iscal)/csrij
+      hintt(1) = (temp*visten(1,iel) + rkl)/distbf
+      hintt(2) = (temp*visten(2,iel) + rkl)/distbf
+      hintt(3) = (temp*visten(3,iel) + rkl)/distbf
+      hintt(4) =  temp*visten(4,iel)       /distbf
+      hintt(5) =  temp*visten(5,iel)       /distbf
+      hintt(6) =  temp*visten(6,iel)       /distbf
+    endif
+
+    ! Gradient BCs
+    coefav(1,ifac) = 0.d0
+    coefav(2,ifac) = 0.d0
+    coefav(3,ifac) = 0.d0
+
+    coefbv(1,1,ifac) = 1.d0-rnx**2
+    coefbv(2,2,ifac) = 1.d0-rny**2
+    coefbv(3,3,ifac) = 1.d0-rnz**2
+
+    coefbv(1,2,ifac) = -rnx*rny
+    coefbv(1,3,ifac) = -rnx*rnz
+    coefbv(2,1,ifac) = -rny*rnx
+    coefbv(2,3,ifac) = -rny*rnz
+    coefbv(3,1,ifac) = -rnz*rnx
+    coefbv(3,2,ifac) = -rnz*rny
+
+    ! Flux BCs
+    cofafv(1,ifac) = 0.d0
+    cofafv(2,ifac) = 0.d0
+    cofafv(3,ifac) = 0.d0
+
+    cofbfv(1,1,ifac) = hintt(1)*rnx**2  + hintt(4)*rnx*rny + hintt(6)*rnx*rnz
+    cofbfv(2,2,ifac) = hintt(4)*rnx*rny + hintt(2)*rny**2  + hintt(5)*rny*rnz
+    cofbfv(3,3,ifac) = hintt(6)*rnx*rnz + hintt(5)*rny*rnz + hintt(3)*rnz**2
+
+    cofbfv(1,2,ifac) = hintt(1)*rnx*rny + hintt(4)*rny**2  + hintt(6)*rny*rnz
+    cofbfv(2,1,ifac) = hintt(1)*rnx*rny + hintt(4)*rny**2  + hintt(6)*rny*rnz
+    cofbfv(1,3,ifac) = hintt(1)*rnx*rnz + hintt(4)*rny*rnz + hintt(6)*rnz**2
+    cofbfv(3,1,ifac) = hintt(1)*rnx*rnz + hintt(4)*rny*rnz + hintt(6)*rnz**2
+    cofbfv(2,3,ifac) = hintt(4)*rnx*rnz + hintt(2)*rny*rnz + hintt(5)*rnz**2
+    cofbfv(3,2,ifac) = hintt(4)*rnx*rnz + hintt(2)*rny*rnz + hintt(5)*rnz**2
+
   endif
   ! Test on velocity symmetry condition: end
 
