@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -42,7 +42,7 @@ subroutine cou1di &
 ! nfabor           ! i  ! <-- ! number of boundary faces                       !
 ! isvtb            ! e  ! <-- ! numero du scalaire couple                      !
 ! icodcl           ! te ! --> ! code de condition limites aux faces            !
-!  (nfabor,nvarcl) !    !     !  de bord                                       !
+!  (nfabor,nvar)   !    !     !  de bord                                       !
 !                  !    !     ! = 1   -> dirichlet                             !
 !                  !    !     ! = 3   -> densite de flux                       !
 !                  !    !     ! = 4   -> glissemt et u.n=0 (vitesse)           !
@@ -51,7 +51,7 @@ subroutine cou1di &
 !                  !    !     ! = 9   -> entree/sortie libre (vitesse          !
 !                  !    !     !  entrante eventuelle     bloquee               !
 ! rcodcl           ! tr ! --> ! valeur des conditions aux limites              !
-!  (nfabor,nvarcl) !    !     !  aux faces de bord                             !
+!  (nfabor,nvar)   !    !     !  aux faces de bord                             !
 !                  !    !     ! rcodcl(1) = valeur du dirichlet                !
 !                  !    !     ! rcodcl(2) = valeur du coef. d'echange          !
 !                  !    !     !  ext. (infinie si pas d'echange)               !
@@ -61,7 +61,7 @@ subroutine cou1di &
 !                  !    !     ! pour les vitesses (vistl+visct)*gradu          !
 !                  !    !     ! pour la pression             dt*gradp          !
 !                  !    !     ! pour les scalaires                             !
-!                  !    !     !        cp*(viscls+visct/sigmas)*gradt          !
+!                  !    !     !        cp*(viscls+visct/turb_schmidt)*gradt    !
 !__________________!____!_____!________________________________________________!
 
 !     Type: i (integer), r (real), s (string), a (array), l (logical),
@@ -78,10 +78,12 @@ use numvar
 use optcal
 use cstnum
 use cstphy
+use dimens, only: nvar
 use entsor
 use pointe
 use field
 use radiat
+use cs_c_bindings
 
 !===============================================================================
 
@@ -90,8 +92,8 @@ implicit none
 ! Arguments
 
 integer          nfabor
-integer          isvtb  , icodcl(nfabor,nvarcl)
-double precision rcodcl(nfabor,nvarcl,3)
+integer          isvtb  , icodcl(nfabor,nvar)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
@@ -102,6 +104,9 @@ integer          icldef
 double precision, dimension(:), allocatable :: h_b
 double precision, dimension(:), pointer :: b_temp
 
+integer, dimension(:), pointer :: ifpt1d
+double precision, dimension(:), pointer :: tppt1d
+
 !===============================================================================
 
 interface
@@ -122,6 +127,10 @@ interface
 
 !===============================================================================
 
+! Get the 1D wall thermal module arrays
+call cs_1d_wall_thermal_get_faces(ifpt1d)
+call cs_1d_wall_thermal_get_temp(tppt1d)
+
 ! Update boundary temperature field for radiative transfer
 
 if (iirayo.ge.1.and.nfpt1d.gt.0) then
