@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -64,6 +64,11 @@ BEGIN_C_DECLS
 
 extern const int  *cs_glob_bc_type;
 
+/*! boundary zone number associated with each boundary face
+  (specific physical models)*/
+
+extern const int  *cs_glob_bc_face_zone;
+
 /*============================================================================
  * Public function prototypes
  *============================================================================*/
@@ -85,7 +90,7 @@ extern const int  *cs_glob_bc_type;
  * attribute than the boundary type, for appropriate error reporting.
  *
  * parameters:
- * \param[in]  bc_type     array of BC type ids
+ *   bc_flag   <-- array of BC type ids
  *   type_name <-- name of attribute in error, or NULL
  *----------------------------------------------------------------------------*/
 
@@ -156,12 +161,12 @@ cs_boundary_conditions_map(cs_mesh_location_type_t    location_type,
  *   faces           <-- list of selected boundary faces (0 to n-1),
  *                       or NULL if no indirection is needed
  *   balance_w       <-- optional balance weight, or NULL
- *   nvarcl          <-- number of variables requiring BC's
+ *   nvar            <-- number of variables requiring BC's
  *   rcodcl          <-> boundary condition values
  *----------------------------------------------------------------------------*/
 
 void
-cs_boundary_conditions_mapped_set(cs_field_t                *f,
+cs_boundary_conditions_mapped_set(const cs_field_t          *f,
                                   ple_locator_t             *locator,
                                   cs_mesh_location_type_t    location_type,
                                   int                        normalize,
@@ -169,22 +174,22 @@ cs_boundary_conditions_mapped_set(cs_field_t                *f,
                                   cs_lnum_t                  n_faces,
                                   const cs_lnum_t           *faces,
                                   cs_real_t                 *balance_w,
-                                  int                        nvarcl,
+                                  int                        nvar,
                                   cs_real_t                  rcodcl[]);
 
 /*----------------------------------------------------------------------------
- * Create the boundary conditions type array bc_type
+ * Create the boundary conditions face type and face zone arrays
  *----------------------------------------------------------------------------*/
 
 void
-cs_boundary_conditions_type_create(void);
+cs_boundary_conditions_create(void);
 
 /*----------------------------------------------------------------------------
- * Free the boundary conditions type array bc_type
+ * Free the boundary conditions face type and face zone arrays
  *----------------------------------------------------------------------------*/
 
 void
-cs_boundary_conditions_type_free(void);
+cs_boundary_conditions_free(void);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -265,7 +270,9 @@ cs_boundary_conditions_set_dirichlet_scalar(cs_real_t  *a,
   }
 }
 
-/*! \brief Set convective oulet boundary condition for a scalar
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set convective oulet boundary condition for a scalar
  *
  * Parameters:
  * \param[out]    coefa         explicit BC coefficient for gradients
@@ -278,13 +285,14 @@ cs_boundary_conditions_set_dirichlet_scalar(cs_real_t  *a,
  */
 /*----------------------------------------------------------------------------*/
 
-void cs_boundary_conditions_set_convective_outlet_scalar(cs_real_t *coefa ,
-                                                         cs_real_t *cofaf,
-                                                         cs_real_t *coefb,
-                                                         cs_real_t *cofbf,
-                                                         cs_real_t  pimp,
-                                                         cs_real_t  cfl,
-                                                         cs_real_t  hint);
+void
+cs_boundary_conditions_set_convective_outlet_scalar(cs_real_t *coefa ,
+                                                    cs_real_t *cofaf,
+                                                    cs_real_t *coefb,
+                                                    cs_real_t *cofbf,
+                                                    cs_real_t  pimp,
+                                                    cs_real_t  cfl,
+                                                    cs_real_t  hint);
 
 /*----------------------------------------------------------------------------*/
 
