@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -175,6 +175,40 @@ cs_field_interpolate(cs_field_t              *f,
                      const cs_real_3_t        point_coords[],
                      cs_real_t               *val);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Find local extrema of a given scalar field at each cell
+ *
+ * This assumes the field values have been synchronized.
+ *
+ * \param[in]      field id    The scalar field id
+ * \param[in]      halo_type   Halo type
+ * \param[in,out]  local_max   The local maximum value
+ * \param[in,out]  local_min   The local minimum value
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_field_local_extrema_scalar(int              f_id,
+                              cs_halo_type_t   halo_type,
+                              cs_real_t       *local_max,
+                              cs_real_t       *local_min);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Synchronize current parallel and periodic field values.
+ *
+ * This function currently only upates fields based on CS_MESH_LOCATION_CELLS.
+ *
+ * \param[in, out]   f           pointer to field
+ * \param[in]        halo_type   halo type
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_field_synchronize(cs_field_t      *f,
+                     cs_halo_type_t   halo_type);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
