@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,11 +32,14 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
+#include "cs_domain.h"
+#include "cs_gwf_tracer.h"
+#include "cs_gwf_soil.h"
 #include "cs_mesh.h"
 #include "cs_mesh_quantities.h"
 #include "cs_mesh_bad_cells.h"
-
-#include "cs_domain.h"
+#include "cs_probe.h"
+#include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -127,16 +130,6 @@ extern void CS_PROCF (findpt, FINDPT)
        cs_int_t   *ndrang    /* --> rank of associated process */
 );
 
-/*----------------------------------------------------------------------------
- * Generator for distribution function of p's
- *----------------------------------------------------------------------------*/
-
-extern void CS_PROCF (fische, FISCHE)
-(
- const cs_int_t   *n,
- const cs_real_t  *mu,
-       cs_int_t    p[]);
-
 /*----------------------------------------------------------------------------
  * Check necessity of extended mesh from FORTRAN options.
  *
@@ -161,10 +154,13 @@ extern void CS_PROCF (initi1, INITI1)
 );
 
 /*----------------------------------------------------------------------------
- * Free Fortran allocated memory
+ * Set the CDO mode in the FORTRAN part
  *----------------------------------------------------------------------------*/
 
-extern void CS_PROCF (memfin, MEMFIN) (void);
+extern void CS_PROCF (set_cdo_mode, SET_CDO_MODE)
+(
+ const cs_int_t   *mode     /* <-- -1: no CDO, 1: with CDO, 2: CDO only */
+);
 
 /*----------------------------------------------------------------------------
  * User function for enthalpy <-> temperature conversion
@@ -197,28 +193,37 @@ void CS_PROCF (usvpst, USVPST)
  const cs_int_t   lstfbr[]   /* <-- list of post-processed boundary faces */
 );
 
-extern void CS_PROCF (usipes, USIPES)
-(
- cs_int_t *nmodpp
-);
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute GUI-defined head losses for a given volume zone.
+ *
+ * Head loss tensor coefficients for each cell are organized as follows:
+ * cku11, cku22, cku33, cku12, cku13, cku23.
+ *
+ * \param[in]       zone  pointer to zone structure
+ * \param[in, out]  cku   head loss coefficients
+ */
+/*----------------------------------------------------------------------------*/
 
-/*-------------------------------------------------------------------------------
+void
+cs_user_head_losses(const cs_zone_t  *zone,
+                    cs_real_t         cku[][6]);
+
+/*----------------------------------------------------------------------------
  * Absorption coefficient for radiative module
- *-------------------------------------------------------------------------------*/
+ *----------------------------------------------------------------------------*/
 
 void
 cs_user_rad_transfer_absorption(const int         bc_type[],
-                                const int         izfrdp[],
                                 const cs_real_t   dt[],
                                 cs_real_t         ck[]);
 
-/*-------------------------------------------------------------------------------
+/*----------------------------------------------------------------------------
  * Compute the net radiation flux
- *-------------------------------------------------------------------------------*/
+ *----------------------------------------------------------------------------*/
 
 void
 cs_user_rad_transfer_net_flux(const int        itypfb[],
-                              const int        izfrdp[],
                               const cs_real_t  dt[],
                               const cs_real_t  coefap[],
                               const cs_real_t  coefbp[],
@@ -232,35 +237,6 @@ cs_user_rad_transfer_net_flux(const int        itypfb[],
                               const cs_real_t  ck[],
                               cs_real_t        net_flux[]);
 
-/*----------------------------------------------------------------------------
- * Uniform random number generator
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (zufalli, zufalli)
-(
- const cs_int_t   *n              /* --> size of the vector */
-);
-
-/*----------------------------------------------------------------------------
- * Uniform random number generator
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (zufall, zufall)
-(
- const cs_int_t   *n,             /* --> size of the vector */
- const cs_real_t  *a              /* <-- generated random number vector */
-);
-
-/*----------------------------------------------------------------------------
- * Gaussian random number generator
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (normalen, normalen)
-(
- const cs_int_t   *n,             /* --> size of the vector */
- const cs_real_t  *x              /* <-- generated random number vector */
-);
-
 /*----------------------------------------------------------------------------
  * Convert temperature to enthalpy at boundary
  *----------------------------------------------------------------------------*/
@@ -272,7 +248,7 @@ void CS_PROCF (b_t_to_h, b_t_to_h)
                                     conversion is requested */
  const cs_real_t *t_b,           /* --> temperature at boundary */
  cs_real_t       *h_b            /* --> enthalpy at boundary */
- );
+);
 
 /*----------------------------------------------------------------------------
  * Convert enthalpy to temperature at cells
@@ -282,7 +258,7 @@ void CS_PROCF (c_h_to_t, c_h_to_t)
 (
  const cs_real_t *h,           /* --> enthalpy */
  cs_real_t       *t            /* --> temperature */
- );
+);
 
 /*----------------------------------------------------------------------------
  * Add field indexes associated with a new non-user solved variable,
@@ -298,22 +274,6 @@ void CS_PROCF (c_h_to_t, c_h_to_t)
 int
 cs_add_model_field_indexes(int f_id);
 
-/*----------------------------------------------------------------------------
- * Map a field to time plot (probes) activation array.
- *
- * This will set the "post_id" keyword for the given field, if not
- * already set.
-
- * parameters:
- *   f_id <--   field id
- *
- * returns:
- *   starting position of this field in the \ref ihisvr array.
- *----------------------------------------------------------------------------*/
-
-int
-cs_field_post_id(int f_id);
-
 /*----------------------------------------------------------------------------
  * Return Lagrangian model status.
  *
@@ -332,6 +292,14 @@ cs_lagr_status(int  *model_flag,
  *  User function prototypes
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Data Entry of the 1D wall thermal module.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_1d_wall_thermal(int iappel,
+                        int isuit1);
+
 /*----------------------------------------------------------------------------
  * Define global options for couplings.
  *
@@ -347,10 +315,9 @@ cs_user_coupling(void);
  *----------------------------------------------------------------------------*/
 
 void
-cs_user_boundary_conditions(int         nvarcl,
+cs_user_boundary_conditions(int         nvar,
                             int         icodcl[],
                             int         bc_type[],
-                            int         izfrdp[],
                             cs_real_t   rcodcl[]);
 
 /*----------------------------------------------------------------------------
@@ -370,6 +337,45 @@ cs_user_extra_operations(void);
 void
 cs_user_initialization(void);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define internal coupling options.
+ *
+ * Options are usually defined using cs_internal_coupling_add_entity.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_internal_coupling(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define volumes as internal coupling zones.
+ *
+ * These zones will be separated from the rest of the domain using automatically
+ * defined thin walls.
+ *
+ * \param[in, out] mesh  pointer to a cs_mesh_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_internal_coupling_add_volumes(cs_mesh_t  *mesh);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define volumesi from separated meshes as internal coupling zones.
+ *
+ * These zones must be disjoint and the face selection criteria must be
+ * specified.
+ *
+ * \param[in, out]  mesh  pointer to a cs_mesh_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_internal_coupling_from_disjoint_meshes(cs_mesh_t  *mesh);
+
 /*----------------------------------------------------------------------------
  * This function is called each time step to define physical properties.
  *----------------------------------------------------------------------------*/
@@ -378,6 +384,19 @@ void
 cs_user_physical_properties(const cs_mesh_t             *mesh,
                             const cs_mesh_quantities_t  *mesh_quantities);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute the porosity (volume factor \f$ \epsilon \f$
+ *        when the porosity model is activated
+ *        (iporos greater than 1 in cs_user_parameters.f90).
+ *
+ * This function is called at the begin of the simulation only.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_porosity(void);
+
 /*----------------------------------------------------------------------------
  * Define mesh joinings.
  *----------------------------------------------------------------------------*/
@@ -395,6 +414,15 @@ cs_user_join(void);
 void
 cs_user_linear_solvers(void);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define or modify output user parameters.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_output(void);
+
 /*----------------------------------------------------------------------------
  * Tag bad cells within the mesh based on geometric criteria.
  *----------------------------------------------------------------------------*/
@@ -418,11 +446,11 @@ void
 cs_user_mesh_modify(cs_mesh_t  *mesh);
 
 /*----------------------------------------------------------------------------
- * Insert thin wall into a mesh.
+ * Insert boundary wall into a mesh.
  *----------------------------------------------------------------------------*/
 
 void
-cs_user_mesh_thinwall(cs_mesh_t  *mesh);
+cs_user_mesh_boundary(cs_mesh_t  *mesh);
 
 /*----------------------------------------------------------------------------
  * Mesh smoothing.
@@ -516,10 +544,9 @@ cs_user_radiative_transfer_parameters(void);
  *-------------------------------------------------------------------------------*/
 
 void
-cs_user_radiative_transfer_bcs(int               nvarcl,
+cs_user_radiative_transfer_bcs(int               nvar,
                                const int         bc_type[],
                                int               icodcl[],
-                               int               izfrdp[],
                                int               isothp[],
                                cs_real_t        *tmin,
                                cs_real_t        *tmax,
@@ -573,6 +600,25 @@ cs_user_postprocess_probes(void);
 void
 cs_user_postprocess_meshes(void);
 
+/*----------------------------------------------------------------------------
+ * User function for output of values on a post-processing mesh.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_postprocess_values(const char            *mesh_name,
+                           int                    mesh_id,
+                           int                    cat_id,
+                           cs_probe_set_t        *probes,
+                           cs_lnum_t              n_cells,
+                           cs_lnum_t              n_i_faces,
+                           cs_lnum_t              n_b_faces,
+                           cs_lnum_t              n_vertices,
+                           const cs_lnum_t        cell_list[],
+                           const cs_lnum_t        i_face_list[],
+                           const cs_lnum_t        b_face_list[],
+                           const cs_lnum_t        vertex_list[],
+                           const cs_time_step_t  *ts);
+
 /*----------------------------------------------------------------------------
  * Override default frequency or calculation end based output.
  *
@@ -639,6 +685,32 @@ cs_user_turbomachinery(void);
 void
 cs_user_turbomachinery_rotor(void);
 
+/*----------------------------------------------------------------------------
+ * Define rotation velocity of rotor.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_turbomachinery_set_rotation_velocity(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define volume and surface zones.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_zones(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define scaling parameter for electric model
+*/
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_scaling_elec(const cs_mesh_t             *mesh,
+                     const cs_mesh_quantities_t  *mesh_quantities,
+                     cs_real_t                   *dt);
 
 /*============================================================================
  *  CDO User function prototypes
@@ -646,71 +718,17 @@ cs_user_turbomachinery_rotor(void);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Activate or not the CDO module
- */
-/*----------------------------------------------------------------------------*/
-
-bool
-cs_user_cdo_activated(void);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Specify additional mesh locations
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_user_cdo_add_mesh_locations(void);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Specify for the computational domain:
- *         -- which type of boundaries closed the computational domain
- *         -- the settings for the time step
+ * \brief  After the first step: cs_user_cdo_init_setup(), this second step
+ *         concludes the setup of properties, equations, source terms...
+ *         At this step, mesh quantities and connectivities are build as well
+ *         as the field arrays.
  *
  * \param[in, out]   domain    pointer to a cs_domain_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_user_cdo_init_domain(cs_domain_t   *domain);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Associate material property and/or convection field to user-defined
- *         equations and specify boundary conditions, source terms, initial
- *         values for these additional equations
- *
- * \param[in, out]   domain    pointer to a cs_domain_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_user_cdo_set_domain(cs_domain_t   *domain);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Setup advanced features concerning the way geometric quantities
- *         are built
- *
- * \return the type of computation to evaluate the cell center
- */
-/*----------------------------------------------------------------------------*/
-
-cs_cdo_cc_algo_t
-cs_user_cdo_geometric_settings(void);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Setup advanced features concerning the numerical parameters
- *         of the equation resolved during the computation
- *
- * \param[in, out]  domain  pointer to a cs_domain_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_user_cdo_numeric_settings(cs_domain_t   *domain);
+cs_user_cdo_finalize_setup(cs_domain_t   *domain);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -749,17 +767,32 @@ void
 cs_user_cdo_end_extra_op(const cs_domain_t     *domain);
 
 /*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Retrieve the bulk density related to a soil structure
+ *
+ * \param[in]  soil      pointer to a cs_gwf_soil_t structure
+ * \param[out] density   return value for the density
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_gwf_get_soil_density(const cs_gwf_soil_t   *soil,
+                             cs_real_t             *density);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Define scaling parameter for electric model
+ * \brief  Specify for each soil and tracer how is defined each term of the
+ *         the tracer equation. Soils and tracer equations have to be added
+ *         previously
+ *
+ * \param[in, out]   domain    pointer to a cs_domain_t structure
 */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_user_scaling_elec(const cs_mesh_t             *mesh,
-                     const cs_mesh_quantities_t  *mesh_quantities,
-                           cs_real_t             *dt);
+cs_user_gwf_setup(cs_domain_t   *domain);
+
+/*----------------------------------------------------------------------------*/
 
 END_C_DECLS
 
