@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -242,7 +242,7 @@ cs_selector_get_b_face_list(const char  *criteria,
 
     cs_mesh_t *mesh = cs_glob_mesh;
 
-    bool del_class_defs = (mesh->class_defs != NULL) ? true : false;
+    bool del_class_defs = (mesh->class_defs == NULL) ? true : false;
 
     cs_mesh_init_group_classes(mesh);
 
@@ -318,7 +318,7 @@ cs_selector_get_i_face_list(const char  *criteria,
 
     cs_mesh_t *mesh = cs_glob_mesh;
 
-    bool del_class_defs = (mesh->class_defs != NULL) ? true : false;
+    bool del_class_defs = (mesh->class_defs == NULL) ? true : false;
 
     cs_mesh_init_group_classes(mesh);
 
@@ -395,7 +395,7 @@ cs_selector_get_cell_list(const char  *criteria,
 
     cs_mesh_t *mesh = cs_glob_mesh;
 
-    bool del_class_defs = (mesh->class_defs != NULL) ? true : false;
+    bool del_class_defs = (mesh->class_defs == NULL) ? true : false;
 
     cs_mesh_init_group_classes(mesh);
 
@@ -457,7 +457,7 @@ cs_selector_get_cells_boundary(const char  *criteria,
   /* Mark cells inside zone selection */
 
   BFT_MALLOC(cell_list, mesh->n_cells, cs_lnum_t);
-  BFT_MALLOC(cell_flag, mesh->n_cells, cs_lnum_t);
+  BFT_MALLOC(cell_flag, mesh->n_cells_with_ghosts, cs_lnum_t);
 
   for (ii = 0; ii < mesh->n_cells; ii++)
     cell_flag[ii] = 0;
@@ -467,7 +467,7 @@ cs_selector_get_cells_boundary(const char  *criteria,
   cs_selector_get_cell_list(criteria, &n_cells, cell_list);
 
   for (ii = 0; ii < n_cells; ii++)
-    cell_flag[cell_list[ii] - 1] = 1;
+    cell_flag[cell_list[ii]] = 1;
 
   BFT_FREE(cell_list);
 
