@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2015 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -111,7 +111,7 @@ integer          inc, iccocg
 integer          nswrgp, imligp
 integer          f_id
 integer          nbrval, nclptr
-integer          ipccou, ipcfou, ntcam1
+integer          ntcam1
 
 double precision epsrgp, climgp, extrap
 double precision cfmax,cfmin, w1min, w2min, w3min
@@ -132,6 +132,8 @@ double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: brom, crom
 double precision, dimension(:), pointer :: viscl, visct, cpro_cour, cpro_four
 
+type(var_cal_opt) :: vcopt_u, vcopt_p
+
 !===============================================================================
 
 !===============================================================================
@@ -157,14 +159,12 @@ endif
 ! Allocate work arrays
 allocate(w1(ncelet), w2(ncelet), w3(ncelet))
 
-call field_get_val_s(iprpfl(iviscl), viscl)
-call field_get_val_s(iprpfl(ivisct), visct)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 call field_get_val_s(icrom, crom)
 call field_get_val_s(ibrom, brom)
-call field_get_val_s(iprpfl(icour), cpro_cour)
-call field_get_val_s(iprpfl(ifour), cpro_four)
-ipccou  = ipproc(icour)
-ipcfou  = ipproc(ifour)
+call field_get_val_s(icour, cpro_cour)
+call field_get_val_s(ifour, cpro_four)
 
 if (ntlist.gt.0) then
   modntl = mod(ntcabs,ntlist)
@@ -174,17 +174,19 @@ else
   modntl = 1
 endif
 
-if (                                                             &
-   .not. (iconv(iu).ge.1 .and. ipccou.ge.1) .and.                &
-   .not. (idiff(iu).ge.1 .and. ipcfou.ge.1) .and.                &
-   .not. ((iconv(iu).ge.1 .or.idiff(iu).ge.1).and.               &
-           (iwarnp.ge.2.or.modntl.eq.0)) .and.                   &
-   .not. (ippmod(icompf).ge.0.and.                               &
-           (iwarnp.ge.2.or.modntl.eq.0)) .and.                   &
-   .not. (idtvar.eq.-1.or.idtvar.eq.1.or.idtvar.eq.2.or.         &
-           ((iwarnp.ge.2.or.modntl.eq.0).and.                    &
-             (idiff(iu).ge.1.or.iconv(iu).ge.1                   &
-                               .or.ippmod(icompf).ge.0)))        &
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
+
+if (.not. (vcopt_u%iconv.ge.1 .and. icour.ge.1) .and.              &
+    .not. (vcopt_u%idiff.ge.1 .and. ifour.ge.1) .and.              &
+    .not. ((vcopt_u%iconv.ge.1 .or.vcopt_u%idiff.ge.1).and.        &
+           (iwarnp.ge.2.or.modntl.eq.0)) .and.                     &
+    .not. (ippmod(icompf).ge.0.and.                                &
+           (iwarnp.ge.2.or.modntl.eq.0)) .and.                     &
+    .not. (idtvar.eq.-1.or.idtvar.eq.1.or.idtvar.eq.2.or.          &
+           ((iwarnp.ge.2.or.modntl.eq.0).and.                      &
+            (vcopt_u%idiff.ge.1.or.vcopt_u%iconv.ge.1.or.          &
+             ippmod(icompf).ge.0)))                                &
    ) then
 
   return
@@ -213,9 +215,9 @@ endif
 ! 2. Compute the diffusivity at the faces
 !===============================================================================
 
-if (idiff(iu).ge. 1) then
+if (vcopt_u%idiff.ge.1) then
   do iel = 1, ncel
-    w1(iel) = viscl(iel) + idifft(iu)*visct(iel)
+    w1(iel) = viscl(iel) + vcopt_u%idifft*visct(iel)
   enddo
   call viscfa(imvisf, w1, viscf, viscb)
 
@@ -238,8 +240,8 @@ if (idtvar.ge.0) then
 
     if (bmasfl(ifac).lt.0.d0) then
       iel = ifabor(ifac)
-      hint = idiff(iu)*(  viscl(iel)                         &
-                        + idifft(iu)*visct(iel))/distb(ifac)
+      hint = vcopt_u%idiff*(  viscl(iel)                         &
+                        + vcopt_u%idifft*visct(iel))/distb(ifac)
       coefbt(ifac) = 0.d0
       cofbft(ifac) = hint
     else
@@ -268,7 +270,7 @@ else
 endif
 
 !===============================================================================
-! 4. Steady algorithme
+! 4. Steady algorithm
 !===============================================================================
 
 if (idtvar.ge.0) then
@@ -290,11 +292,11 @@ if (idtvar.ge.0) then
       coefbr(ifac) = 0.d0
     enddo
 
-    nswrgp = nswrgr(ipr)
-    imligp = imligr(ipr)
-    iwarnp = iwarni(ipr)
-    epsrgp = epsrgr(ipr)
-    climgp = climgr(ipr)
+    nswrgp = vcopt_p%nswrgr
+    imligp = vcopt_p%imligr
+    iwarnp = vcopt_p%iwarni
+    epsrgp = vcopt_p%epsrgr
+    climgp = vcopt_p%climgr
     extrap = 0.d0
 
     f_id = -1
@@ -328,7 +330,7 @@ if (idtvar.ge.0) then
     ! 4.1.1 Courant limitation
     ! ========================
 
-    if (coumax.gt.0.d0.and.iconv(iu).ge.1) then
+    if (coumax.gt.0.d0.and.vcopt_u%iconv.ge.1) then
 
       ! ICOU = 1 marque l'existence d'une limitation par le COURANT
       icou = 1
@@ -341,7 +343,8 @@ if (idtvar.ge.0) then
       isym = 2
 
       call matrdt &
-     ( iconv(iu), idiff0, isym, coefbt, cofbft, imasfl, bmasfl, viscf, viscb, dam)
+     ( vcopt_u%iconv, idiff0, isym, coefbt, cofbft, imasfl, bmasfl, viscf, &
+       viscb, dam)
 
       do iel = 1, ncel
         w1(iel) = dam(iel)/(crom(iel)*volume(iel))
@@ -374,7 +377,7 @@ if (idtvar.ge.0) then
     ! 4.1.2 Fourier limitation
     ! ========================
 
-    if (foumax.gt.0.d0.and.idiff(iu).ge.1) then
+    if (foumax.gt.0.d0.and.vcopt_u%idiff.ge.1) then
 
       ! IFOU = 1 marque l'existence d'une limitation par le FOURIER
       ifou = 1
@@ -387,7 +390,8 @@ if (idtvar.ge.0) then
 
       call matrdt &
       !==========
- (iconv0, idiff(iu), isym, coefbt, cofbft, imasfl, bmasfl, viscf, viscb, dam)
+ (iconv0, vcopt_u%idiff, isym, coefbt, cofbft, imasfl, bmasfl, &
+  viscf, viscb, dam)
 
       do iel = 1, ncel
         w2(iel) = dam(iel)/(crom(iel)*volume(iel))
@@ -568,7 +572,7 @@ if (idtvar.ge.0) then
       call log_iteration_clipping_field(flid, icfmin(1), icfmax(1), dt, dt,icfmin(1), icfmax(1))
 
       ttcabs = ttcabs + (dtloc - dt(1))
-      if (imobil.eq.1 .or. iturbo.eq.2) then
+      if (iturbo.eq.2) then
         ttcmob = ttcmob + (dtloc - dt(1))
       endif
 
@@ -638,7 +642,7 @@ if (idtvar.ge.0) then
 ! 4.2  CALCUL DU NOMBRE DE COURANT POUR AFFICHAGE
 !===============================================================================
 
-  if (iconv(iu).ge.1 .and. ipccou.ge.1) then
+  if (vcopt_u%iconv.ge.1 .and. icour.ge.1) then
 
     idiff0 = 0
     cnom   =' COURANT'
@@ -651,7 +655,8 @@ if (idtvar.ge.0) then
 
     call matrdt &
     !==========
- (iconv(iu), idiff0, isym, coefbt, cofbft, imasfl, bmasfl, viscf, viscb, dam)
+ (vcopt_u%iconv, idiff0, isym, coefbt, cofbft, imasfl, bmasfl, &
+  viscf, viscb, dam)
 
     do iel = 1, ncel
       w1(iel) = dam(iel)/(crom(iel)*volume(iel))
@@ -681,23 +686,30 @@ if (idtvar.ge.0) then
         endif
       enddo
 
-      xyzmin(1) = xyzcen(1,icfmin(1))
-      xyzmin(2) = xyzcen(2,icfmin(1))
-      xyzmin(3) = xyzcen(3,icfmin(1))
-      xyzmax(1) = xyzcen(1,icfmax(1))
-      xyzmax(2) = xyzcen(2,icfmax(1))
-      xyzmax(3) = xyzcen(3,icfmax(1))
+      xyzmin(1) = xyzcen(1,max(icfmin(1), 1))
+      xyzmin(2) = xyzcen(2,max(icfmin(1), 1))
+      xyzmin(3) = xyzcen(3,max(icfmin(1), 1))
+      xyzmax(1) = xyzcen(1,max(icfmax(1), 1))
+      xyzmax(2) = xyzcen(2,max(icfmax(1), 1))
+      xyzmax(3) = xyzcen(3,max(icfmax(1), 1))
 
       if (irangp.ge.0) then
         nbrval = 3
         call parmnl (nbrval, cfmin, xyzmin)
-        !==========
         call parmxl (nbrval, cfmax, xyzmax)
-        !==========
       endif
 
-      write(nfecra,1001) cnom, cfmax, xyzmax(1), xyzmax(2), xyzmax(3)
-      write(nfecra,1002) cnom, cfmin, xyzmin(1), xyzmin(2), xyzmin(3)
+      if (icfmin(1).gt.0) then
+        write(nfecra,1001) cnom,cfmax,xyzmax(1),xyzmax(2),xyzmax(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
+
+      if (icfmax(1).gt.0) then
+        write(nfecra,1002) cnom,cfmin,xyzmin(1),xyzmin(2),xyzmin(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
 
     endif
 
@@ -707,10 +719,10 @@ if (idtvar.ge.0) then
 ! 4.3  CALCUL DU NOMBRE DE FOURIER POUR AFFICHAGE
 !===============================================================================
 
-  if (idiff(iu).ge.1 .and. ipcfou.ge.1) then
+  if (vcopt_u%idiff.ge.1 .and. ifour.ge.1) then
 
     iconv0 = 0
-    CNOM   =' FOURIER'
+    cnom   =' FOURIER'
 
     !                              2
     ! CONSTRUCTION DE      +2.NU/DX  (       FOURIER) =W1
@@ -721,7 +733,8 @@ if (idtvar.ge.0) then
 
     call matrdt &
     !==========
- (iconv0, idiff(iu), isym, coefbt, cofbft, imasfl, bmasfl, viscf, viscb, dam)
+ (iconv0, vcopt_u%idiff, isym, coefbt, cofbft, imasfl, bmasfl, &
+  viscf, viscb, dam)
 
     do iel = 1, ncel
       w1(iel) = dam(iel)/(crom(iel)*volume(iel))
@@ -780,7 +793,8 @@ if (idtvar.ge.0) then
 !     En incompressible uniquement (en compressible, on preferera
 !       afficher la contrainte liee a la masse volumique)
 
-  if ((idiff(iu).ge.1.or.iconv(iu).ge.1) .and. (ippmod(icompf).lt.0)) then
+  if (      (vcopt_u%idiff.ge.1.or.vcopt_u%iconv.ge.1) &
+      .and. (ippmod(icompf).lt.0)) then
 
     cnom   =' COU/FOU'
 
@@ -790,11 +804,12 @@ if (idtvar.ge.0) then
     ! MATRICE A PRIORI NON SYMETRIQUE
 
     isym = 1
-    if (iconv(iu).gt.0) isym = 2
+    if (vcopt_u%iconv.gt.0) isym = 2
 
     call matrdt &
     !==========
- (iconv(iu), idiff(iu), isym, coefbt, cofbft, imasfl, bmasfl, viscf, viscb, dam)
+ (vcopt_u%iconv, vcopt_u%idiff, isym, coefbt, cofbft, imasfl, bmasfl, &
+  viscf, viscb, dam)
 
     do iel = 1, ncel
       w1(iel) = dam(iel)/(crom(iel)*volume(iel))
@@ -828,23 +843,30 @@ if (idtvar.ge.0) then
         endif
       enddo
 
-      xyzmin(1) = xyzcen(1,icfmin(1))
-      xyzmin(2) = xyzcen(2,icfmin(1))
-      xyzmin(3) = xyzcen(3,icfmin(1))
-      xyzmax(1) = xyzcen(1,icfmax(1))
-      xyzmax(2) = xyzcen(2,icfmax(1))
-      xyzmax(3) = xyzcen(3,icfmax(1))
+      xyzmin(1) = xyzcen(1,max(icfmin(1), 1))
+      xyzmin(2) = xyzcen(2,max(icfmin(1), 1))
+      xyzmin(3) = xyzcen(3,max(icfmin(1), 1))
+      xyzmax(1) = xyzcen(1,max(icfmax(1), 1))
+      xyzmax(2) = xyzcen(2,max(icfmax(1), 1))
+      xyzmax(3) = xyzcen(3,max(icfmax(1), 1))
 
       if (irangp.ge.0) then
         nbrval = 3
         call parmnl (nbrval, cfmin, xyzmin)
-        !==========
         call parmxl (nbrval, cfmax, xyzmax)
-        !==========
       endif
 
-      write(nfecra,1001) cnom,cfmax,xyzmax(1),xyzmax(2),xyzmax(3)
-      write(nfecra,1002) cnom,cfmin,xyzmin(1),xyzmin(2),xyzmin(3)
+      if (icfmin(1).gt.0) then
+        write(nfecra,1001) cnom,cfmax,xyzmax(1),xyzmax(2),xyzmax(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
+
+      if (icfmax(1).gt.0) then
+        write(nfecra,1002) cnom,cfmin,xyzmin(1),xyzmin(2),xyzmin(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
 
     endif
 
@@ -888,23 +910,30 @@ if (idtvar.ge.0) then
         endif
       enddo
 
-      xyzmin(1) = xyzcen(1,icfmin(1))
-      xyzmin(2) = xyzcen(2,icfmin(1))
-      xyzmin(3) = xyzcen(3,icfmin(1))
-      xyzmax(1) = xyzcen(1,icfmax(1))
-      xyzmax(2) = xyzcen(2,icfmax(1))
-      xyzmax(3) = xyzcen(3,icfmax(1))
+      xyzmin(1) = xyzcen(1,max(icfmin(1), 1))
+      xyzmin(2) = xyzcen(2,max(icfmin(1), 1))
+      xyzmin(3) = xyzcen(3,max(icfmin(1), 1))
+      xyzmax(1) = xyzcen(1,max(icfmax(1), 1))
+      xyzmax(2) = xyzcen(2,max(icfmax(1), 1))
+      xyzmax(3) = xyzcen(3,max(icfmax(1), 1))
 
       if (irangp.ge.0) then
         nbrval = 3
         call parmnl (nbrval, cfmin, xyzmin)
-        !==========
         call parmxl (nbrval, cfmax, xyzmax)
-        !==========
       endif
 
-      write(nfecra,1001) cnom,cfmax,xyzmax(1),xyzmax(2),xyzmax(3)
-      write(nfecra,1002) cnom,cfmin,xyzmin(1),xyzmin(2),xyzmin(3)
+      if (icfmin(1).gt.0) then
+        write(nfecra,1001) cnom,cfmax,xyzmax(1),xyzmax(2),xyzmax(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
+
+      if (icfmax(1).gt.0) then
+        write(nfecra,1002) cnom,cfmin,xyzmin(1),xyzmin(2),xyzmin(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
 
     endif
 
@@ -916,17 +945,17 @@ if (idtvar.ge.0) then
 else
 
   isym = 1
-  if (iconv(iu).gt.0) isym = 2
+  if (vcopt_u%iconv.gt.0) isym = 2
 
   call matrdt &
   !==========
- ( iconv(iu), idiff(iu), isym,                                  &
+ ( vcopt_u%iconv, vcopt_u%idiff, isym,                          &
    coefbt, cofbft,                                              &
    imasfl, bmasfl,                                              &
    viscf, viscb, dt )
 
   do iel = 1, ncel
-    dt(iel) =  relaxv(iu)*crom(iel)*volume(iel)/max(dt(iel),epzero)
+    dt(iel) =  vcopt_u%relaxv*crom(iel)*volume(iel)/max(dt(iel),epzero)
   enddo
 
 endif
