@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -69,7 +69,6 @@ use lagran
 use ppppar
 use ppthch
 use ppincl
-use radiat, only: iirayo
 use cplsat
 use mesh
 use field
@@ -86,7 +85,7 @@ implicit none
 ! Arguments
 
 integer          nummai , numtyp
-integer          nvar   , nscal
+integer          nvar
 integer          ncelps , nfbrps
 
 integer          lstcel(ncelps), lstfbr(nfbrps)
@@ -125,7 +124,7 @@ double precision, dimension(:), pointer :: cvar_pr
 ipp = 0
 
 !===============================================================================
-! 1.1. Fluid domain
+! Fluid domain
 !===============================================================================
 
 if (numtyp .eq. -1) then
@@ -134,21 +133,8 @@ if (numtyp .eq. -1) then
   call field_get_val_s(ivarfl(ipr), cvar_pr)
   call field_get_val_v(ivarfl(iu), vel)
 
-  !  1.1.2 Automatic additional variables
-  !  ------------------------------------
-
-  ! Wall distance (if LES+VanDriest or Rij+Echo or K-w SST)
-
-  if (ineedy.eq.1) then
-
-    idimt = 1
-    ientla = .true.
-    ivarpr = .true.
-
-    call post_write_var(nummai, 'DistWall', idimt, ientla, ivarpr,  &
-                        ntcabs, ttcabs, dispar, rbid, rbid)
-
-  endif
+  !  Automatic additional variables
+  !  ------------------------------
 
   ! Yplus (if LES+VanDriest)
 
@@ -167,9 +153,9 @@ if (numtyp .eq. -1) then
 
   endif
 
-  ! Vitesse et pression absolues en cas de calcul en repre relatif
+  ! Relative pressure and velocity in case of turbomachinery
 
-  if (icorio.eq.1) then
+  if (iturbo.ne.0) then
 
     call field_get_val_s(icrom, crom)
 
@@ -180,50 +166,13 @@ if (numtyp .eq. -1) then
     do iloc = 1, ncelps
 
       iel = lstcel(iloc)
-      call rotation_velocity(1, xyzcen(:,iel), vr)
-
-      tracel(iloc) = cvar_pr(iel) + &
-             0.5d0*crom(iel)*(vr(1)**2 + vr(2)**2 + vr(3)**2)
-
-    enddo
-
-    call post_write_var(nummai, 'Pressure', idimt, ientla, ivarpr,  &
-                        ntcabs, ttcabs, tracel, rbid, rbid)
-
-    idimt = 3
-    ientla = .true.
-    ivarpr = .false.
-
-    do iloc = 1, ncelps
-
-      iel = lstcel(iloc)
-      call rotation_velocity(1, xyzcen(:,iel), vr)
-
-      tracel(1 + (iloc-1)*idimt) = vel(1,iel) + vr(1)
-      tracel(2 + (iloc-1)*idimt) = vel(2,iel) + vr(2)
-      tracel(3 + (iloc-1)*idimt) = vel(3,iel) + vr(3)
-
-    enddo
-
-    call post_write_var(nummai, 'Velocity', idimt, ientla, ivarpr,  &
-                        ntcabs, ttcabs, tracel, rbid, rbid)
-
-  endif
-
-  ! Vitesse et pression relatives en cas de calcul en repre fixe
-
-  if (imobil.eq.1 .or. iturbo.eq.1 .or. iturbo.eq.2) then
-
-    call field_get_val_s(icrom, crom)
-
-    idimt = 1
-    ientla = .true.
-    ivarpr = .false.
-
-    do iloc = 1, ncelps
-
-      iel = lstcel(iloc)
-      call rotation_velocity(irotce(iel), xyzcen(:,iel), vr)
+      if (irotce(iel).gt.0) then
+        call rotation_velocity(irotce(iel), xyzcen(:,iel), vr)
+      else
+        vr(1) = 0
+        vr(2) = 0
+        vr(3) = 0
+      endif
 
       tracel(iloc) =   cvar_pr(iel) &
                      - crom(iel)*0.5d0*(vr(1)**2 + vr(2)**2 + vr(3)**2)
@@ -240,7 +189,13 @@ if (numtyp .eq. -1) then
     do iloc = 1, ncelps
 
       iel = lstcel(iloc)
-      call rotation_velocity(irotce(iel), xyzcen(:,iel), vr)
+      if (irotce(iel).gt.0) then
+        call rotation_velocity(irotce(iel), xyzcen(:,iel), vr)
+      else
+        vr(1) = 0
+        vr(2) = 0
+        vr(3) = 0
+      endif
 
       tracel(1 + (iloc-1)*idimt) = vel(1,iel) - vr(1)
       tracel(2 + (iloc-1)*idimt) = vel(2,iel) - vr(2)
@@ -253,9 +208,51 @@ if (numtyp .eq. -1) then
 
   endif
 
+  ! Absolute pressure and velocity in case of relative coordinate system
+
+  if (icorio.eq.1) then
+
+    call field_get_val_s(icrom, crom)
+
+    idimt = 1
+    ientla = .true.
+    ivarpr = .false.
+
+    do iloc = 1, ncelps
+
+      iel = lstcel(iloc)
+      call rotation_velocity(0, xyzcen(:,iel), vr)
+
+      tracel(iloc) = cvar_pr(iel) + &
+             0.5d0*crom(iel)*(vr(1)**2 + vr(2)**2 + vr(3)**2)
+
+    enddo
+
+    call post_write_var(nummai, 'Abs Pressure', idimt, ientla, ivarpr,  &
+                        ntcabs, ttcabs, tracel, rbid, rbid)
+
+    idimt = 3
+    ientla = .true.
+    ivarpr = .false.
+
+    do iloc = 1, ncelps
+
+      iel = lstcel(iloc)
+      call rotation_velocity(0, xyzcen(:,iel), vr)
+
+      tracel(1 + (iloc-1)*idimt) = vel(1,iel) + vr(1)
+      tracel(2 + (iloc-1)*idimt) = vel(2,iel) + vr(2)
+      tracel(3 + (iloc-1)*idimt) = vel(3,iel) + vr(3)
+
+    enddo
+
+    call post_write_var(nummai, 'Abs Velocity', idimt, ientla, ivarpr,  &
+                        ntcabs, ttcabs, tracel, rbid, rbid)
+
+  endif
 
 !===============================================================================
-! 1.2. Boundary
+! Boundary
 !===============================================================================
 
 else if (numtyp .eq. -2) then
@@ -277,13 +274,12 @@ else if (numtyp .eq. -2) then
 
     call field_get_key_int(fldid, keyvis, iflpst)
 
-    if (iand(iflpst, 2) .eq. 0) cycle ! nothing to do for this field
+    if (iand(iflpst, POST_BOUNDARY_NR) .eq. 0) cycle ! nothing to do
 
     call field_get_dim (fldid, idimt)
     call field_get_name(fldid, name80(4:80))
     name80(1:3) = 'bc_'
 
-
     !  Compute non-reconstructed values at boundary faces
 
     if (idimt.ne.1) then
@@ -485,30 +481,7 @@ else if (numtyp .eq. -2) then
 endif ! end of test on postprocessing mesh number
 
 !===============================================================================
-!     2.2. VARIABLES RADIATIVES AUX FRONTIERES
-!===============================================================================
-
-if (numtyp.eq.-2) then
-
-  if (iirayo.gt.0) then
-
-    do iloc = 1, nfbrps
-      ifac = lstfbr(iloc)
-      trafbr(iloc) = izfrad(ifac)
-    enddo
-
-    idimt  = 1
-    ientla = .true.
-    ivarpr = .false.
-
-    call post_write_var(nummai, 'radiative_boundary_zones', idimt,           &
-                        ientla, ivarpr, ntcabs, ttcabs, rbid, rbid, trafbr)
-
-  endif
-endif
-
-!===============================================================================
-! 2.3. Electric module variables
+! Electric module variables
 !===============================================================================
 
 if (numtyp.eq.-1) then
@@ -518,33 +491,10 @@ if (numtyp.eq.-1) then
 
     allocate(grad(3,ncelet))
 
-    if (.true.) then
-
-      ! Gradient of the real potential
-
-      call field_get_id('elec_pot_r', f_id)
-
-      inc = 1
-      iprev = 0
-      iccocg = 1
-
-      call field_gradient_scalar(f_id, iprev, imrgra, inc,                   &
-                                 iccocg,                                     &
-                                 grad)
-      idimt  = 3
-      ientla = .true.
-      ivarpr = .true.
-
-      call post_write_var(nummai, 'Pot_Gradient_R', idimt, ientla, ivarpr,   &
-                          ntcabs, ttcabs, grad, rbid, rbid)
-
-    endif
-
     ! For Joule Heating by direct conduction:
     !   gradient of the imaginary component of the potential
 
-    if (.true.                                                               &
-        .and. (ippmod(ieljou).eq.2 .or. ippmod(ieljou).eq.4)) then
+    if (ippmod(ieljou).eq.2 .or. ippmod(ieljou).eq.4) then
 
       call field_get_id('elec_pot_i', f_id)
 
@@ -568,8 +518,7 @@ if (numtyp.eq.-1) then
     ! For Joule heating by direct conduction:
     !   imaginary component of the current density
 
-    if (.true.                                                               &
-        .and. (ippmod(ieljou).eq.2 .or. ippmod(ieljou).eq.4)) then
+    if (ippmod(ieljou).eq.2 .or. ippmod(ieljou).eq.4) then
 
       call field_get_id('elec_pot_i', f_id)
 
@@ -604,83 +553,9 @@ if (numtyp.eq.-1) then
 
     endif
 
-    ! For electric arcs: electromagnetic field calculation
-
-    if (.true. .and. ippmod(ielarc).ge.2) then
-
-      ! Ax Component
-
-      call field_get_id('vec_potential_01', f_id)
-
-      inc = 1
-      iprev = 0
-      iccocg = 1
-
-      call field_gradient_scalar(f_id, iprev, imrgra, inc,                   &
-                                 iccocg,                                     &
-                                 grad)
-
-      ! B = rot A ( B = curl A)
-
-      do iloc = 1, ncelps
-        iel = lstcel(iloc)
-        tracel(1 + (iloc-1)*idimt) =  zero
-        tracel(2 + (iloc-1)*idimt) =  grad(3,iel)
-        tracel(3 + (iloc-1)*idimt) = -grad(2,iel)
-      enddo
-
-      ! Ay component
-
-      call field_get_id('vec_potential_02', f_id)
-
-      inc = 1
-      iprev = 0
-      iccocg = 1
-
-      call field_gradient_scalar(f_id, iprev, imrgra, inc,                   &
-                                 iccocg,                                     &
-                                 grad)
-
-      ! B = rot A (B = curl A)
-
-      do iloc = 1, ncelps
-        iel = lstcel(iloc)
-        tracel(1 + (iloc-1)*idimt) = tracel(1 + (iloc-1)*idimt) - grad(3,iel)
-        tracel(3 + (iloc-1)*idimt) = tracel(3 + (iloc-1)*idimt) + grad(1,iel)
-      enddo
-
-      ! Az component
-
-      call field_get_id('vec_potential_03', f_id)
-
-      inc = 1
-      iprev = 0
-      iccocg = 1
-
-      call field_gradient_scalar(f_id, iprev, imrgra, inc,                   &
-                                 iccocg,                                     &
-                                 grad)
-
-      ! B = rot A (B = curl A)
-
-      do iloc = 1, ncelps
-        iel = lstcel(iloc)
-        tracel(1 + (iloc-1)*idimt) = tracel(1 + (iloc-1)*idimt) + grad(2,iel)
-        tracel(2 + (iloc-1)*idimt) = tracel(2 + (iloc-1)*idimt) - grad(1,iel)
-      enddo
-
-      idimt  = 3
-      ientla = .true.
-      ivarpr = .false.
-
-      call post_write_var(nummai, 'Magnetic_field', idimt, ientla, ivarpr,  &
-                          ntcabs, ttcabs, tracel, rbid, rbid)
-
-    endif
-
     ! Calculation of Module and Argument of the complex potential if IELJOU = 4
 
-    if (.true. .and. ippmod(ieljou).eq.4) then
+    if (ippmod(ieljou).eq.4) then
 
       ivar = 0
 
@@ -741,10 +616,6 @@ if (numtyp.eq.-1) then
 
 endif
 
-!--------
-! Formats
-!--------
-
 !----
 ! End
 !----
