@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -95,6 +95,46 @@ module field
 
     !---------------------------------------------------------------------------
 
+    ! Interface to C function assigning integer bit values to a key
+
+    !> \brief Set integer bits matching a mask to 1 for a given key for a field.
+
+    !> If the key id is not valid, or the value type or field category is not
+    !> compatible, a fatal error is provoked.
+
+    !> \param[in]   f_id  field id
+    !> \param[in]   k_id  id of associated key
+    !> \param[in]   mask  associated mask
+
+    subroutine field_set_key_int_bits(f_id, k_id, k_value)  &
+      bind(C, name='cs_f_field_set_key_int_bits')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      integer(c_int), value :: f_id, k_id, k_value
+    end subroutine field_set_key_int_bits
+
+    !---------------------------------------------------------------------------
+
+    ! Interface to C function assigning integer bit values to a key
+
+    !> \brief Set integer bits matching a mask to 0 for a given key for a field.
+
+    !> If the key id is not valid, or the value type or field category is not
+    !> compatible, a fatal error is provoked.
+
+    !> \param[in]   f_id  field id
+    !> \param[in]   k_id  id of associated key
+    !> \param[in]   mask  associated mask
+
+    subroutine field_clear_key_int_bits(f_id, k_id, k_value)  &
+      bind(C, name='cs_f_field_clear_key_int_bits')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      integer(c_int), value :: f_id, k_id, k_value
+    end subroutine field_clear_key_int_bits
+
+    !---------------------------------------------------------------------------
+
     ! Interface to C function assigning floating-point value to a key
 
     !> \brief Assign a floating point value for a given key to a field.
@@ -362,6 +402,23 @@ module field
 
     !---------------------------------------------------------------------------
 
+    ! Interface to C function returning field's value pointer and dimensions.
+
+    ! If the field id is not valid, a fatal error is provoked.
+
+    subroutine cs_f_field_var_ptr_by_id_try(id, p_type, p_rank, f_dim, c_p)  &
+      bind(C, name='cs_f_field_var_ptr_by_id_try')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      integer(c_int), value        :: id
+      integer(c_int), value        :: p_type
+      integer(c_int), value        :: p_rank
+      integer(c_int), dimension(2) :: f_dim
+      type(c_ptr), intent(out)     :: c_p
+    end subroutine cs_f_field_var_ptr_by_id_try
+
+    !---------------------------------------------------------------------------
+
     ! Interface to C function returning field's boundary condition
     ! coefficient values pointer and dimensions.
 
@@ -773,7 +830,7 @@ contains
   !> \param[in]   f_id  field id
   !> \param[out]  name  field's name
 
-  subroutine field_get_name (f_id, name)
+  subroutine field_get_name(f_id, name)
 
     use, intrinsic :: iso_c_binding
     implicit none
@@ -1207,7 +1264,7 @@ contains
   !> \param[in]   name  key name
   !> \param[out]  id    associated key id
 
-  subroutine field_get_key_id (name, id)
+  subroutine field_get_key_id(name, id)
 
     use, intrinsic :: iso_c_binding
     implicit none
@@ -1242,7 +1299,7 @@ contains
   !> \param[in]   k_id     id of associated key
   !> \param[out]  k_value  integer value associated with key id for this field
 
-  subroutine field_get_key_int (f_id, k_id, k_value)
+  subroutine field_get_key_int(f_id, k_id, k_value)
 
     use, intrinsic :: iso_c_binding
     implicit none
@@ -1278,7 +1335,7 @@ contains
   !> \param[in]   k_name   key name
   !> \param[out]  k_value  integer value associated with key id for this field
 
-  subroutine field_get_key_int_by_name (f_id, k_name, k_value)
+  subroutine field_get_key_int_by_name(f_id, k_name, k_value)
 
     use, intrinsic :: iso_c_binding
     implicit none
@@ -1319,7 +1376,7 @@ contains
   !> \param[in]   k_id     id of associated key
   !> \param[out]  k_value  integer value associated with key id for this field
 
-  subroutine field_get_key_double (f_id, k_id, k_value)
+  subroutine field_get_key_double(f_id, k_id, k_value)
 
     use, intrinsic :: iso_c_binding
     implicit none
@@ -1356,7 +1413,7 @@ contains
   !> \param[in]   k_id  id of associated key
   !> \param[in]   str   string associated with key
 
-  subroutine field_set_key_str (f_id, k_id, str)
+  subroutine field_set_key_str(f_id, k_id, str)
 
     use, intrinsic :: iso_c_binding
     implicit none
@@ -1392,7 +1449,7 @@ contains
   !> \param[in]   k_id  id of associated key
   !> \param[out]  str   string associated with key
 
-  subroutine field_get_key_str (f_id, k_id, str)
+  subroutine field_get_key_str(f_id, k_id, str)
 
     use, intrinsic :: iso_c_binding
     implicit none
@@ -1433,7 +1490,7 @@ contains
   ! compared strings are identical except for the last character, respectively
   ! Y, y, V,v, or 2 and Z, z, W, w, or 3.
 
-  subroutine fldsnv (name1, name2, name3)
+  subroutine fldsnv(name1, name2, name3)
 
     implicit none
 
@@ -1523,7 +1580,7 @@ contains
   !> \param[in]   f_id  field id
   !> \param[out]  str   string associated with key
 
-  subroutine field_get_label (f_id, str)
+  subroutine field_get_label(f_id, str)
 
     use, intrinsic :: iso_c_binding
     implicit none
@@ -1564,13 +1621,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to scalar field values
 
-  subroutine field_get_val_s (field_id, p)
+  subroutine field_get_val_s(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                  :: field_id
-    double precision, dimension(:), pointer, intent(out) :: p
+    integer, intent(in)                                    :: field_id
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1594,13 +1651,13 @@ contains
   !> \param[in]     name      name of given field (which must be scalar)
   !> \param[out]    p         pointer to scalar field values
 
-  subroutine field_get_val_s_by_name (name, p)
+  subroutine field_get_val_s_by_name(name, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    character(len=*), intent(in)                         :: name
-    double precision, dimension(:), pointer, intent(out) :: p
+    character(len=*), intent(in)                           :: name
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1629,13 +1686,13 @@ contains
   !> \param[out]    p         pointer to scalar field values at the previous
   !>                          iteration
 
-  subroutine field_get_val_prev_s_by_name (name, p)
+  subroutine field_get_val_prev_s_by_name(name, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    character(len=*), intent(in)                         :: name
-    double precision, dimension(:), pointer, intent(out) :: p
+    character(len=*), intent(in)                           :: name
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1664,13 +1721,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be vectorial)
   !> \param[out]    p         pointer to vector field values
 
-  subroutine field_get_val_v (field_id, p)
+  subroutine field_get_val_v(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                    :: field_id
-    double precision, dimension(:,:), pointer, intent(out) :: p
+    integer, intent(in)                                      :: field_id
+    double precision, dimension(:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1694,13 +1751,13 @@ contains
   !> \param[in]     name      name of given field (which must be vectorial)
   !> \param[out]    p         pointer to scalar field values
 
-  subroutine field_get_val_v_by_name (name, p)
+  subroutine field_get_val_v_by_name(name, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    character(len=*), intent(in)                           :: name
-    double precision, dimension(:,:), pointer, intent(out) :: p
+    character(len=*), intent(in)                             :: name
+    double precision, dimension(:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1720,6 +1777,40 @@ contains
 
   end subroutine field_get_val_v_by_name
 
+  !=============================================================================
+
+  !> \brief Return pointer to the array's previous values of a given vector
+  !>        field
+
+  !> \param[in]     name      name of given field (which must be vectorial)
+  !> \param[out]    p         pointer to vector field values at the previous
+  !>                          iteration
+
+  subroutine field_get_val_prev_v_by_name(name, p)
+
+    use, intrinsic :: iso_c_binding
+    implicit none
+
+    character(len=*), intent(in)                             :: name
+    double precision, dimension(:,:), pointer, intent(inout) :: p
+
+    ! Local variables
+
+    character(len=len_trim(name)+1, kind=c_char) :: c_name
+    integer(c_int) :: f_id, p_type, p_rank
+    integer(c_int), dimension(2) :: f_dim
+    type(c_ptr) :: c_p
+
+    c_name = trim(name)//c_null_char
+
+    f_id = cs_f_field_id_by_name(c_name)
+    p_type = 2
+    p_rank = 2
+
+    call cs_f_field_var_ptr_by_id(f_id, p_type, p_rank, f_dim, c_p)
+    call c_f_pointer(c_p, p, [f_dim(1), f_dim(2)])
+
+  end subroutine field_get_val_prev_v_by_name
 
   !=============================================================================
 
@@ -1728,13 +1819,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to previous scalar field values
 
-  subroutine field_get_val_prev_s (field_id, p)
+  subroutine field_get_val_prev_s(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                  :: field_id
-    double precision, dimension(:), pointer, intent(out) :: p
+    integer, intent(in)                                    :: field_id
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1753,18 +1844,49 @@ contains
 
   !=============================================================================
 
-  !> \brief Return pointer to the previous values array of a given vector field
+  !> \brief Return pointer to the previous values array of a given scalar field
+  !> if it exists, to the current value otherwise
 
-  !> \param[in]     field_id  id of given field (which must be vectorial)
-  !> \param[out]    p         pointer to previous vector field values
+  !> \param[in]     field_id  id of given field (which must be scalar)
+  !> \param[out]    p         pointer to previous scalar field values
 
-  subroutine field_get_val_prev_v (field_id, p)
+  subroutine field_get_val_prev_s_try(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
     integer, intent(in)                                    :: field_id
-    double precision, dimension(:,:), pointer, intent(out) :: p
+    double precision, dimension(:), pointer, intent(inout) :: p
+
+    ! Local variables
+
+    integer(c_int) :: f_id, p_type, p_rank
+    integer(c_int), dimension(3) :: f_dim
+    type(c_ptr) :: c_p
+
+    f_id = field_id
+    p_type = 2
+    p_rank = 1
+
+    call cs_f_field_var_ptr_by_id_try(f_id, p_type, p_rank, f_dim, c_p)
+    call c_f_pointer(c_p, p, [f_dim(1)])
+
+  end subroutine field_get_val_prev_s_try
+
+  !=============================================================================
+
+  !> \brief Return pointer to the previous values array of a given vector field
+
+  !> \param[in]     field_id  id of given field (which must be vectorial)
+  !> \param[out]    p         pointer to previous vector field values
+
+  subroutine field_get_val_prev_v(field_id, p)
+
+    use, intrinsic :: iso_c_binding
+    implicit none
+
+    integer, intent(in)                                      :: field_id
+    double precision, dimension(:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1788,13 +1910,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to scalar field BC coefa values
 
-  subroutine field_get_coefa_s (field_id, p)
+  subroutine field_get_coefa_s(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                  :: field_id
-    double precision, dimension(:), pointer, intent(out) :: p
+    integer, intent(in)                                    :: field_id
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1818,13 +1940,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be a vector)
   !> \param[out]    p         pointer to vector field BC coefa values
 
-  subroutine field_get_coefa_v (field_id, p)
+  subroutine field_get_coefa_v(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                    :: field_id
-    double precision, dimension(:,:), pointer, intent(out) :: p
+    integer, intent(in)                                      :: field_id
+    double precision, dimension(:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1849,13 +1971,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to vector field BC coefa values
 
-  subroutine field_get_coefad_s (field_id, p)
+  subroutine field_get_coefad_s(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                  :: field_id
-    double precision, dimension(:), pointer, intent(out) :: p
+    integer, intent(in)                                    :: field_id
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1880,13 +2002,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to vector field BC coefa values
 
-  subroutine field_get_coefad_v (field_id, p)
+  subroutine field_get_coefad_v(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                    :: field_id
-    double precision, dimension(:,:), pointer, intent(out) :: p
+    integer, intent(in)                                      :: field_id
+    double precision, dimension(:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1910,7 +2032,7 @@ contains
   !> \param[in]     field_id  id of given field (which must be a vector)
   !> \param[out]    p         pointer to vector field BC coefa values
 
-  subroutine field_get_coefac_v (field_id, p)
+  subroutine field_get_coefac_v(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
@@ -1940,13 +2062,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to scalar field BC coefa values
 
-  subroutine field_get_coefb_s (field_id, p)
+  subroutine field_get_coefb_s(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                  :: field_id
-    double precision, dimension(:), pointer, intent(out) :: p
+    integer, intent(in)                                    :: field_id
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -1970,13 +2092,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to scalar field BC coefbc values
 
-  subroutine field_get_coefbc_s (field_id, p)
+  subroutine field_get_coefbc_s(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                  :: field_id
-    double precision, dimension(:), pointer, intent(out) :: p
+    integer, intent(in)                                    :: field_id
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2000,13 +2122,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be a vector)
   !> \param[out]    p         pointer to vector field BC coefa values
 
-  subroutine field_get_coefb_uv (field_id, p)
+  subroutine field_get_coefb_uv(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                    :: field_id
-    double precision, dimension(:,:), pointer, intent(out) :: p
+    integer, intent(in)                                      :: field_id
+    double precision, dimension(:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2030,13 +2152,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be a vector)
   !> \param[out]    p         pointer to vector field BC coefbc values
 
-  subroutine field_get_coefbc_uv (field_id, p)
+  subroutine field_get_coefbc_uv(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                    :: field_id
-    double precision, dimension(:,:), pointer, intent(out) :: p
+    integer, intent(in)                                      :: field_id
+    double precision, dimension(:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2060,13 +2182,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be a vector)
   !> \param[out]    p         pointer to vector field BC coefa values
 
-  subroutine field_get_coefb_v (field_id, p)
+  subroutine field_get_coefb_v(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                      :: field_id
-    double precision, dimension(:,:,:), pointer, intent(out) :: p
+    integer, intent(in)                                        :: field_id
+    double precision, dimension(:,:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2090,13 +2212,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be a vector)
   !> \param[out]    p         pointer to vector field BC coefa values
 
-  subroutine field_get_coefbc_v (field_id, p)
+  subroutine field_get_coefbc_v(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                      :: field_id
-    double precision, dimension(:,:,:), pointer, intent(out) :: p
+    integer, intent(in)                                        :: field_id
+    double precision, dimension(:,:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2120,13 +2242,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to scalar field BC coefa values
 
-  subroutine field_get_coefaf_s (field_id, p)
+  subroutine field_get_coefaf_s(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                  :: field_id
-    double precision, dimension(:), pointer, intent(out) :: p
+    integer, intent(in)                                    :: field_id
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2150,13 +2272,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to scalar field BC coefa values
 
-  subroutine field_get_coefac_s (field_id, p)
+  subroutine field_get_coefac_s(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                  :: field_id
-    double precision, dimension(:), pointer, intent(out) :: p
+    integer, intent(in)                                    :: field_id
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2180,13 +2302,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to vector field BC coefa values
 
-  subroutine field_get_coefaf_v (field_id, p)
+  subroutine field_get_coefaf_v(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                    :: field_id
-    double precision, dimension(:,:), pointer, intent(out) :: p
+    integer, intent(in)                                      :: field_id
+    double precision, dimension(:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2210,13 +2332,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to scalar field BC coefb values
 
-  subroutine field_get_coefbf_s (field_id, p)
+  subroutine field_get_coefbf_s(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                  :: field_id
-    double precision, dimension(:), pointer, intent(out) :: p
+    integer, intent(in)                                    :: field_id
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2240,13 +2362,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to vector field BC coefb values
 
-  subroutine field_get_coefbf_uv (field_id, p)
+  subroutine field_get_coefbf_uv(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                    :: field_id
-    double precision, dimension(:,:), pointer, intent(out) :: p
+    integer, intent(in)                                      :: field_id
+    double precision, dimension(:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2270,13 +2392,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to vector field BC coefb values
 
-  subroutine field_get_coefbf_v (field_id, p)
+  subroutine field_get_coefbf_v(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                      :: field_id
-    double precision, dimension(:,:,:), pointer, intent(out) :: p
+    integer, intent(in)                                        :: field_id
+    double precision, dimension(:,:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2301,13 +2423,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to vector field BC coefb values
 
-  subroutine field_get_coefbd_s (field_id, p)
+  subroutine field_get_coefbd_s(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                  :: field_id
-    double precision, dimension(:), pointer, intent(out) :: p
+    integer, intent(in)                                    :: field_id
+    double precision, dimension(:), pointer, intent(inout) :: p
 
     ! Local variables
 
@@ -2332,13 +2454,13 @@ contains
   !> \param[in]     field_id  id of given field (which must be scalar)
   !> \param[out]    p         pointer to vector field BC coefa values
 
-  subroutine field_get_coefbd_v (field_id, p)
+  subroutine field_get_coefbd_v(field_id, p)
 
     use, intrinsic :: iso_c_binding
     implicit none
 
-    integer, intent(in)                                      :: field_id
-    double precision, dimension(:,:,:), pointer, intent(out) :: p
+    integer, intent(in)                                        :: field_id
+    double precision, dimension(:,:,:), pointer, intent(inout) :: p
 
     ! Local variables
 
