@@ -3,7 +3,7 @@
 !     This file is part of the Code_Saturne Kernel, element of the
 !     Code_Saturne CFD tool.
 
-!     Copyright (C) 1998-2016 EDF S.A., France
+!     Copyright (C) 1998-2018 EDF S.A., France
 
 !     contact: saturne-support@edf.fr
 
@@ -68,6 +68,7 @@ use ihmpre
 use radiat
 use cplsat
 use mesh
+use post
 use field
 use cs_c_bindings
 use darcy_module
@@ -81,27 +82,27 @@ implicit none
 ! Local variables
 
 integer          ii, ivar
-integer          keycpl, iflid, kcvlim, ifctsl
-integer          kdiftn, kturt, kfturt, kislts, keyvar
+integer          iflid, kcvlim, ifctsl, clip_id
+integer          kturt, kfturt, kislts, keyvar, kclipp
 integer          itycat, ityloc, idim1, idim3, idim6
 logical          iprev, inoprv
-integer          f_id, kscavr, f_vis, f_log, f_dften, f_type
-integer          iopchr
+integer          f_id, kscavr, f_type
+integer          iopchr, ilog
 integer          iscdri, icla, iclap
 integer          keyccl, keydri
 integer          idfm, iggafm, nfld
 integer          iflidp, idimf, n_fans
+integer          f_dim
 
 character(len=80) :: name, f_name
 
 type(gas_mix_species_prop) sasp
-
-type(var_cal_opt) vcopt
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 
 !===============================================================================
-! 1. Initialisation
+! Initialisation
 !===============================================================================
 
 ! The itycat variable is used to define field categories. It is used in Fortran
@@ -119,17 +120,12 @@ inoprv = .false.   ! variables have no previous value
 call field_get_key_id('log', keylog)
 call field_get_key_id('post_vis', keyvis)
 call field_get_key_id('label', keylbl)
-call field_get_key_id('coupled', keycpl)
 call field_get_key_id("variable_id", keyvar)
 
 ! If a scalar is a variance, store the id of the parent scalar
 call field_get_key_id("first_moment_id", kscavr)
 
-! Key id for diffusivity tensor
-call field_get_key_id("diffusivity_tensor", kdiftn)
-
 ! Keys not stored globally
-
 call field_get_key_id('turbulent_flux_model', kturt)
 call field_get_key_id('turbulent_flux_id', kfturt)
 
@@ -143,103 +139,32 @@ call field_get_key_id("drift_scalar_model", keydri)
 call field_get_n_fields(nfld)
 
 !===============================================================================
-! 2. Set keywords and add some additional fields
+! Set keywords and add some additional fields
 !===============================================================================
 
-! Copy field calculation options into the field structure
-
-do f_id = 0, nfld - 1
-
-  call field_get_type(f_id, f_type)
-
-  ! Is the field of type FIELD_VARIABLE?
-  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
-    call field_get_key_struct_var_cal_opt(f_id, vcopt)
-
-    call field_get_key_int(f_id, keyvar, ivar)
-
-    vcopt%iwarni= iwarni(ivar)
-    vcopt%iconv = iconv (ivar)
-    vcopt%istat = istat (ivar)
-    vcopt%idiff = idiff (ivar)
-    vcopt%idifft= idifft(ivar)
-    vcopt%idften= idften(ivar)
-    vcopt%iswdyn= iswdyn(ivar)
-    vcopt%ischcv= ischcv(ivar)
-    vcopt%ibdtso= ibdtso(ivar)
-    vcopt%isstpc= isstpc(ivar)
-    vcopt%nswrgr= nswrgr(ivar)
-    vcopt%nswrsm= nswrsm(ivar)
-    vcopt%imrgra= imrgra
-    vcopt%imligr= imligr(ivar)
-    vcopt%ircflu= ircflu(ivar)
-    vcopt%iwgrec= iwgrec(ivar)
-    vcopt%thetav= thetav(ivar)
-    vcopt%blencv= blencv(ivar)
-    vcopt%epsilo= epsilo(ivar)
-    vcopt%epsrsm= epsrsm(ivar)
-    vcopt%epsrgr= epsrgr(ivar)
-    vcopt%climgr= climgr(ivar)
-    vcopt%extrag= extrag(ivar)
-    vcopt%relaxv= relaxv(ivar)
-
-    call field_set_key_struct_var_cal_opt(f_id, vcopt)
-  endif
-enddo
-
-
 ! User variables
 !---------------
 
-do ivar = 1, nvar
-  ! Init key word: tensorial diffusivity
-  call field_set_key_int(ivarfl(ivar), kdiftn, idften(ivar))
-enddo
-
 idfm = 0
 iggafm = 0
 
 do ii = 1, nscal
-
   if (isca(ii) .gt. 0) then
 
     ivar = isca(ii)
-    f_id = ivarfl(ivar)
-
-    call field_get_key_int(ivarfl(ivar), keyvis, f_vis)
-    call field_get_key_int(ivarfl(ivar), keylog, f_log)
-
-    call field_get_key_int(ivarfl(ivar), kdiftn, f_dften)
+    call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
 
     if (ityturt(ii).gt.0) then
-      call field_get_name (f_id, name)
-      f_name = trim(name)//'_turbulent_flux'
-
       if (ityturt(ii).eq.3) then
-        itycat = FIELD_INTENSIVE + FIELD_VARIABLE  ! for variables
-      else
-        itycat = FIELD_INTENSIVE + FIELD_PROPERTY  ! for properties
+        idfm = 1
       endif
 
       ! GGDH or AFM on current scalar
       ! and if DFM, GGDH on the scalar variance
       iggafm = 1
 
-      call field_create(f_name, itycat, ityloc, idim3, iprev, iflid)
-      if (ityturt(ii).eq.3) then
-        call field_set_key_int(iflid, keycpl, 1)
-        ! Tensorial diffusivity
-        call field_set_key_int(iflid, kdiftn, 6)
-        idfm = 1
-      endif
-      call field_set_key_int(iflid, keyvis, f_vis)
-      call field_set_key_int(iflid, keylog, f_log)
-
-      call field_set_key_int(ivarfl(ivar), kturt, iturt(ii))
-      call field_set_key_int(ivarfl(ivar), kfturt, iflid)
-
     ! If the user has chosen a tensorial diffusivity
-    else if (f_dften.eq.6) then
+    else if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
       idfm = 1
     endif
 
@@ -249,12 +174,24 @@ do ii = 1, nscal
     ! Additional fields for Drift scalars is done in addfld
 
   endif
-
 enddo
 
 ! Reserved fields whose ids are not saved (may be queried by name)
 !-----------------------------------------------------------------
 
+itycat = FIELD_INTENSIVE
+ityloc = 1 ! cells
+
+if (iphydr.eq.1) then
+  call field_find_or_create('volume_forces', &
+                            itycat, ityloc, idim3, f_id)
+endif
+
+if (iphydr.eq.2) then
+  call field_find_or_create('hydrostatic_pressure_prd', &
+                            itycat, ityloc, idim1, f_id)
+endif
+
 ! friction velocity at the wall, in the case of a LES calculation
 ! with van Driest-wall damping (delayed here rather than placed in
 ! addfld, as idries may be set in modini).
@@ -320,7 +257,7 @@ do iflid = 0, nfld-1
     ! Index of the class, all member of the class share the same mass flux
     call field_get_key_int(iflid, keyccl, icla)
 
-    ! The comming field are not solved, they are properties
+    ! The following fields are not solved, they are properties
     itycat = FIELD_PROPERTY
     ityloc = 2 ! variables defined on interior faces
 
@@ -366,26 +303,33 @@ do iflid = 0, nfld-1
 
     ityloc = 1 ! variables defined on cells
 
-    ! Relaxation time
-    f_name = 'drift_tau_'//trim(name)
-    call field_create(f_name, itycat, ityloc, idim1, inoprv, f_id)
-    call field_set_key_str(f_id, keylbl, f_name)
-
-    ! Set the same visualization options as the scalar
+    ! Get the scalar's output options
+    ! (except non-reconstructed boundary output)
     call field_get_key_int(iflid, keyvis, iopchr)
-    if (iopchr.eq.1) then
-      call field_set_key_int(f_id, keyvis, iopchr)
-    endif
+    call field_get_key_int(iflid, keylog, ilog)
+    if (iand(iopchr, POST_BOUNDARY_NR) .ne. 0) iopchr = iopchr - POST_BOUNDARY_NR
 
-    ! Store the drift velocity
-    f_name = 'drift_vel_'//trim(name)
-    call field_create(f_name, itycat, ityloc, idim3, inoprv, f_id)
-    call field_set_key_str(f_id, keylbl, f_name)
+    ! If the mass flux is imposed, no need of drift_tau nor drift_vel
+    if (.not.(btest(iscdri, DRIFT_SCALAR_IMPOSED_MASS_FLUX))) then
 
-    ! Set the same visualization options as the scalar
-    call field_get_key_int(iflid, keyvis, iopchr)
-    if (iopchr.eq.1) then
+      ! Relaxation time
+      f_name = 'drift_tau_'//trim(name)
+      call field_create(f_name, itycat, ityloc, idim1, inoprv, f_id)
+      call field_set_key_str(f_id, keylbl, f_name)
+
+      ! Set the same visualization options as the scalar,
       call field_set_key_int(f_id, keyvis, iopchr)
+      call field_set_key_int(f_id, keylog, ilog)
+
+      ! Store the drift velocity
+      f_name = 'drift_vel_'//trim(name)
+      call field_create(f_name, itycat, ityloc, idim3, inoprv, f_id)
+      call field_set_key_str(f_id, keylbl, f_name)
+
+      ! Set the same visualization options as the scalar
+      call field_set_key_int(f_id, keyvis, iopchr)
+      call field_set_key_int(f_id, keyvis, ilog)
+
     endif
 
     ! Interaction time particle--eddies
@@ -393,12 +337,10 @@ do iflid = 0, nfld-1
       f_name = 'drift_turb_tau_'//trim(name)
       call field_create(f_name, itycat, ityloc, idim1, inoprv, f_id)
       call field_set_key_str(f_id, keylbl, f_name)
-    endif
 
-    ! Set the same visualization options as the scalar
-    call field_get_key_int(iflid, keyvis, iopchr)
-    if (iopchr.eq.1) then
+      ! Set the same visualization options as the scalar
       call field_set_key_int(f_id, keyvis, iopchr)
+      call field_set_key_int(f_id, keyvis, ilog)
     endif
 
   endif
@@ -410,23 +352,26 @@ itycat = FIELD_PROPERTY
 ityloc = 1         ! variables defined on cells
 idimf  = -1        ! Field dimension
 
-do ivar = 1, nvar
-  if (iwgrec(ivar).eq.1) then
 
-    if (idiff(ivar).lt.1) cycle
-    iflid = ivarfl(ivar)
-    if (iflid.eq.iflidp) cycle
-    iflidp = iflid
-    call field_get_name(iflid, name)
-    f_name = 'gradient_weighting_'//trim(name)
-    if (idften(ivar).eq.1) then
-      idimf = 1
-    elseif (idften(ivar).eq.6) then
-      idimf = 6
+do f_id = 0, nfld - 1
+  call field_get_type(f_id, f_type)
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+
+    call field_get_key_struct_var_cal_opt(f_id, vcopt)
+    if (vcopt%iwgrec.eq.1 .and. vcopt%idiff .ge. 1) then
+
+      call field_get_name(f_id, name)
+      f_name = 'gradient_weighting_'//trim(name)
+      if (iand(vcopt%idften, ISOTROPIC_DIFFUSION).ne.0) then
+        idimf = 1
+      else if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
+        idimf = 6
+      endif
+      call field_create(f_name, itycat, ityloc, idimf, inoprv, iflid)
+      call field_set_key_int(f_id, kwgrec, iflid)
+
     endif
-    call field_create(f_name, itycat, ityloc, idimf, inoprv, f_id)
-    call field_set_key_int(iflid, kwgrec, f_id)
-
   endif
 enddo
 
@@ -437,23 +382,51 @@ call field_get_key_id("slope_test_upwind_id", kislts)
 itycat = FIELD_POSTPROCESS
 ityloc = 1 ! cells
 
-do ii = 1, nvar
-  f_id = ivarfl(ii)
-  call field_get_key_int(f_id, kislts, ifctsl)
-  if (ifctsl.eq.0) then
-   call field_get_key_struct_var_cal_opt(f_id, vcopt)
+do f_id = 0, nfld - 1
+  call field_get_type(f_id, f_type)
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    call field_get_key_int(f_id, kislts, ifctsl)
+    if (ifctsl.ge.0) then
+      call field_get_key_struct_var_cal_opt(f_id, vcopt)
 
-   ! Now create matching field
-    if (vcopt%iconv.gt.0 .and. vcopt%blencv.gt.0 .and. vcopt%isstpc.eq.0) then
+      ! Now create matching field
+      if (vcopt%iconv.gt.0 .and. vcopt%blencv.gt.0 .and. vcopt%isstpc.eq.0) then
+        ! Build name and label
+        call field_get_name(f_id, f_name)
+        name  = trim(f_name) // '_slope_upwind'
+        call field_create(name, itycat, ityloc, idim1, inoprv, ifctsl)
+        call field_set_key_int(ifctsl, keyvis, POST_ON_LOCATION)
+        call field_set_key_int(f_id, kislts, ifctsl)
+      endif
+    endif
+  endif
+enddo
+
+! Postprocessing of clippings
+
+call field_get_key_id("clipping_id", kclipp)
+
+itycat = FIELD_POSTPROCESS
+ityloc = 1 ! cells
+
+do f_id = 0, nfld - 1
+  call field_get_type(f_id, f_type)
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    call field_get_key_int(f_id, kclipp, clip_id)
+    if (clip_id.ge.0) then
+
+      ! Now create matching field
       ! Build name and label
       call field_get_name(f_id, f_name)
-      name  = trim(f_name) // '_slope_upwind'
-      call field_create(name, itycat, ityloc, idim1, inoprv, ifctsl)
-      call field_set_key_int(ifctsl, keyvis, 1)
-    else
-      ifctsl = -1
+
+      call field_get_dim(f_id, f_dim)
+      name  = trim(f_name) // '_clipped'
+      call field_create(name, itycat, ityloc, f_dim, inoprv, clip_id)
+      call field_set_key_int(clip_id, keyvis, POST_ON_LOCATION)
+      call field_set_key_int(f_id, kclipp, clip_id)
     endif
-    call field_set_key_int(f_id, kislts, ifctsl)
   endif
 enddo
 
@@ -468,7 +441,7 @@ if (n_fans .gt. 0) then
   ityloc = 1 ! cells
 
   call field_create(name, itycat, ityloc, idim1, inoprv, ifctsl)
-  call field_set_key_int(ifctsl, keyvis, 1)
+  call field_set_key_int(ifctsl, keyvis, POST_ON_LOCATION)
   call field_set_key_int(ifctsl, keylog, 1)
 
 endif
@@ -479,60 +452,57 @@ call field_get_key_id("convection_limiter_id", kcvlim)
 
 itycat = FIELD_PROPERTY
 
-do ii = 1, nvar
-  f_id = ivarfl(ii)
+do f_id = 0, nfld - 1
 
-  call field_get_key_struct_var_cal_opt(f_id, vcopt)
+  call field_get_type(f_id, f_type)
 
-  ! Beta limiter
-  if (vcopt%isstpc.eq.2) then
-    ! Now create matching field
-    ! Build name and label
-    call field_get_name(f_id, f_name)
-    name  = trim(f_name) // '_conv_lim'
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
 
-    ityloc = 1 ! cells
+    call field_get_key_struct_var_cal_opt(f_id, vcopt)
 
-    call field_create(name, itycat, ityloc, idim1, inoprv, ifctsl)
-    call field_set_key_int(ifctsl, keyvis, 1)
-    call field_set_key_int(ifctsl, keylog, 1)
+    ! Beta limiter or Roe-Sweby limiter
+    if (vcopt%isstpc.eq.2 .or. vcopt%isstpc.eq.3) then
+      ! Now create matching field
+      ! Build name and label
+      call field_get_name(f_id, f_name)
 
-    call field_set_key_int(f_id, kcvlim, ifctsl)
-  !Roe-Sweby limiter
-  else if (vcopt%isstpc.eq.3) then
-    ! Now create matching field
-    ! Build name and label
-    call field_get_name(f_id, f_name)
-    name  = trim(f_name) // '_conv_lim'
+      call field_get_dim(f_id, f_dim)
+      name = trim(f_name) // '_conv_lim'
 
-    ityloc = 2 ! Interior faces
+      if (vcopt%isstpc.eq.2) then
+        ityloc = 1 ! cells
+      else
+        ityloc = 2 ! Interior faces
+      endif
 
-    call field_create(name, itycat, ityloc, idim1, inoprv, ifctsl)
-    call field_set_key_int(ifctsl, keyvis, 1)
-    call field_set_key_int(ifctsl, keylog, 1)
+      call field_create(name, itycat, ityloc, f_dim, inoprv, ifctsl)
+      call field_set_key_int(ifctsl, keyvis, POST_ON_LOCATION)
+      call field_set_key_int(ifctsl, keylog, 1)
 
-    call field_set_key_int(f_id, kcvlim, ifctsl)
+      call field_set_key_int(f_id, kcvlim, ifctsl)
+    endif
   endif
 enddo
 
 !===============================================================================
 
-! Cavitation: the void fraction has its spectific convective flux
-!----------------------------------------------------------------
+! VOF algorithm: the void fraction has its spectific convective flux
+!-------------------------------------------------------------------
 
-if (icavit.ge.0) then
+if (ivofmt.ge.0) then
 
   itycat = FIELD_EXTENSIVE + FIELD_PROPERTY
 
   ityloc = 2  ! inner faces
   f_name = 'inner_void_fraction_flux'
   call field_create(f_name, itycat, ityloc, idim1, inoprv, f_id)
-  call field_set_key_int(ivarfl(ivoidf), kimasf, f_id)
+  call field_set_key_int(ivarfl(ivolf2), kimasf, f_id)
 
   ityloc = 3 ! boundary faces
   f_name = 'boundary_void_fraction_flux'
   call field_create(f_name, itycat, ityloc, idim1, inoprv, f_id)
-  call field_set_key_int(ivarfl(ivoidf), kbmasf, f_id)
+  call field_set_key_int(ivarfl(ivolf2), kbmasf, f_id)
 
 endif
 
@@ -573,13 +543,25 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1 &
   if (iturb.eq.32.and.iggafm.eq.1) then
     call field_create('anisotropic_turbulent_viscosity_scalar', itycat, &
                       ityloc, idim6, inoprv, ivstes)
- endif
+  endif
 endif
 
 !===============================================================================
-! 3. Set some field keys
+! Set some field keys
 !===============================================================================
 
+! Copy imrgra into the field structure
+do f_id = 0, nfld - 1
+  call field_get_type(f_id, f_type)
+
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    call field_get_key_struct_var_cal_opt(f_id, vcopt)
+    vcopt%imrgra= imrgra
+    call field_set_key_struct_var_cal_opt(f_id, vcopt)
+  endif
+enddo
+
 ! Copy field physical properties of species into the field structure
 
 if (ippmod(igmix).ge.0) then
@@ -686,6 +668,10 @@ if (ippmod(igmix).ge.0) then
 
 endif
 
+! Check if scalars are buoyant and set n_buoyant_scal accordingly.
+! It is then used in tridim to update buoyant scalars and density in U-P loop
+call cs_parameters_set_n_buoyant_scalars
+
 ! Some C mappings
 call cs_field_pointer_map_boundary
 
