@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -45,8 +45,10 @@ use cplsat
 use post
 use ppincl
 use rotation
-use cs_c_bindings
 use darcy_module
+use turbomachinery
+use vof
+use cs_c_bindings
 
 !===============================================================================
 
@@ -57,15 +59,22 @@ implicit none
 
 ! Local variables
 
-integer          n_fields, f_id, f_dim, n_moments
-integer          ii, jj, ivar, imom, iok, ikw
-integer          icompt, ipp, nbccou, keyvar
+integer          n_fields, f_id, n_moments
+integer          ii, jj, imom, iok, ikw
+integer          nbccou, keyvar, flag
 integer          nscacp, iscal
 integer          imrgrp
+integer          kcpsyr, icpsyr
+integer          nfld, f_type
 
 logical          is_set
 
-double precision relxsp, omgnrm
+double precision relxsp
+
+character(len=80) :: name
+
+type(var_cal_opt) :: vcopt , vcopt1, vcopt2, vcopt3, vcopt4
+type(var_cal_opt) :: vcopt5, vcopt6
 
 !===============================================================================
 
@@ -76,20 +85,12 @@ call field_get_n_fields(n_fields)
 
 call field_get_key_id("variable_id", keyvar)
 
-call cs_f_turb_complete_constants
+call field_get_key_id("syrthes_coupling", kcpsyr)
 
 !===============================================================================
 ! 1. ENTREES SORTIES entsor
 !===============================================================================
 
-! ---> Niveau d'impression listing
-!       Non initialise -> standard
-do ii = 1, nvarmx
-  if (iwarni(ii).eq.-10000) then
-    iwarni(ii) = 0
-  endif
-enddo
-
 !---> sorties chrono?
 !     Sauf mention contraire de l'utilisateur, on sort a la fin les
 !        variables de calcul, la viscosite, rho, le pas de temps s'il
@@ -109,75 +110,26 @@ endif
 !      Si une valeur non modifiee par l'utilisateur (=-999)
 !        on la met a sa valeur par defaut
 !      On sort toutes les variables a tous les pas de temps par defaut
-!      IHISVR nb de sonde et numero par variable (-999 non initialise)
-!             -1 : toutes les sondes
 !      NTHIST = -1 : on ne sort pas d'historiques
 !      NTHIST =  n : on sort des historiques tous les n pas de temps
 !      NTHSAV = -1 : on sauvegarde a la fin uniquement
 !      NTHSAV =  0 : periode par defaut (voir caltri)
 !             > 0  : periode
 
-do f_id = 0, n_fields-1
-  call field_get_key_int(f_id, keyvar, ivar)
-  if (ivar.ge.1) then
-    call field_get_key_int(f_id, keyipp, ipp)
-    call field_get_dim(f_id, f_dim)
-    do ii = 1, f_dim
-      if (ihisvr(ipp + ii-1,1).eq.-999) ihisvr(ipp + ii-1,1) = -1
-    enddo
-  endif
-enddo
-if (ihisvr(ippdt, 1).eq.-999) ihisvr(ippdt, 1) = -1
-if (ipucou.ne.1) then
-  ihisvr(ipptx,1) = 0
-  ihisvr(ippty,1) = 0
-  ihisvr(ipptz,1) = 0
-endif
-ipp = ipppro(ipproc(ivisct))
-if ((iturb.eq.10 .or. itytur.eq.2                        &
-     .or. itytur.eq.5 .or. iturb.eq.60                   &
-     .or. iturb.eq.70)                                   &
-     .and.ihisvr(ipp,1).eq.-999) ihisvr(ipp,1) = -1
-if (idtvar.lt.0) then
-  ihisvr(ipppro(ipproc(icour)),1) = 0
-  ihisvr(ipppro(ipproc(ifour)),1) = 0
-endif
-
 n_moments = cs_time_moment_n_moments()
 
 do imom = 1, n_moments
   f_id = time_moment_field_id(imom)
   if (f_id.lt.0) cycle
   call field_is_key_set(f_id, keyvis, is_set)
-  if (.not. is_set) call field_set_key_int(f_id, keyvis, 1)
+  if (.not. is_set) then
+    flag = POST_ON_LOCATION + POST_MONITOR
+    call field_set_key_int(f_id, keyvis, flag)
+  endif
   call field_is_key_set(f_id, keylog, is_set)
   if (.not. is_set) call field_set_key_int(f_id, keylog, 1)
-  ipp = field_post_id(f_id)
-  call field_get_dim(f_id, f_dim)
-  do ii = 1, f_dim
-    if (ihisvr(ipp + ii-1,1).eq.-999) ihisvr(ipp + ii-1,1) = -1
-  enddo
 enddo
 
-do ii = 1, nvppmx
-  if (ihisvr(ii,1).eq.-999) then
-    ihisvr(ii,1) = 0
-  endif
-enddo
-
-!     Si on est en ALE, on a un test equivalent dans strini.F
-if (iale.eq.0) then
-  icompt = 0
-  do ii = 2, nvppmx
-    if (ihisvr(ii,1).ne.0) icompt = icompt+1
-  enddo
-
-  if (icompt.eq.0.or.ncapt.eq.0) then
-    nthist = -1
-    frhist = -1.d0
-  endif
-endif
-
 ! Adapt the output frequency parameters according to the time scheme.
 if (idtvar.lt.0.or.idtvar.eq.2) then
   frhist = -1.d0
@@ -187,17 +139,10 @@ else
   endif
 endif
 
-! Variable labels
-
-if (icorio.eq.1) then
-  call field_set_key_str(ivarfl(ipr), keylbl, 'Rel Pressure')
-  call field_set_key_str(ivarfl(iu), keylbl, 'Rel Velocity')
-endif
-
 ! Logging and postprocessing output
 
-if (irovar.eq.0) then
-  call hide_property(irom)
+if (irovar.eq.0.and.ivofmt.lt.0) then
+  call hide_property(icrom)
   call field_set_key_int(ibrom, keylog, 0)
 endif
 
@@ -319,204 +264,79 @@ do iscal = 1, nscal
   endif
 enddo
 
-! Velocity and Pressure (thetav for the pressure is taken without worring)
-if (abs(thetav(iu)+999.d0).gt.epzero.or.                 &
-    abs(thetav(iv)+999.d0).gt.epzero.or.                 &
-    abs(thetav(iw)+999.d0).gt.epzero.or.                 &
-    abs(thetav(ipr)+999.d0).gt.epzero) then
-  write(nfecra,1131) 'VITESSE-PRESSION ','THETAV'
-elseif (ischtp.eq.1) then
-  thetav(iu) = 1.d0
-  thetav(iv) = 1.d0
-  thetav(iw) = 1.d0
-  thetav(ipr) = 1.d0
-elseif (ischtp.eq.2) then
-  thetav(iu) = 0.5d0
-  thetav(iv) = 0.5d0
-  thetav(iw) = 0.5d0
-  thetav(ipr) = 1.d0
-endif
+! Loop on on field variables
+call field_get_n_fields(nfld)
 
-!     Taux de vide en diphasique homogene
-if (icavit.ge.0) then
-  if (abs(thetav(ivoidf)+999.d0).gt.epzero) then
-    write(nfecra,1031) 'TAUX DE VIDE','THETAV'
-    iok = iok + 1
-  elseif (ischtp.eq.1) then
-    thetav(ivoidf) = 1.d0
-  elseif (ischtp.eq.2) then
-    thetav(ivoidf) = 0.5d0
-  endif
-endif
-
-!     Turbulence (en k-eps : ordre 1)
-if (itytur.eq.2) then
-  if (abs(thetav(ik )+999.d0).gt.epzero.or.               &
-      abs(thetav(iep)+999.d0).gt.epzero) then
-    write(nfecra,1031) 'VARIABLES   K-EPS','THETAV'
-    iok = iok + 1
-  elseif (ischtp.eq.1) then
-    thetav(ik ) = 1.d0
-    thetav(iep) = 1.d0
-  elseif (ischtp.eq.2) then
-    !     pour le moment, on ne peut pas passer par ici (cf varpos)
-    thetav(ik ) = 0.5d0
-    thetav(iep) = 0.5d0
-  endif
-elseif (itytur.eq.3) then
-  if (abs(thetav(ir11)+999.d0).gt.epzero.or.              &
-      abs(thetav(ir22)+999.d0).gt.epzero.or.              &
-      abs(thetav(ir33)+999.d0).gt.epzero.or.              &
-      abs(thetav(ir12)+999.d0).gt.epzero.or.              &
-      abs(thetav(ir13)+999.d0).gt.epzero.or.              &
-      abs(thetav(ir23)+999.d0).gt.epzero.or.              &
-      abs(thetav(iep )+999.d0).gt.epzero) then
-    write(nfecra,1031) 'VARIABLES  RIJ-EP','THETAV'
-    iok = iok + 1
-  elseif (ischtp.eq.1) then
-    thetav(ir11) = 1.d0
-    thetav(ir22) = 1.d0
-    thetav(ir33) = 1.d0
-    thetav(ir12) = 1.d0
-    thetav(ir13) = 1.d0
-    thetav(ir23) = 1.d0
-    thetav(iep ) = 1.d0
-  elseif (ischtp.eq.2) then
-    thetav(ir11) = 0.5d0
-    thetav(ir22) = 0.5d0
-    thetav(ir33) = 0.5d0
-    thetav(ir12) = 0.5d0
-    thetav(ir13) = 0.5d0
-    thetav(ir23) = 0.5d0
-    thetav(iep ) = 0.5d0
-  endif
-
-  ! Diffusivity model:
-
-  ! Daly Harlow (GGDH) on Rij and epsilon by default
-  if (idirsm.ne.0) then
-    idften(ir11) = 6
-    idften(ir22) = 6
-    idften(ir33) = 6
-    idften(ir12) = 6
-    idften(ir23) = 6
-    idften(ir13) = 6
-    idften(iep)  = 6
-
-  ! Scalar diffusivity (Shir model) elswhere (idirsm = 0)
-  else
-    idften(ir11) = 1
-    idften(ir22) = 1
-    idften(ir33) = 1
-    idften(ir12) = 1
-    idften(ir23) = 1
-    idften(ir13) = 1
-    idften(iep)  = 1
-  endif
-
-  if (iturb.eq.32) then
-    if (abs(thetav(ial)+999.d0).gt.epzero) then
-      write(nfecra,1031) 'VARIABLES  RIJ-EB','THETAV'
-      iok = iok + 1
-    elseif (ischtp.eq.1) then
-      thetav(ial) = 1.d0
-    elseif (ischtp.eq.2) then
-      thetav(ial) = 0.5d0
+do f_id = 0, nfld - 1
+  call field_get_type(f_id, f_type)
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    call field_get_key_struct_var_cal_opt(f_id, vcopt)
+    if (abs(vcopt%thetav+999.d0).gt.epzero) then
+      call field_get_name(f_id, name)
+      write(nfecra,1131) trim(name),'THETAV'
+    else
+      if (vcopt%istat.eq.0) then
+        vcopt%thetav = 1.d0
+      else if (ischtp.eq.1) then
+        vcopt%thetav = 1.d0
+      else if (ischtp.eq.2) then
+        vcopt%thetav = 0.5d0
+      endif
     endif
-  endif
-
-elseif (iturb.eq.50) then
-  if (abs(thetav(ik  )+999.d0).gt.epzero.or.              &
-      abs(thetav(iep )+999.d0).gt.epzero.or.              &
-      abs(thetav(iphi)+999.d0).gt.epzero.or.              &
-      abs(thetav(ifb )+999.d0).gt.epzero) then
-    write(nfecra,1031) 'VARIABLES     V2F','THETAV'
-    iok = iok + 1
-  elseif (ischtp.eq.1) then
-    thetav(ik  ) = 1.d0
-    thetav(iep ) = 1.d0
-    thetav(iphi) = 1.d0
-    thetav(ifb ) = 1.d0
-  elseif (ischtp.eq.2) then
-    !     pour le moment, on ne peut pas passer par ici (cf varpos)
-    thetav(ik  ) = 0.5d0
-    thetav(iep ) = 0.5d0
-    thetav(iphi) = 0.5d0
-    thetav(ifb ) = 0.5d0
-  endif
-elseif (iturb.eq.51) then
-  if (abs(thetav(ik  )+999.d0).gt.epzero.or.              &
-      abs(thetav(iep )+999.d0).gt.epzero.or.              &
-      abs(thetav(iphi)+999.d0).gt.epzero.or.              &
-      abs(thetav(ial )+999.d0).gt.epzero) then
-    write(nfecra,1031) 'VARIABLES BL-V2/K','THETAV'
-    iok = iok + 1
-  elseif (ischtp.eq.1) then
-    thetav(ik  ) = 1.d0
-    thetav(iep ) = 1.d0
-    thetav(iphi) = 1.d0
-    thetav(ial ) = 1.d0
-  elseif (ischtp.eq.2) then
-    !     pour le moment, on ne peut pas passer par ici (cf varpos)
-    thetav(ik  ) = 0.5d0
-    thetav(iep ) = 0.5d0
-    thetav(iphi) = 0.5d0
-    thetav(ial ) = 0.5d0
-  endif
-elseif (iturb.eq.60) then
-  if (abs(thetav(ik  )+999.d0).gt.epzero.or.              &
-      abs(thetav(iomg)+999.d0).gt.epzero ) then
-    write(nfecra,1031) 'VARIABLES K-OMEGA','THETAV'
-    iok = iok + 1
-  elseif (ischtp.eq.1) then
-    thetav(ik  ) = 1.d0
-    thetav(iomg) = 1.d0
-  elseif (ischtp.eq.2) then
-    !     pour le moment, on ne peut pas passer par ici (cf varpos)
-    thetav(ik  ) = 0.5d0
-    thetav(iomg) = 0.5d0
-  endif
-elseif (iturb.eq.70) then
-  if (abs(thetav(inusa)+999.d0).gt.epzero) then
-    write(nfecra,1031) 'VARIABLE NU_tilde de SA','THETAV'
-    iok = iok + 1
-  elseif (ischtp.eq.1) then
-    thetav(inusa) = 1.d0
-  elseif (ischtp.eq.2) then
-    !     pour le moment, on ne peut pas passer par ici (cf varpos)
-    thetav(inusa) = 0.5d0
-  endif
-endif
-
-! Scalares
-do iscal = 1, nscal
-  ivar  = isca(iscal)
-  if (abs(thetav(ivar)+999.d0).gt.epzero) then
-    write(nfecra,1041) 'SCALAIRE',ISCAL,'THETAV'
-  elseif (ischtp.eq.1) then
-    thetav(ivar) = 1.d0
-  elseif (ischtp.eq.2) then
-    thetav(ivar) = 0.5d0
+    call field_set_key_struct_var_cal_opt(f_id, vcopt)
   endif
 enddo
 
-! Mesh velocity for ALE
-if (iale.eq.1) then
-  if (abs(thetav(iuma)+999.d0).gt.epzero.or.                       &
-      abs(thetav(ivma)+999.d0).gt.epzero.or.                       &
-      abs(thetav(iwma)+999.d0).gt.epzero) then
-    write(nfecra,1032) 'THETAV'
-    iok = iok + 1
-  elseif (ischtp.eq.1) then
-    thetav(iuma) = 1.d0
-    thetav(ivma) = 1.d0
-    thetav(iwma) = 1.d0
-  elseif (ischtp.eq.2) then
-!     pour le moment, on ne peut pas passer par ici (cf varpos)
-    thetav(iuma) = 0.5d0
-    thetav(ivma) = 0.5d0
-    thetav(iwma) = 0.5d0
+! Diffusivity model:
+! Daly Harlow (GGDH) on Rij and epsilon by default
+if (itytur.eq.3) then
+  if (irijco.eq.1) then
+    call field_get_key_struct_var_cal_opt(ivarfl(irij), vcopt1)
+    call field_get_key_struct_var_cal_opt(ivarfl(iep), vcopt)
+
+    ! Diffusivity model:
+    ! Daly Harlow (GGDH) on Rij and epsilon by default
+    if (idirsm.ne.0) then
+      vcopt1%idften = ANISOTROPIC_RIGHT_DIFFUSION
+      vcopt%idften  = ANISOTROPIC_RIGHT_DIFFUSION
+      ! Scalar diffusivity (Shir model) elswhere (idirsm = 0)
+    else
+      vcopt1%idften = ISOTROPIC_DIFFUSION
+      vcopt%idften  = ISOTROPIC_DIFFUSION
+    endif
+
+    call field_set_key_struct_var_cal_opt(ivarfl(irij), vcopt1)
+    call field_set_key_struct_var_cal_opt(ivarfl(iep), vcopt)
+  else
+    call field_get_key_struct_var_cal_opt(ivarfl(ir11), vcopt1)
+    call field_get_key_struct_var_cal_opt(ivarfl(ir22), vcopt2)
+    call field_get_key_struct_var_cal_opt(ivarfl(ir33), vcopt3)
+    call field_get_key_struct_var_cal_opt(ivarfl(ir12), vcopt4)
+    call field_get_key_struct_var_cal_opt(ivarfl(ir13), vcopt5)
+    call field_get_key_struct_var_cal_opt(ivarfl(ir23), vcopt6)
+    call field_get_key_struct_var_cal_opt(ivarfl(iep), vcopt)
+
+    ! Diffusivity model:
+    ! Daly Harlow (GGDH) on Rij and epsilon by default
+    if (idirsm.ne.0) then
+      vcopt1%idften = ANISOTROPIC_RIGHT_DIFFUSION
+      vcopt%idften  = ANISOTROPIC_RIGHT_DIFFUSION
+      ! Scalar diffusivity (Shir model) elswhere (idirsm = 0)
+    else
+      vcopt1%idften = ISOTROPIC_DIFFUSION
+      vcopt%idften  = ISOTROPIC_DIFFUSION
+    endif
+
+    call field_set_key_struct_var_cal_opt(ivarfl(ir11), vcopt1)
+    call field_set_key_struct_var_cal_opt(ivarfl(ir22), vcopt1)
+    call field_set_key_struct_var_cal_opt(ivarfl(ir33), vcopt1)
+    call field_set_key_struct_var_cal_opt(ivarfl(ir12), vcopt1)
+    call field_set_key_struct_var_cal_opt(ivarfl(ir13), vcopt1)
+    call field_set_key_struct_var_cal_opt(ivarfl(ir23), vcopt1)
+    call field_set_key_struct_var_cal_opt(ivarfl(iep), vcopt)
   endif
+
 endif
 
 ! ---> ISSTPC
@@ -526,20 +346,26 @@ endif
 
 if (itytur.eq.4) then
   ii = iu
-  if (isstpc(ii).eq.-999) isstpc(ii) = 1
-  ii = iv
-  if (isstpc(ii).eq.-999) isstpc(ii) = 1
-  ii = iw
-  if (isstpc(ii).eq.-999) isstpc(ii) = 1
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  if (vcopt%isstpc.eq.-999) then
+    vcopt%isstpc = 1
+    call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  endif
   do jj = 1, nscal
     ii = isca(jj)
-    if (isstpc(ii).eq.-999) isstpc(ii) = 0
-  enddo
+    call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+    if (vcopt%isstpc.eq.-999) then
+      vcopt%isstpc = 0
+      call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+    endif
+ enddo
 endif
 
-do ii = 1, nvarmx
-  if (isstpc(ii).eq.-999) then
-    isstpc(ii) = 0
+do ii = 1, nvar
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  if (vcopt%isstpc.eq.-999) then
+    vcopt%isstpc = 0
+    call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
   endif
 enddo
 
@@ -555,38 +381,49 @@ enddo
 !  upwind et on affiche un message si l'utilisateur avait specifie autre chose
 
 ii = iu
-if (abs(blencv(ii)+999.d0).lt.epzero) blencv(ii) = 1.d0
-ii = iv
-if (abs(blencv(ii)+999.d0).lt.epzero) blencv(ii) = 1.d0
-ii = iw
-if (abs(blencv(ii)+999.d0).lt.epzero) blencv(ii) = 1.d0
+call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+if (abs(vcopt%blencv+999.d0).lt.epzero) then
+  vcopt%blencv = 1.d0
+  call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+endif
 
 if (icavit.ge.0) then
-  ii = ivoidf
-  if (abs(blencv(ii)).gt.epzero) then
-    if (abs(blencv(ii)+999.d0).gt.epzero) &
-         write(nfecra,3000) 0.d0, blencv(ii)
-    blencv(ii) = 0.d0
+  ii = ivolf2
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  if (abs(vcopt%blencv+999.d0).lt.epzero) then
+    if (abs(vcopt%blencv+999.d0).gt.epzero) &
+         write(nfecra,3000) 0.d0, vcopt%blencv
+    vcopt%blencv = 0.d0
+    call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
   endif
 endif
 
 do jj = 1, nscaus
   ii = isca(jj)
-  if (abs(blencv(ii)+999.d0).lt.epzero) blencv(ii) = 1.d0
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  if (abs(vcopt%blencv+999.d0).lt.epzero) then
+    vcopt%blencv = 1.d0
+    call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  endif
 enddo
 
 if (iscalt.gt.0) then
   ii = isca(iscalt)
-  if (abs(blencv(ii)+999.d0).lt.epzero) blencv(ii) = 1.d0
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  if (abs(vcopt%blencv+999.d0).lt.epzero) then
+    vcopt%blencv = 1.d0
+    call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  endif
 endif
 
-do ii = 1, nvarmx
-  if (abs(blencv(ii)+999.d0).lt.epzero) then
-    blencv(ii) = 0.d0
+do ii = 1, nvar
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  if (abs(vcopt%blencv+999.d0).lt.epzero) then
+    vcopt%blencv = 0.d0
+    call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
   endif
 enddo
 
-
 ! ---> NSWRSM, EPSRSM ET EPSILO
 !        Si l'utilisateur n'a rien specifie  (NSWRSM=-999),
 !        On impose
@@ -604,35 +441,37 @@ enddo
 
 if (ischtp.eq.2) then
   ii = ipr
-  if (nswrsm(ii).eq.-999) nswrsm(ii) = 5
-  if (abs(epsilo(ii)+999.d0).lt.epzero) epsilo(ii) = 1.d-5
-  if (abs(epsrsm(ii)+999.d0).lt.epzero) epsrsm(ii) = 10.d0*epsilo(ii)
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  if (vcopt%nswrsm.eq.-999) vcopt%nswrsm = 5
+  if (abs(vcopt%epsilo+999.d0).lt.epzero) vcopt%epsilo = 1.d-5
+  if (abs(vcopt%epsrsm+999.d0).lt.epzero) vcopt%epsrsm = 10.d0*vcopt%epsilo
+  call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
   ii = iu
-  if (nswrsm(ii).eq.-999) nswrsm(ii) = 10
-  if (abs(epsilo(ii)+999.d0).lt.epzero) epsilo(ii) = 1.d-5
-  if (abs(epsrsm(ii)+999.d0).lt.epzero) epsrsm(ii) = 10.d0*epsilo(ii)
-  ii = iv
-  if (nswrsm(ii).eq.-999) nswrsm(ii) = 10
-  if (abs(epsilo(ii)+999.d0).lt.epzero) epsilo(ii) = 1.d-5
-  if (abs(epsrsm(ii)+999.d0).lt.epzero) epsrsm(ii) = 10.d0*epsilo(ii)
-  ii = iw
-  if (nswrsm(ii).eq.-999) nswrsm(ii) = 10
-  if (abs(epsilo(ii)+999.d0).lt.epzero) epsilo(ii) = 1.d-5
-  if (abs(epsrsm(ii)+999.d0).lt.epzero) epsrsm(ii) = 10.d0*epsilo(ii)
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  if (vcopt%nswrsm.eq.-999) vcopt%nswrsm = 10
+  if (abs(vcopt%epsilo+999.d0).lt.epzero) vcopt%epsilo = 1.d-5
+  if (abs(vcopt%epsrsm+999.d0).lt.epzero) vcopt%epsrsm = 10.d0*vcopt%epsilo
+  call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
   do jj = 1, nscal
     ii = isca(jj)
-    if (nswrsm(ii).eq.-999) nswrsm(ii) = 10
-    if (abs(epsilo(ii)+999.d0).lt.epzero) epsilo(ii) = 1.d-5
-    if (abs(epsrsm(ii)+999.d0).lt.epzero) epsrsm(ii) = 10.d0*epsilo(ii)
+    call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+    if (vcopt%nswrsm.eq.-999) vcopt%nswrsm = 10
+    if (abs(vcopt%epsilo+999.d0).lt.epzero) vcopt%epsilo = 1.d-5
+    if (abs(vcopt%epsrsm+999.d0).lt.epzero) vcopt%epsrsm = 10.d0*vcopt%epsilo
+    call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
   enddo
 endif
 ii = ipr
-if (nswrsm(ii).eq.-999) nswrsm(ii) = 2
+call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+if (vcopt%nswrsm.eq.-999) vcopt%nswrsm = 2
+call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
 
-do ii = 1, nvarmx
-  if (nswrsm(ii).eq.-999) nswrsm(ii) = 1
-  if (abs(epsilo(ii)+999.d0).lt.epzero) epsilo(ii) = 1.d-8
-  if (abs(epsrsm(ii)+999.d0).lt.epzero) epsrsm(ii) = 10.d0*epsilo(ii)
+do ii = 1, nvar
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  if (vcopt%nswrsm.eq.-999) vcopt%nswrsm = 1
+  if (abs(vcopt%epsilo+999.d0).lt.epzero) vcopt%epsilo = 1.d-8
+  if (abs(vcopt%epsrsm+999.d0).lt.epzero) vcopt%epsrsm = 10.d0*vcopt%epsilo
+  call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
 enddo
 
 ! ---> ANOMAX
@@ -653,15 +492,19 @@ imrgrp = abs(imrgra)
 if (imrgrp.ge.10) imrgrp = imrgrp - 10
 
 if (imrgrp.eq.0.or.imrgrp.ge.4) then
-  do ii = 1, nvarmx
-    if (imligr(ii).eq.-999) then
-      imligr(ii) = -1
+  do ii = 1, nvar
+    call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+    if (vcopt%imligr.eq.-999) then
+      vcopt%imligr = -1
+      call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
     endif
   enddo
 else
-  do ii = 1, nvarmx
-    if (imligr(ii).eq.-999) then
-      imligr(ii) = 1
+  do ii = 1, nvar
+    call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+    if (vcopt%imligr.eq.-999) then
+      vcopt%imligr = 1
+      call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
     endif
   enddo
 endif
@@ -760,29 +603,13 @@ endif
 ! Idem en Spalart-Allmaras.
 
 if (ypluli.lt.-grand) then
-  if (iwallf.eq.4 .or. itytur.eq.4 .or. iturb.eq.70.or.iwallf.eq.6) then
+  if (iwallf.eq.4 .or. itytur.eq.4 .or. iturb.eq.70.or.iwallf.eq.6.or.iturb.eq.60) then
     ypluli = 10.88d0
   else
     ypluli = 1.d0/xkappa
   endif
 endif
 
-
-! ---> Van Driest
-if (idries.eq.-1) then
-  !   On met 1 en supposant qu'en periodicite ou parallele on utilise le
-  !     mode de calcul de la distance a la paroi qui les prend en charge
-  !     (ICDPAR=+/-1, valeur par defaut)
-  if (iturb.eq.40) then
-    idries = 1
-  elseif (iturb.eq.41) then
-    idries = 0
-  elseif (iturb.eq.42) then
-    idries = 0
-  endif
-endif
-
-
 ! ---> ICPSYR
 !      Si l'utilisateur n'a pas modifie ICPSYR, on prend par defaut :
 !        s'il n y a pas de couplage
@@ -807,7 +634,8 @@ if (nscal.gt.0) then
 !       On compte le nombre de scalaires couples
     nscacp = 0
     do iscal = 1, nscal
-      if (icpsyr(iscal).eq.1) then
+      call field_get_key_int(ivarfl(isca(iscal)), kcpsyr, icpsyr)
+      if (icpsyr.eq.1) then
         nscacp = nscacp + 1
       endif
     enddo
@@ -817,7 +645,8 @@ if (nscal.gt.0) then
 
 !         On couple le scalaire temperature de la phase
       if (iscalt.gt.0.and.iscalt.le.nscal) then
-        icpsyr(iscalt) = 1
+        icpsyr = 1
+        call field_set_key_int(ivarfl(isca(iscalt)), kcpsyr, icpsyr)
         goto 100
       endif
  100        continue
@@ -826,14 +655,6 @@ if (nscal.gt.0) then
 
   endif
 
-!     Pour tous les autres scalaires, non renseignes pas l'utilisateur
-!       on ne couple pas
-  do iscal = 1, nscamx
-    if (icpsyr(iscal).eq.-999) then
-      icpsyr(iscal) = 0
-    endif
-  enddo
-
 endif
 
 ! Temperature scale
@@ -891,16 +712,8 @@ endif
 if (icdpar.eq.-1 .and. ikw.eq.1 .and. isuite.eq.1)                &
      write(nfecra,2001)
 
-! ---> INEEDY, IMLIGY
+! ---> IMLIGY
 !      Calcul de la distance a la paroi
-!       (une seule phase ...)
-
-ineedy = 0
-if ((iturb.eq.30.and.irijec.eq.1).or.              &
-     (itytur.eq.4.and.idries.eq.1).or.              &
-     iturb.eq.60.or.iturb.eq.70      ) then
-  ineedy = 1
-endif
 
 if (imrgrp.eq.0 .or. imrgrp.ge.4) then
   if (imligy.eq.-999) then
@@ -913,8 +726,9 @@ else
 endif
 
 !     Warning : non initialise => comme la vitesse
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
 if (iwarny.eq.-999) then
-  iwarny = iwarni(iu)
+  iwarny = vcopt%iwarni
 endif
 
 
@@ -938,37 +752,67 @@ endif
 if (idtvar.lt.0) then
   relxsp = 1.d0-relxst
   if (relxsp.le.epzero) relxsp = relxst
-  if (abs(relaxv(ipr)+999.d0).le.epzero)                 &
-       relaxv(ipr) = relxsp
-  do ii = 1, nvarmx
-    if (abs(relaxv(ii)+999.d0).le.epzero) relaxv(ii) = relxst
+  call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+  if (abs(vcopt%relaxv+999.d0).le.epzero) then
+       vcopt%relaxv = relxsp
+       call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+  endif
+  do ii = 1, nvar
+    call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+    if (abs(vcopt%relaxv+999.d0).le.epzero) then
+      vcopt%relaxv = relxst
+      call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+    endif
   enddo
 else
   if (ikecou.eq.0) then
     if (itytur.eq.5) then !FIXME
-      if (abs(relaxv(ik)+999.d0).lt.epzero)              &
-           relaxv(ik) = 0.7d0
-      if (abs(relaxv(iep)+999.d0).lt.epzero)             &
-           relaxv(iep) = 0.7d0
+      call field_get_key_struct_var_cal_opt(ivarfl(ik), vcopt)
+      if (abs(vcopt%relaxv+999.d0).lt.epzero) then
+        vcopt%relaxv = 1.d0
+        call field_set_key_struct_var_cal_opt(ivarfl(ik), vcopt)
+      endif
+      call field_get_key_struct_var_cal_opt(ivarfl(iep), vcopt)
+      if (abs(vcopt%relaxv+999.d0).lt.epzero) then
+        vcopt%relaxv = 1.d0
+        call field_set_key_struct_var_cal_opt(ivarfl(iep), vcopt)
+      endif
     else if (itytur.eq.2) then
-      if (abs(relaxv(ik)+999.d0).lt.epzero)              &
-           relaxv(ik) = 1.d0
-      if (abs(relaxv(iep)+999.d0).lt.epzero)             &
-           relaxv(iep) = 1.d0
+      call field_get_key_struct_var_cal_opt(ivarfl(ik), vcopt)
+      if (abs(vcopt%relaxv+999.d0).lt.epzero) then
+        vcopt%relaxv = 1.d0
+        call field_set_key_struct_var_cal_opt(ivarfl(ik), vcopt)
+      endif
+      call field_get_key_struct_var_cal_opt(ivarfl(iep), vcopt)
+      if (abs(vcopt%relaxv+999.d0).lt.epzero) then
+        vcopt%relaxv = 1.d0
+        call field_set_key_struct_var_cal_opt(ivarfl(iep), vcopt)
+      endif
     else if (itytur.eq.6) then
-      if (abs(relaxv(ik)+999.d0).lt.epzero)              &
-           relaxv(ik) = 1.d0
-      if (abs(relaxv(iomg)+999.d0).lt.epzero)            &
-           relaxv(iomg) = 1.d0
+      call field_get_key_struct_var_cal_opt(ivarfl(ik), vcopt)
+      if (abs(vcopt%relaxv+999.d0).lt.epzero)  then
+           vcopt%relaxv = 1.d0
+           call field_set_key_struct_var_cal_opt(ivarfl(ik), vcopt)
+      endif
+      call field_get_key_struct_var_cal_opt(ivarfl(iomg), vcopt)
+      if (abs(vcopt%relaxv+999.d0).lt.epzero) then
+           vcopt%relaxv = 1.d0
+           call field_set_key_struct_var_cal_opt(ivarfl(iomg), vcopt)
+      endif
     endif
   endif
   if (iturb.eq.70) then
-    if (abs(relaxv(inusa)+999.d0).lt.epzero) then
-      relaxv(inusa) = 1.D0
+    call field_get_key_struct_var_cal_opt(ivarfl(inusa), vcopt)
+    if (abs(vcopt%relaxv+999.d0).lt.epzero) then
+      vcopt%relaxv = 1.d0
+      call field_set_key_struct_var_cal_opt(ivarfl(inusa), vcopt)
     endif
   endif
-  if (abs(relaxv(ipr)+999.d0).lt.epzero)                 &
-       relaxv(ipr) = 1.d0
+  call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+  if (abs(vcopt%relaxv+999.d0).lt.epzero) then
+     vcopt%relaxv = 1.d0
+     call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+  endif
 endif
 
 ! ---> SPECIFIQUE STATIONNAIRE
@@ -976,10 +820,13 @@ if (idtvar.lt.0) then
   dtref = 1.d0
   dtmin = 1.d0
   dtmax = 1.d0
-  do ii = 1, nvarmx
-    istat(ii) = 0
+  do ii = 1, nvar
+    call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+    vcopt%istat = 0
+    call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
   enddo
-  arak = arak/max(relaxv(iu),epzero)
+  call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
+  arak = arak/max(vcopt%relaxv,epzero)
 endif
 
 !===============================================================================
@@ -994,14 +841,6 @@ cpow    = apow**(2.d0/(1.d0-bpow))
 dpow    = 1.d0/(1.d0+bpow)
 cmu025 = cmu**0.25d0
 
-if (iturb.eq.30.or.iturb.eq.31) then
-  sigmae = 1.22d0
-else if (iturb.eq.32) then
-  sigmae = 1.15d0
-else
-  sigmae = 1.30d0
-endif
-
 if (idirsm.eq.0) then
   csrij = 0.11d0
 else
@@ -1012,6 +851,12 @@ else
   endif
 endif
 
+if (iturb.eq.60) then !sst-ddes
+  cddes = 0.65d0
+elseif (iturb.eq.51) then !phif-ddes
+  cddes = 0.60d0
+endif
+
 ! ---> ICLVFL
 !      Si l'utilisateur n'a pas modifie ICLVFL, on prend par defaut :
 !        0 pour les variances
@@ -1088,21 +933,31 @@ if (nscal.gt.0) then
   do iscal = 1, nscal
     ! AFM and GGDH on the scalar
     if (ityturt(iscal).eq.1.or.ityturt(iscal).eq.2) then
-      idften(isca(iscal)) = 6
+      call field_get_key_struct_var_cal_opt(ivarfl(isca(iscal)), vcopt)
+      vcopt%idften = ANISOTROPIC_RIGHT_DIFFUSION
       ctheta(iscal) = cthafm
-
+      call field_set_key_struct_var_cal_opt(ivarfl(isca(iscal)), vcopt)
     ! DFM on the scalar
     elseif (ityturt(iscal).eq.3) then
-      idifft(isca(iscal)) = 0
-      idften(isca(iscal)) = 1
-      ctheta(iscal) = cthdfm
+      call field_get_key_struct_var_cal_opt(ivarfl(isca(iscal)), vcopt)
+      vcopt%idifft = 0
+      vcopt%idften = ISOTROPIC_DIFFUSION
+      if (iturt(iscal).eq.31) then
+        ctheta(iscal) = cthebdfm
+        c2trit = 0.3d0
+      else
+        ctheta(iscal) = cthdfm
+      end if
+      call field_set_key_struct_var_cal_opt(ivarfl(isca(iscal)), vcopt)
       ! GGDH on the thermal fluxes is automatically done
 
       ! GGDH on the variance of the thermal scalar
       do ii = 1, nscal
         if (iscavr(ii).eq.iscal) then
-          idften(isca(ii)) = 6
+          call field_get_key_struct_var_cal_opt(ivarfl(isca(ii)), vcopt)
+          vcopt%idften = ANISOTROPIC_RIGHT_DIFFUSION
           ctheta(ii) = csrij
+          call field_set_key_struct_var_cal_opt(ivarfl(isca(ii)), vcopt)
         endif
       enddo
     else
@@ -1115,12 +970,16 @@ endif
 if (ippmod(idarcy).eq.1) then
 
   if (darcy_anisotropic_permeability.eq.1) then
-    idften(ipr) = 6
-  endif
+    call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+    vcopt%idften = ANISOTROPIC_LEFT_DIFFUSION
+    call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+ endif
 
   if (darcy_anisotropic_dispersion.eq.1) then
     do iscal = 1, nscal
-      idften(isca(iscal)) = 6
+      call field_get_key_struct_var_cal_opt(ivarfl(isca(iscal)), vcopt)
+      vcopt%idften = ANISOTROPIC_LEFT_DIFFUSION
+      call field_set_key_struct_var_cal_opt(ivarfl(isca(iscal)), vcopt)
     enddo
   endif
 
@@ -1175,23 +1034,27 @@ if (cfopre.lt.-0.5d0*grand) cfopre = 2.0d0
 call nbccpl(nbrcpl)
 !==========
 
-if (nbrcpl.ge.1) then
-  ! Si on est en couplage rotor/stator avec resolution en repere absolu
-  call angular_velocity(1, omgnrm)
-  omgnrm = abs(omgnrm)
-  if (omgnrm.ge.epzero) then
-    ! Couplage avec interpolation aux faces
-    ifaccp = 1
-    ! Mobile mesh
-    if (icorio.eq.0) then
-      imobil = 1
-      call cs_post_set_deformable
-    endif
+if (nbrcpl.ge.1.and.iturbo.ne.0) then
+  ifaccp = 1
+endif
+
+!===============================================================================
+! 8. Parameters of VOF/cavitation modules
+!===============================================================================
+
+if (ivofmt.ge.0) then
+  if (clvfmn.lt.-grand) then
+    clvfmn = 0.d0
+    if (icavit.gt.0) clvfmn = epzero
+  endif
+  if (clvfmx.gt.grand) then
+    clvfmx = 1.d0
+    if (icavit.gt.0) clvfmx = 1.d0-epzero
   endif
 endif
 
 !===============================================================================
-! 8. STOP SI PB
+! 9. STOP SI PB
 !===============================================================================
 
 if (iok.ne.0) then
@@ -1247,22 +1110,6 @@ endif
 '@  Verifier cs_user_parameters.f90,'                           /,&
 '@',                                                            /,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 1031 format(                                                     &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES',               /,&
-'@    =========,'                                               /,&
-'@    ',a17,                                                    /,&
-'@    ',a6,' SERA INITIALISE AUTOMATIQUEMENT.,'                 /,&
-'@    NE PAS LE MODIFIER.,'                                     /,&
-'@',                                                            /,&
-'@  Le calcul ne sera pas execute.',                            /,&
-'@',                                                            /,&
-'@  Verifier cs_user_parameters.f90,'                           /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@',                                                            /)
  1131 format( &
 '@',                                                            /,&
@@ -1274,33 +1121,6 @@ endif
 '@    ',a6,'.'                                                  /,&
 '@',                                                            /,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 1032 format(                                                     &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES',               /,&
-'@    =========,'                                               /,&
-'@    VITESSE DE MAILLAGE EN ALE',                              /,&
-'@    ',a6,' SERA INITIALISE AUTOMATIQUEMENT.,'                 /,&
-'@    NE PAS LE MODIFIER.,'                                     /,&
-'@',                                                            /,&
-'@  Le calcul ne sera pas execute.',                            /,&
-'@',                                                            /,&
-'@  Verifier cs_user_parameters.f90,'                           /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 1041 format(                                                     &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : MODIFICATION AVANCEE DE ',                    /,&
-'@    =========,'                                               /,&
-'@    ',a8,' ',i10,                                             /,&
-'@    ',a6,'.'                                                  /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@',                                                            /)
  1061 format(                                                     &
 '@',                                                            /,&
@@ -1458,22 +1278,6 @@ endif
 '@  Check cs_user_parameters.f90',                              /,&
 '@',                                                            /,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 1031 format(                                                     &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ WARNING: ABORT IN THE DATA SPECIFICATION',                /,&
-'@    ========',                                                /,&
-'@    ',a17,                                                    /,&
-'@    ',a6,' WILL BE INITIALIZED AUTOMATICALLY',                /,&
-'@    DO NOT MODIFY IT.,'                                       /,&
-'@',                                                            /,&
-'@  The calculation will not be run.',                          /,&
-'@',                                                            /,&
-'@  Check cs_user_parameters.f90',                              /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@',                                                            /)
  1131 format( &
 '@',                                                            /,&
@@ -1487,33 +1291,6 @@ endif
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@',                                                            /)
 
- 1032 format(                                                     &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ WARNING: ABORT IN THE DATA SPECIFICATION',                /,&
-'@    ========',                                                /,&
-'@    MESH VELOCITY IN ALE',                                    /,&
-'@    ',a6,' WILL BE INITIALIZED AUTOMATICALLY',                /,&
-'@    DO NOT MODIFY IT.,'                                       /,&
-'@',                                                            /,&
-'@  The calculation will not be run.',                          /,&
-'@',                                                            /,&
-'@  Check cs_user_parameters.f90',                              /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 1041 format(                                                     &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ WARNING: ADVANCED MODIFICATION OF',                       /,&
-'@    ========',                                                /,&
-'@    ',a8,' ',i10,                                             /,&
-'@    ',a6,'.',                                                 /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
  1061 format(                                                     &
 '@',                                                            /,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
