@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,6 +27,8 @@ module numvar
 
   !=============================================================================
 
+  use, intrinsic :: iso_c_binding
+
   use paramx
 
   implicit none
@@ -64,9 +66,9 @@ module numvar
   !> velocity component \f$ u_z \f$
   integer, save :: iw
 
-  !> \anchor ivoidf
-  !> void fraction for cavitation modelling
-  integer, save :: ivoidf
+  !> \anchor ivolf2
+  !> void fraction for VOF method
+  integer, save :: ivolf2
 
   !> \anchor ik
   !> turbulent kinetic energy \f$ k \f$
@@ -133,7 +135,7 @@ module numvar
   integer, save :: iscapp(nscamx)
 
   !> \anchor nscaus
-  !> number of user scalars
+  !> number of user scalars solutions of an advection equation
   integer, save :: nscaus
 
   !> \anchor nscapp
@@ -142,7 +144,7 @@ module numvar
 
   !> \anchor nscasp
   !> number of species scalars
-  integer, save :: nscasp
+  integer(c_int), pointer, save :: nscasp
 
   !> \anchor iuma
   !> mesh velocity component \f$ w_x \f$
@@ -163,21 +165,15 @@ module numvar
   !----------------------------------------------------------------------------
 
   !> \defgroup physical_prop Physical properties
-  !> \brief Physical properties are stored in propce.
+  !> \brief Physical properties are using the field API.
   !> See \ref cs_user_boundary_conditions for some examples.
 
   !> \addtogroup physical_prop
   !> \{
 
-  !> pointer to cell properties (propce)
-  integer, save :: ipproc(npromx)
-
-  !> Density at the current time step
+  !> Density at the current time step (equal to icrom, kept for compatibility)
   integer, save :: irom
 
-  !> Density at the previous time step
-  integer, save :: iroma
-
   !> Density at the second previous time step
   integer, save :: iromaa
 
@@ -187,15 +183,6 @@ module numvar
   !> dynamic turbulent viscosity
   integer, save :: ivisct
 
-  !> dynamic molecular viscosity (in kg/(m.s)) at the previous time-step
-  integer, save :: ivisla
-
-  !> dynamic turbulent viscosity at the previous time-step
-  integer, save :: ivista
-
-  !> specific heat \f$ C_p \f$ at the previous time-step
-  integer, save :: icpa
-
   !> error estimator for Navier-Stokes
   integer, save :: iestim(nestmx)
 
@@ -208,12 +195,18 @@ module numvar
   !> variable diffusivity field id key for scalars
   integer, save :: kivisl
 
+  !> variable density field id key for scalars
+  integer, save :: kromsl
+
   !> source terms at previous time step for 2nd order
   integer, save :: kstprv
 
   !> source terms at the current time step (used for limiters)
   integer, save :: kst
 
+  !> turbulent schmidt key for scalars
+  integer, save :: ksigmas
+
   !> convective mass flux of the variables at the previous time-step
   integer, save :: ifluaa(nvarmx)
 
@@ -248,7 +241,7 @@ module numvar
 
   !> Mesh velocity viscosity for the ALE module
   !> \remark might be orthotropic
-  integer, save :: ivisma(3)
+  integer, save :: ivisma
 
   !> pointer for dilatation source terms
   integer, save :: iustdy(nscamx)
@@ -273,6 +266,23 @@ module numvar
 
   !>  field id of temperature at boundary
   integer, save ::  itempb
+
+  !> field id of the square of the norm of the deviatoric
+  !> part of the deformation rate tensor (\f$S^2=2S_{ij}^D S_{ij}^D\f$).
+  !> Field defined only with the \f$k-\omega\f$ (SST) turbulence model
+  integer, save :: is2kw
+
+  !> field id of the divergence of the velocity. More precisely,
+  !> it is the trace of the velocity gradient (and not a finite volume
+  !> divergence term). In the cell \c iel,  \f$div(\vect{u})\f$ is given
+  !> by \c divukw(iel1). This array is defined only with the \f$k-\omega\f$ SST
+  !> turbulence model (because in this case it may be calculated at the same
+  !> time as \f$S^2\f$)
+  integer, save :: idivukw
+
+  !> field id of the strain rate tensor at the previous time step
+  integer, save :: istraio
+
   !> \}
   !----------------------------------------------------------------------------
   ! Numerical properties
@@ -299,9 +309,6 @@ module numvar
   !> Field id for variable i
   integer, save :: ivarfl(nvarmx)
 
-  !> Field id for property i
-  integer, save :: iprpfl(npromx)
-
   !> Field id for the dttens tensor
   integer, save :: idtten
 
@@ -309,6 +316,81 @@ module numvar
 
   !=============================================================================
 
+  interface
+
+    !---------------------------------------------------------------------------
+
+    !> \cond DOXYGEN_SHOULD_SKIP_THIS
+
+    !---------------------------------------------------------------------------
+
+    ! Interface to C function retrieving the number of species in the gas mix
+    ! if gas mix model is enabled (igmix)
+
+    subroutine cs_f_gas_mix_get_pointers(nscasp) &
+      bind(C, name='cs_f_gas_mix_get_pointers')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      type(c_ptr), intent(out) :: nscasp
+    end subroutine cs_f_gas_mix_get_pointers
+
+    !---------------------------------------------------------------------------
+
+    !> (DOXYGEN_SHOULD_SKIP_THIS) \endcond
+
+    !---------------------------------------------------------------------------
+
+  end interface
+
+  !=============================================================================
+
+contains
+
+  !=============================================================================
+
+  !> \addtogroup field_map
+  !> \{
+
+  !> \anchor iprpfl
+  !> Identity function for compatibility with deprecated iprpfl array
+
+  elemental pure function iprpfl(f_id) result(r_f_id)
+
+    implicit none
+
+    ! Parameters
+
+    integer, intent(in) :: f_id
+    integer             :: r_f_id
+
+    ! Function body
+
+    r_f_id = f_id
+
+  end function iprpfl
+
+  !> \}
+
+  !> \brief Initialize Fortran gas mix API.
+  !> This maps Fortran pointers to global C variables.
+
+  subroutine gas_mix_options_init
+
+    use, intrinsic :: iso_c_binding
+    implicit none
+
+    ! Local variables
+
+    type(c_ptr) :: c_nscasp
+
+    call cs_f_gas_mix_get_pointers(c_nscasp)
+
+    call c_f_pointer(c_nscasp, nscasp)
+
+  end subroutine gas_mix_options_init
+
+  !=============================================================================
+
   !> \}
 
 end module numvar
