@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -62,6 +62,7 @@ use period
 use mesh
 use field
 use field_operator
+use cs_c_bindings
 
 !===============================================================================
 
@@ -78,7 +79,7 @@ double precision, dimension(nfbrps), intent(out)   :: bflux
 integer ::         inc, iccocg
 integer ::         ifac, iloc, ivar
 integer ::         iel
-integer ::         ifccp, iflmab
+integer ::         iflmab
 
 double precision :: cpp   , srfbn
 double precision :: flumab, diipbx, diipby, diipbz, tcel
@@ -89,6 +90,8 @@ double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
 double precision, allocatable, dimension(:,:) :: grad
 double precision, dimension(:), pointer :: bmasfl, tscalp
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 ! Initialize variables to avoid compiler warnings
@@ -111,11 +114,8 @@ if (iscalt.gt.0) then
 
   call field_get_val_prev_s(ivarfl(ivar), tscalp)
 
-  if (iscacp(iscalt).eq.1 .and. icp.gt.0) then
-    ifccp  = iprpfl(icp)
-    call field_get_val_s(ifccp, cpro_cp)
-  else
-    ifccp = -1
+  if (iscacp(iscalt).eq.1 .and. icp.ge.0) then
+    call field_get_val_s(icp, cpro_cp)
   endif
 
   call field_get_key_int(ivarfl(ivar), kbmasf, iflmab)
@@ -123,8 +123,10 @@ if (iscalt.gt.0) then
 
   ! Compute variable values at boundary faces
 
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
   ! Reconstructed fluxes
-  if (ircflu(ivar) .gt. 0 .and. itbrrb.eq.1) then
+  if (vcopt%ircflu .gt. 0 .and. itbrrb.eq.1) then
 
     ! Compute gradient of temperature / enthalpy
 
@@ -153,7 +155,7 @@ if (iscalt.gt.0) then
              + diipbx*grad(1,iel) + diipby*grad(2,iel) + diipbz*grad(3,iel)
 
       if (iscacp(iscalt).eq.1) then
-        if (ifccp.ge.0) then
+        if (icp.ge.0) then
           cpp = cpro_cp(iel)
         else
           cpp = cp0
@@ -184,7 +186,7 @@ if (iscalt.gt.0) then
       tcel = tscalp(iel)
 
       if (iscacp(iscalt).eq.1) then
-        if (ifccp.ge.0) then
+        if (icp.ge.0) then
           cpp = cpro_cp(iel)
         else
           cpp = cp0
@@ -260,6 +262,7 @@ use period
 use mesh
 use field
 use field_operator
+use cs_c_bindings
 
 !===============================================================================
 
@@ -288,6 +291,8 @@ double precision, allocatable, dimension(:,:) :: grad
 double precision, dimension(:), pointer :: tplusp, tstarp
 double precision, dimension(:), pointer :: tscalp
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 ! pointers to T+ and T* if saved
@@ -318,8 +323,10 @@ if (itstar.ge.0 .and. itplus.ge.0) then
 
   ! Compute variable values at boundary faces
 
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
   ! Reconstructed fluxes
-  if (ircflu(ivar) .gt. 0 .and. itbrrb.eq.1) then
+  if (vcopt%ircflu .gt. 0 .and. itbrrb.eq.1) then
 
     ! Boundary condition pointers for gradients and advection
 
@@ -658,4 +665,3 @@ enddo
 
 return
 end subroutine post_stress_tangential
-
