@@ -90,13 +90,10 @@
 !>                              composing the hydrostatic pressure
 !> \param[in]     tpucou        non scalar time step in case of
 !>                               velocity pressure coupling
-!> \param[in]     trav          right hand side for the normalizing
-!>                               the residual
 !> \param[in]     viscf         visc*surface/dist aux faces internes
 !> \param[in]     viscb         visc*surface/dist aux faces de bord
-!> \param[in]     dpvar         tableau de travail pour increment
+!> \param[in]     phi           potential to be solved (pressure increment)
 !> \param[in]     tslagr        coupling term for the Lagrangian module
-!> \param[in]     trava         tableau de travail pour couplage
 !_______________________________________________________________________________
 
 subroutine resopv &
@@ -105,10 +102,9 @@ subroutine resopv &
    dt     , vel    ,                                              &
    coefav , coefbv , coefa_dp        , coefb_dp ,                 &
    smacel , spcond , svcond ,                                     &
-   frcxt  , dfrcxt , tpucou , trav   ,                            &
+   frcxt  , dfrcxt , tpucou ,                                     &
    viscf  , viscb  ,                                              &
-   dpvar  , tslagr ,                                              &
-   trava  )
+   phi    , tslagr )
 
 !===============================================================================
 
@@ -158,11 +154,9 @@ double precision smacel(ncesmp,nvar), spcond(nfbpcd,nvar)
 double precision svcond(ncelet,nvar)
 double precision frcxt(3,ncelet), dfrcxt(3,ncelet)
 double precision, dimension (1:6,1:ncelet), target :: tpucou
-double precision trav(3,ncelet)
 double precision viscf(nfac), viscb(nfabor)
-double precision dpvar(ncelet)
+double precision phi(ncelet)
 double precision tslagr(ncelet,*)
-double precision trava(ndim,ncelet)
 double precision coefav(3  ,nfabor)
 double precision coefbv(3,3,nfabor)
 double precision vel   (3  ,ncelet)
@@ -210,7 +204,7 @@ type(var_cal_opt) :: vcopt_p, vcopt_u
 double precision rvoid(1)
 
 double precision, allocatable, dimension(:) :: dam, xam
-double precision, allocatable, dimension(:) :: res, presa
+double precision, allocatable, dimension(:) :: res, phia
 double precision, dimension(:,:), allocatable :: gradp
 double precision, allocatable, dimension(:) :: coefaf_dp, coefbf_dp
 double precision, allocatable, dimension(:) :: coefap, coefbp, coefa_dp2
@@ -218,10 +212,10 @@ double precision, allocatable, dimension(:) :: coefa_rho, coefb_rho
 double precision, allocatable, dimension(:) :: cofafp, cofbfp, coefaf_dp2
 double precision, allocatable, dimension(:) :: rhs, rovsdt
 double precision, allocatable, dimension(:) :: hydro_pres
-double precision, allocatable, dimension(:) :: velflx, velflb, ddpvar
+double precision, allocatable, dimension(:) :: velflx, velflb, ddphi
 double precision, allocatable, dimension(:,:) :: coefar, cofafr
 double precision, allocatable, dimension(:,:,:) :: coefbr, cofbfr
-double precision, allocatable, dimension(:) :: adxk, adxkm1, dpvarm1, rhs0
+double precision, allocatable, dimension(:) :: adxk, adxkm1, dphim1, rhs0
 double precision, allocatable, dimension(:,:) :: weighf
 double precision, allocatable, dimension(:) :: weighb
 double precision, allocatable, dimension(:,:) :: frchy, dfrchy
@@ -236,13 +230,14 @@ double precision, dimension(:), pointer :: viscap
 double precision, dimension(:,:), pointer :: vitenp
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: brom, crom, croma
-double precision, dimension(:), pointer :: cvar_pr, cvara_pr
+double precision, dimension(:), pointer :: cvar_pr
 double precision, dimension(:), pointer :: cpro_divu
 double precision, dimension(:), pointer :: cpro_wgrec_s
 double precision, dimension(:,:), pointer :: cpro_wgrec_v
 double precision, dimension(:), pointer :: c_estim_der
 double precision, dimension(:), pointer :: cpro_tsrho
-double precision, allocatable, dimension(:) :: surfbm
+double precision, allocatable, dimension(:) :: surfbm, dphi
+double precision, allocatable, dimension(:,:) :: trav
 
 !===============================================================================
 
@@ -259,12 +254,14 @@ call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
 
 ! Allocate temporary arrays
 allocate(dam(ncelet), xam(nfac))
-allocate(res(ncelet), presa(ncelet))
+allocate(res(ncelet), phia(ncelet))
 allocate(rhs(ncelet), rovsdt(ncelet))
 allocate(iflux(nfac), bflux(ndimfb))
+allocate(dphi(ncelet))
+allocate(trav(3, ncelet))
 iswdyp = vcopt_p%iswdyn
 if (iswdyp.ge.1) allocate(adxk(ncelet), adxkm1(ncelet),   &
-                          dpvarm1(ncelet), rhs0(ncelet))
+                          dphim1(ncelet), rhs0(ncelet))
 if (icalhy.eq.1) allocate(frchy(ndim,ncelet),             &
                           dfrchy(ndim,ncelet), hydro_pres(ncelet))
 
@@ -321,7 +318,6 @@ endif
 call field_get_val_s(ibrom, brom)
 
 call field_get_val_s(ivarfl(ipr), cvar_pr)
-call field_get_val_prev_s(ivarfl(ipr), cvara_pr)
 
 call field_get_key_int(ivarfl(ipr), kimasf, iflmas)
 call field_get_key_int(ivarfl(ipr), kbmasf, iflmab)
@@ -386,153 +382,7 @@ if (ivofmt.ge.0.or.idilat.eq.4) then
 endif
 
 !===============================================================================
-! 2. Norm residual
-!===============================================================================
-
-!     Test d'un residu de normalisation de l'etape de pression
-!       plus comprehensible = div(rho u* + dt gradP^(n))-Gamma
-!       i.e. second membre du systeme en pression hormis la partie
-!       pression (sinon a convergence, on tend vers 0)
-!       Represente les termes que la pression doit equilibrer
-!     On calcule ici div(rho dt/rho gradP^(n)) et on complete a la fin
-!       avec  div(rho u*)
-!     Pour grad P^(n) on suppose que des CL de Neumann homogenes
-!       s'appliquent partout : on peut donc utiliser les CL de la
-!       vitesse pour u*+dt/rho gradP^(n). Comme on calcule en deux fois,
-!       on utilise les CL de vitesse homogenes pour dt/rho gradP^(n)
-!       ci-dessous et les CL de vitesse completes pour u* a la fin.
-
-if (iphydr.eq.1) then
-  do iel = 1, ncel
-    unsvom = -1.d0/volume(iel)
-    trav(1,iel) = trav(1,iel)*unsvom + frcxt(1 ,iel) + dfrcxt(1 ,iel)
-    trav(2,iel) = trav(2,iel)*unsvom + frcxt(2 ,iel) + dfrcxt(2 ,iel)
-    trav(3,iel) = trav(3,iel)*unsvom + frcxt(3 ,iel) + dfrcxt(3 ,iel)
-  enddo
-else
-  if (isno2t.gt.0 .and. nterup.gt.1) then
-    do iel = 1, ncel
-      unsvom = -1.d0/volume(iel)
-      romro0 = crom(iel)-ro0
-      trav(1,iel) = (trav(1,iel)+trava(1,iel))*unsvom + romro0*gx
-      trav(2,iel) = (trav(2,iel)+trava(2,iel))*unsvom + romro0*gy
-      trav(3,iel) = (trav(3,iel)+trava(3,iel))*unsvom + romro0*gz
-    enddo
-  else
-    do iel = 1, ncel
-      unsvom = -1.d0/volume(iel)
-      romro0 = crom(iel)-ro0
-      trav(1,iel) = trav(1,iel)*unsvom + romro0*gx
-      trav(2,iel) = trav(2,iel)*unsvom + romro0*gy
-      trav(3,iel) = trav(3,iel)*unsvom + romro0*gz
-    enddo
-  endif
-endif
-do iel = 1, ncel
-  dtsrom = dt(iel)/crom(iel)
-  do isou = 1, 3
-    trav(isou,iel) = vel(isou,iel) +dtsrom*trav(isou,iel)
-  enddo
-enddo
-
-!---> TRAITEMENT DU PARALLELISME ET DE LA PERIODICITE
-
-if (irangp.ge.0.or.iperio.eq.1) then
-  call synvin(trav)
-endif
-
-! To save time, no space reconstruction
-init   = 1
-inc    = 1
-iflmb0 = 1
-if (iale.eq.1) iflmb0 = 0
-nswrgp = 1
-imligp = vcopt_u%imligr
-iwarnp = vcopt_p%iwarni
-epsrgp = vcopt_u%epsrgr
-climgp = vcopt_u%climgr
-itypfl = 1
-! VOF algorithm: the pressure step corresponds to the
-! correction of the volumetric flux, not the mass flux
-if (idilat.ge.4.or.ivofmt.ge.0) itypfl = 0
-
-call inimav &
-(f_id0  , itypfl ,                                              &
- iflmb0 , init   , inc    , imrgra , nswrgp , imligp ,          &
- iwarnp ,                                                       &
- epsrgp , climgp ,                                              &
- crom, brom   ,                                                 &
- trav   ,                                                       &
- coefav , coefbv ,                                              &
- iflux  , bflux  )
-
-init = 1
-call divmas(init, iflux, bflux, res)
-
-! --- Weakly compressible algorithm: semi analytic scheme
-if (idilat.ge.4) then
-  do iel = 1, ncel
-    res(iel) = res(iel)*crom(iel)
-  enddo
-endif
-
-! --- Volumic Gamma source term adding for volumic mass flow rate
-if (ncesmp.gt.0) then
-  do ii = 1, ncesmp
-    iel = icetsm(ii)
-    res(iel) = res(iel) - cell_f_vol(iel)*smacel(ii,ipr)
-  enddo
-endif
-
-! --- Surface Gamma source term adding for surface condensation modelling
-if (nfbpcd.gt.0) then
-  do ii = 1, nfbpcd
-    ifac= ifbpcd(ii)
-    iel = ifabor(ifac)
-    res(iel) = res(iel) - surfbn(ifac)*spcond(ii,ipr)
-  enddo
-endif
-
-! --- volume Gamma source term adding for volume condensation modelling
-if (icondv.eq.0) then
-  allocate(surfbm(ncelet))
-  surfbm(:) = 0.d0
-
-  do ii = 1, ncmast
-    iel= ltmast(ii)
-    surfbm(iel) = s_metal*volume(iel)/voltot
-    res(iel) = res(iel) - surfbm(iel)*svcond(iel,ipr)
-  enddo
-
-  deallocate(surfbm)
-endif
-
-
-! Cavitation source term
-if (icavit.gt.0) then
-  do iel = 1, ncel
-    res(iel) = res(iel) -cell_f_vol(iel)*gamcav(iel)*(1.d0/rho2 - 1.d0/rho1)
-  enddo
-endif
-
-! Lagrangian: coupling
-if (iilagr.eq.2 .and. ltsmas.eq.1) then
-
-  do iel = 1, ncel
-    res(iel) = res(iel) -tslagr(iel,itsmas)
-  enddo
-
-endif
-
-rnormp = sqrt(cs_gdot(ncel,res,res))
-
-if (vcopt_p%iwarni.ge.2) then
-  write(nfecra,1300)chaine(1:16) ,rnormp
-endif
-sinfo%nbivar = 0
-
-!===============================================================================
-! 3. Compute an approximated pressure increment if needed
+! 2. Compute an approximated pressure increment if needed
 !    that is when there are buoyancy terms (gravity and variable density)
 !    with a free outlet.
 !===============================================================================
@@ -664,7 +514,7 @@ if (iphydr.eq.1.and.icalhy.eq.1) then
       cofafp , cofbfp ,                       &
       viscf  , viscb  ,                       &
       dam    , xam    ,                       &
-      dpvar  , rhs    )
+      dphi   , rhs    ) !FIXME remove work arrays.
 
     ! Free memory
     deallocate(coefap, cofafp)
@@ -819,7 +669,7 @@ if (iphydr.eq.1.or.iifren.eq.1) then
 endif
 
 !===============================================================================
-! 4. Building of the linear system to solve
+! 3. Building of the linear system to solve
 !===============================================================================
 
 ! ---> Implicit term
@@ -919,11 +769,8 @@ call matrix &
    imasfl , bmasfl , viscf  , viscb  ,                            &
    rvoid  , dam    , xam    )
 
-! Free memory
-deallocate(iflux, bflux)
-
 !===============================================================================
-! 5. Mass flux initialization
+! 4. Mass flux initialization
 !===============================================================================
 
 ! --- Flux de masse predit et premiere composante Rhie et Chow
@@ -932,7 +779,7 @@ deallocate(iflux, bflux)
 allocate(gradp(3,ncelet))
 
 iccocg = 1
-iprev  = 1
+iprev  = 0
 inc    = 1
 
 ! Pressure gradient
@@ -961,69 +808,80 @@ endif
 !     The RHS contains rho div(u*) and not div(rho u*)
 !     so this term will be add afterwards
 if (idilat.ge.4) then
-  if (iand(vcopt_p%idften, ISOTROPIC_DIFFUSION).ne.0) then
+  if (arak.gt.0.d0 .and. iand(vcopt_p%idften, ISOTROPIC_DIFFUSION).ne.0) then
     do iel = 1, ncel
       ardtsr  = arak*(dt(iel)/crom(iel))
       do isou = 1, 3
         trav(isou,iel) = ardtsr*trav(isou,iel)
       enddo
     enddo
-  else if (iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0) then
+  else if (arak.gt.0.d0 .and. iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0) then
     do iel = 1, ncel
       arsr  = arak/crom(iel)
 
       trav(1,iel) = arsr*(                                 &
-                           vitenp(1,iel)*(trav(1,iel))     &
-                         + vitenp(4,iel)*(trav(2,iel))     &
-                         + vitenp(6,iel)*(trav(3,iel))     &
+                           vitenp(1,iel)*trav(1,iel)      &
+                         + vitenp(4,iel)*trav(2,iel)      &
+                         + vitenp(6,iel)*trav(3,iel)      &
                          )
       trav(2,iel) = arsr*(                                 &
-                           vitenp(4,iel)*(trav(1,iel))     &
-                         + vitenp(2,iel)*(trav(2,iel))     &
-                         + vitenp(5,iel)*(trav(3,iel))     &
+                           vitenp(4,iel)*trav(1,iel)      &
+                         + vitenp(2,iel)*trav(2,iel)      &
+                         + vitenp(5,iel)*trav(3,iel)      &
                          )
       trav(3,iel) = arsr*(                                 &
-                           vitenp(6,iel)*(trav(1,iel))     &
-                         + vitenp(5,iel)*(trav(2,iel))     &
-                         + vitenp(3,iel)*(trav(3,iel))     &
+                           vitenp(6,iel)*trav(1,iel)      &
+                         + vitenp(5,iel)*trav(2,iel)      &
+                         + vitenp(3,iel)*trav(3,iel)      &
                          )
 
     enddo
+  else
+    !$omp parallel do private(isou)
+    do iel = 1, ncel
+      do isou = 1, 3
+        trav(isou,iel) = 0.d0
+      enddo
+    enddo
   endif
 
 ! Standard algorithm
 else
-  if (iand(vcopt_p%idften, ISOTROPIC_DIFFUSION).ne.0) then
+  if (arak.gt.0.d0 .and. iand(vcopt_p%idften, ISOTROPIC_DIFFUSION).ne.0) then
     do iel = 1, ncel
       ardtsr  = arak*(dt(iel)/crom(iel))
       do isou = 1, 3
         trav(isou,iel) = vel(isou,iel) + ardtsr*trav(isou,iel)
       enddo
     enddo
-  else if (iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0) then
+  else if (arak.gt.0.d0 .and. iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0) then
     do iel = 1, ncel
       arsr  = arak/crom(iel)
 
-      trav(1,iel) = vel(1,iel)                             &
-                  + arsr*(                                 &
-                           vitenp(1,iel)*(trav(1,iel))     &
-                         + vitenp(4,iel)*(trav(2,iel))     &
-                         + vitenp(6,iel)*(trav(3,iel))     &
+      trav(1,iel) = vel(1,iel) + arsr*(                   &
+                           vitenp(1,iel)*trav(1,iel)      &
+                         + vitenp(4,iel)*trav(2,iel)      &
+                         + vitenp(6,iel)*trav(3,iel)      &
                          )
-      trav(2,iel) = vel(2,iel)                             &
-                  + arsr*(                                 &
-                           vitenp(4,iel)*(trav(1,iel))     &
-                         + vitenp(2,iel)*(trav(2,iel))     &
-                         + vitenp(5,iel)*(trav(3,iel))     &
+      trav(2,iel) = vel(2,iel) + arsr*(                   &
+                           vitenp(4,iel)*trav(1,iel)      &
+                         + vitenp(2,iel)*trav(2,iel)      &
+                         + vitenp(5,iel)*trav(3,iel)      &
                          )
-      trav(3,iel) = vel(3,iel)                             &
-                  + arsr*(                                 &
-                           vitenp(6,iel)*(trav(1,iel))     &
-                         + vitenp(5,iel)*(trav(2,iel))     &
-                         + vitenp(3,iel)*(trav(3,iel))     &
+      trav(3,iel) = vel(3,iel) + arsr*(                   &
+                           vitenp(6,iel)*trav(1,iel)      &
+                         + vitenp(5,iel)*trav(2,iel)      &
+                         + vitenp(3,iel)*trav(3,iel)      &
                          )
 
     enddo
+  else
+    !$omp parallel do private(isou)
+    do iel = 1, ncel
+      do isou = 1, 3
+        trav(isou, iel) = vel(isou, iel)
+      enddo
+    enddo
   endif
 endif
 
@@ -1108,8 +966,8 @@ iccocg = 1
 !----------------------
 if (arak.gt.0.d0) then
 
-! --- Prise en compte de Arak : la viscosite face est multipliee
-!       Le pas de temps aussi. On retablit plus bas.
+  ! --- Prise en compte de Arak : la viscosite face est multipliee
+  !       Le pas de temps aussi. On retablit plus bas.
   do ifac = 1, nfac
     viscf(ifac) = arak*viscf(ifac)
   enddo
@@ -1147,8 +1005,8 @@ if (arak.gt.0.d0) then
    0      , iwarnp ,                                                           &
    epsrgp , climgp , extrap ,                                                  &
    frcxt  ,                                                                    &
-   cvara_pr          ,                                                         &
-   coefa_p , coefb_p , coefaf_p , coefbf_p ,                                   &
+   cvar_pr,                                                                    &
+   coefa_p, coefb_p, coefaf_p        , coefbf_p        ,                       &
    viscf  , viscb  ,                                                           &
    viscap ,                                                                    &
    imasfl , bmasfl )
@@ -1217,7 +1075,7 @@ if (arak.gt.0.d0) then
    iphydr , 0      , iwarnp ,                                              &
    epsrgp , climgp , extrap ,                                              &
    frcxt  ,                                                                &
-   cvara_pr,                                                               &
+   cvar_pr,                                                                &
    coefa_p , coefb_p , coefaf_p , coefbf_p ,                               &
    viscf  , viscb  ,                                                       &
    vitenp ,                                                                &
@@ -1322,21 +1180,21 @@ if (arak.gt.0.d0) then
 endif
 
 !===============================================================================
-! 6. Solving (Loop over the non-orthogonalities)
+! 5. Solving (Loop over the non-orthogonalities)
 !===============================================================================
 
 ! --- Number of sweeps
 nswmpr = vcopt_p%nswrsm
 
 ! --- Variables are set to 0
-!       cvar_pr    is the increment of the pressure
-!       dpvar      is the increment of the increment between sweeps
+!       phi        is the increment of the pressure
+!       dphi       is the increment of the increment between sweeps
 !       cpro_divu       is the initial divergence of the predicted mass flux
 
 do iel = 1, ncel
-  cvar_pr(iel) = 0.d0
-  dpvar(iel) = 0.d0
-  presa(iel) = 0.d0
+  phi(iel)  = 0.d0
+  dphi(iel) = 0.d0
+  phia(iel) = 0.d0
 enddo
 
 relaxp = vcopt_p%relaxv
@@ -1459,7 +1317,7 @@ if (icondv.eq.0) then
   do ii = 1, ncmast
     iel= ltmast(ii)
     surfbm(iel) = s_metal*volume(iel)/voltot
-    res(iel) = res(iel) - surfbm(iel)*svcond(iel,ipr)
+    cpro_divu(iel) = cpro_divu(iel) - surfbm(iel)*svcond(iel,ipr)
   enddo
 
   deallocate(surfbm)
@@ -1490,15 +1348,85 @@ endif
 
 ! --- Initial right hand side
 do iel = 1, ncel
-  rhs(iel) = - cpro_divu(iel) - rovsdt(iel)*cvar_pr(iel)
+  rhs(iel) = - cpro_divu(iel) - rovsdt(iel)*phi(iel)
 enddo
 
-
 ! --- Right hand side residual
 residu = sqrt(cs_gdot(ncel,rhs,rhs))
 
 sinfo%rnsmbr = residu
 
+! --- Norm resiudal
+! Historical norm for the pressure step:
+!       div(rho u* + dt gradP^(n))-Gamma
+!       i.e.  RHS of the pressure + div(dt gradP^n) (otherwise there is a risk
+!       a 0 norm at steady states...). Represents terms that pressure has to
+!       balance.
+
+do iel = 1, ncel
+  dtsrom = dt(iel) / crom(iel)
+  do isou = 1, 3
+    trav(isou, iel) = dtsrom * gradp(isou,iel)
+  enddo
+enddo
+
+!---> Parallelism
+if (irangp.ge.0.or.iperio.eq.1) then
+  call synvin(trav)
+endif
+
+! To save time, no space reconstruction
+init   = 1
+inc    = 1
+iflmb0 = 1
+if (iale.eq.1) iflmb0 = 0
+nswrgp = 1
+imligp = vcopt_u%imligr
+iwarnp = vcopt_p%iwarni
+epsrgp = vcopt_u%epsrgr
+climgp = vcopt_u%climgr
+itypfl = 1
+! VOF algorithm: the pressure step corresponds to the
+! correction of the volumetric flux, not the mass flux
+if (idilat.ge.4.or.ivofmt.ge.0) itypfl = 0
+
+call inimav &
+(f_id0  , itypfl ,                                              &
+ iflmb0 , init   , inc    , imrgra , nswrgp , imligp ,          &
+ iwarnp ,                                                       &
+ epsrgp , climgp ,                                              &
+ crom   , brom   ,                                              &
+ trav   ,                                                       &
+ coefav , coefbv ,                                              &
+ iflux  , bflux  )
+
+init = 1
+call divmas(init, iflux, bflux, res)
+
+! Free memory
+deallocate(iflux, bflux)
+
+! --- Weakly compressible algorithm: semi analytic scheme
+if (idilat.ge.4) then
+  do iel = 1, ncel
+    res(iel) = res(iel)*crom(iel)
+  enddo
+endif
+
+! It is: div(dt/rho*rho grad P) + div(rho u*) - Gamma
+! NB: if iphydr=1, div(rho u*) contains div(d fext).
+do iel = 1, ncel
+  res(iel) = res(iel) + cpro_divu(iel)
+enddo
+
+! --- Pressure norm
+rnormp = sqrt(cs_gdot(ncel,res,res))
+
+if (vcopt_p%iwarni.ge.2) then
+  write(nfecra,1300)chaine(1:16) ,rnormp
+endif
+sinfo%nbivar = 0
+
 ! Pressure derive for the log
 if (rnormp.lt.epzero) then
   sinfo%dervar = - sinfo%rnsmbr
@@ -1547,7 +1475,7 @@ if (iswdyp.ge.1) then
    iwarnp ,                                                                    &
    epsrgp , climgp , extrap ,                                                  &
    dfrcxt ,                                                                    &
-   dpvar  ,                                                                    &
+   dphi   ,                                                                    &
    coefa_dp  , coefb_dp  ,                                                     &
    coefaf_dp , coefbf_dp ,                                                     &
    viscf  , viscb  ,                                                           &
@@ -1562,7 +1490,7 @@ if (iswdyp.ge.1) then
    iphydr , iwarnp ,                                                       &
    epsrgp , climgp , extrap ,                                              &
    dfrcxt ,                                                                &
-   dpvar  ,                                                                &
+   dphi   ,                                                                &
    coefa_dp  , coefb_dp  ,                                                 &
    coefaf_dp , coefbf_dp ,                                                 &
    viscf  , viscb  ,                                                       &
@@ -1579,16 +1507,16 @@ endif
 
 do while (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp)
 
-  ! Solving on the increment dpvar
+  ! Solving on the increment dphi
   !-------------------------------
   if (iswdyp.eq.0) then
     do iel = 1, ncel
-      dpvar(iel) = 0.d0
+      dphi(iel) = 0.d0
     enddo
   else
     do iel = 1, ncel
-      dpvarm1(iel) = dpvar(iel)
-      dpvar(iel) = 0.d0
+      dphim1(iel) = dphi(iel)
+      dphi(iel) = 0.d0
     enddo
   endif
 
@@ -1600,7 +1528,7 @@ do while (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp)
 
   call sles_solve_native(ivarfl(ipr), '',                            &
                          isym, ibsize, iesize, dam, xam,             &
-                         epsilp, rnormp, niterf, ressol, rhs, dpvar)
+                         epsilp, rnormp, niterf, ressol, rhs, dphi)
 
   ! Dynamic relaxation of the system
   !---------------------------------
@@ -1634,7 +1562,7 @@ do while (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp)
      iwarnp ,                                                  &
      epsrgp , climgp , extrap ,                                &
      dfrcxt ,                                                  &
-     dpvar  ,                                                  &
+     dphi   ,                                                  &
      coefa_dp  , coefb_dp  ,                                   &
      coefaf_dp , coefbf_dp ,                                   &
      viscf  , viscb  ,                                         &
@@ -1643,13 +1571,15 @@ do while (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp)
 
     else if (iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0) then
 
+      ircflp = vcopt_p%ircflu
+
       call itrgrv &
       !==========
    ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp , &
      iphydr , iwarnp ,                                                       &
      epsrgp , climgp , extrap ,                                              &
      dfrcxt ,                                                                &
-     dpvar  ,                                                                &
+     dphi   ,                                                                &
      coefa_dp  , coefb_dp  ,                                                 &
      coefaf_dp , coefbf_dp ,                                                 &
      viscf  , viscb  ,                                                       &
@@ -1720,25 +1650,25 @@ do while (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp)
   if (iswdyp.eq.0) then
     if (idtvar.ge.0.and.isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp) then
       do iel = 1, ncel
-        presa(iel) = cvar_pr(iel)
-        cvar_pr(iel) = presa(iel) + vcopt_p%relaxv*dpvar(iel)
+        phia(iel) = phi(iel)
+        phi(iel) = phi(iel) + vcopt_p%relaxv*dphi(iel)
       enddo
     ! If it is the last sweep, update with the total increment
     else
       do iel = 1, ncel
-        presa(iel) = cvar_pr(iel)
-        cvar_pr(iel) = presa(iel) + dpvar(iel)
+        phia(iel) = phi(iel)
+        phi(iel) = phi(iel) + dphi(iel)
       enddo
     endif
   elseif (iswdyp.eq.1) then
      do iel = 1, ncel
-      presa(iel) = cvar_pr(iel)
-      cvar_pr(iel) = presa(iel) + alph*dpvar(iel)
+      phia(iel) = phi(iel)
+      phi(iel) = phi(iel) + alph*dphi(iel)
     enddo
   elseif (iswdyp.ge.2) then
     do iel = 1, ncel
-      presa(iel) = cvar_pr(iel)
-      cvar_pr(iel) = presa(iel) + alph*dpvar(iel) + beta*dpvarm1(iel)
+      phia(iel) = phi(iel)
+      phi(iel) = phi(iel) + alph*dphi(iel) + beta*dphim1(iel)
     enddo
   endif
 
@@ -1765,7 +1695,7 @@ do while (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp)
    iwarnp ,                                                                    &
    epsrgp , climgp , extrap ,                                                  &
    dfrcxt ,                                                                    &
-   cvar_pr   ,                                                                 &
+   phi    ,                                                                    &
    coefa_dp  , coefb_dp  ,                                                     &
    coefaf_dp , coefbf_dp ,                                                     &
    viscf  , viscb  ,                                                           &
@@ -1774,13 +1704,15 @@ do while (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp)
 
   else if (iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0) then
 
+    ircflp = vcopt_p%ircflu
+
     call itrgrv &
     !==========
  ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp , &
    iphydr , iwarnp ,                                                       &
    epsrgp , climgp , extrap ,                                              &
    dfrcxt ,                                                                &
-   cvar_pr   ,                                                             &
+   phi    ,                                                                &
    coefa_dp  , coefb_dp  ,                                                 &
    coefaf_dp , coefbf_dp ,                                                 &
    viscf  , viscb  ,                                                       &
@@ -1791,7 +1723,7 @@ do while (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp)
   endif
 
   do iel = 1, ncel
-    rhs(iel) = - cpro_divu(iel) - rhs(iel) - rovsdt(iel)*cvar_pr(iel)
+    rhs(iel) = - cpro_divu(iel) - rhs(iel) - rovsdt(iel)*phi(iel)
   enddo
 
   ! --- Convergence test
@@ -1880,7 +1812,7 @@ if (iand(vcopt_p%idften, ISOTROPIC_DIFFUSION).ne.0) then
    0      , iwarnp ,                                                           &
    epsrgp , climgp , extrap ,                                                  &
    dfrcxt ,                                                                    &
-   presa  ,                                                                    &
+   phia   ,                                                                    &
    coefa_dp  , coefb_dp  ,                                                     &
    coefaf_dp , coefbf_dp ,                                                     &
    viscf  , viscb  ,                                                           &
@@ -1898,7 +1830,7 @@ if (iand(vcopt_p%idften, ISOTROPIC_DIFFUSION).ne.0) then
    0      , iwarnp ,                                                           &
    epsrgp , climgp , extrap ,                                                  &
    dfrcxt ,                                                                    &
-   dpvar  ,                                                                    &
+   dphi   ,                                                                    &
    coefa_dp  , coefb_dp  ,                                                     &
    coefaf_dp , coefbf_dp ,                                                     &
    viscf  , viscb  ,                                                           &
@@ -1907,13 +1839,15 @@ if (iand(vcopt_p%idften, ISOTROPIC_DIFFUSION).ne.0) then
 
 else if (iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0) then
 
+  ircflp = vcopt_p%ircflu
+
   call itrmav &
   !==========
  ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp , &
    iphydr , 0      , iwarnp ,                                              &
    epsrgp , climgp , extrap ,                                              &
    dfrcxt ,                                                                &
-   presa  ,                                                                &
+   phia   ,                                                                &
    coefa_dp  , coefb_dp  ,                                                 &
    coefaf_dp , coefbf_dp ,                                                 &
    viscf  , viscb  ,                                                       &
@@ -1934,7 +1868,7 @@ else if (iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0) then
    iphydr , 0      , iwarnp ,                                              &
    epsrgp , climgp , extrap ,                                              &
    dfrcxt ,                                                                &
-   dpvar  ,                                                                &
+   dphi   ,                                                                &
    coefa_dp  , coefb_dp  ,                                                 &
    coefaf_dp , coefbf_dp ,                                                 &
    viscf  , viscb  ,                                                       &
@@ -1945,20 +1879,20 @@ else if (iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0) then
 endif
 
 !===============================================================================
-! 8. Suppression of the mesh hierarchy
+! 6. Suppression of the mesh hierarchy
 !===============================================================================
 
 call sles_free_native(ivarfl(ipr), '')
 
 !===============================================================================
-! 9. Weakly compressible algorithm: semi analytic scheme
+! 7. Weakly compressible algorithm: semi analytic scheme
 !    2nd step solving a convection diffusion equation
 !===============================================================================
 
 if (idilat.eq.5) then
 
   ! Allocate temporary arrays
-  allocate(ddpvar(ncelet))
+  allocate(ddphi(ncelet))
   allocate(coefar(3,ndimfb), cofafr(3,ndimfb))
   allocate(coefbr(3,3,ndimfb), cofbfr(3,3,ndimfb))
 
@@ -2142,7 +2076,7 @@ if (idilat.eq.5) then
    ischcp , isstpp , inc    , imrgra , iccocg ,                   &
    iwarnp , imucpp , idftnp , imasac ,                            &
    blencp , epsrgp , climgp , extrap , relaxp , thetap ,          &
-   cvar_pr  , cvar_pr       ,                                     &
+   phi    , phi    ,                                              &
    coefa_dp2       , coefb_p, coefaf_dp2      , coefbf_p,         &
    velflx , velflb , viscf  , viscb  , rvoid  , rvoid  ,          &
    rvoid  , rvoid  ,                                              &
@@ -2152,8 +2086,8 @@ if (idilat.eq.5) then
   ! --- Initialization of the variable to solve
   do iel = 1, ncel
     rovsdt(iel) = 340.d0/dt(iel) * cell_f_vol(iel)
-    dpvar(iel)   = 0.d0
-    ddpvar(iel)  = 0.d0
+    dphi(iel)   = 0.d0
+    ddphi(iel)  = 0.d0
     rhs(iel)    = - rhs(iel)
   enddo
 
@@ -2198,13 +2132,13 @@ if (idilat.eq.5) then
      iwarnp ,                                                       &
      blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
      relaxp , thetap ,                                              &
-     dpvar  , dpvar  ,                                              &
+     dphi   , dphi   ,                                              &
      coefa_dp2       , coefb_p, coefaf_dp2      ,coefbf_p,          &
      velflx , velflb ,                                              &
      viscf  , viscb  , viscf  , viscb  , rvoid  ,                   &
      weighf , weighb ,                                              &
      icvflb , ivoid  ,                                              &
-     rovsdt , rhs    , dpvar  , ddpvar ,                            &
+     rovsdt , rhs    , dphi   , ddphi  ,                            &
      rvoid  , rvoid  )
 
   call sles_pop(ivarfl(ipr))
@@ -2212,9 +2146,9 @@ if (idilat.eq.5) then
   ! --- Update the increment of Pressure
 
   do iel = 1, ncel
-    cvar_pr(iel) = cvar_pr(iel) + dpvar(iel)
+    phi(iel) = phi(iel) + dphi(iel)
     ! Remove the last increment
-    dpvar(iel) = dpvar(iel) - ddpvar(iel)
+    dphi(iel) = dphi(iel) - ddphi(iel)
   enddo
 
   ! --- Update the Mass flux
@@ -2235,7 +2169,7 @@ if (idilat.eq.5) then
    0      , iwarnp ,                                                           &
    epsrgp , climgp , extrap ,                                                  &
    dfrcxt ,                                                                    &
-   dpvar  ,                                                                    &
+   dphi   ,                                                                    &
    coefa_dp2       , coefb_p, coefaf_dp2      ,coefbf_p,                       &
    viscf  , viscb  ,                                                           &
    dt     ,                                                                    &
@@ -2252,7 +2186,7 @@ if (idilat.eq.5) then
    0      , iwarnp ,                                                           &
    epsrgp , climgp , extrap ,                                                  &
    dfrcxt ,                                                                    &
-   ddpvar ,                                                                    &
+   ddphi  ,                                                                    &
    coefa_dp2       , coefb_p, coefaf_dp2      ,coefbf_p,                       &
    viscf  , viscb  ,                                                           &
    dt     ,                                                                    &
@@ -2265,7 +2199,7 @@ if (idilat.eq.5) then
      iphydr , 0      , iwarnp ,                                              &
      epsrgp , climgp , extrap ,                                              &
      dfrcxt ,                                                                &
-     dpvar  ,                                                                &
+     dphi   ,                                                                &
      coefa_dp2       , coefb_p, coefaf_dp2      ,coefbf_p,                   &
      viscf  , viscb  ,                                                       &
      vitenp ,                                                                &
@@ -2283,7 +2217,7 @@ if (idilat.eq.5) then
      iphydr , 0      , iwarnp ,                                              &
      epsrgp , climgp , extrap ,                                              &
      dfrcxt ,                                                                &
-     ddpvar ,                                                                &
+     ddphi  ,                                                                &
      coefa_dp2       , coefb_p, coefaf_dp2      ,coefbf_p,                   &
      viscf  , viscb  ,                                                       &
      vitenp ,                                                                &
@@ -2293,7 +2227,7 @@ if (idilat.eq.5) then
   endif
 
   ! Free memory
-  deallocate(ddpvar)
+  deallocate(ddphi)
   deallocate(coefa_dp2, coefaf_dp2)
   deallocate(coefar, coefbr)
   deallocate(cofafr, cofbfr)
@@ -2302,16 +2236,16 @@ if (idilat.eq.5) then
 endif
 
 !===============================================================================
-! 10. Update the pressure field
+! 8. Update the pressure field
 !===============================================================================
 
 if (idtvar.lt.0) then
   do iel = 1, ncel
-    cvar_pr(iel) = cvara_pr(iel) + vcopt_p%relaxv*cvar_pr(iel)
+    cvar_pr(iel) = cvar_pr(iel) + vcopt_p%relaxv*phi(iel)
   enddo
 else
   do iel = 1, ncel
-    cvar_pr(iel) = cvara_pr(iel) + cvar_pr(iel)
+    cvar_pr(iel) = cvar_pr(iel) + phi(iel)
   enddo
 endif
 
@@ -2334,13 +2268,14 @@ endif
 
 ! Free memory
 deallocate(dam, xam)
-deallocate(res, presa)
+deallocate(trav)
+deallocate(res, phia, dphi)
 if (allocated(divu)) deallocate(divu)
 deallocate(gradp)
 deallocate(coefaf_dp, coefbf_dp)
 deallocate(rhs, rovsdt)
 if (allocated(weighf)) deallocate(weighf, weighb)
-if (iswdyp.ge.1) deallocate(adxk, adxkm1, dpvarm1, rhs0)
+if (iswdyp.ge.1) deallocate(adxk, adxkm1, dphim1, rhs0)
 if (icalhy.eq.1) deallocate(frchy, dfrchy, hydro_pres)
 if (ivofmt.ge.0.or.idilat.eq.4) then
   if (allocated(xdtsro)) deallocate(xdtsro)
