@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -31,13 +31,15 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
-#include "cs_cdo.h"
 #include "cs_cdo_connect.h"
 #include "cs_cdo_local.h"
 #include "cs_cdo_quantities.h"
+#include "cs_mesh_location.h"
 #include "cs_param.h"
 #include "cs_property.h"
 #include "cs_time_step.h"
+#include "cs_xdef.h"
+#include "cs_xdef_eval.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -51,15 +53,38 @@ BEGIN_C_DECLS
  * Type definitions
  *============================================================================*/
 
-typedef struct _cs_adv_field_t cs_adv_field_t;
+#define CS_ADVECTION_FIELD_POST_COURANT (1 << 0)  // postprocess Courant number
+#define CS_ADVECTION_FIELD_STEADY       (1 << 1)  // steady-state field
+
+typedef struct {
+
+  int             id;
+  char  *restrict name;
+
+  cs_flag_t   loc_flag;      // Short descriptor for localization
+  cs_flag_t   flag;          // Short descriptor dedicated to postprocessing
+  int         vtx_field_id;  // id among cs_field_t structures (-1 if not used)
+  int         cell_field_id; // id among cs_field_t structures (-1 if not used)
+
+  /* We assume that there is only one definition assicated to an advection
+     field */
+  cs_xdef_t               *definition;
+
+  /* Function pointers */
+  cs_xdef_eval_t          *get_eval_all_vertices;
+  cs_xdef_eval_t          *get_eval_at_cell;
+  cs_xdef_eval_cw_t       *get_eval_at_cell_cw;
+  cs_xdef_eval_cw_xyz_t   *get_eval_at_xyz_cw;
+
+} cs_adv_field_t;
 
 /* List of available keys for setting an advection field */
 typedef enum {
 
-  CS_ADVKEY_POST,
-  CS_ADVKEY_POST_UNITV,
-  CS_ADVKEY_CELL_FIELD,
-  CS_ADVKEY_VERTEX_FIELD,
+  CS_ADVKEY_DEFINE_AT_CELLS,
+  CS_ADVKEY_DEFINE_AT_VERTICES,
+  CS_ADVKEY_POST_COURANT,
+  CS_ADVKEY_STATE_STEADY,
   CS_ADVKEY_N_KEYS
 
 } cs_advection_field_key_t;
@@ -83,35 +108,72 @@ typedef enum {
 /*----------------------------------------------------------------------------*/
 
 void
-cs_advection_field_set_shared_pointers(const cs_cdo_quantities_t   *quant,
-                                       const cs_cdo_connect_t      *connect,
-                                       const cs_time_step_t        *time_step);
+cs_advection_field_set_shared_pointers(const cs_cdo_quantities_t  *quant,
+                                       const cs_cdo_connect_t     *connect,
+                                       const cs_time_step_t       *time_step);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Create and initialize a new advection field structure
+ * \brief  Get the number of allocated cs_adv_field_t structures
+ *
+ * \return the number of advection fields
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_advection_field_get_n_fields(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Search in the array of advection field structures which one has
+ *         the name given in argument
  *
  * \param[in]  name        name of the advection field
  *
- * \return a pointer to a new allocated cs_adv_field_t structure
+ * \return a pointer to a cs_adv_field_t structure or NULL if not found
  */
 /*----------------------------------------------------------------------------*/
 
 cs_adv_field_t *
-cs_advection_field_create(const char   *name);
+cs_advection_field_by_name(const char   *name);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Free a cs_adv_field_t structure
+ * \brief  Search in the array of advection field structures which one has
+ *         the id given in argument
  *
- * \param[in, out]  adv      pointer to a cs_adv_field_t structure to free
+ * \param[in]  id        identification number
+ *
+ * \return a pointer to a cs_adv_field_t structure or NULL if not found
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_adv_field_t *
+cs_advection_field_by_id(int      id);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Add and initialize a new advection field structure
+ *
+ * \param[in]  name        name of the advection field
+ *
+ * \return a pointer to the new allocated cs_adv_field_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_adv_field_t *
+cs_advection_field_add(const char   *name);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Free all alllocated cs_adv_field_t structures and its related array
  *
  * \return a NULL pointer
  */
 /*----------------------------------------------------------------------------*/
 
-cs_adv_field_t *
-cs_advection_field_free(cs_adv_field_t   *adv);
+void
+cs_advection_field_destroy_all(void);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -179,19 +241,17 @@ cs_advection_field_get_name(const cs_adv_field_t   *adv);
  */
 /*----------------------------------------------------------------------------*/
 
-cs_param_def_type_t
+cs_xdef_type_t
 cs_advection_field_get_deftype(const cs_adv_field_t   *adv);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Print a summary of a cs_adv_field_t structure
- *
- * \param[in]  adv      pointer to a cs_adv_field_t structure to summarize
+ * \brief  Print all setup information related to cs_adv_field_t structures
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_advection_field_summary(const cs_adv_field_t   *adv);
+cs_advection_field_log_setup(void);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -199,27 +259,25 @@ cs_advection_field_summary(const cs_adv_field_t   *adv);
  *
  * \param[in, out]  adv       pointer to a cs_adv_field_t structure
  * \param[in]       key       key related to the member of adv to set
- * \param[in]       keyval    accessor to the value to set
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_advection_field_set_option(cs_adv_field_t            *adv,
-                              cs_advection_field_key_t   key,
-                              const char                *keyval);
+                              cs_advection_field_key_t   key);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Define the value of a cs_adv_field_t structure
  *
  * \param[in, out]  adv       pointer to a cs_adv_field_t structure
- * \param[in]       keyval    accessor to the value to set
+ * \param[in]       vector    values to set
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_advection_field_def_by_value(cs_adv_field_t    *adv,
-                                const char        *val);
+                                cs_real_t          vector[3]);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -227,38 +285,85 @@ cs_advection_field_def_by_value(cs_adv_field_t    *adv,
  *
  * \param[in, out]  adv     pointer to a cs_adv_field_t structure
  * \param[in]       func    pointer to a function
+ * \param[in]       input   NULL or pointer to a structure cast on-the-fly
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_advection_field_def_by_analytic(cs_adv_field_t        *adv,
-                                   cs_analytic_func_t    *func);
+                                   cs_analytic_func_t    *func,
+                                   void                  *input);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Define a cs_adv_field_t structure thanks to an array of values
  *
  * \param[in, out]  adv       pointer to a cs_adv_field_t structure
- * \param[in]       desc      information about this array
+ * \param[in]       loc       information to know where are located values
  * \param[in]       array     pointer to an array
+ * \param[in]       index     optional pointer to the array index
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_advection_field_def_by_array(cs_adv_field_t     *adv,
-                                cs_desc_t           desc,
-                                const cs_real_t    *array);
+cs_advection_field_def_by_array(cs_adv_field_t    *adv,
+                                cs_flag_t          loc,
+                                cs_real_t         *array,
+                                cs_lnum_t         *index);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Create a cs_field_t structure related to an advection field
+ * \brief  Define a cs_adv_field_t structure thanks to an array of values
  *
- * \param[in, out] adv     pointer to a cs_adv_field_t structure
+ * \param[in, out]  adv       pointer to a cs_adv_field_t structure
+ * \param[in]       field     pointer to a cs_field_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_advection_field_create_field(cs_adv_field_t   *adv);
+cs_advection_field_def_by_field(cs_adv_field_t    *adv,
+                                cs_field_t        *field);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Create all needed cs_field_t structures related to an advection
+ *         field
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_advection_field_create_fields(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Get a cs_field_t structure related to an advection field and a mesh
+ *         location
+ *
+ * \param[in]  adv         pointer to a cs_adv_field_t structure
+ * \param[in]  ml_type     type of mesh location (cells or vertices)
+ *
+ * \return a pointer to a cs_field_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_field_t *
+cs_advection_field_get_field(cs_adv_field_t           *adv,
+                             cs_mesh_location_type_t   ml_type);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the value of the advection field at the cell center
+ *
+ * \param[in]      cm      pointer to a cs_cell_mesh_t structure
+ * \param[in]      adv     pointer to a cs_adv_field_t structure
+ * \param[in, out] vect    pointer to a cs_nvec3_t structure (meas + unitv)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_advection_field_in_cell(const cs_cell_mesh_t   *cm,
+                           const cs_adv_field_t   *adv,
+                           cs_nvec3_t             *vect);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -271,15 +376,16 @@ cs_advection_field_create_field(cs_adv_field_t   *adv);
 /*----------------------------------------------------------------------------*/
 
 void
-cs_advection_field_get_cell_vector(cs_lnum_t              c_id,
-                                   const cs_adv_field_t  *adv,
-                                   cs_nvec3_t            *vect);
+cs_advection_field_get_cell_vector(cs_lnum_t               c_id,
+                                   const cs_adv_field_t   *adv,
+                                   cs_nvec3_t             *vect);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the value of the advection field for a given face
  *
  * \param[in]      adv     pointer to a cs_adv_field_t structure
+ * \param[in]      cm      pointer to a cs_cell_mesh_t structure
  * \param[in]      xyz     coordinates where to evaluate the advection field
  * \param[in, out] vect    pointer to a cs_nvec3_t structure (meas + unitv)
  */
@@ -287,6 +393,7 @@ cs_advection_field_get_cell_vector(cs_lnum_t              c_id,
 
 void
 cs_advection_field_get_at_xyz(const cs_adv_field_t   *adv,
+                              const cs_cell_mesh_t   *cm,
                               const cs_real_3_t       xyz,
                               cs_nvec3_t             *vect);
 
@@ -307,8 +414,8 @@ cs_advection_field_at_cells(const cs_adv_field_t  *adv,
 /*!
  * \brief  Compute the value of the advection field at vertices
  *
- * \param[in]      adv     pointer to a cs_adv_field_t structure
- * \param[in, out] vect    pointer to a cs_nvec3_t structure (meas/unitv)
+ * \param[in]      adv          pointer to a cs_adv_field_t structure
+ * \param[in, out] vtx_values   array storing the results
  */
 /*----------------------------------------------------------------------------*/
 
@@ -316,34 +423,19 @@ void
 cs_advection_field_at_vertices(const cs_adv_field_t  *adv,
                                cs_real_t             *vtx_values);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Compute the max. value of the advection field among cells
- *
- * \param[in]      adv       pointer to a cs_adv_field_t structure
- *
- * \return the  max. value of the magnitude of the field at cell centers
- */
-/*----------------------------------------------------------------------------*/
-
-double
-cs_advection_field_get_cell_max(const cs_adv_field_t      *adv);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the value of the flux of the advection field across the
  *         the dual faces of a cell
  *
- * \param[in]      c_id     id of the current cell
- * \param[in]      a_info   set of parameters for the advection operator
+ * \param[in]      cm       pointer to a cs_cell_mesh_t structure
  * \param[in]      adv      pointer to a cs_adv_field_t structure
  * \param[in, out] fluxes   array of values attached to dual faces of a cell
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_advection_field_get_flux_dfaces(cs_lnum_t                     c_id,
-                                   const cs_param_advection_t    a_info,
+cs_advection_field_get_flux_dfaces(const cs_cell_mesh_t         *cm,
                                    const cs_adv_field_t         *adv,
                                    cs_real_t                    *fluxes);
 
@@ -354,8 +446,8 @@ cs_advection_field_get_flux_dfaces(cs_lnum_t                     c_id,
  *         of a face.
  *
  * \param[in]  adv       pointer to a cs_adv_field_t structure
- * \param[in]  a_info    set of parameters for the advection operator
  * \param[in]  cm        pointer to a cs_cell_mesh_t structure
+ * \param[in]  tef_meas  area of the triangle tef
  * \param[in]  f         id of the face in the current cell
  * \param[in]  e         id of the edge in the current cell
  * \param[in]  v1        id of the first vertex in the current cell
@@ -367,8 +459,8 @@ cs_advection_field_get_flux_dfaces(cs_lnum_t                     c_id,
 
 cs_real_t
 cs_advection_field_get_flux_tef(const cs_adv_field_t        *adv,
-                                const cs_param_advection_t   a_info,
                                 const cs_cell_mesh_t        *cm,
+                                const cs_real_t              tef_meas,
                                 short int                    f,
                                 short int                    e,
                                 short int                    v1,
@@ -376,86 +468,15 @@ cs_advection_field_get_flux_tef(const cs_adv_field_t        *adv,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Compute the value of the flux of the advection field across the
- *         triangle defined by a vertex, the face and edge barycenters
+ * \brief  For each cs_adv_field_t structures, update the values of the related
+ *         field(s)
  *
- * \param[in]  v_id     id of the current vertex
- * \param[in]  e_id     id of the current edge
- * \param[in]  f_id     id of the current face
- * \param[in]  a_info   set of parameters for the advection operator
- * \param[in]  adv      pointer to a cs_adv_field_t structure
- *
- * \return the value of the flux across s(v,e,f)
- */
-/*----------------------------------------------------------------------------*/
-
-cs_real_t
-cs_advection_field_get_flux_svef(cs_lnum_t                    v_id,
-                                 cs_lnum_t                    e_id,
-                                 cs_lnum_t                    f_id,
-                                 const cs_param_advection_t   a_info,
-                                 const cs_adv_field_t        *adv);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Update the values of the related field(s)
- *
- * \param[in, out]     adv     pointer to a cs_adv_field_t structure
+ * \param[in]      cur2prev    true or false
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_advection_field_update(cs_adv_field_t   *adv);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Check if additional predefined postprocessing is requested
- *
- * \param[in]     adv     pointer to a cs_adv_field_t structure
- *
- * \return true or false
- */
-/*----------------------------------------------------------------------------*/
-
-bool
-cs_advection_field_needs_post(const cs_adv_field_t   *adv);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Predefined post-processing output for advection fields.
- *         Prototype of this function is fixed since it is a function pointer
- *         defined in cs_post.h (cs_post_time_mesh_dep_output_t)
- *
- * \param[in, out] input        pointer to a optional structure (here a
- *                              cs_groundwater_t structure)
- * \param[in]      mesh_id      id of the output mesh for the current call
- * \param[in]      cat_id       category id of the output mesh for this call
- * \param[in]      ent_flag     indicate global presence of cells (ent_flag[0]),
- *                              interior faces (ent_flag[1]), boundary faces
- *                              (ent_flag[2]), particles (ent_flag[3]) or probes
- *                              (ent_flag[4])
- * \param[in]      n_cells      local number of cells of post_mesh
- * \param[in]      n_i_faces    local number of interior faces of post_mesh
- * \param[in]      n_b_faces    local number of boundary faces of post_mesh
- * \param[in]      cell_list    list of cells (1 to n)
- * \param[in]      i_face_list  list of interior faces (1 to n)
- * \param[in]      b_face_list  list of boundary faces (1 to n)
- * \param[in]      time_step    pointer to a cs_time_step_t struct.
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_advection_field_extra_post(void                      *input,
-                              int                        mesh_id,
-                              int                        cat_id,
-                              int                        ent_flag[5],
-                              cs_lnum_t                  n_cells,
-                              cs_lnum_t                  n_i_faces,
-                              cs_lnum_t                  n_b_faces,
-                              const cs_lnum_t            cell_list[],
-                              const cs_lnum_t            i_face_list[],
-                              const cs_lnum_t            b_face_list[],
-                              const cs_time_step_t      *time_step);
+cs_advection_field_update(bool   cur2prev);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -472,21 +493,6 @@ cs_advection_get_peclet(const cs_adv_field_t        *adv,
                         const cs_property_t         *diff,
                         cs_real_t                    peclet[]);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief   Compute the Courant number in each cell
- *
- * \param[in]      adv        pointer to the advection field struct.
- * \param[in]      dt         value of the current time step
- * \param[in, out] courant    pointer to an array storing Courant numbers
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_advection_get_courant(const cs_adv_field_t        *adv,
-                         double                       dt,
-                         cs_real_t                    courant[]);
-
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
