@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -61,11 +61,6 @@ subroutine cfdttv &
 ! viscb(nfabor     ! tr ! --- ! tab de trav aux faces de bord                  !
 !__________________!____!_____!________________________________________________!
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!===============================================================================
 
 !===============================================================================
 ! Module files
@@ -105,7 +100,7 @@ double precision wflmas(nfac), wflmab(nfabor), viscb(nfabor)
 
 ! Local variables
 
-integer          ifac, iel, iterns
+integer          ifac, iel, iterns, idtcfl
 integer          iconvp, idiffp, isym
 
 double precision, allocatable, dimension(:) :: viscf
@@ -142,14 +137,14 @@ call field_get_val_s(ivarfl(ipr), cvar_pr)
 
 ! Map specific heats fields for sound celerity computation
 
-if (icp.gt.0) then
-  call field_get_val_s(iprpfl(icp), cpro_cp)
+if (icp.ge.0) then
+  call field_get_val_s(icp, cpro_cp)
 else
   cpro_cp => rvoid1
 endif
 
-if (icv.gt.0) then
-  call field_get_val_s(iprpfl(icv), cpro_cv)
+if (icv.ge.0) then
+  call field_get_val_s(icv, cpro_cv)
 else
   cpro_cv => rvoid1
 endif
@@ -164,9 +159,10 @@ do ifac = 1, nfabor
 enddo
 
 iterns = 1
+idtcfl = 1
 call cfmsfp                                                       &
 !==========
- ( nvar   , nscal  , iterns , ncepdp , ncesmp ,                   &
+ ( nvar   , nscal  , idtcfl , iterns , ncepdp , ncesmp ,          &
    icepdc , icetsm , itypsm ,                                     &
    dt     , vela   ,                                              &
    ckupdc , smacel ,                                              &
@@ -189,7 +185,6 @@ do ifac = 1, nfabor
 enddo
 
 call matrdt &
-!==========
  ( iconvp  , idiffp  , isym   , coefbt , cofbft , wflmas ,          &
    wflmab , viscf  , viscb  , w1 )
 
@@ -198,7 +193,6 @@ call matrdt &
 allocate(c2(ncelet))
 
 call cs_cf_thermo_c_square(cpro_cp, cpro_cv, cvar_pr, crom, c2, ncel)
-!=========================
 
 ! Compute the coefficient CFL/dt
 
@@ -208,13 +202,13 @@ if (iporos.ge.1) then
       wcf(iel) = epzero
     else
       wcf(iel) =  w1(iel) * c2(iel) * crom(iel)                      &
-                / (cvar_pr(iel)*cell_f_vol(iel))
+                / ((cvar_pr(iel) + psginf)*cell_f_vol(iel))
     endif
   enddo
 else
   do iel = 1, ncel
     wcf(iel) =  w1(iel) * c2(iel) * crom(iel)                        &
-              / (cvar_pr(iel)*cell_f_vol(iel))
+              / ((cvar_pr(iel) + psginf)*cell_f_vol(iel))
   enddo
 endif
 
@@ -225,11 +219,11 @@ deallocate(c2)
 deallocate(coefbt,cofbft)
 
 !--------
-! FORMATS
+! Formats
 !--------
 
 !----
-! FIN
+! End
 !----
 
 return
