@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -25,6 +25,8 @@
 !> eventual source terms) for total energy over a time step. It is the third
 !> step of the compressible algorithm at each time iteration.
 !>
+!> Please refer to the <a href="../../theory.pdf#cfener"><b>cfener</b></a> section
+!> of the theory guide for more informations.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -109,6 +111,7 @@ integer          iconvp, idiffp, ndircp
 integer          nswrsp, ircflp, ischcp, isstpp, iescap
 double precision epsrgp, climgp, extrap, blencp, epsilp
 double precision sclnor, thetap, epsrsp, relaxp
+double precision turb_schmidt
 
 integer          inc    , iccocg , imucpp , idftnp , iswdyp
 integer          f_id0  , ii, jj
@@ -145,6 +148,7 @@ double precision, dimension(:), pointer :: cvar_tempk, cvar_yk
 double precision, dimension(:), pointer :: visct, cpro_cp, cpro_cv, cpro_viscls
 
 type(gas_mix_species_prop) :: s_k
+type(var_cal_opt) :: vcopt_u, vcopt_p, vcopt_e
 
 !===============================================================================
 
@@ -157,14 +161,14 @@ call field_get_val_s(ivarfl(ivar), cvar_energ)
 call field_get_val_s(ivarfl(isca(itempk)), cvar_tempk)
 call field_get_val_v(ivarfl(iu), vel)
 
-if (icp.gt.0) then
-  call field_get_val_s(iprpfl(icp), cpro_cp)
+if (icp.ge.0) then
+  call field_get_val_s(icp, cpro_cp)
 else
   cpro_cp => rvoid1
 endif
 
-if (icv.gt.0) then
-  call field_get_val_s(iprpfl(icv), cpro_cv)
+if (icv.ge.0) then
+  call field_get_val_s(icv, cpro_cv)
 else
   cpro_cv => rvoid1
 endif
@@ -189,7 +193,7 @@ call field_get_val_prev_s(icrom, cromo)
 
 call field_get_val_s(ivarfl(ipr), cvar_pr)
 
-call field_get_val_s(iprpfl(ivisct), visct)
+call field_get_val_s(ivisct, visct)
 
 call field_get_key_int(ivarfl(ivar), kimasf, iflmas)
 call field_get_key_int(ivarfl(ivar), kbmasf, iflmab)
@@ -204,7 +208,11 @@ endif
 ! Prints
 call field_get_label(ivarfl(ivar), chaine)
 
-if(iwarni(ivar).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt_e)
+
+if(vcopt_e%iwarni.ge.1) then
   write(nfecra,1000) chaine(1:8)
 endif
 
@@ -268,14 +276,14 @@ endif
 
 do iel = 1, ncel
   rovsdt(iel) = rovsdt(iel)                                                    &
-                + istat(ivar)*(cromo(iel)/dt(iel))*cell_f_vol(iel)
+                + vcopt_e%istat*(cromo(iel)/dt(iel))*cell_f_vol(iel)
 enddo
 
 !                                       __        v
 !     VISCOUS DISSIPATION TERM        : >  ((SIGMA *U).n)  *S
 !     ========================          --               ij  ij
 
-if (idiff(iu).ge. 1) then
+if (vcopt_u%idiff.ge. 1) then
   call cfdivs(smbrs, vel)
 endif
 
@@ -303,12 +311,12 @@ if (irecpt.eq.1) then
   iii = ipr
   inc = 1
   iccocg = 1
-  nswrgp = nswrgr(iii)
-  imligp = imligr(iii)
-  iwarnp = iwarni(iii)
-  epsrgp = epsrgr(iii)
-  climgp = climgr(iii)
-  extrap = extrag(iii)
+  nswrgp = vcopt_p%nswrgr
+  imligp = vcopt_p%imligr
+  iwarnp = vcopt_p%iwarni
+  epsrgp = vcopt_p%epsrgr
+  climgp = vcopt_p%climgr
+  extrap = vcopt_p%extrag
 
   allocate(coefap(nfabor))
   allocate(coefbp(nfabor))
@@ -408,9 +416,9 @@ do iel = 1, ncel
                             + gz*vel(3,iel) )
 enddo
 
-!                                  Kij*Sij           LAMBDA   Cp   MUT
-!     FACE DIFFUSION "VELOCITY" : --------- avec K = ------ + -- .------
-!     =========================    IJ.nij              Cv     Cv  SIGMAS
+!                                  Kij*Sij           LAMBDA   Cp      MUT
+!     FACE DIFFUSION "VELOCITY" : --------- avec K = ------ + -- .------------
+!     =========================    IJ.nij              Cv     Cv  TURB_SCHMIDT
 
 ! Only SGDH available
 
@@ -418,14 +426,16 @@ allocate(w1(ncelet))
 allocate(viscf(nfac))
 allocate(viscb(nfabor))
 
-if( idiff(ivar).ge. 1 ) then
+if( vcopt_e%idiff.ge. 1 ) then
 
-!     MUT/SIGMAS
+  call field_get_key_double(ivarfl(isca(iscal)), ksigmas, turb_schmidt)
+
+!     MUT/TURB_SCHMIDT
   do iel = 1, ncel
-    w1(iel) = visct(iel)/sigmas(iscal)
+    w1(iel) = visct(iel)/turb_schmidt
   enddo
-!     CP*MUT/SIGMAS
-  if(icp.gt.0) then
+!     CP*MUT/TURB_SCHMIDT
+  if(icp.ge.0) then
     do iel = 1, ncel
       w1(iel) = w1(iel)*cpro_cp(iel)
     enddo
@@ -434,8 +444,8 @@ if( idiff(ivar).ge. 1 ) then
       w1(iel) = w1(iel)*cp0
     enddo
   endif
-!     (CP/CV)*MUT/SIGMAS
-  if(icv.gt.0) then
+!     (CP/CV)*MUT/TURB_SCHMIDT
+  if(icv.ge.0) then
     do iel = 1, ncel
       w1(iel) = w1(iel)/cpro_cv(iel)
     enddo
@@ -444,7 +454,7 @@ if( idiff(ivar).ge. 1 ) then
       w1(iel) = w1(iel)/cv0
     enddo
   endif
-!     (CP/CV)*MUT/SIGMAS+LAMBDA/CV
+!     (CP/CV)*MUT/TURB_SCHMIDT+LAMBDA/CV
   if(ifcvsl.lt.0)then
     do iel = 1, ncel
       w1(iel) = w1(iel) + visls0(iscal)
@@ -496,12 +506,12 @@ if( idiff(ivar).ge. 1 ) then
   iii = iu
   inc = 1
   iccocg = 1
-  nswrgp = nswrgr(iii)
-  imligp = imligr(iii)
-  iwarnp = iwarni(iii)
-  epsrgp = epsrgr(iii)
-  climgp = climgr(iii)
-  extrap = extrag(iii)
+  nswrgp = vcopt_u%nswrgr
+  imligp = vcopt_u%imligr
+  iwarnp = vcopt_u%iwarni
+  epsrgp = vcopt_u%epsrgr
+  climgp = vcopt_u%climgr
+  extrap = vcopt_u%extrag
 
 ! Allocate temporary arrays
   allocate(coefap(nfabor))
@@ -784,29 +794,29 @@ endif
 ! 4. Solving
 !===============================================================================
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
+iconvp = vcopt_e%iconv
+idiffp = vcopt_e%idiff
 ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
-thetap = thetav(ivar)
+nswrsp = vcopt_e%nswrsm
+nswrgp = vcopt_e%nswrgr
+imligp = vcopt_e%imligr
+ircflp = vcopt_e%ircflu
+ischcp = vcopt_e%ischcv
+isstpp = vcopt_e%isstpc
+iwarnp = vcopt_e%iwarni
+blencp = vcopt_e%blencv
+epsilp = vcopt_e%epsilo
+epsrsp = vcopt_e%epsrsm
+epsrgp = vcopt_e%epsrgr
+climgp = vcopt_e%climgr
+extrap = vcopt_e%extrag
+relaxp = vcopt_e%relaxv
+thetap = vcopt_e%thetav
 iescap = 0
 
 !  idtvar = 1  => unsteady
 imucpp = 0  ! not a thermal scalar
-idftnp = 1  ! scalar viscosity
+idftnp = ISOTROPIC_DIFFUSION
 iswdyp = 0  ! no dynamic relaxation
 
 ! impose boundary convective at some faces (face indicator icvfli)
@@ -819,7 +829,7 @@ call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 call codits                                                      &
 !==========
-( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
   ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
   iwarnp ,                                                       &
@@ -849,12 +859,12 @@ call cs_cf_check_internal_energy(cvar_energ, ncel, vel)
 
 ! Explicit balance (see codits : the increment is removed)
 
-if (iwarni(ivar).ge.2) then
+if (vcopt_e%iwarni.ge.2) then
   do iel = 1, ncel
     smbrs(iel) = smbrs(iel)                                                    &
-            - istat(ivar)*(crom(iel)/dt(iel))*cell_f_vol(iel)                  &
+            - vcopt_e%istat*(crom(iel)/dt(iel))*cell_f_vol(iel)                  &
                 *(cvar_energ(iel)-cvara_energ(iel))                            &
-                * max(0,min(nswrsm(ivar)-2,1))
+                * max(0,min(vcopt_e%nswrsm-2,1))
   enddo
   sclnor = sqrt(cs_gdot(ncel,smbrs,smbrs))
   write(nfecra,1200)chaine(1:8) ,sclnor
