@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2015 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -67,6 +67,7 @@ use coincl
 use cpincl
 use ppincl
 use ppcpfu
+use cs_c_bindings
 
 !===============================================================================
 
@@ -76,7 +77,9 @@ implicit none
 
 integer          ii , jj, iok
 integer          icha , isc , is
-double precision wmolme
+double precision wmolme, turb_schmidt
+
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 ! 1. VARIABLES TRANSPORTEES
@@ -111,7 +114,8 @@ do isc = 1, nscapp
 
 ! ---- Schmidt ou Prandtl turbulent
 
-  sigmas(jj) = 0.7d0
+  turb_schmidt = 0.7d0
+  call field_set_key_double(ivarfl(isca(jj)), ksigmas, turb_schmidt)
 
 ! ---- Coeff dissipation des fluctuations
 
@@ -119,12 +123,7 @@ do isc = 1, nscapp
 
   ii = isca(iscapp(isc))
 
-! ------ Niveau de detail des impressions pour les variables et
-!          donc les scalaires (valeurs 0 ou 1)
-!          Si = -10000 non modifie par l'utilisateur -> niveau 1
-  if(iwarni(ii).eq.-10000) then
-    iwarni(ii) = 1
-  endif
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
 
 ! ---- Informations relatives a la resolution des scalaires
 
@@ -134,21 +133,23 @@ do isc = 1, nscapp
 !         - Schema convectif % schema 2ieme ordre
 !           = 0 : upwind
 !           = 1 : second ordre
-  blencv(ii) = 0.d0
+  vcopt%blencv = 0.d0
 
 !         - Type de schema convetif second ordre (utile si BLENCV > 0)
 !           = 0 : Second Order Linear Upwind
 !           = 1 : Centre
-  ischcv(ii) = 1
+  vcopt%ischcv = 1
 
 !         - Test de pente pour basculer d'un schema centre vers l'upwind
 !           = 0 : utilisation automatique du test de pente
 !           = 1 : calcul sans test de pente
-  isstpc(ii) = 0
+  vcopt%isstpc = 0
 
 !         - Reconstruction des flux de convetion et de diffusion aux faces
 !           = 0 : pas de reconstruction
-  ircflu(ii) = 0
+  vcopt%ircflu = 0
+
+  call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
 
 enddo
 
