@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,8 @@
 !> for transport.
 !> In order to ensure the exact conservation of mass, the mass fluxes are
 !> computed following the procedure of the standard subroutine resopv
-!> (see theory guide).
+!> (See the <a href="../../theory.pdf#resopv"><b>resopv</b></a>
+!> section of the theory guide for more informations).
 !>
 !> The hydraulic head is contained in the pressure array, while the velocities
 !> are contained in the velocity arrays.
@@ -62,6 +63,8 @@
 !>
 !> This routine is essentially inspired from navstv, resopv and codits.
 !>
+!> Please refer to the <a href="../../theory.pdf#groundwater"><b>groundwater flows</b></a>
+!> section of the theory guide for more theoretical informations.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -97,6 +100,7 @@ use field
 use field_operator
 use cs_c_bindings
 use darcy_module
+use cs_c_bindings
 
 !===============================================================================
 
@@ -113,9 +117,9 @@ integer iccocg, inc   , iel, isou, init
 integer nswrgp, imligp, iwarnp
 integer imucpp, ircflp, isweep, isym, lchain
 integer ndircp, niterf, nswmpr
-integer iinvpe, iflmas, iflmab, iesize, idiffp, iconvp, ibsize
+integer iflmas, iflmab, iesize, idiffp, iconvp, ibsize
 integer fid
-integer iflid , iflwgr, f_dim, f_id0, iprev, iitsm
+integer iflid , iflwgr, f_dim, f_id0, iwgrp, iprev, iitsm
 
 double precision epsrgp, climgp, extrap
 double precision thetap, xdu, xdv, xdw, xnrmul
@@ -124,6 +128,7 @@ double precision relaxp, residu, ressol, epsilp
 character(len=80) :: chaine
 
 type(solving_info) sinfo
+type(var_cal_opt) :: vcopt_p
 
 double precision rvoid(1)
 
@@ -208,7 +213,9 @@ endif
 ! Index of the field
 iflid = ivarfl(ipr)
 
-if (iwgrec(ipr).eq.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
+
+if (vcopt_p%iwgrec.eq.1) then
   ! Id weighting field for gradient
   call field_get_key_int(iflid, kwgrec, iflwgr)
   call field_get_dim(iflwgr, f_dim)
@@ -224,7 +231,8 @@ call field_get_name(ivarfl(ipr), chaine)
 lchain = 16
 
 f_id0 = -1
-if (iwgrec(ipr).eq.1) f_id0 = -2
+iwgrp = 0
+if (vcopt_p%iwgrec.eq.1) iwgrp = 1
 
 ! --- Boundary conditions
 call field_get_coefa_s(ivarfl(ipr), coefa_p)
@@ -270,11 +278,10 @@ endif
 ! Computation of diffusion coefficients at the centers of faces
 if (darcy_anisotropic_permeability.eq.0) then
   call viscfa &
-  !==========
   ( imvisf ,                                  &
     cpro_permeability     ,                   &
     viscf  , viscb  )
-  if (iwgrec(ipr).eq.1) then
+  if (vcopt_p%iwgrec.eq.1) then
     ! Weighting for gradient
     do iel = 1, ncel
       cpro_wgrec_s(iel) = cpro_permeability(iel)
@@ -284,13 +291,12 @@ if (darcy_anisotropic_permeability.eq.0) then
 else if (darcy_anisotropic_permeability.eq.1) then
   allocate(weighf(2,nfac))
   allocate(weighb(ndimfb))
-  iwarnp = iwarni(ipr)
+  iwarnp = vcopt_p%iwarni
   call vitens &
-  !==========
   ( cpro_permeability_6 , iwarnp ,       &
     weighf , weighb ,                    &
     viscf  , viscb  )
-  if (iwgrec(ipr).eq.1) then
+  if (vcopt_p%iwgrec.eq.1) then
     ! Weighting for gradient
     do iel = 1, ncel
       do isou = 1, 6
@@ -307,12 +313,11 @@ ndircp = ndircl(ipr) ! no diagonal stepped aside
 thetap = 1.d0 ! implicit scheme
 imucpp = 0 ! do not multiply the convectiv term by anything
 
-! Note that the computed matrrix is always the same in the loop over iterns
+! Note that the computed matrix is always the same in the loop over iterns
 ! (loop of the Newton scheme, handled by tridim)
 ! Another option would be to store the arrays dam and xam in a global array
 ! rather than computing the matrix at each call to 'richards'.
 call matrix &
-!==========
 ( iconvp , idiffp , ndircp , isym   ,                            &
   thetap , imucpp ,                                              &
   coefb_p , coefbf_p     , rovsdt ,                              &
@@ -327,7 +332,7 @@ call matrix &
 ! The way used to handle non-orthogonalities is the same.
 ! The difference is the instationnary term.
 
-nswmpr = nswrsm(ipr)
+nswmpr = vcopt_p%nswrsm
 
 ! Initialization of dpvar for avoiding warnings
 do iel = 1, ncel
@@ -336,23 +341,22 @@ enddo
 
 ! We compute the first residue (rhs)
 
-relaxp = relaxv(ipr) ! relaxation for loop over isweep
+relaxp = vcopt_p%relaxv ! relaxation for loop over isweep
 isweep = 1 ! counter for non-orthogonalities
 iccocg = 1 ! no calculation of cocg. What does it mean?
 init = 1 ! it is an initialization of the residue
 inc  = 1 ! 0 increment, 1 otherwise
-nswrgp = nswrgr(ipr) ! number of sweeps for gradients reconstruction
-imligp = imligr(ipr)
-iwarnp = iwarni(ipr)
-epsrgp = epsrgr(ipr)
-climgp = climgr(ipr)
-extrap = extrag(ipr)
-ircflp = ircflu(ipr)
+nswrgp = vcopt_p%nswrgr ! number of sweeps for gradients reconstruction
+imligp = vcopt_p%imligr
+iwarnp = vcopt_p%iwarni
+epsrgp = vcopt_p%epsrgr
+climgp = vcopt_p%climgr
+extrap = vcopt_p%extrag
+ircflp = vcopt_p%ircflu
 
 if (darcy_anisotropic_permeability.eq.0) then
 
   call itrgrp &
-  !==========
 ( ivarfl(ipr), init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydr , &
   iwarnp ,                                                                    &
   epsrgp , climgp , extrap ,                                                  &
@@ -361,13 +365,12 @@ if (darcy_anisotropic_permeability.eq.0) then
   coefa_p  , coefb_p  ,                                                       &
   coefaf_p , coefbf_p ,                                                       &
   viscf  , viscb  ,                                                           &
-  cpro_permeability, cpro_permeability, cpro_permeability,                    &
+  cpro_permeability,                                                          &
   rhs   )
 
 else if (darcy_anisotropic_permeability.eq.1) then
 
   call itrgrv &
-  !==========
 ( ivarfl(ipr), init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,        &
   iphydr , iwarnp ,                                                                  &
   epsrgp , climgp , extrap ,                                                         &
@@ -420,8 +423,7 @@ sinfo%rnsmbr = residu
 ! taking into account the boundary conditions.
 ! This part is inspired from codits (call to promav)
 allocate(w1(ncelet))
-iinvpe = 1 ! for processing communication before calculation
-call promav(isym, ibsize, iesize, iinvpe, dam, xam, cvar_pr, w1)
+call promav(isym, ibsize, iesize, ivarfl(ipr), dam, xam, cvar_pr, w1)
 
 !$omp parallel do
 do iel = 1, ncel
@@ -433,30 +435,29 @@ rnormp = sqrt(cs_gdot(ncel, w1, w1))
 deallocate(w1)
 
 ! Writing
-if (iwarni(ipr).ge.2) then
+if (vcopt_p%iwarni.ge.2) then
   write(nfecra,1400)chaine(1:16), isweep, residu, relaxp
 endif
 
 ! Loop for non-orthogonalities
 ! TODO: dynamic relaxation
-nswmpr = nswrsm(ipr)
-do while ( (isweep.le.nswmpr.and.residu.gt.epsrsm(ipr)*rnormp) &
+nswmpr = vcopt_p%nswrsm
+do while ( (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp) &
             .or. (isweep.eq.1) )
             ! We pass at least once to ensure exactness of mass flux
 
-  iwarnp = iwarni(ipr)
-  epsilp = epsilo(ipr)
-  iinvpe = 1
+  iwarnp = vcopt_p%iwarni
+  epsilp = vcopt_p%epsilo
   ressol = residu
 
   call sles_solve_native(ivarfl(ipr), chaine,                         &
-                         isym, ibsize, iesize, dam, xam, iinvpe,      &
+                         isym, ibsize, iesize, dam, xam,              &
                          epsilp, rnormp, niterf, ressol, rhs, dpvar)
 
-  if (isweep.le.nswmpr.and.residu.gt.epsrsm(ipr)*rnormp) then
+  if (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp) then
     do iel = 1, ncel
       presa(iel) = cvar_pr(iel)
-      cvar_pr(iel) = presa(iel) + relaxv(ipr)*dpvar(iel)
+      cvar_pr(iel) = presa(iel) + vcopt_p%relaxv*dpvar(iel)
     enddo
 
     ! If it is the last sweep, update with the total increment
@@ -474,7 +475,6 @@ do while ( (isweep.le.nswmpr.and.residu.gt.epsrsm(ipr)*rnormp) &
   if (darcy_anisotropic_permeability.eq.0) then
 
     call itrgrp &
-    !==========
   ( ivarfl(ipr), init  , inc , imrgra , iccocg , nswrgp , imligp , iphydr ,  &
     iwarnp ,                                                                 &
     epsrgp , climgp , extrap ,                                               &
@@ -483,13 +483,12 @@ do while ( (isweep.le.nswmpr.and.residu.gt.epsrsm(ipr)*rnormp) &
     coefa_p  , coefb_p  ,                                                    &
     coefaf_p , coefbf_p ,                                                    &
     viscf  , viscb  ,                                                        &
-    cpro_permeability, cpro_permeability, cpro_permeability,                 &
+    cpro_permeability,                                                       &
     rhs   )
 
   else if (darcy_anisotropic_permeability.eq.1) then
 
     call itrgrv &
-    !==========
     ( ivarfl(ipr), init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,        &
     iphydr , iwarnp ,                                                                    &
     epsrgp , climgp , extrap ,                                                           &
@@ -519,7 +518,7 @@ do while ( (isweep.le.nswmpr.and.residu.gt.epsrsm(ipr)*rnormp) &
   ! Writing
   sinfo%nbivar = sinfo%nbivar + niterf
 
-  if (iwarni(ipr).ge.2) then
+  if (vcopt_p%iwarni.ge.2) then
     write(nfecra,1400)chaine(1:16), isweep, residu, relaxp
   endif
 
@@ -541,7 +540,7 @@ else
 endif
 
 ! Writing
-if (iwarni(ipr).ge.2) then
+if (vcopt_p%iwarni.ge.2) then
   if (isweep.gt.nswmpr) then
     write(nfecra,1600) chaine(1:16), nswmpr
   endif
@@ -557,12 +556,12 @@ deallocate(dam, xam, rhs)
 iccocg = 1
 init = 1
 inc = 1
-nswrgp = nswrgr(ipr)
-imligp = imligr(ipr)
-iwarnp = iwarni(ipr)
-epsrgp = epsrgr(ipr)
-climgp = climgr(ipr)
-extrap = extrag(ipr)
+nswrgp = vcopt_p%nswrgr
+imligp = vcopt_p%imligr
+iwarnp = vcopt_p%iwarni
+epsrgp = vcopt_p%epsrgr
+climgp = vcopt_p%climgr
+extrap = vcopt_p%extrag
 
 ! We compute the new mass flux, taking care not to reconstruct
 ! the last increment of the lopp on isweep, to ensure an
@@ -571,15 +570,14 @@ extrap = extrag(ipr)
 if (darcy_anisotropic_permeability.eq.0) then
 
   call itrmas &
-  !==========
  ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydr , &
-   iwarnp ,                                                       &
+   iwgrp  , iwarnp ,                                              &
    epsrgp , climgp , extrap ,                                     &
    rvoid  ,                                                       &
    presa  ,                                                       &
    coefa_p , coefb_p , coefaf_p , coefbf_p ,                      &
    viscf  , viscb  ,                                              &
-   cpro_permeability, cpro_permeability, cpro_permeability,       &
+   cpro_permeability,                                             &
    imasfl , bmasfl )
 
   ! The last increment is not reconstructed to fullfill exactly the continuity
@@ -590,23 +588,21 @@ if (darcy_anisotropic_permeability.eq.0) then
   init = 0
 
   call itrmas &
-  !==========
  ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydr ,     &
-   iwarnp ,                                                                    &
+   iwgrp  , iwarnp ,                                                           &
    epsrgp , climgp , extrap ,                                                  &
    rvoid  ,                                                                    &
    dpvar  ,                                                                    &
    coefa_p , coefb_p , coefaf_p , coefbf_p ,                                   &
    viscf  , viscb  ,                                                           &
-   cpro_permeability, cpro_permeability, cpro_permeability,                    &
+   cpro_permeability,                                                          &
    imasfl , bmasfl )
 
 else if (darcy_anisotropic_permeability.eq.1) then
 
   call itrmav &
-  !==========
  ( f_id0, init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp , &
-   iphydr , iwarnp ,                                                     &
+   iphydr , iwgrp  , iwarnp ,                                            &
    epsrgp , climgp , extrap ,                                            &
    rvoid  ,                                                              &
    presa  ,                                                              &
@@ -625,9 +621,8 @@ else if (darcy_anisotropic_permeability.eq.1) then
   ircflp = 0
 
   call itrmav &
-  !==========
  ( f_id0, init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,  &
-   iphydr , iwarnp ,                                                      &
+   iphydr , iwgrp  , iwarnp ,                                             &
    epsrgp , climgp , extrap ,                                             &
    rvoid  ,                                                               &
    dpvar  ,                                                               &
@@ -695,7 +690,7 @@ endif
 ! update pressure head (h = H - z) for post-processing
 ! Only used when gravity is taken into account
 if (darcy_gravity.ge.1) then
-  call field_get_val_s(iprpfl(iprtot), cpro_prtot)
+  call field_get_val_s(iprtot, cpro_prtot)
   !$omp parallel do
   do iel = 1, ncel
     cpro_prtot(iel) = cvar_pr(iel) - xyzcen(1,iel)*darcy_gravity_x             &
