@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -38,19 +38,6 @@
 #include <unistd.h>
 #include <assert.h>
 
-/*----------------------------------------------------------------------------
- * libxml2 library headers
- *----------------------------------------------------------------------------*/
-
-#if defined(HAVE_LIBXML2)
-
-#include <libxml/tree.h>
-#include <libxml/parser.h>
-#include <libxml/xpath.h>
-#include <libxml/xpathInternals.h>
-
-#endif
-
 /*----------------------------------------------------------------------------
  * Local headers
  *----------------------------------------------------------------------------*/
@@ -137,7 +124,7 @@ _output_value(const char  *const param,
   if (cs_gui_strcmp(param, "auxiliary_restart_file_writing")) {
 
     cs_xpath_add_attribute(&path, "status");
-    if(cs_gui_get_status(path, &result)) *keyword = result;
+    if (cs_gui_get_status(path, &result)) *keyword = result;
 
   }
   else {
@@ -272,9 +259,6 @@ _attribute_value(char               **path,
 
   cs_xpath_add_attribute(path, "status");
 
-  if (   cs_gui_strcmp(child, "postprocessing_recording")
-      || cs_gui_strcmp(child, "listing_printing")) *keyword = -999;
-
   if (cs_gui_get_status(*path, &result))
     *keyword = result;
 
@@ -304,71 +288,6 @@ _variable_attribute(const char  *name,
   _attribute_value(&path, child, keyword);
 }
 
-/*-----------------------------------------------------------------------------
- * Return number of <probe recording> tags in the <variable> tag.
- *----------------------------------------------------------------------------*/
-
-static int
-_variable_number_probes(const char  *variable)
-{
-  char *path = NULL;
-  char *choice = NULL;
-  int   nb_probes;
-
-  path = cs_xpath_short_path();
-  cs_xpath_add_element(&path, "variable");
-  cs_xpath_add_test_attribute(&path, "name", variable);
-  cs_xpath_add_element(&path, "probes");
-  cs_xpath_add_attribute(&path, "choice");
-  choice = cs_gui_get_attribute_value(path);
-
-  if (choice)
-    nb_probes = atoi(choice);
-  else
-    nb_probes = -1;
-
-  BFT_FREE(choice);
-  BFT_FREE(path);
-
-  return nb_probes;
-}
-
-/*-----------------------------------------------------------------------------
- * Return probe number for <probe_recording> tag for variable.
- *
- * parameters:
- *   variable   <--  name of variable
- *   num_probe  <--  number of <probe_recording> tags
- *----------------------------------------------------------------------------*/
-
-static int
-_variable_probe_name(const char *variable,
-                     int         num_probe)
-{
-  char *path = NULL;
-  char *strvalue = NULL;
-  int   intvalue;
-
-  path = cs_xpath_short_path();
-  cs_xpath_add_element(&path, "variable");
-  cs_xpath_add_test_attribute(&path, "name", variable);
-  cs_xpath_add_element(&path, "probes");
-  cs_xpath_add_element_num(&path, "probe_recording", num_probe);
-  cs_xpath_add_attribute(&path, "name");
-
-  strvalue = cs_gui_get_attribute_value(path);
-
-  if (strvalue == NULL)
-    bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
-
-  intvalue = atoi(strvalue);
-
-  BFT_FREE(strvalue);
-  BFT_FREE(path);
-
-  return intvalue;
-}
-
 /*-----------------------------------------------------------------------------
  * Return variable label.
  *
@@ -398,37 +317,22 @@ _variable_label(const char *variable)
  * Post-processing options for all variables (velocity, pressure, ...)
  *
  * parameters:
- *   f_id               <-- field id
- *   ihisvr             --> histo output
- *   nvppmx             <-- number of printed variables
+ *   f_id <-- field id
  *----------------------------------------------------------------------------*/
 
 static void
-_variable_post(int         f_id,
-               int        *ihisvr,
-               const int  *nvppmx)
+_variable_post(int  f_id)
 {
-  int   nb_probes;
-  int   iprob;
   char *label = NULL;
-  int   num_probe;
 
   cs_field_t  *f = cs_field_by_id(f_id);
-  const int var_key_id = cs_field_key_id("post_id");
-  int ipp = cs_field_get_key_int(f, var_key_id);
 
-  if (ipp == -1) return;
-
-  int f_post = 0, f_log = 0;
-  const int k_post = cs_field_key_id("post_vis");
+  int f_post = -999, f_log = -999, f_monitor = -999;
   const int k_log  = cs_field_key_id("log");
   const int k_lbl = cs_field_key_id("label");
+  const int k_post = cs_field_key_id("post_vis");
 
-  _variable_attribute(f->name,
-                      "postprocessing_recording",
-                      &f_post);
-  if (f_post != -999)
-    cs_field_set_key_int(f, k_post, f_post);
+  /* Listing output */
 
   _variable_attribute(f->name,
                       "listing_printing",
@@ -436,23 +340,23 @@ _variable_post(int         f_id,
   if (f_log != -999)
     cs_field_set_key_int(f, k_log, f_log);
 
-  nb_probes = _variable_number_probes(f->name);
+  /* Postprocessing outputs */
 
-  ihisvr[0 + (ipp - 1)] = nb_probes;
+  _variable_attribute(f->name,
+                      "postprocessing_recording",
+                      &f_post);
+  if (f_post == 1)
+    cs_field_set_key_int_bits(f, k_post, CS_POST_ON_LOCATION);
+  else if (f_post == 0)
+    cs_field_clear_key_int_bits(f, k_post, CS_POST_ON_LOCATION);
 
-  if (f->dim > 1)
-    for (int idim = 1; idim < f->dim; idim++)
-      ihisvr[0 + (ipp - 1) + idim] = nb_probes;
-
-  if (nb_probes > 0) {
-    for (iprob = 0; iprob < nb_probes; iprob++) {
-      num_probe = _variable_probe_name(f->name, iprob+1);
-      ihisvr[(iprob+1)*(*nvppmx) + (ipp - 1)] = num_probe;
-      if (f->dim > 1)
-        for (int idim = 1; idim < f->dim; idim++)
-          ihisvr[(iprob+1)*(*nvppmx) + (ipp - 1) + idim] = num_probe;
-    }
-  }
+  _variable_attribute(f->name,
+                      "probes_recording",
+                      &f_monitor);
+  if (f_monitor == 1)
+    cs_field_set_key_int_bits(f, k_post, CS_POST_MONITOR);
+  else if (f_monitor == 0)
+    cs_field_clear_key_int_bits(f, k_post, CS_POST_MONITOR);
 
   label = _variable_label(f->name);
   if (label != NULL)
@@ -484,72 +388,6 @@ _property_output_status(const char  *name,
   _attribute_value(&path, child, keyword);
 }
 
-/*-----------------------------------------------------------------------------
- * Return probe number for sub tag "probe_recording" for property.
- *
- * parameters:
- *   property   <--  name of property
- *   num_probe  <-- number of <probe_recording> tags
- *----------------------------------------------------------------------------*/
-
-static int
-_property_probe_name(const char  *property,
-                     int          num_probe)
-{
-  char *path = NULL;
-  char *strvalue = NULL;
-  int   value;
-
-  path = cs_xpath_short_path();
-  cs_xpath_add_element(&path, "property");
-  cs_xpath_add_test_attribute(&path, "name", property);
-  cs_xpath_add_element(&path, "probes");
-  cs_xpath_add_element_num(&path, "probe_recording", num_probe);
-  cs_xpath_add_attribute(&path, "name");
-
-  strvalue = cs_gui_get_attribute_value(path);
-
-  if (strvalue == NULL)
-    bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
-
-  value = atoi(strvalue);
-
-  BFT_FREE(path);
-  BFT_FREE(strvalue);
-
-  return value;
-}
-
-/*-----------------------------------------------------------------------------
- * Return number of sub tags <probe_recording> for property.
- *----------------------------------------------------------------------------*/
-
-static int
-_property_number_probes(const char  *property)
-{
-  char *path = NULL;
-  char *choice = NULL;
-  int   nb_probes;
-
-  path = cs_xpath_short_path();
-  cs_xpath_add_element(&path, "property");
-  cs_xpath_add_test_attribute(&path, "name", property);
-  cs_xpath_add_element(&path, "probes");
-  cs_xpath_add_attribute(&path, "choice");
-  choice = cs_gui_get_attribute_value(path);
-
-  if (choice) {
-    nb_probes = atoi(choice);
-    BFT_FREE(choice);
-  }
-  else
-    nb_probes = -1;
-
-  BFT_FREE(path);
-
-  return nb_probes;
-}
-
 /*-----------------------------------------------------------------------------
  * Return the label model's property.
  *
@@ -579,61 +417,64 @@ _get_property_label(const char  *property)
  * Post-processing options for properties
  *
  * parameters:
- *   f_id               <-- field id
- *   ihisvr             --> histo output
- *   nvppmx             <-- number of printed variables
+ *   f_id <-- field id
  *----------------------------------------------------------------------------*/
 
 static void
-_property_post(int         f_id,
-               int        *ihisvr,
-               const int  *nvppmx)
+_property_post(int  f_id)
 {
-  int nb_probes;
-  int iprob;
   char *label = NULL;
-  int num_probe;
 
   cs_field_t  *f = cs_field_by_id(f_id);
-  const int var_key_id = cs_field_key_id("post_id");
-  int ipp = cs_field_get_key_int(f, var_key_id);
 
-  if (ipp == -1) return;
+  /* Some properties are not handled by the GUI */
 
-  int f_post = 0, f_log = 0;
-  const int k_post = cs_field_key_id("post_vis");
+  if (strcmp(f->name, "porosity") == 0)
+    return;
+
+  /* Now check for options */
+
+  int f_post = -999, f_log = -999, f_monitor = -999;
+  bool in_tree = false;
   const int k_log  = cs_field_key_id("log");
   const int k_lbl = cs_field_key_id("label");
+  const int k_post = cs_field_key_id("post_vis");
 
-  /* Visualization outputs frequency */
-  _property_output_status(f->name,
-                          "postprocessing_recording",
-                          &f_post);
-  if (f_post != -999)
-    cs_field_set_key_int(f, k_post, f_post);
-
-  /* Listing output frequency */
+  /* Listing output */
   _property_output_status(f->name,
                           "listing_printing",
                           &f_log);
   if (f_log != -999)
     cs_field_set_key_int(f, k_log, f_log);
-
-  /* Activated probes */
-  nb_probes = _property_number_probes(f->name);
-
-  ihisvr[0 + (ipp - 1)] = nb_probes;
-
-  if (nb_probes > 0) {
-    for (iprob=0; iprob<nb_probes; iprob++){
-      num_probe = _property_probe_name(f->name, iprob+1);
-      ihisvr[(iprob + 1)*(*nvppmx) + (ipp - 1)] = num_probe;
-      if (f->dim > 1)
-        for (int idim = 1; idim < f->dim; idim++)
-          ihisvr[(iprob+1)*(*nvppmx) + (ipp - 1) + idim] = num_probe;
-    }
+  else {
+    char *pl = _get_property_label(f->name);
+    if (pl != NULL)
+      in_tree = true;
+    BFT_FREE(pl);
   }
 
+  /* Postprocessing outputs */
+
+  _property_output_status(f->name,
+                          "postprocessing_recording",
+                          &f_post);
+  if (f_post == 1)
+    cs_field_set_key_int_bits(f, k_post, CS_POST_ON_LOCATION);
+  else if (f_post == 0)
+    cs_field_clear_key_int_bits(f, k_post, CS_POST_ON_LOCATION);
+  else if (in_tree)
+    cs_field_set_key_int_bits(f, k_post, CS_POST_ON_LOCATION);
+
+  _property_output_status(f->name,
+                          "probes_recording",
+                          &f_monitor);
+  if (f_monitor == 1)
+    cs_field_set_key_int_bits(f, k_post, CS_POST_MONITOR);
+  else if (f_monitor == 0)
+    cs_field_clear_key_int_bits(f, k_post, CS_POST_MONITOR);
+  else if (in_tree)
+    cs_field_set_key_int_bits(f, k_post, CS_POST_MONITOR);
+
   /* Take into account labels */
 
   label = _get_property_label(f->name);
@@ -665,131 +506,50 @@ _time_moment_output_status(int          moment_id,
   _attribute_value(&path, child, keyword);
 }
 
-/*-----------------------------------------------------------------------------
- * Return probe number for sub tag "probe_recording" for moment.
- *
- * parameters:
- *   moment_id  <-- id of moment
- *   num_probe  <-- number of <probe_recording> tags
- *----------------------------------------------------------------------------*/
-
-static int
-_time_moment_probe_name(int  moment_id,
-                        int  num_probe)
-{
-  char *path = NULL;
-  char *strvalue = NULL;
-  int   value;
-
-  path = cs_xpath_short_path();
-  cs_xpath_add_element_num(&path, "time_average", moment_id+1);
-  cs_xpath_add_element(&path, "probes");
-  cs_xpath_add_element_num(&path, "probe_recording", num_probe);
-  cs_xpath_add_attribute(&path, "name");
-
-  strvalue = cs_gui_get_attribute_value(path);
-
-  if (strvalue == NULL)
-    bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
-
-  value = atoi(strvalue);
-
-  BFT_FREE(path);
-  BFT_FREE(strvalue);
-
-  return value;
-}
-
-/*-----------------------------------------------------------------------------
- * Return number of sub tags <probe_recording> for moment.
- *
- * parameters:
- *   moment_id  <-- id of moment
- *----------------------------------------------------------------------------*/
-
-static int
-_time_moment_number_probes(int moment_id)
-{
-  char *path = NULL;
-  char *choice = NULL;
-  int   nb_probes;
-
-  path = cs_xpath_short_path();
-  cs_xpath_add_element_num(&path, "time_average", moment_id+1);
-  cs_xpath_add_element(&path, "probes");
-  cs_xpath_add_attribute(&path, "choice");
-  choice = cs_gui_get_attribute_value(path);
-
-  if (choice) {
-    nb_probes = atoi(choice);
-    BFT_FREE(choice);
-  }
-  else
-    nb_probes = -1;
-
-  BFT_FREE(path);
-
-  return nb_probes;
-}
-
 /*-----------------------------------------------------------------------------
  * Post-processing options for time moments
  *
  * parameters:
- *   f_id               <-- field id
- *   ihisvr             --> histo output
- *   nvppmx             <-- number of printed variables
+ *   f_id <-- field id
  *----------------------------------------------------------------------------*/
 
 static void
-_time_moment_post(int         f_id,
-                  int         moment_id,
-                  int        *ihisvr,
-                  const int  *nvppmx)
+_time_moment_post(int  f_id,
+                  int  moment_id)
 {
-  int nb_probes;
-  int iprob;
   char *label = NULL;
-  int num_probe;
 
   cs_field_t  *f = cs_field_by_id(f_id);
-  const int var_key_id = cs_field_key_id("post_id");
-  int ipp = cs_field_get_key_int(f, var_key_id);
 
-  if (ipp == -1) return;
-
-  int f_post = 0, f_log = 0;
-  const int k_post = cs_field_key_id("post_vis");
+  int f_post = -999, f_log = -999, f_monitor = -999;
   const int k_log  = cs_field_key_id("log");
+  const int k_post = cs_field_key_id("post_vis");
 
-  /* Visualization outputs frequency */
-  _time_moment_output_status(moment_id,
-                             "postprocessing_recording",
-                             &f_post);
-  if (f_post != -999)
-    cs_field_set_key_int(f, k_post, f_post);
-
-  /* Listing output frequency */
+  /* Listing output */
   _time_moment_output_status(moment_id,
                              "listing_printing",
                              &f_log);
   if (f_log != -999)
     cs_field_set_key_int(f, k_log, f_log);
 
-  /* Activated probes */
-  nb_probes = _time_moment_number_probes(moment_id);
+  /* Postprocessing outputs */
 
-  ihisvr[0 + (ipp - 1)] = nb_probes;
+  _time_moment_output_status(moment_id,
+                             "postprocessing_recording",
+                             &f_post);
+  if (f_post == 1)
+    cs_field_set_key_int_bits(f, k_post, CS_POST_ON_LOCATION);
+  else if (f_post == 0)
+    cs_field_clear_key_int_bits(f, k_post, CS_POST_ON_LOCATION);
 
-  if (nb_probes > 0) {
-    for (iprob=0; iprob<nb_probes; iprob++){
-      num_probe = _time_moment_probe_name(moment_id, iprob+1);
-      ihisvr[(iprob + 1)*(*nvppmx) + (ipp - 1)] = num_probe;
-      if (f->dim > 1)
-        for (int idim = 1; idim < f->dim; idim++)
-          ihisvr[(iprob+1)*(*nvppmx) + (ipp - 1) + idim] = num_probe;
-    }
-  }
+  _time_moment_output_status(moment_id,
+                             "probes_recording",
+                             &f_monitor);
+
+  if (f_monitor == 1)
+    cs_field_set_key_int_bits(f, k_post, CS_POST_MONITOR);
+  else if (f_monitor == 0)
+    cs_field_clear_key_int_bits(f, k_post, CS_POST_MONITOR);
 
   BFT_FREE(label);
 }
@@ -1013,6 +773,43 @@ _output_type_choice(const char  *type,
   return description;
 }
 
+/*-----------------------------------------------------------------------------
+ * add notebook variable to formula
+ *----------------------------------------------------------------------------*/
+
+static void
+_add_notebook_variables(mei_tree_t *ev_law)
+{
+  char *path = NULL;
+
+  /* number of variable */
+  int nbvar = cs_gui_get_tag_count("/physical_properties/notebook/var\n", 1);
+
+  if (nbvar == 0)
+    return;
+
+  for (int ivar = 0; ivar < nbvar; ivar++) {
+    path = cs_xpath_init_path();
+    cs_xpath_add_elements(&path, 2, "physical_properties", "notebook");
+    cs_xpath_add_element_num(&path, "var", ivar +1);
+    cs_xpath_add_attribute(&path, "name");
+    char *name = cs_gui_get_attribute_value(path);
+    BFT_FREE(path);
+
+    path = cs_xpath_init_path();
+    cs_xpath_add_elements(&path, 2, "physical_properties", "notebook");
+    cs_xpath_add_element_num(&path, "var", ivar +1);
+    cs_xpath_add_attribute(&path, "value");
+    char *value = cs_gui_get_attribute_value(path);
+    double val = atof(value);
+    BFT_FREE(path);
+
+    mei_tree_insert(ev_law, name, val);
+    BFT_FREE(name);
+    BFT_FREE(value);
+  }
+}
+
 /*-----------------------------------------------------------------------------
  * Initialize mei tree and check for symbols existence.
  *
@@ -1044,6 +841,9 @@ _init_mei_tree(const int        num,
   mei_tree_insert(tree, "niter", *ntcabs );
   mei_tree_insert(tree, "t", *ttcabs);
 
+  /* add variable from notebook */
+  _add_notebook_variables(tree);
+
   /* try to build the interpreter */
   if (mei_tree_builder(tree))
     bft_error(__FILE__, __LINE__, 0,
@@ -1129,7 +929,7 @@ void CS_PROCF (cspstb, CSPSTB) (cs_int_t        *ipstdv)
 
     if (_surfacic_variable_post("yplus", true))
       ipstdv[1] = 1;
-    if (_surfacic_variable_post("tplus", true))
+    if (_surfacic_variable_post("tplus", false))
       ipstdv[2] = 1;
     if (_surfacic_variable_post("thermal_flux", true))
       ipstdv[3] = 1;
@@ -1141,7 +941,7 @@ void CS_PROCF (cspstb, CSPSTB) (cs_int_t        *ipstdv)
       }
     }
 
-    if (_surfacic_variable_post("boundary_layer_nusselt", true))
+    if (_surfacic_variable_post("boundary_layer_nusselt", false))
       ipstdv[4] = 1;
   }
 }
@@ -1150,64 +950,13 @@ void CS_PROCF (cspstb, CSPSTB) (cs_int_t        *ipstdv)
  * Determine output options.
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (csenso, CSENSO) (const cs_int_t  *nvppmx,
-                                cs_int_t        *ncapt,
-                                cs_int_t        *nthist,
-                                cs_real_t       *frhist,
-                                cs_int_t        *iecaux,
-                                cs_int_t        *ihisvr,
-                                cs_int_t        *tplfmt,
-                                cs_real_t       *xyzcap)
+void CS_PROCF (csenso, CSENSO) (cs_int_t  *iecaux)
 {
-  int i;
-  int ipp;
-  int *ippfld;
-  char fmtprb[16];
-
   if (!cs_gui_file_is_loaded())
     return;
 
-  {
-    int n_fields = cs_field_n_fields();
-    int ipp_max = 1;
-    const int k_pp = cs_field_key_id("post_id");
-    for (int f_id = 0; f_id < n_fields; f_id++) {
-      const cs_field_t *f = cs_field_by_id(f_id);
-      int ipp1 = cs_field_get_key_int(f, k_pp);
-      int ipp2 = ipp1 + f->dim - 1;
-      ipp_max = CS_MAX(ipp_max, ipp2);
-    }
-    BFT_MALLOC(ippfld, ipp_max, int);
-    for (ipp = 0; ipp < ipp_max; ipp++)
-      ippfld[ipp] = -1;
-    for (int f_id = 0; f_id < n_fields; f_id++) {
-      const cs_field_t *f = cs_field_by_id(f_id);
-      ipp = cs_field_get_key_int(f, k_pp);
-      if (ipp > 1)
-        ippfld[ipp - 1] = f_id;
-    }
-  }
-
   _output_value("auxiliary_restart_file_writing", iecaux);
   _output_value("listing_printing_frequency", &cs_glob_log_frequency);
-  _output_value("probe_recording_frequency", nthist);
-  _output_time_value("probe_recording_frequency_time", frhist);
-
-  _output_choice("probe_format", fmtprb, sizeof(fmtprb) - 1);
-
-  /* Time plot (probe) format */
-  if (!strcmp(fmtprb, "DAT"))
-    *tplfmt = 1;
-  else if (!strcmp(fmtprb, "CSV"))
-    *tplfmt = 2;
-
-  *ncapt = cs_gui_get_tag_count("/analysis_control/output/probe", 1);
-  for (i = 0; i < *ncapt; i++) {
-    xyzcap[0 + i*3] = _probe_coordinate(i+1, "probe_x");
-    xyzcap[1 + i*3] = _probe_coordinate(i+1, "probe_y");
-    xyzcap[2 + i*3] = _probe_coordinate(i+1, "probe_z");
-  }
-
   const int n_fields = cs_field_n_fields();
 
   /* temporary field -> moment ids */
@@ -1228,12 +977,12 @@ void CS_PROCF (csenso, CSENSO) (const cs_int_t  *nvppmx,
   for (int f_id = 0; f_id < n_fields; f_id++) {
     const cs_field_t  *f = cs_field_by_id(f_id);
     if (f->type & CS_FIELD_VARIABLE)
-      _variable_post(f->id, ihisvr, nvppmx);
+      _variable_post(f->id);
     else if (f->type & CS_FIELD_PROPERTY)
-      _property_post(f->id, ihisvr, nvppmx);
+      _property_post(f->id);
     else if (moment_id != NULL) {
       if (moment_id[f_id] > -1)
-        _time_moment_post(f->id, moment_id[f_id], ihisvr, nvppmx);
+        _time_moment_post(f->id, moment_id[f_id]);
     }
   }
 
@@ -1243,29 +992,7 @@ void CS_PROCF (csenso, CSENSO) (const cs_int_t  *nvppmx,
   bft_printf("==>CSENSO\n");
   bft_printf("--iecaux = %i\n", *iecaux);
   bft_printf("--ntlist = %i\n", cs_glob_log_frequency);
-  bft_printf("--nthist = %i\n", *nthist);
-  bft_printf("--frhist = %f\n", *frhist);
-  bft_printf("--ncapt  = %i\n", *ncapt);
-  bft_printf("--tplfmt = %i\n", *tplfmt);
-  for (i=0; i < *ncapt; i++) {
-    bft_printf("--xyzcap[%i][0] = %f\n", i, xyzcap[0 +i*3]);
-    bft_printf("--xyzcap[%i][1] = %f\n", i, xyzcap[1 +i*3]);
-    bft_printf("--xyzcap[%i][2] = %f\n", i, xyzcap[2 +i*3]);
-  }
-  const int k_post = cs_field_key_id("post_id");
-  for (int f_id = 0; f_id < cs_field_n_fields(); f_id++) {
-    const cs_field_t *f = cs_field_by_id(f_id);
-    const int ipp = cs_field_get_key_int(f, k_post);
-    bft_printf("-->variable post_id[%i] = %s\n", ipp, f->name);
-    bft_printf("--ihisvr[0][%i]= %i \n", ipp, ihisvr[0 + (ipp-1)]);
-    if (ihisvr[0 + (ipp-1)]>0)
-      for (j=0; j<ihisvr[0 + (ipp-1)]; j++)
-        bft_printf("--ihisvr[%i][%i]= %i \n", j+1, ipp,
-                   ihisvr[(j+1)*(*nvppmx) + (ipp-1)]);
-  }
 #endif
-
-  BFT_FREE(ippfld);
 }
 
 /*============================================================================
@@ -1349,6 +1076,54 @@ cs_gui_postprocess_meshes(void)
     BFT_FREE(type);
     BFT_FREE(path);
   }
+
+  /* Probe definitions */
+
+  int n_probes = cs_gui_get_tag_count("/analysis_control/output/probe", 1);
+
+  if (n_probes > 0) {
+
+    cs_real_3_t *p_coords;
+    BFT_MALLOC(p_coords, n_probes, cs_real_3_t);
+
+    for (i = 0; i < n_probes; i++) {
+      p_coords[i][0] = _probe_coordinate(i+1, "probe_x");
+      p_coords[i][1] = _probe_coordinate(i+1, "probe_y");
+      p_coords[i][2] = _probe_coordinate(i+1, "probe_z");
+    }
+
+    cs_probe_set_create_from_array("probes",
+                                   n_probes,
+                                   (const cs_real_3_t *)p_coords,
+                                   NULL);
+
+    BFT_FREE(p_coords);
+
+    int frequency_n = 1;
+    cs_real_t frequency_t = -1.;
+
+    _output_value("probe_recording_frequency", &frequency_n);
+    _output_time_value("probe_recording_frequency_time", &frequency_t);
+
+    /* Time plot (probe) format */
+    char fmt_opts[16], fmtprb[16];
+    _output_choice("probe_format", fmtprb, sizeof(fmtprb) - 1);
+    if (!strcmp(fmtprb, "DAT"))
+      strncpy(fmt_opts, "dat", 16);
+    else if (!strcmp(fmtprb, "CSV"))
+      fmt_opts[0] = '\0';
+
+    cs_post_define_writer(CS_POST_WRITER_PROBES,   /* writer_id */
+                          "",                      /* case_name */
+                          "monitoring",            /* dir_name */
+                          "time_plot",
+                          fmt_opts,
+                          FVM_WRITER_FIXED_MESH,
+                          false,                   /* output_at_start */
+                          false,                   /* output_at_end */
+                          frequency_n,
+                          frequency_t);
+  }
 }
 
 /*----------------------------------------------------------------------------
@@ -1365,6 +1140,7 @@ cs_gui_postprocess_writers(void)
   char *format_options = NULL;
   char *time_dependency = NULL;
   char *frequency_choice = NULL;
+  char *output_start_s = NULL;
   char *output_end_s = NULL;
   char *id_s = NULL;
 
@@ -1379,6 +1155,7 @@ cs_gui_postprocess_writers(void)
 
     int id = 0;
     fvm_writer_time_dep_t  time_dep = FVM_WRITER_FIXED_MESH;
+    bool output_at_start = false;
     bool output_at_end = true;
     cs_int_t time_step = -1;
     cs_real_t time_value = -1.0;
@@ -1388,6 +1165,7 @@ cs_gui_postprocess_writers(void)
     label = _output_type_choice("writer", "label", i);
     directory = _output_type_options("writer", "name", "directory", i);
     frequency_choice = _output_type_options("writer", "period", "frequency", i);
+    output_start_s = _output_type_options("writer", "status", "output_at_start", i);
     output_end_s = _output_type_options("writer", "status", "output_at_end", i);
     if (cs_gui_strcmp(frequency_choice, "none")) {
       time_step = -1;
@@ -1402,6 +1180,8 @@ cs_gui_postprocess_writers(void)
       time_step = -1;
       time_value = -1.;
     }
+    if (cs_gui_strcmp(output_start_s, "on"))
+      output_at_start = true;
     if (cs_gui_strcmp(output_end_s, "off"))
       output_at_end = false;
     format_name = _output_type_options("writer", "name", "format",i);
@@ -1420,6 +1200,7 @@ cs_gui_postprocess_writers(void)
                           format_name,
                           format_options,
                           time_dep,
+                          output_at_start,
                           output_at_end,
                           time_step,
                           time_value);
@@ -1428,6 +1209,7 @@ cs_gui_postprocess_writers(void)
     BFT_FREE(format_name);
     BFT_FREE(format_options);
     BFT_FREE(time_dependency);
+    BFT_FREE(output_start_s);
     BFT_FREE(output_end_s);
     BFT_FREE(frequency_choice);
     BFT_FREE(directory);
