@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,7 +45,9 @@
 #include "bft_error.h"
 #include "bft_printf.h"
 
+#include "cs_blas.h"
 #include "cs_interface.h"
+#include "cs_math.h"
 #include "cs_mesh.h"
 #include "cs_mesh_quantities.h"
 #include "cs_post.h"
@@ -412,10 +414,10 @@ _compute_weighting_offsetting(const cs_mesh_t             *mesh,
     /* Compute center offsetting coefficient */
     /*---------------------------------------*/
 
-                for (i = 0; i < dim; i++) {
+    for (i = 0; i < dim; i++) {
       v1[i] = mesh_quantities->dofij[face_id*3 + i];
       v2[i] = mesh_quantities->i_face_normal[face_id*3 + i];
-                }
+    }
     double of_s = _MODULE_3D(v1) * _MODULE_3D(v2);
 
     offsetting[cell1] = CS_MAX(offsetting[cell1],
@@ -734,37 +736,176 @@ _vtx_from_max_face(const cs_mesh_t     *mesh,
                          vtx_val);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define the cell -> faces connectivity
+ *
+ * \param[in]  mesh        pointer to a cs_mesh_t structure
+ * \param[in]  p_c2f_idx   pointer to the array of indexes
+ * \param[in]  p_c2f_lst   pointer to the list of face ids
+ */
+/*----------------------------------------------------------------------------*/
+
+static void
+_build_c2f(const cs_mesh_t   *mesh,
+           cs_lnum_t         *p_c2f_idx[],
+           cs_lnum_t         *p_c2f_ids[])
+{
+  int  idx_size = 0;
+  int  *cell_shift = NULL;
+  cs_lnum_t  *c2f_idx = NULL;
+  cs_lnum_t  *c2f_ids = NULL;
+
+  const int  n_cells = mesh->n_cells;
+  const int  n_i_faces = mesh->n_i_faces;
+  const int  n_b_faces = mesh->n_b_faces;
+
+  BFT_MALLOC(c2f_idx, n_cells + 1, cs_lnum_t);
+  BFT_MALLOC(cell_shift, n_cells, int);
+
+# pragma omp parallel for if (n_cells > CS_THR_MIN)
+  for (cs_lnum_t i = 0; i < n_cells; i++)
+    cell_shift[i] = c2f_idx[i] = 0;
+  c2f_idx[n_cells] = 0;
+
+  for (cs_lnum_t i = 0; i < n_b_faces; i++) {
+    c2f_idx[mesh->b_face_cells[i]+1] += 1;
+    idx_size += 1;
+  }
+
+  for (cs_lnum_t i = 0; i < n_i_faces; i++) {
+
+    const int  c1_id = mesh->i_face_cells[i][0];
+    const int  c2_id = mesh->i_face_cells[i][1];
+
+    if (c1_id < n_cells) // cell owned by the local rank
+      c2f_idx[c1_id+1] += 1, idx_size += 1;
+    if (c2_id < n_cells) // cell owned by the local rank
+      c2f_idx[c2_id+1] += 1, idx_size += 1;
+
+  }
+
+  for (cs_lnum_t i = 0; i < n_cells; i++)
+    c2f_idx[i+1] += c2f_idx[i];
+
+  assert(c2f_idx[n_cells] == idx_size);
+
+  BFT_MALLOC(c2f_ids, idx_size, cs_lnum_t);
+
+  for (cs_lnum_t f_id = 0; f_id < n_i_faces; f_id++) {
+
+    const cs_lnum_t  c1_id = mesh->i_face_cells[f_id][0];
+    const cs_lnum_t  c2_id = mesh->i_face_cells[f_id][1];
+
+    if (c1_id < n_cells) { /* Don't want ghost cells */
+
+      const cs_lnum_t  shift = c2f_idx[c1_id] + cell_shift[c1_id];
+      c2f_ids[shift] = f_id;
+      cell_shift[c1_id] += 1;
+
+    }
+
+    if (c2_id < n_cells) { /* Don't want ghost cells */
+
+      const cs_lnum_t  shift = c2f_idx[c2_id] + cell_shift[c2_id];
+      c2f_ids[shift] = f_id;
+      cell_shift[c2_id] += 1;
+
+    }
+
+  } /* End of loop on internal faces */
+
+  for (cs_lnum_t  f_id = 0; f_id < n_b_faces; f_id++) {
+
+    const cs_lnum_t  c_id = mesh->b_face_cells[f_id];
+    const cs_lnum_t  shift = c2f_idx[c_id] + cell_shift[c_id];
+
+    c2f_ids[shift] = n_i_faces + f_id;
+    cell_shift[c_id] += 1;
+
+  } /* End of loop on border faces */
+
+  /* Free memory */
+  BFT_FREE(cell_shift);
+
+  /* Return pointers */
+  *p_c2f_idx = c2f_idx;
+  *p_c2f_ids = c2f_ids;
+}
+
 /*----------------------------------------------------------------------------
- * Evaluate boundary thickness.
+ * Compute cellwise the warping error
+ *  Id = 1/|c| \sum_(f \in F_c) x_f \otimes \vect{f}
+ * Froebinus norm is used to get a scalar-valued quantity
  *
  * parameters:
- *   mesh             <-- pointer to mesh structure
+ *   mesh             <-- pointer to mesh structure.
  *   mesh_quantities  <-- pointer to mesh quantities structures.
- *   b_thickness      --> boundary thickness
+ *   warp_error       <-> array of values to compute
  *----------------------------------------------------------------------------*/
 
 static void
-_get_boundary_thickness(const cs_mesh_t             *mesh,
-                        const cs_mesh_quantities_t  *mesh_quantities,
-                        cs_real_t                    b_thickness[])
+_compute_warp_error(const cs_mesh_t              *mesh,
+                    const cs_mesh_quantities_t   *mesh_quantities,
+                    cs_real_t                     warp_error[])
 {
-  const cs_real_3_t  *cell_cen
-    = (const cs_real_3_t  *)(mesh_quantities->cell_cen);
-  const cs_real_3_t  *b_face_cog
-    = (const cs_real_3_t  *)(mesh_quantities->b_face_cog);
-  const cs_real_3_t  *b_face_normal
-    = (const cs_real_3_t  *)(mesh_quantities->b_face_normal);
-  const cs_real_t  *b_face_surf
-    = (const cs_real_t *)(mesh_quantities->b_face_surf);
+  const cs_real_t  *vol = mesh_quantities->cell_vol;
 
-  for (cs_lnum_t f_id = 0; f_id < mesh->n_b_faces; f_id++) {
-    cs_lnum_t c_id = mesh->b_face_cells[f_id];
-    b_thickness[f_id]
-      = (  (b_face_cog[f_id][0] - cell_cen[c_id][0])*b_face_normal[f_id][0]
-         + (b_face_cog[f_id][1] - cell_cen[c_id][1])*b_face_normal[f_id][1]
-         + (b_face_cog[f_id][2] - cell_cen[c_id][2])*b_face_normal[f_id][2])
-        * 2.0 / b_face_surf[f_id];
-  }
+  cs_lnum_t  *c2f_ids = NULL;
+  cs_lnum_t  *c2f_idx = NULL;
+
+  /* Build cell -> face connectivity */
+  _build_c2f(mesh, &c2f_idx, &c2f_ids);
+
+  for (cs_lnum_t c_id = 0; c_id < mesh->n_cells; c_id++) {
+
+    const cs_real_t  invvol_c = 1/vol[c_id];
+    const cs_real_t  *xc = mesh_quantities->cell_cen + 3*c_id;
+
+    cs_real_33_t   tens = { {0, 0, 0}, {0, 0, 0}, {0, 0, 0} };
+
+    for (cs_lnum_t i = c2f_idx[c_id]; i < c2f_idx[c_id+1]; i++) {
+
+      cs_lnum_t  f_id = c2f_ids[i];
+      cs_real_t  *xf = NULL, *surf = NULL;
+      int  sgn = 1;
+
+      if (f_id < mesh->n_i_faces) {
+
+        const cs_lnum_t  c2_id = mesh->i_face_cells[f_id][1];
+        if (c_id == c2_id)
+          sgn = -1;
+
+        xf = mesh_quantities->i_face_cog + 3*f_id;
+        surf = mesh_quantities->i_f_face_normal + 3*f_id;
+
+      }
+      else {
+
+        f_id -= mesh->n_i_faces; // Border face
+        xf = mesh_quantities->b_face_cog + 3*f_id;
+        surf = mesh_quantities->b_f_face_normal + 3*f_id;
+
+      }
+
+      for (int ki = 0; ki < 3; ki++)
+        for (int kj = 0; kj < 3; kj++)
+          tens[ki][kj] += sgn*(xf[ki] - xc[ki]) * surf[kj];
+
+    } // Loop on face cells
+
+    for (int ki = 0; ki < 3; ki++)
+      for (int kj = 0; kj < 3; kj++)
+        tens[ki][kj] *= invvol_c;
+
+    warp_error[c_id] =
+      fabs(cs_math_33_determinant((const cs_real_t (*)[3])tens) - 1);
+
+  } // Loop on cells
+
+  /* Free memory */
+  BFT_FREE(c2f_ids);
+  BFT_FREE(c2f_idx);
 }
 
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
@@ -871,6 +1012,7 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
   bool  compute_orthogonality = true;
   bool  compute_warping = true;
   bool  compute_thickness = true;
+  bool  compute_warp_error = true;
   bool  vol_fields = false;
   bool  brd_fields = false;
 
@@ -885,6 +1027,8 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
   const cs_lnum_t  n_cells = mesh->n_cells;
   const cs_lnum_t  n_cells_wghosts = mesh->n_cells_with_ghosts;
 
+  const cs_time_step_t *ts = cs_glob_time_step;
+
   /* Check input data */
 
   assert(mesh_quantities->i_face_normal != NULL);
@@ -916,10 +1060,11 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
      For the moment, we export the mesh at this stage; this should be moved
      once PSTEMA has been moved from CALTRI to an earlier step. */
 
-  cs_post_write_meshes(NULL);
-
   cs_post_activate_writer(0, 1);
 
+  if (vol_fields || brd_fields)
+    cs_post_write_meshes(ts);
+
   /* Evaluate mesh quality criteria */
   /*--------------------------------*/
 
@@ -969,7 +1114,8 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                             b_face_warping,
                             face_to_cell);
 
-        cs_post_write_var(-1,
+        cs_post_write_var(CS_POST_MESH_VOLUME,
+                          CS_POST_WRITER_ALL_ASSOCIATED,
                           "Face_Warp_c_max",
                           1,
                           false,
@@ -978,7 +1124,7 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                           face_to_cell,
                           NULL,
                           NULL,
-                          NULL);
+                          ts);
 
       }
       if (face_to_vtx != NULL) {
@@ -989,20 +1135,22 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                            b_face_warping,
                            face_to_vtx);
 
-        cs_post_write_vertex_var(-1,
+        cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                                 CS_POST_WRITER_ALL_ASSOCIATED,
                                  "Face_Warp_v_max",
                                  1,
                                  false,
                                  true,
                                  CS_POST_TYPE_cs_real_t,
                                  face_to_vtx,
-                                 NULL);
+                                 ts);
       }
 
     } /* End of post-processing on volume */
 
     if (brd_fields == true)
-      cs_post_write_var(-2,
+      cs_post_write_var(CS_POST_MESH_BOUNDARY,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
                         "Face_Warp",
                         1,
                         false,
@@ -1011,7 +1159,7 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                         NULL,
                         NULL,
                         b_face_warping,
-                        NULL);
+                        ts);
 
     BFT_FREE(working_array);
 
@@ -1057,7 +1205,8 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
       if (face_to_cell != NULL) {
 
         _cell_from_max_face(mesh, 0.5, weighting, NULL, face_to_cell);
-        cs_post_write_var(-1,
+        cs_post_write_var(CS_POST_MESH_VOLUME,
+                          CS_POST_WRITER_ALL_ASSOCIATED,
                           "Weighting_c_max",
                           1,
                           false,
@@ -1066,23 +1215,25 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                           face_to_cell,
                           NULL,
                           NULL,
-                          NULL);
+                          ts);
 
       }
       if (face_to_vtx != NULL) {
 
         _vtx_from_max_face(mesh, 0.5, weighting, NULL, face_to_vtx);
-        cs_post_write_vertex_var(-1,
+        cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                                 CS_POST_WRITER_ALL_ASSOCIATED,
                                  "Weighting_v_max",
                                  1,
                                  false,
                                  true,
                                  CS_POST_TYPE_cs_real_t,
                                  face_to_vtx,
-                                 NULL);
+                                 ts);
 
       }
-      cs_post_write_var(-1,
+      cs_post_write_var(CS_POST_MESH_VOLUME,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
                         "Offset",
                         1,
                         false,
@@ -1091,7 +1242,7 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                         offsetting,
                         NULL,
                         NULL,
-                        NULL);
+                        ts);
 
     } /* End of post-processing on volume */
 
@@ -1141,7 +1292,8 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
       if (face_to_cell != NULL) {
 
         _cell_from_max_face(mesh, 0., i_face_ortho, b_face_ortho, face_to_cell);
-        cs_post_write_var(-1,
+        cs_post_write_var(CS_POST_MESH_VOLUME,
+                          CS_POST_WRITER_ALL_ASSOCIATED,
                           "Non_Ortho_c_max",
                           1,
                           false,
@@ -1150,26 +1302,28 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                           face_to_cell,
                           NULL,
                           NULL,
-                          NULL);
+                          ts);
 
       }
       if (face_to_vtx != NULL) {
 
         _vtx_from_max_face(mesh, 0., i_face_ortho, b_face_ortho, face_to_vtx);
-        cs_post_write_vertex_var(-1,
+        cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                                 CS_POST_WRITER_ALL_ASSOCIATED,
                                  "Non_Ortho_v_max",
                                  1,
                                  false,
                                  true,
                                  CS_POST_TYPE_cs_real_t,
                                  face_to_vtx,
-                                 NULL);
+                                 ts);
       }
 
     } /* End of post-processing on volume */
 
     if (brd_fields == true)
-      cs_post_write_var(-2,
+      cs_post_write_var(CS_POST_MESH_BOUNDARY,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
                         "Non_Ortho",
                         1,
                         false,
@@ -1178,7 +1332,7 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                         NULL,
                         NULL,
                         b_face_ortho,
-                        NULL);
+                        ts);
 
     BFT_FREE(working_array);
 
@@ -1201,7 +1355,8 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
     /* Post processing */
 
     if (vol_fields == true)
-      cs_post_write_var(-1,
+      cs_post_write_var(CS_POST_MESH_VOLUME,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
                         "Cell_Volume",
                         1,
                         false,
@@ -1210,7 +1365,7 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                         mesh_quantities->cell_vol,
                         NULL,
                         NULL,
-                        NULL);
+                        ts);
 
   } /* End of cell volume treatment */
 
@@ -1223,7 +1378,7 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
     cs_real_t *b_thickness;
     BFT_MALLOC(b_thickness, mesh->n_b_faces, cs_real_t);
 
-    _get_boundary_thickness(mesh, mesh_quantities, b_thickness);
+    cs_mesh_quantities_b_thickness_f(mesh, mesh_quantities, 0, b_thickness);
 
     /* Display histograms */
 
@@ -1233,7 +1388,8 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
     /* Post processing */
 
     if (vol_fields == true)
-      cs_post_write_var(-2,
+      cs_post_write_var(CS_POST_MESH_BOUNDARY,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
                         "Boundary_Cell_Thickness",
                         1,
                         false,
@@ -1242,11 +1398,53 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                         NULL,
                         NULL,
                         b_thickness,
-                        NULL);
+                        ts);
 
     BFT_FREE(b_thickness);
 
   } /* End of boundary cell thickness treatment */
+
+  /*---------------*/
+  /* warping error */
+  /*---------------*/
+
+  if (compute_warp_error == true) {
+
+    cs_real_t  *warp_error = NULL;
+    BFT_MALLOC(warp_error, mesh->n_cells ,cs_real_t);
+
+    _compute_warp_error(mesh, mesh_quantities, warp_error);
+
+    /* Display histograms */
+
+    bft_printf(_("\n  Histogram of the cellwise warping error :\n\n"));
+    _histogram(n_cells, warp_error);
+
+    /* Post processing */
+
+    if (vol_fields == true)
+      cs_post_write_var(CS_POST_MESH_VOLUME,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
+                        "Warp_Error",
+                        1,
+                        false,
+                        true,
+                        CS_POST_TYPE_cs_real_t,
+                        warp_error,
+                        NULL,
+                        NULL,
+                        ts);
+
+    double  l2_error = cs_gres(mesh->n_cells,
+                               mesh_quantities->cell_vol,
+                               warp_error,
+                               warp_error);
+
+    bft_printf(" L2-error norm induced by warping : %5.3e\n", sqrt(l2_error));
+    BFT_FREE(warp_error);
+
+  } /* End of cell volume treatment */
+
 }
 
 /*----------------------------------------------------------------------------*/
