@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2015 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -33,15 +33,49 @@
 
 #include "cs_base.h"
 #include "cs_mesh.h"
+#include "cs_internal_coupling.h"
 
 /*----------------------------------------------------------------------------*/
 
 BEGIN_C_DECLS
 
 /*=============================================================================
- * Local Macro definitions
+ * Macro definitions
  *============================================================================*/
 
+/*!
+ * @defgroup bad_cells_flags Flags specifying bad cells treatment
+ *
+ * @{
+ */
+
+/*
+ * Field property type
+ */
+
+/*! Correct bad cells warping for gradients */
+#define CS_BAD_CELLS_WARPED_CORRECTION (1 << 0)
+
+/*! Regularise on bad cells */
+#define CS_BAD_CELLS_REGULARISATION (1 << 1)
+
+/*! Recompute face centers */
+#define CS_CELL_FACE_CENTER_CORRECTION (1 << 2)
+
+/*! Recompute cell centers */
+#define CS_CELL_CENTER_CORRECTION (1 << 3)
+
+/*! Clip face distance when negative or too small */
+#define CS_FACE_DISTANCE_CLIP (1 << 4)
+
+/*! Clip geometrical quantities used in flux reconstuction */
+#define CS_FACE_RECONSTRUCTION_CLIP (1 << 5)
+
+/*! Limit cells volumle ratio */
+#define CS_CELL_VOLUME_RATIO_CORRECTION (1 << 6)
+
+/*! @} */
+
 /*============================================================================
  * Type definition
  *============================================================================*/
@@ -88,20 +122,27 @@ typedef struct {
   cs_real_t      max_vol;        /* Maximum cell volume */
   cs_real_t      tot_vol;        /* Total volume */
 
-  cs_real_33_t  *cocgb_s_it;     /* coupling of gradient compponents for
+  cs_real_t      min_f_vol;        /* Minimum cell volume */
+  cs_real_t      max_f_vol;        /* Maximum cell volume */
+  cs_real_t      tot_f_vol;        /* Total volume */
+
+  cs_real_33_t  *cocgb_s_it;     /* coupling of gradient components for
                                     iterative reconstruction at boundary */
-  cs_real_33_t  *cocg_s_it;      /* coupling of gradient compponents for
+  cs_real_33_t  *cocg_s_it;      /* coupling of gradient components for
                                     iterative reconstruction */
-  cs_real_33_t  *cocgb_s_lsq;    /* coupling of gradient compponents for
+  cs_real_33_t  *cocgb_s_lsq;    /* coupling of gradient components for
                                     least-square reconstruction at boundary */
-  cs_real_33_t  *cocg_s_lsq;     /* coupling of gradient compponents for
-                                    least-square reconstruction */
 
   cs_real_33_t  *cocg_it;        /* Interleaved cocg matrix
                                     for iterative gradients */
   cs_real_33_t  *cocg_lsq;       /* Interleaved cocg matrix
                                     for least square gradients */
 
+  cs_real_t     *corr_grad_lin_det;  /* Determinant of geometrical matrix
+                                        linear gradient correction */
+  cs_real_33_t  *corr_grad_lin;      /* Geometrical matrix
+                                        linear gradient correction */
+
   cs_int_t      *b_sym_flag;     /* Symmetry flag for boundary faces */
   cs_int_t      *c_solid_flag;   /* Is the fluid volume 0 flag */
   unsigned      *bad_cell_flag;  /* Flag (mask) for bad cells detected */
@@ -116,6 +157,9 @@ typedef struct {
 
 extern cs_mesh_quantities_t  *cs_glob_mesh_quantities;
 
+/* Flag (mask) to activate bad cells correction */
+extern unsigned cs_glob_mesh_quantities_flag;
+
 /* Choice of the porous model */
 extern int cs_glob_porous_model;
 
@@ -236,6 +280,29 @@ cs_mesh_quantities_create(void);
 cs_mesh_quantities_t *
 cs_mesh_quantities_destroy(cs_mesh_quantities_t  *mesh_quantities);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Reset a mesh quantities structure to its empty initial state.
+ *
+ * \param[in]   mq           pointer to mesh quantities structures.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_mesh_quantities_free_all(cs_mesh_quantities_t  *mq);
+
+/*----------------------------------------------------------------------------
+ * Compute mesh quantities needed fo preprocessing
+ *
+ * parameters:
+ *   mesh            <-- pointer to a cs_mesh_t structure
+ *   mesh_quantities <-> pointer to a cs_mesh_quantities_t structure
+ *----------------------------------------------------------------------------*/
+
+void
+cs_mesh_quantities_compute_preprocess(const cs_mesh_t       *mesh,
+                                      cs_mesh_quantities_t  *mesh_quantities);
+
 /*----------------------------------------------------------------------------
  * Compute mesh quantities
  *
@@ -248,6 +315,30 @@ void
 cs_mesh_quantities_compute(const cs_mesh_t       *mesh,
                            cs_mesh_quantities_t  *mesh_quantities);
 
+/*----------------------------------------------------------------------------
+ * Compute fluid mesh quantities
+ *
+ * parameters:
+ *   mesh            <-- pointer to a cs_mesh_t structure
+ *   mesh_quantities <-> pointer to a cs_mesh_quantities_t structure
+ *----------------------------------------------------------------------------*/
+
+void
+cs_mesh_quantities_fluid_compute(const cs_mesh_t       *mesh,
+                                 cs_mesh_quantities_t  *mesh_quantities);
+
+/*----------------------------------------------------------------------------
+ * Compute fluid section mesh quantities at the initial step
+ *
+ * parameters:
+ *   mesh            <-- pointer to a cs_mesh_t structure
+ *   mesh_quantities <-> pointer to a cs_mesh_quantities_t structure
+ *----------------------------------------------------------------------------*/
+
+void
+cs_mesh_init_fluid_sections(const cs_mesh_t       *mesh,
+                            cs_mesh_quantities_t  *mesh_quantities);
+
 /*----------------------------------------------------------------------------
  * Compute mesh quantities
  *
@@ -308,6 +399,21 @@ cs_mesh_quantities_b_faces(const cs_mesh_t   *mesh,
                            cs_real_t         *p_b_face_cog[],
                            cs_real_t         *p_b_face_normal[]);
 
+/*----------------------------------------------------------------------------
+ * Compute cell centers.
+ *
+ * The corresponding array is allocated by this function, and it is the
+ * caller's responsibility to free it when they are no longer needed.
+ *
+ * parameters:
+ *   mesh       <-- pointer to a cs_mesh_t structure
+ *   p_cell_cen <-> pointer to the cell centers array
+ *----------------------------------------------------------------------------*/
+
+void
+cs_mesh_quantities_cell_cen(const cs_mesh_t  *mesh,
+                            cs_real_t        *cell_cen[]);
+
 /*----------------------------------------------------------------------------
  * Check that no negative volumes are present, and exit on error otherwise.
  *
@@ -345,6 +451,41 @@ cs_mesh_quantities_reduce_extended(const cs_mesh_t       *mesh,
 int
 cs_mesh_quantities_compute_count(void);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Determine local boundary thickness around each vertex.
+ *
+ * \param[in]   m            pointer to mesh structure
+ * \param[in]   mq           pointer to mesh quantities structures.
+ * \param[in]   n_passes     number of smoothing passes
+ * \param[out]  b_thickness  thickness for each mesh vertex
+ *                           (0 at non-boundary vertices)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_mesh_quantities_b_thickness_v(const cs_mesh_t             *m,
+                                 const cs_mesh_quantities_t  *mq,
+                                 int                          n_passes,
+                                 cs_real_t                    b_thickness[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Determine local boundary thickness around each boundary face.
+ *
+ * \param[in]   m            pointer to mesh structure
+ * \param[in]   mq           pointer to mesh quantities structures.
+ * \param[in]   n_passes     number of optional smoothing passes
+ * \param[out]  b_thickness  thickness for each mesh boundary face
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_mesh_quantities_b_thickness_f(const cs_mesh_t             *m,
+                                 const cs_mesh_quantities_t  *mq,
+                                 int                          n_passes,
+                                 cs_real_t                    b_thickness[]);
+
 /*----------------------------------------------------------------------------
  * Dump a cs_mesh_quantities_t structure
  *
@@ -357,6 +498,36 @@ void
 cs_mesh_quantities_dump(const cs_mesh_t             *mesh,
                         const cs_mesh_quantities_t  *mesh_quantities);
 
+/*----------------------------------------------------------------------------
+ * Compute 3x3 matrix cocg for the scalar gradient least squares algorithm
+ * adapted for internal coupling.
+ *
+ * parameters:
+ *   m    <--  mesh
+ *   fvq  <->  mesh quantities
+ *   ce   <->  coupling
+ *----------------------------------------------------------------------------*/
+
+void
+cs_compute_cell_cocg_lsq_coupling(const cs_mesh_t         *m,
+                                  cs_mesh_quantities_t    *fvq,
+                                  cs_internal_coupling_t  *ce);
+
+/*----------------------------------------------------------------------------
+ * Compute 3x3 matrix cocg for the scalar gradient iterative algorithm
+ * adapted for internal coupling.
+ *
+ * parameters:
+ *   m    <--  mesh
+ *   fvq  <->  mesh quantities
+ *   ce   <->  coupling
+ *----------------------------------------------------------------------------*/
+
+void
+cs_compute_cell_cocg_it_coupling(const cs_mesh_t         *m,
+                                 cs_mesh_quantities_t    *fvq,
+                                 cs_internal_coupling_t  *ce);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
