@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -68,7 +68,7 @@ implicit none
 
 ! Local variables
 
-integer         ipp, icla, icha, igg, it, ir, ih, if, izone
+integer         icla, icha, igg, it, ir, ih, if, izone
 integer         isol, ige, iat , iii
 integer         idirac
 
@@ -93,8 +93,9 @@ ihm = 0 ! enthalpy, common to many models
 i_comb_drift = 0
 
 !> --- Specific condensation modelling
-!>     (icond = -1 : not activated by default)
-icond = -1
+!>     (icondb, icondv = -1 : not activated by default)
+icondb = -1
+icondv = -1
 
 ! ---> Initialisation pour la combustion gaz
 !       Variables transportees
@@ -108,8 +109,6 @@ iyfm   = 0
 iyfp2m = 0
 icoyfp = 0
 !       Variables algebriques ou d'etat
-nsalpp = 0
-nsalto = 0
 itemp  = 0
 do igg = 1, ngazgm
   iym(igg) = 0
@@ -731,4 +730,3 @@ enddo
 
 return
 end subroutine ppinii
-
