@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -82,6 +82,7 @@ double precision g11, g22, g33, g12, g13, g23, gkks3
 double precision g11p, g22p, g33p
 double precision phit11, phit22, phit33, phit12, phit13, phit23
 double precision aa, bb
+double precision turb_schmidt
 
 double precision, dimension(:), pointer :: cvara_ep
 double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
@@ -101,7 +102,8 @@ else
 endif
 
 if(iscalt.gt.0.and.nscal.ge.iscalt) then
-  prdtur = sigmas(iscalt)
+  call field_get_key_double(ivarfl(isca(iscalt)), ksigmas, turb_schmidt)
+  prdtur = turb_schmidt
 else
   prdtur = 1.d0
 endif
@@ -126,7 +128,7 @@ call field_get_val_prev_s(ivarfl(ir23), cvara_r23)
 !                 Rit = Rik drom/dxk (sum on k)
 !===============================================================================
 
-
+! FIXME use beta instead and be consistant with the model chosen...
 if     (ivar.eq.ir11) then
 
   do iel = 1, ncel
@@ -308,6 +310,7 @@ elseif (ivar.eq.iep ) then
     g22p = const*2.d0*(r2t*gy)
     g33p = const*2.d0*(r3t*gz)
 
+    !FIXME for EB-DFM and EBRSM
     aa = 0.d0
     bb = 0.5d0*(g11p+g22p+g33p)
     smbr(iel) = smbr(iel) + ce1*max(aa,bb)*volume(iel)
