@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -28,6 +28,11 @@
 !>
 !> \brief Solving the \f$ k - \epsilon \f$ for incompressible flows or slightly
 !> compressible flows for one time step.
+!>
+!> Please refer to the
+!> <a href="../../theory.pdf#keps"><b>\f$k - \epsilon \f$ model</b></a>
+!> section of the theory guide for more informations, as well as the
+!> <a href="../../theory.pdf#turbke"><b>turbke</b></a> section.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -76,6 +81,7 @@ use mesh
 use field
 use field_operator
 use cs_c_bindings
+use atincl, only: kopint
 
 !===============================================================================
 
@@ -105,7 +111,7 @@ integer          iconvp, idiffp, ndircp
 integer          nswrsp, ircflp, ischcp, isstpp, iescap
 integer          iflmas, iflmab
 integer          iwarnp
-integer          istprv
+integer          istprv, f_oi_id
 integer          iphydp, iprev
 integer          imucpp, idftnp, iswdyp
 
@@ -123,6 +129,9 @@ double precision thetp1, thetak, thetae, thets, thetap
 double precision tuexpk, tuexpe
 double precision cmueta, sqrcmu, xs
 double precision hint
+double precision turb_schmidt
+double precision xnoral, xnal(3)
+double precision utaurf,ut2,ypa,ya,xunorm, limiter, nu0,alpha
 
 double precision rvoid(1)
 
@@ -151,6 +160,10 @@ double precision, dimension(:), pointer :: cvara_al, cvara_phi
 double precision, dimension(:), pointer :: cpro_pcvto, cpro_pcvlo
 double precision, dimension(:), pointer :: viscl, cvisct
 double precision, dimension(:), pointer :: c_st_k_p, c_st_eps_p
+double precision, dimension(:,:), pointer :: vel
+double precision, dimension(:), pointer :: cvar_al
+
+type(var_cal_opt) :: vcopt_k, vcopt_e
 
 !===============================================================================
 
@@ -182,8 +195,8 @@ endif
 call field_get_coefa_v(ivarfl(iu), coefau)
 call field_get_coefb_v(ivarfl(iu), coefbu)
 
-call field_get_val_s(iprpfl(ivisct), cvisct)
-call field_get_val_s(iprpfl(iviscl), viscl)
+call field_get_val_s(ivisct, cvisct)
+call field_get_val_s(iviscl, viscl)
 
 call field_get_key_int(ivarfl(iu), kimasf, iflmas)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
@@ -213,20 +226,23 @@ call field_get_val_s(icrom, crom)
 call field_get_val_s(ibrom, brom)
 call field_get_val_s(icrom, cromo)
 call field_get_val_s(ibrom, bromo)
-call field_get_val_s(iprpfl(ivisct), cpro_pcvto)
-call field_get_val_s(iprpfl(iviscl), cpro_pcvlo)
+call field_get_val_s(ivisct, cpro_pcvto)
+call field_get_val_s(iviscl, cpro_pcvlo)
 if (istprv.ge.0) then
   if (iroext.gt.0) then
     call field_get_val_prev_s(icrom, cromo)
     call field_get_val_prev_s(ibrom, bromo)
   endif
   if (iviext.gt.0) then
-    call field_get_val_s(iprpfl(ivista), cpro_pcvto)
-    call field_get_val_s(iprpfl(ivisla), cpro_pcvlo)
+    call field_get_val_prev_s(ivisct, cpro_pcvto)
+    call field_get_val_prev_s(iviscl, cpro_pcvlo)
   endif
 endif
 
-if (iwarni(ik).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ik), vcopt_k)
+call field_get_key_struct_var_cal_opt(ivarfl(iep), vcopt_e)
+
+if (vcopt_k%iwarni.ge.1) then
   if (iturb.eq.20) then
     write(nfecra,1000)
   else if (iturb.eq.21) then
@@ -242,6 +258,107 @@ sqrcmu = sqrt(cmu)
 d2s3 = 2.d0/3.d0
 d1s3 = 1.d0/3.d0
 
+!===============================================================================
+! 1.1 Advanced reinit
+!===============================================================================
+
+! Automatic reinitialization at the end of the first iteration:
+! wall distance y^+ is computed with -C log(1-alpha), where C=CL*Ceta*L*kappa,
+! then y so we have an idea of the wall distance in complex geometries.
+! Then U is initialized with a Reichard layer,
+! Epsilon by 1/(kappa y), clipped next to the wall at its value for y^+=15.
+! k is given by a blending between eps/(2 nu)*y^2 and utau/sqrt(Cmu)
+! The blending function is chosen so that the asymptotic behavior
+! and give the correct peak of k (not the same blending than for the EBRSM
+! because k profile is not the same for k-omega).
+
+!TODO FIXME: Are the BC uncompatible?
+if (ntcabs.eq.1.and.reinit_turb.eq.1.and.iturb.eq.51) then
+
+  allocate(grad(3,ncelet))
+
+  ! Compute the gradient of Alpha
+  iprev  = 0
+  inc    = 1
+  iccocg = 1
+
+  call field_gradient_scalar(ivarfl(ial), iprev, imrgra, inc,     &
+                             iccocg,                              &
+                             grad)
+
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+  call field_get_val_s(ivarfl(ial), cvar_al)
+  call field_get_val_v(ivarfl(iu), vel)
+
+  utaurf = 0.05d0*uref
+  nu0 = viscl0 / ro0
+
+  do iel = 1, ncel
+    ! Compute the velocity magnitude
+    xunorm = vel(1,iel)**2 + vel(2,iel)**2 + vel(3,iel)**2
+    xunorm = sqrt(xunorm)
+
+    ! y+ is bounded by 400, because in the Reichard profile,
+    ! it corresponds to saturation (u>uref)
+    cvar_al(iel) = max(min(cvar_al(iel),(1.d0-exp(-400.d0/50.d0))),0.d0)
+
+    call field_current_to_previous(ivarfl(ial))
+
+    ! Compute the magnitude of the alpha gradient
+    xnoral = ( grad(1,iel)*grad(1,iel)          &
+           +   grad(2,iel)*grad(2,iel)          &
+           +   grad(3,iel)*grad(3,iel) )
+    xnoral = sqrt(xnoral)
+   ! Compute the unitary vector of Alpha
+    if (xnoral.le.epzero/cell_f_vol(iel)**(1.d0/3.d0)) then
+      xnal(1) = 1.d0/sqrt(3.d0)
+      xnal(2) = 1.d0/sqrt(3.d0)
+      xnal(3) = 1.d0/sqrt(3.d0)
+    else
+      xnal(1) = grad(1,iel)/xnoral
+      xnal(2) = grad(2,iel)/xnoral
+      xnal(3) = grad(3,iel)/xnoral
+    endif
+
+    alpha = cvar_al(iel)
+
+    ! Compute YA, therefore alpha is given by 1-exp(-YA/(50 nu/utau))
+    ! NB: y^+ = 50 give the best compromise
+    ya = -dlog(1.d0-cvar_al(iel))*50.d0*nu0/utaurf
+    ypa = ya/(nu0/utaurf)
+
+    ! Velocity magnitude is imposed (limited only), the direction is
+    ! conserved
+    if (xunorm.le.1.d-12*uref) then
+      limiter = 1.d0
+    else
+      limiter = min( utaurf/xunorm*(2.5d0*dlog(1.d0+0.4d0*ypa)  &
+                    +7.8d0*(1.d0-dexp(-ypa/11.d0)               &
+                    -(ypa/11.d0)*dexp(-0.33d0*ypa))),           &
+                    1.d0)
+    endif
+
+    vel(1,iel) = limiter*vel(1,iel)
+    vel(2,iel) = limiter*vel(2,iel)
+    vel(3,iel) = limiter*vel(3,iel)
+
+    ut2 = 0.05d0*uref
+
+    cvar_ep(iel) = utaurf**3*min(1.d0/(xkappa*15.d0*nu0/utaurf), &
+                                 1.d0/(xkappa*ya))
+    cvar_k(iel) =  cvar_ep(iel)/2.d0/nu0*ya**2                 &
+                   * exp(-ypa/25.d0)**2                        &
+                 + ut2**2/sqrt(cmu)*(1.d0-exp(-ypa/25.d0))**2
+  enddo
+
+  call field_current_to_previous(ivarfl(iu))
+  call field_current_to_previous(ivarfl(ik))
+  call field_current_to_previous(ivarfl(iep)) !TODO phi ?
+
+  deallocate(grad)
+endif
+
+
 !===============================================================================
 ! 2. Compute the scalar strain rate SijSij and the trace of the velocity
 !    gradient
@@ -288,9 +405,9 @@ deallocate(gradv)
 
 do iel = 1, ncel
   rho = crom(iel)
-  romvsd = rho*volume(iel)/dt(iel)
-  tinstk(iel) = istat(ik)*romvsd
-  tinste(iel) = istat(iep)*romvsd
+  romvsd = rho*cell_f_vol(iel)/dt(iel)
+  tinstk(iel) = vcopt_k%istat*romvsd
+  tinste(iel) = vcopt_e%istat*romvsd
 enddo
 
 !===============================================================================
@@ -379,12 +496,12 @@ else if (igrake.eq.1) then
 
   iccocg = 1
   inc = 1
-  nswrgp = nswrgr(ik)
-  epsrgp = epsrgr(ik)
-  imligp = imligr(ik)
-  iwarnp = iwarni(ik)
-  climgp = climgr(ik)
-  extrap = extrag(ik)
+  nswrgp = vcopt_k%nswrgr
+  epsrgp = vcopt_k%epsrgr
+  imligp = vcopt_k%imligr
+  iwarnp = vcopt_k%iwarni
+  climgp = vcopt_k%climgr
+  extrap = vcopt_k%extrag
 
   ! Dirichlet boundary condition on the gradient of rho
   do ifac = 1, nfabor
@@ -403,7 +520,8 @@ else if (igrake.eq.1) then
   !   smbrk = P+G
   !   smbre = P+(1-ce3)*G
   if (iscalt.gt.0.and.nscal.ge.iscalt) then
-    prdtur = sigmas(iscalt)
+    call field_get_key_double(ivarfl(isca(iscalt)), ksigmas, turb_schmidt)
+    prdtur = turb_schmidt
   else
     prdtur = 1.d0
   endif
@@ -420,7 +538,7 @@ else if (igrake.eq.1) then
            / (rho*prdtur)
 
     ! Implicit Buoyant terms when negativ
-    tinstk(iel) = tinstk(iel) + max(-rho*volume(iel)*cmu*ttke*gravke, 0.d0)
+    tinstk(iel) = tinstk(iel) + max(-rho*cell_f_vol(iel)*cmu*ttke*gravke, 0.d0)
 
     ! Explicit Buoyant terms
     smbre(iel) = smbre(iel) + visct*max(gravke, zero)
@@ -484,12 +602,12 @@ if (iturb.eq.51) then
   inc = 1
   init = 1
 
-  nswrgp = nswrgr(ivar)
-  imligp = imligr(ivar)
-  iwarnp = iwarni(ivar)
-  epsrgp = epsrgr(ivar)
-  climgp = climgr(ivar)
-  extrap = extrag(ivar)
+  nswrgp = vcopt_k%nswrgr
+  imligp = vcopt_k%imligr
+  iwarnp = vcopt_k%iwarni
+  epsrgp = vcopt_k%epsrgr
+  climgp = vcopt_k%climgr
+  extrap = vcopt_k%extrag
   iphydp = 0
 
   call itrgrp &
@@ -500,7 +618,7 @@ if (iturb.eq.51) then
   cvara_k         ,                                                            &
   coefap , coefbp , cofafp , cofbfp ,                                          &
   viscf  , viscb  ,                                                            &
-  w3     , w3     , w3     ,                                                   &
+  w3     ,                                                                     &
   w10    )
 
   do iel = 1, ncel
@@ -572,12 +690,12 @@ if (itytur.eq.2) then
 
     rho   = cromo(iel)
 
-    smbrk(iel) = volume(iel)*                                     &
+    smbrk(iel) = cell_f_vol(iel)*                                     &
                ( smbrk(iel) - rho*cvara_ep(iel)                       &
                - d2s3*rho*cvara_k(iel)*divu(iel)                      &
                )
 
-    smbre(iel) = volume(iel)*                                              &
+    smbre(iel) = cell_f_vol(iel)*                                              &
                ( cvara_ep(iel)/cvara_k(iel)*( ce1*smbre(iel)                       &
                                             - ce2rc(iel)*rho*cvara_ep(iel)     &
                                             )                              &
@@ -593,10 +711,10 @@ if (itytur.eq.2) then
       xk   = cvara_k(iel)
       rho = crom(iel)
       ttke = xk / xeps
-      tinstk(iel) = tinstk(iel) + rho*volume(iel)/ttke            &
-                  + max(d2s3*rho*volume(iel)*divu(iel), 0.d0)
-      tinste(iel) = tinste(iel) + ce2rc(iel)*rho*volume(iel)/ttke &
-                  + max(d2s3*ce1*rho*volume(iel)*divu(iel), 0.d0)
+      tinstk(iel) = tinstk(iel) + rho*cell_f_vol(iel)/ttke            &
+                  + max(d2s3*rho*cell_f_vol(iel)*divu(iel), 0.d0)
+      tinste(iel) = tinste(iel) + ce2rc(iel)*rho*cell_f_vol(iel)/ttke &
+                  + max(d2s3*ce1*rho*cell_f_vol(iel)*divu(iel), 0.d0)
     enddo
   endif
 
@@ -617,27 +735,26 @@ else if (iturb.eq.50) then
     tt = max(ttke, ttmin)
 
     ! Explicit part
-    smbrk(iel) = volume(iel)*                                     &
+    smbrk(iel) = cell_f_vol(iel)*                                     &
                ( smbrk(iel) - rho*cvara_ep(iel)                       &
                - d2s3*rho*cvara_k(iel)*divu(iel)                      &
                )
 
-    smbre(iel) = volume(iel)*                                       &
+    smbre(iel) = cell_f_vol(iel)*                                       &
                ( 1.d0/tt*(ceps1*smbre(iel) - ce2rc(iel)*rho*xeps)   &
                - d2s3*rho*ceps1*xk*divu(iel)                        &
                )
 
-    ! We store the part with Pk in PRDV2F which will be reused in RESV2F
+    ! We store the part with Pk in prdv2f which will be reused in resv2f
     prdv2f(iel) = prdv2f(iel)                               &
                 - d2s3*rho*cvara_k(iel)*divu(iel)!FIXME this term should be removed
 
     ! Implicit part
-    if (xk.gt.1.d-12) then !FIXME make it dimensionless
-      tinstk(iel) = tinstk(iel) + rho*volume(iel)/ttke
-    endif
-    tinstk(iel) = tinstk(iel) + max(d2s3*rho*volume(iel)*divu(iel), 0.d0)
-    tinste(iel) = tinste(iel) + ce2rc(iel)*rho*volume(iel)/tt                &
-                + max(d2s3*ceps1*ttke/tt*rho*volume(iel)*divu(iel), 0.d0)
+    tinstk(iel) = tinstk(iel) + rho*cell_f_vol(iel)/max(ttke, epzero * ttmin)
+
+    tinstk(iel) = tinstk(iel) + max(d2s3*rho*cell_f_vol(iel)*divu(iel), 0.d0)
+    tinste(iel) = tinste(iel) + ce2rc(iel)*rho*cell_f_vol(iel)/tt                &
+                + max(d2s3*ceps1*ttke/tt*rho*cell_f_vol(iel)*divu(iel), 0.d0)
 
   enddo
 
@@ -656,30 +773,31 @@ else if (iturb.eq.51) then
     tt = sqrt(ttke**2.d0+ttmin**2.d0)
 
     ! Explicit part
-    smbrk(iel) = volume(iel)*                                     &
+    smbrk(iel) = cell_f_vol(iel)*                                     &
                ( smbrk(iel)                                       &
                - rho*xeps                                         &
                - rho*w11(iel)*xk                                  &
                - d2s3*rho*xk*divu(iel)                            &
                )
 
-    smbre(iel) = volume(iel)*                                               &
+    smbre(iel) = cell_f_vol(iel)*                                               &
                ( 1.d0/tt*(cpale1*smbre(iel) - w10(iel)*rho*xeps)            &
                - d2s3*rho*cpale1*xk/tt*divu(iel)                            &
                )
 
-    ! We store the part with Pk in PRDV2F which will be reused in RESV2F
+    ! We store the part with Pk in prdv2f which will be reused in resv2f
     prdv2f(iel) = prdv2f(iel)                               &
                 - d2s3*rho*cvara_k(iel)*divu(iel)!FIXME this term should be removed
 
     ! Implicit part
-    if (xk.gt.1.d-12) then !FIXME make it dimensionless
-      tinstk(iel) = tinstk(iel) + rho*volume(iel)/ttke
-    endif
-    tinstk(iel) = tinstk(iel) + max(d2s3*rho*volume(iel)*divu(iel), 0.d0)
-    tinstk(iel) = tinstk(iel) + w11(iel)*rho*volume(iel)
-    tinste(iel) = tinste(iel) + w10(iel)*rho*volume(iel)/tt                  &
-                + max(d2s3*cpale1*ttke/tt*rho*volume(iel)*divu(iel), 0.d0)
+    tinstk(iel) = tinstk(iel) + rho*cell_f_vol(iel)/max(ttke, epzero * ttmin)
+
+    tinstk(iel) = tinstk(iel) + max(d2s3*rho*cell_f_vol(iel)*divu(iel), 0.d0)
+    ! Note that w11 is positiv
+    tinstk(iel) = tinstk(iel) + w11(iel)*rho*cell_f_vol(iel)
+    ! Note that w10 is positiv
+    tinste(iel) = tinste(iel) + w10(iel)*rho*cell_f_vol(iel)/tt                  &
+                + max(d2s3*cpale1*ttke/tt*rho*cell_f_vol(iel)*divu(iel), 0.d0)
 
   enddo
 
@@ -717,11 +835,25 @@ call cs_user_turbulence_source_terms &
    ckupdc , smacel ,                                              &
    w8     , usimpe )
 
+if (ippmod(iatmos).ge.0) then
+  ! Nudging towards optimal interpolation for k
+  call field_get_key_int(ivarfl(ik), kopint, f_oi_id)
+  if (f_oi_id.ge.0) then
+    call cs_at_data_assim_source_term(ivarfl(ik), w7, usimpk)
+  endif
+
+  ! Nudging towards optimal interpolation for epsilon
+  call field_get_key_int(ivarfl(iep), kopint, f_oi_id)
+  if (f_oi_id.ge.0) then
+    call cs_at_data_assim_source_term(ivarfl(ik), w8, usimpe)
+  endif
+endif
+
 ! If source terms are extrapolated over time
 if (istprv.ge.0) then
 
-  thetak = thetav(ik)
-  thetae = thetav(iep)
+  thetak = vcopt_k%thetav
+  thetae = vcopt_e%thetav
 
   do iel = 1, ncel
 
@@ -871,10 +1003,10 @@ if (ikecou.eq.1) then
   call field_get_coefaf_s(ivarfl(ivar), cofafp)
   call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
-  if (idiff(ivar).ge. 1) then
+  if (vcopt_k%idiff.ge. 1) then
 
     do iel = 1, ncel
-      w4(iel) = viscl(iel) + idifft(ivar)*cvisct(iel)/sigmak
+      w4(iel) = viscl(iel) + vcopt_k%idifft*cvisct(iel)/sigmak
     enddo
 
     call viscfa(imvisf, w4, viscf, viscb)
@@ -892,40 +1024,40 @@ if (ikecou.eq.1) then
 
   iccocg = 1
   inc    = 1
-  iconvp = iconv (ivar)
+  iconvp = vcopt_k%iconv
   imasac = 1
-  idiffp = idiff (ivar)
-  nswrgp = nswrgr(ivar)
-  imligp = imligr(ivar)
-  ircflp = ircflu(ivar)
-  ischcp = ischcv(ivar)
-  isstpp = isstpc(ivar)
-  iwarnp = iwarni(ivar)
+  idiffp = vcopt_k%idiff
+  nswrgp = vcopt_k%nswrgr
+  imligp = vcopt_k%imligr
+  ircflp = vcopt_k%ircflu
+  ischcp = vcopt_k%ischcv
+  isstpp = vcopt_k%isstpc
+  iwarnp = vcopt_k%iwarni
   imucpp = 0
-  idftnp = 1 ! no tensorial diffusivity
-  blencp = blencv(ivar)
-  epsrgp = epsrgr(ivar)
-  climgp = climgr(ivar)
-  extrap = extrag(ivar)
-  relaxp = relaxv(ivar)
-  thetap = thetav(ivar)
+  idftnp = ISOTROPIC_DIFFUSION
+  blencp = vcopt_k%blencv
+  epsrgp = vcopt_k%epsrgr
+  climgp = vcopt_k%climgr
+  extrap = vcopt_k%extrag
+  relaxp = vcopt_k%relaxv
+  thetap = vcopt_k%thetav
   ! all boundary convective flux with upwind
   icvflb = 0
 
   call bilsca &
- ( idtvar , ivar   , iconvp , idiffp , nswrgp , imligp , ircflp , &
-   ischcp , isstpp , inc    , imrgra , iccocg ,                   &
-   iwarnp , imucpp , idftnp , imasac ,                            &
-   blencp , epsrgp , climgp , extrap , relaxp , thetap ,          &
-   cvara_k         , cvara_k         ,                            &
-   coefap , coefbp , cofafp , cofbfp ,                            &
-   imasfl , bmasfl ,                                              &
-   viscf  , viscb  , rvoid  , rvoid  ,                            &
-   rvoid  , rvoid  ,                                              &
-   icvflb , ivoid  ,                                              &
+ ( idtvar , ivarfl(ivar)    , iconvp , idiffp , nswrgp , imligp , ircflp , &
+   ischcp , isstpp , inc    , imrgra , iccocg ,                            &
+   iwarnp , imucpp , idftnp , imasac ,                                     &
+   blencp , epsrgp , climgp , extrap , relaxp , thetap ,                   &
+   cvara_k         , cvara_k         ,                                     &
+   coefap , coefbp , cofafp , cofbfp ,                                     &
+   imasfl , bmasfl ,                                                       &
+   viscf  , viscb  , rvoid  , rvoid  ,                                     &
+   rvoid  , rvoid  ,                                                       &
+   icvflb , ivoid  ,                                                       &
    w7     )
 
-  if (iwarni(ivar).ge.2) then
+  if (vcopt_k%iwarni.ge.2) then
     rnorm = sqrt(cs_gdot(ncel,smbrk,smbrk))
     write(nfecra,1100) chaine(1:8) ,rnorm
   endif
@@ -940,10 +1072,10 @@ if (ikecou.eq.1) then
   call field_get_coefaf_s(ivarfl(ivar), cofafp)
   call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
-  if (idiff(ivar).ge. 1) then
+  if (vcopt_e%idiff.ge. 1) then
     do iel = 1, ncel
       w4(iel) = viscl(iel)                                        &
-              + idifft(ivar)*cvisct(iel)/sigmae
+              + vcopt_e%idifft*cvisct(iel)/sigmae
     enddo
 
     call viscfa(imvisf, w4, viscf, viscb)
@@ -961,40 +1093,40 @@ if (ikecou.eq.1) then
 
   iccocg = 1
   inc    = 1
-  iconvp = iconv (ivar)
+  iconvp = vcopt_e%iconv
   imasac = 1
-  idiffp = idiff (ivar)
-  nswrgp = nswrgr(ivar)
-  imligp = imligr(ivar)
-  ircflp = ircflu(ivar)
-  ischcp = ischcv(ivar)
-  isstpp = isstpc(ivar)
-  iwarnp = iwarni(ivar)
+  idiffp = vcopt_e%idiff
+  nswrgp = vcopt_e%nswrgr
+  imligp = vcopt_e%imligr
+  ircflp = vcopt_e%ircflu
+  ischcp = vcopt_e%ischcv
+  isstpp = vcopt_e%isstpc
+  iwarnp = vcopt_e%iwarni
   imucpp = 0
-  idftnp = 1 ! no tensorial diffusivity
-  blencp = blencv(ivar)
-  epsrgp = epsrgr(ivar)
-  climgp = climgr(ivar)
-  extrap = extrag(ivar)
-  relaxp = relaxv(ivar)
-  thetap = thetav(ivar)
+  idftnp = ISOTROPIC_DIFFUSION
+  blencp = vcopt_e%blencv
+  epsrgp = vcopt_e%epsrgr
+  climgp = vcopt_e%climgr
+  extrap = vcopt_e%extrag
+  relaxp = vcopt_e%relaxv
+  thetap = vcopt_e%thetav
   ! all boundary convective flux with upwind
   icvflb = 0
 
   call bilsca &
- ( idtvar , ivar   , iconvp , idiffp , nswrgp , imligp , ircflp , &
-   ischcp , isstpp , inc    , imrgra , iccocg ,                   &
-   iwarnp , imucpp , idftnp , imasac ,                            &
-   blencp , epsrgp , climgp , extrap , relaxp , thetap ,          &
-   cvara_ep        , cvara_ep        ,                            &
-   coefap , coefbp , cofafp , cofbfp ,                            &
-   imasfl , bmasfl ,                                              &
-   viscf  , viscb  , rvoid  , rvoid  ,                            &
-   rvoid  , rvoid  ,                                              &
-   icvflb , ivoid  ,                                              &
+ ( idtvar , ivarfl(ivar)    , iconvp , idiffp , nswrgp , imligp , ircflp , &
+   ischcp , isstpp , inc    , imrgra , iccocg ,                            &
+   iwarnp , imucpp , idftnp , imasac ,                                     &
+   blencp , epsrgp , climgp , extrap , relaxp , thetap ,                   &
+   cvara_ep        , cvara_ep        ,                                     &
+   coefap , coefbp , cofafp , cofbfp ,                                     &
+   imasfl , bmasfl ,                                                       &
+   viscf  , viscb  , rvoid  , rvoid  ,                                     &
+   rvoid  , rvoid  ,                                                       &
+   icvflb , ivoid  ,                                                       &
    w8     )
 
-  if (iwarni(ivar).ge.2) then
+  if (vcopt_e%iwarni.ge.2) then
     rnorm = sqrt(cs_gdot(ncel,smbre,smbre))
     write(nfecra,1100) chaine(1:8) ,rnorm
   endif
@@ -1043,7 +1175,7 @@ if (ikecou.eq.1) then
       delte = (-a21*smbrk(iel) +a11*smbre(iel) )*unsdet
 
       ! New source term for the iterative process
-      romvsd = rho*volume(iel)/dt(iel)
+      romvsd = rho*cell_f_vol(iel)/dt(iel)
 
       smbrk(iel) = romvsd*deltk
       smbre(iel) = romvsd*delte
@@ -1095,15 +1227,15 @@ call field_get_coefaf_s(ivarfl(ivar), cofafp)
 call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 ! Face viscosity
-if (idiff(ivar).ge.1) then
+if (vcopt_k%idiff.ge.1) then
 
   do iel = 1, ncel
     if (iturb.eq.51) then
       w1(iel) = viscl(iel)/2.d0                                   &
-              + idifft(ivar)*cvisct(iel)/sigmak
+              + vcopt_k%idifft*cvisct(iel)/sigmak
     else
       w1(iel) = viscl(iel)                                        &
-              + idifft(ivar)*cvisct(iel)/sigmak
+              + vcopt_k%idifft*cvisct(iel)/sigmak
     endif
   enddo
 
@@ -1121,33 +1253,34 @@ else
 endif
 
 ! Solving k
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
-ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+iconvp = vcopt_k%iconv
+idiffp = vcopt_k%idiff
+ndircp = ndircl(ik)
+nswrsp = vcopt_k%nswrsm
+nswrgp = vcopt_k%nswrgr
+imligp = vcopt_k%imligr
+ircflp = vcopt_k%ircflu
+ischcp = vcopt_k%ischcv
+isstpp = vcopt_k%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
-thetap = thetav(ivar)
+idftnp = vcopt_k%idften
+iswdyp = vcopt_k%iswdyn
+iwarnp = vcopt_k%iwarni
+blencp = vcopt_k%blencv
+epsilp = vcopt_k%epsilo
+epsrsp = vcopt_k%epsrsm
+epsrgp = vcopt_k%epsrgr
+climgp = vcopt_k%climgr
+extrap = vcopt_k%extrag
+relaxp = vcopt_k%relaxv
+thetap = vcopt_k%thetav
 ! all boundary convective flux with upwind
 icvflb = 0
+init   = 1
 
 call codits &
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
    iwarnp ,                                                       &
@@ -1171,14 +1304,14 @@ call field_get_coefaf_s(ivarfl(ivar), cofafp)
 call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 ! Face viscosity
-if (idiff(ivar).ge.1) then
+if (vcopt_e%idiff.ge.1) then
   do iel = 1, ncel
     if (iturb.eq.51) then
       w1(iel) = viscl(iel)/2.d0                                   &
-              + idifft(ivar)*cvisct(iel)/cpalse
+              + vcopt_e%idifft*cvisct(iel)/cpalse
     else
       w1(iel) = viscl(iel)                                        &
-              + idifft(ivar)*cvisct(iel)/sigmae
+              + vcopt_e%idifft*cvisct(iel)/sigmae
     endif
   enddo
 
@@ -1196,33 +1329,33 @@ else
 endif
 
 ! Solving epsilon
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
+iconvp = vcopt_e%iconv
+idiffp = vcopt_e%idiff
 ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+nswrsp = vcopt_e%nswrsm
+nswrgp = vcopt_e%nswrgr
+imligp = vcopt_e%imligr
+ircflp = vcopt_e%ircflu
+ischcp = vcopt_e%ischcv
+isstpp = vcopt_e%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
-thetap = thetav(ivar)
+idftnp = vcopt_e%idften
+iswdyp = vcopt_e%iswdyn
+iwarnp = vcopt_e%iwarni
+blencp = vcopt_e%blencv
+epsilp = vcopt_e%epsilo
+epsrsp = vcopt_e%epsrsm
+epsrgp = vcopt_e%epsrgr
+climgp = vcopt_e%climgr
+extrap = vcopt_e%extrag
+relaxp = vcopt_e%relaxv
+thetap = vcopt_e%thetav
 ! all boundary convective flux with upwind
 icvflb = 0
 
 call codits &
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
    iwarnp ,                                                       &
@@ -1242,8 +1375,7 @@ call codits &
 !===============================================================================
 
 iclip = 1
-iwarnp = iwarni(ik)
-call clipke(ncelet, ncel, nvar, iclip, iwarnp)
+call clipke(ncelet, ncel, iclip)
 
 ! Free memory
 deallocate(viscf, viscb)
