@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -69,7 +69,6 @@ use cstnum
 use cstphy
 use optcal
 use lagran
-use pointe, only: s2kw, divukw, dispar
 use parall
 use mesh
 use field
@@ -104,6 +103,7 @@ integer          nswrsp, ircflp, ischcp, isstpp, iescap
 integer          iflmas, iflmab
 integer          iwarnp
 integer          istprv
+integer          init
 integer          imucpp, idftnp, iswdyp
 
 integer          icvflb, imasac
@@ -118,8 +118,10 @@ double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
 double precision thetp1, thetak, thetaw, thets, thetap, epsrsp
 double precision tuexpk, tuexpw
 double precision cdkw, xarg1, xxf1, xgamma, xbeta, sigma, produc
+double precision xlt, xdelta, xrd, xfd, xs2pw2, xdist, xdiff, fddes
 double precision var, vrmin(2), vrmax(2)
 double precision utaurf,ut2,ypa,ya,xunorm, limiter, nu0
+double precision turb_schmidt
 
 double precision rvoid(1)
 
@@ -127,6 +129,8 @@ double precision, allocatable, dimension(:) :: viscf, viscb
 double precision, allocatable, dimension(:) :: smbrk, smbrw
 double precision, allocatable, dimension(:) :: tinstk, tinstw, xf1
 double precision, allocatable, dimension(:,:) :: gradk, grado, grad
+double precision, allocatable, dimension(:,:,:) :: gradv
+double precision, allocatable, dimension(:) :: s2pw2
 double precision, allocatable, dimension(:) :: w1, w2
 double precision, allocatable, dimension(:) :: gdkgdw
 double precision, allocatable, dimension(:) :: w5, w6
@@ -147,6 +151,11 @@ double precision, dimension(:), pointer :: cpro_pcvto, cpro_pcvlo
 double precision, dimension(:), pointer :: viscl, cvisct
 double precision, dimension(:), pointer :: c_st_k_p, c_st_omg_p
 double precision, dimension(:,:), pointer :: vel
+double precision, dimension(:), pointer :: cpro_divukw, cpro_s2kw
+double precision, dimension(:), pointer :: ddes_fd_coeff
+double precision, dimension(:), pointer :: w_dist
+
+type(var_cal_opt) :: vcopt_w, vcopt_k
 
 !===============================================================================
 
@@ -164,12 +173,16 @@ allocate(w1(ncelet), w2(ncelet))
 allocate(dpvar(ncelet))
 allocate(gdkgdw(ncelet))
 allocate(prodk(ncelet), prodw(ncelet))
+if (iddes.eq.1) then
+  allocate(gradv(3, 3, ncelet))
+  allocate(s2pw2(ncelet))
+endif
 
 epz2 = epzero**2
 iclpkmx(1) = 0
 
-call field_get_val_s(iprpfl(ivisct), cvisct)
-call field_get_val_s(iprpfl(iviscl), viscl)
+call field_get_val_s(ivisct, cvisct)
+call field_get_val_s(iviscl, viscl)
 
 call field_get_key_int(ivarfl(ik), kimasf, iflmas)
 call field_get_key_int(ivarfl(ik), kbmasf, iflmab)
@@ -181,8 +194,8 @@ call field_get_val_s(ibrom, brom)
 call field_get_val_s(icrom, cromo)
 call field_get_val_s(ibrom, bromo)
 
-call field_get_val_s(iprpfl(ivisct), cpro_pcvto)
-call field_get_val_s(iprpfl(iviscl), cpro_pcvlo)
+call field_get_val_s(ivisct, cpro_pcvto)
+call field_get_val_s(iviscl, cpro_pcvlo)
 
 call field_get_val_s(ivarfl(ik), cvar_k)
 call field_get_val_prev_s(ivarfl(ik), cvara_k)
@@ -199,8 +212,14 @@ call field_get_coefb_s(ivarfl(iomg), coefb_o)
 call field_get_coefaf_s(ivarfl(iomg), coefaf_o)
 call field_get_coefbf_s(ivarfl(iomg), coefbf_o)
 
+call field_get_key_struct_var_cal_opt(ivarfl(iomg), vcopt_w)
+call field_get_key_struct_var_cal_opt(ivarfl(ik), vcopt_k)
+
 thets  = thetst
 
+call field_get_val_s(is2kw, cpro_s2kw)
+call field_get_val_s(idivukw, cpro_divukw)
+
 call field_get_key_int(ivarfl(ik), kstprv, istprv)
 if (istprv.ge.0) then
   call field_get_val_s(istprv, c_st_k_p)
@@ -217,12 +236,15 @@ if (istprv.ge.0) then
     call field_get_val_prev_s(ibrom, bromo)
   endif
   if (iviext.gt.0) then
-    call field_get_val_s(iprpfl(ivista), cpro_pcvto)
-    call field_get_val_s(iprpfl(ivisla), cpro_pcvlo)
+    call field_get_val_prev_s(ivisct, cpro_pcvto)
+    call field_get_val_prev_s(iviscl, cpro_pcvlo)
   endif
 endif
 
-if (iwarni(ik).ge.1) then
+call field_get_id("wall_distance", f_id)
+call field_get_val_s(f_id, w_dist)
+
+if (vcopt_k%iwarni.ge.1) then
   write(nfecra,1000)
 endif
 
@@ -248,6 +270,18 @@ call field_gradient_scalar(ivarfl(iomg), iprev, imrgra, inc,        &
                            iccocg,                                  &
                            grado)
 
+if (iddes.eq.1) then
+  call field_get_val_s_by_name("hybrid_blend", ddes_fd_coeff)
+
+  call field_gradient_vector(ivarfl(iu), iprev, imrgra, inc,        &
+                             gradv)
+  do iel = 1, ncel
+    s2pw2(iel) = gradv(1,1,iel)**2.d0 + gradv(2,1,iel)**2.d0 + gradv(3,1,iel)**2.d0 &
+               + gradv(1,2,iel)**2.d0 + gradv(2,2,iel)**2.d0 + gradv(3,2,iel)**2.d0 &
+               + gradv(1,3,iel)**2.d0 + gradv(2,3,iel)**2.d0 + gradv(3,3,iel)**2.d0
+  enddo
+end if
+
 do iel = 1, ncel
   gdkgdw(iel) = gradk(1,iel)*grado(1,iel) &
               + gradk(2,iel)*grado(2,iel) &
@@ -262,7 +296,7 @@ deallocate(gradk, grado)
 !===============================================================================
 
 do iel = 1, ncel
-  w2(iel) = max(dispar(iel),epzero)
+  w2(iel) = max(w_dist(iel),epzero)
 enddo
 
 ! En cas d'ordre 2 on utilise les valeurs en n car le terme en (1-f1)*gdkgdw
@@ -289,8 +323,8 @@ enddo
 do iel = 1, ncel
   rho = crom(iel)
   romvsd = rho*volume(iel)/dt(iel)
-  tinstk(iel) = istat(ik)*romvsd
-  tinstw(iel) = istat(iomg)*romvsd
+  tinstk(iel) = vcopt_k%istat*romvsd
+  tinstw(iel) = vcopt_w%istat*romvsd
 enddo
 
 !===============================================================================
@@ -305,22 +339,22 @@ do iel = 1, ncel
   xeps = cmu*xw*xk
   visct = cpro_pcvto(iel)
   rho = cromo(iel)
-  prodw(iel) = visct*s2kw(iel)                    &
-             - d2s3*rho*xk*divukw(iel)
+  prodw(iel) = visct*cpro_s2kw(iel)                    &
+             - d2s3*rho*xk*cpro_divukw(iel)
 
   ! The negative part is implicited
   xxf1   = xf1(iel)
   xgamma = xxf1*ckwgm1 + (1.d0-xxf1)*ckwgm2
   tinstw(iel) = tinstw(iel)                                         &
               + max( d2s3*rho*volume(iel)                           &
-                   *(rho*xgamma*xk/(visct*xw))*divukw(iel), 0.d0)
+                   *(rho*xgamma*xk/(visct*xw))*cpro_divukw(iel), 0.d0)
 
   ! Take the min between prodw and the low Reynold one
   if (prodw(iel).gt.ckwc1*rho*xeps) then
     prodk(iel) = ckwc1*rho*xeps
   else
     prodk(iel) = prodw(iel)
-    tinstk(iel) = tinstk(iel) + max(d2s3*volume(iel)*rho*divukw(iel), 0.d0)
+    tinstk(iel) = tinstk(iel) + max(d2s3*volume(iel)*rho*cpro_divukw(iel), 0.d0)
   endif
 enddo
 
@@ -365,12 +399,12 @@ if (igrake.eq.1) then
   iccocg = 1
   inc = 1
 
-  nswrgp = nswrgr(ik)
-  epsrgp = epsrgr(ik)
-  imligp = imligr(ik)
-  iwarnp = iwarni(ik)
-  climgp = climgr(ik)
-  extrap = extrag(ik)
+  nswrgp = vcopt_k%nswrgr
+  epsrgp = vcopt_k%epsrgr
+  imligp = vcopt_k%imligr
+  iwarnp = vcopt_k%iwarni
+  climgp = vcopt_k%climgr
+  extrap = vcopt_k%extrag
 
   ! BCs on rho: Dirichlet ROMB
   !  NB: viscb is used as COEFB
@@ -392,7 +426,8 @@ if (igrake.eq.1) then
   !   prodk=min(P,c1*eps)+G
   !   prodw=P+(1-ce3)*G
   if (iscalt.gt.0.and.nscal.ge.iscalt) then
-    prdtur = sigmas(iscalt)
+    call field_get_key_double(ivarfl(isca(iscalt)), ksigmas, turb_schmidt)
+    prdtur = turb_schmidt
   else
     prdtur = 1.d0
   endif
@@ -449,8 +484,8 @@ call cs_user_turbulence_source_terms &
 ! If source terms are extrapolated over time
 if (istprv.ge.0) then
 
-  thetak = thetav(ik)
-  thetaw = thetav(iomg)
+  thetak = vcopt_k%thetav
+  thetaw = vcopt_w%thetav
 
   do iel = 1, ncel
 
@@ -494,31 +529,78 @@ endif
 !      Les termes sont stockes dans     smbrk, smbrw
 !      En sortie de l'etape on conserve smbrk,smbrw,gdkgdw
 !===============================================================================
+! Standard k-w SST RANS model
+if(iddes.ne.1) then
+  do iel = 1, ncel
 
-do iel = 1, ncel
+    visct  = cpro_pcvto(iel)
+    rho    = cromo(iel)
+    xk     = cvara_k(iel)
+    xw     = cvara_omg(iel)
+    xxf1   = xf1(iel)
+    xgamma = xxf1*ckwgm1 + (1.d0-xxf1)*ckwgm2
+    xbeta  = xxf1*ckwbt1 + (1.d0-xxf1)*ckwbt2
 
-  visct  = cpro_pcvto(iel)
-  rho    = cromo(iel)
-  xk     = cvara_k(iel)
-  xw     = cvara_omg(iel)
-  xxf1   = xf1(iel)
-  xgamma = xxf1*ckwgm1 + (1.d0-xxf1)*ckwgm2
-  xbeta  = xxf1*ckwbt1 + (1.d0-xxf1)*ckwbt2
+    smbrk(iel) = smbrk(iel) + volume(iel)*(                         &
+                                            prodk(iel)              &
+                                          - cmu*rho*xw*xk )
 
-  smbrk(iel) = smbrk(iel) + volume(iel)*(                         &
-                                          prodk(iel)              &
-                                        - cmu*rho*xw*xk )
+    smbrw(iel) = smbrw(iel)                                                &
+               + volume(iel)*(                                             &
+                               rho*xgamma/visct*prodw(iel)                 &
+                             - xbeta*rho*xw**2                             &
+                             + 2.d0*rho/xw*(1.d0-xxf1)/ckwsw2*gdkgdw(iel)  &
+                             )
 
-  smbrw(iel) = smbrw(iel)                                                &
-             + volume(iel)*(                                             &
-                             rho*xgamma/visct*prodw(iel)                 &
-                           - xbeta*rho*xw**2                             &
-                           + 2.d0*rho/xw*(1.d0-xxf1)/ckwsw2*gdkgdw(iel)  &
-                           )
+    tinstw(iel) = tinstw(iel) + volume(iel)*max(-2.d0*rho/xw**2*(1.d0-xxf1) &
+                                                /ckwsw2*gdkgdw(iel), 0.d0)
+  enddo
+! DDES mode for k-w SST
+else
+  do iel = 1, ncel
 
-  tinstw(iel) = tinstw(iel) + volume(iel)*max(-2.d0*rho/xw**2*(1.d0-xxf1) &
-                                              /ckwsw2*gdkgdw(iel), 0.d0)
-enddo
+    visct  = cpro_pcvto(iel)
+    xnu    = cpro_pcvlo(iel)
+    rho    = cromo(iel)
+    xk     = cvara_k(iel)
+    xw     = cvara_omg(iel)
+    xxf1   = xf1(iel)
+    xgamma = xxf1*ckwgm1 + (1.d0-xxf1)*ckwgm2
+    xbeta  = xxf1*ckwbt1 + (1.d0-xxf1)*ckwbt2
+    xs2pw2 = max(sqrt(s2pw2(iel)),epzero)
+    xdist  = max(w_dist(iel),epzero)
+
+    xlt    = sqrt(xk)/(cmu*xw)
+    xdelta = volume(iel)**(1.d0/3.d0)
+    xrd    = (visct + xnu)/ ( xs2pw2 *(xkappa**2) *(xdist**2))
+    xfd    = 1.d0 -tanh((8.d0*xrd)**3)
+    xdiff  = max((xlt-(cddes*xdelta)),0.d0)
+    fddes  = xlt/(xlt-xfd*xdiff)
+
+    ddes_fd_coeff(iel) = xfd
+
+    ! Storage of the Fd coefficient and check locally if
+    ! DDES is activated for post-propcessing
+    ! NB: for RANS zones (L_RANS < L_LES) Fd is clipped to 0
+    if ((cddes*xdelta).ge.xlt)then
+      ddes_fd_coeff(iel) = 0.d0
+    endif
+
+    smbrk(iel) = smbrk(iel) + volume(iel)*(                         &
+                                            prodk(iel)              &
+                                          - cmu*rho*xw*xk*fddes)
+
+    smbrw(iel) = smbrw(iel)                                                &
+               + volume(iel)*(                                             &
+                               rho*xgamma/visct*prodw(iel)                 &
+                             - xbeta*rho*xw**2                             &
+                             + 2.d0*rho/xw*(1.d0-xxf1)/ckwsw2*gdkgdw(iel)  &
+                             )
+
+    tinstw(iel) = tinstw(iel) + volume(iel)*max(-2.d0*rho/xw**2*(1.d0-xxf1) &
+                                                /ckwsw2*gdkgdw(iel), 0.d0)
+  enddo
+endif
 
 ! If the solving of k-omega is uncoupled, negative source terms are implicited
 if (ikecou.eq.0) then
@@ -683,13 +765,13 @@ if (ikecou.eq.1) then
 
   call field_get_label(ivarfl(ivar), chaine)
 
-  if (idiff(ivar).ge. 1) then
+  if (vcopt_k%idiff.ge. 1) then
 
     do iel = 1, ncel
       xxf1 = xf1(iel)
       sigma = xxf1*ckwsk1 + (1.d0-xxf1)*ckwsk2
       w7(iel) = viscl(iel)                                        &
-              + idifft(ivar)*cvisct(iel)/sigma
+              + vcopt_k%idifft*cvisct(iel)/sigma
     enddo
     call viscfa &
  ( imvisf ,                                                       &
@@ -710,39 +792,39 @@ if (ikecou.eq.1) then
   iccocg = 1
   inc    = 1
   imucpp = 0
-  idftnp = 1 ! no tensorial diffusivity
+  idftnp = ISOTROPIC_DIFFUSION
   imasac = 1
-  iconvp = iconv (ivar)
-  idiffp = idiff (ivar)
-  nswrgp = nswrgr(ivar)
-  imligp = imligr(ivar)
-  ircflp = ircflu(ivar)
-  ischcp = ischcv(ivar)
-  isstpp = isstpc(ivar)
-  iwarnp = iwarni(ivar)
-  blencp = blencv(ivar)
-  epsrgp = epsrgr(ivar)
-  climgp = climgr(ivar)
-  extrap = extrag(ivar)
-  relaxp = relaxv(ivar)
-  thetap = thetav(ivar)
+  iconvp = vcopt_k%iconv
+  idiffp = vcopt_k%idiff
+  nswrgp = vcopt_k%nswrgr
+  imligp = vcopt_k%imligr
+  ircflp = vcopt_k%ircflu
+  ischcp = vcopt_k%ischcv
+  isstpp = vcopt_k%isstpc
+  iwarnp = vcopt_k%iwarni
+  blencp = vcopt_k%blencv
+  epsrgp = vcopt_k%epsrgr
+  climgp = vcopt_k%climgr
+  extrap = vcopt_k%extrag
+  relaxp = vcopt_k%relaxv
+  thetap = vcopt_k%thetav
   ! all boundary convective flux with upwind
   icvflb = 0
 
   call bilsca &
- ( idtvar , ivar   , iconvp , idiffp , nswrgp , imligp , ircflp , &
-   ischcp , isstpp , inc    , imrgra , iccocg ,                   &
-   iwarnp , imucpp , idftnp , imasac ,                            &
-   blencp , epsrgp , climgp , extrap , relaxp , thetap ,          &
-   cvara_k         , cvara_k         ,                            &
-   coefa_k , coefb_k , coefaf_k , coefbf_k ,                      &
-   imasfl , bmasfl ,                                              &
-   viscf  , viscb  , rvoid  , rvoid  ,                            &
-   rvoid  , rvoid  ,                                              &
-   icvflb , ivoid  ,                                              &
+ ( idtvar , ivarfl(ivar)    , iconvp , idiffp , nswrgp , imligp , ircflp , &
+   ischcp , isstpp , inc    , imrgra , iccocg ,                            &
+   iwarnp , imucpp , idftnp , imasac ,                                     &
+   blencp , epsrgp , climgp , extrap , relaxp , thetap ,                   &
+   cvara_k         , cvara_k         ,                                     &
+   coefa_k , coefb_k , coefaf_k , coefbf_k ,                               &
+   imasfl , bmasfl ,                                                       &
+   viscf  , viscb  , rvoid  , rvoid  ,                                     &
+   rvoid  , rvoid  ,                                                       &
+   icvflb , ivoid  ,                                                       &
    w5     )
 
-  if (iwarni(ivar).ge.2) then
+  if (vcopt_k%iwarni.ge.2) then
     rnorm = sqrt(cs_gdot(ncel,smbrk,smbrk))
     write(nfecra,1100) chaine(1:8) ,rnorm
   endif
@@ -752,12 +834,12 @@ if (ikecou.eq.1) then
 
   call field_get_label(ivarfl(ivar), chaine)
 
-  if (idiff(ivar).ge. 1) then
+  if (vcopt_w%idiff.ge. 1) then
     do iel = 1, ncel
       xxf1 = xf1(iel)
       sigma = xxf1*ckwsw1 + (1.d0-xxf1)*ckwsw2
       w7(iel) = viscl(iel)                                        &
-              + idifft(ivar)*cvisct(iel)/sigma
+              + vcopt_w%idifft*cvisct(iel)/sigma
     enddo
     call viscfa &
  ( imvisf ,                                                       &
@@ -778,39 +860,39 @@ if (ikecou.eq.1) then
   iccocg = 1
   inc    = 1
   imucpp = 0
-  idftnp = 1 ! no tensorial diffusivity
-  iconvp = iconv (ivar)
+  idftnp = ISOTROPIC_DIFFUSION
+  iconvp = vcopt_w%iconv
   imasac = 1
-  idiffp = idiff (ivar)
-  nswrgp = nswrgr(ivar)
-  imligp = imligr(ivar)
-  ircflp = ircflu(ivar)
-  ischcp = ischcv(ivar)
-  isstpp = isstpc(ivar)
-  iwarnp = iwarni(ivar)
-  blencp = blencv(ivar)
-  epsrgp = epsrgr(ivar)
-  climgp = climgr(ivar)
-  extrap = extrag(ivar)
-  relaxp = relaxv(ivar)
-  thetap = thetav(ivar)
+  idiffp = vcopt_w%idiff
+  nswrgp = vcopt_w%nswrgr
+  imligp = vcopt_w%imligr
+  ircflp = vcopt_w%ircflu
+  ischcp = vcopt_w%ischcv
+  isstpp = vcopt_w%isstpc
+  iwarnp = vcopt_w%iwarni
+  blencp = vcopt_w%blencv
+  epsrgp = vcopt_w%epsrgr
+  climgp = vcopt_w%climgr
+  extrap = vcopt_w%extrag
+  relaxp = vcopt_w%relaxv
+  thetap = vcopt_w%thetav
   ! all boundary convective flux with upwind
   icvflb = 0
 
   call bilsca &
- ( idtvar , ivar   , iconvp , idiffp , nswrgp , imligp , ircflp , &
-   ischcp , isstpp , inc    , imrgra , iccocg ,                   &
-   iwarnp , imucpp , idftnp , imasac ,                            &
-   blencp , epsrgp , climgp , extrap , relaxp , thetap ,          &
-   cvara_omg       , cvara_omg       ,                            &
-   coefa_o , coefb_o , coefaf_o , coefbf_o ,                      &
-   imasfl , bmasfl ,                                              &
-   viscf  , viscb  , rvoid  , rvoid  ,                            &
-   rvoid  , rvoid  ,                                              &
-   icvflb , ivoid  ,                                              &
+ ( idtvar , ivarfl(ivar)    , iconvp , idiffp , nswrgp , imligp , ircflp , &
+   ischcp , isstpp , inc    , imrgra , iccocg ,                            &
+   iwarnp , imucpp , idftnp , imasac ,                                     &
+   blencp , epsrgp , climgp , extrap , relaxp , thetap ,                   &
+   cvara_omg       , cvara_omg       ,                                     &
+   coefa_o , coefb_o , coefaf_o , coefbf_o ,                               &
+   imasfl , bmasfl ,                                                       &
+   viscf  , viscb  , rvoid  , rvoid  ,                                     &
+   rvoid  , rvoid  ,                                                       &
+   icvflb , ivoid  ,                                                       &
    w6     )
 
-  if (iwarni(ivar).ge.2) then
+  if (vcopt_w%iwarni.ge.2) then
     rnorm = sqrt(cs_gdot(ncel,smbrw,smbrw))
     write(nfecra,1100) chaine(1:8) ,rnorm
   endif
@@ -851,8 +933,8 @@ if (ikecou.eq.1) then
     romvsd     = 1.d0/(rho*volume(iel))
     smbrk(iel) = smbrk(iel)*romvsd
     smbrw(iel) = smbrw(iel)*romvsd
-    divp23     = d2s3*max(divukw(iel),zero)
-    produc     = w1(iel)*s2kw(iel)+w2(iel)
+    divp23     = d2s3*max(cpro_divukw(iel),zero)
+    produc     = w1(iel)*cpro_s2kw(iel)+w2(iel)
     xk         = cvara_k(iel)
     xw         = cvara_omg(iel)
     xxf1       = xf1(iel)
@@ -901,13 +983,13 @@ endif
 ivar = ik
 
 ! Face viscosity
-if (idiff(ivar).ge. 1) then
+if (vcopt_k%idiff.ge. 1) then
 
   do iel = 1, ncel
     xxf1 = xf1(iel)
     sigma = xxf1*ckwsk1 + (1.d0-xxf1)*ckwsk2
     w1(iel) = viscl(iel)                                          &
-             + idifft(ivar)*cvisct(iel)/sigma
+             + vcopt_k%idifft*cvisct(iel)/sigma
   enddo
   call viscfa &
  ( imvisf ,                                                       &
@@ -926,33 +1008,34 @@ else
 endif
 
 ! Solving k
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
+iconvp = vcopt_k%iconv
+idiffp = vcopt_k%idiff
 ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+nswrsp = vcopt_k%nswrsm
+nswrgp = vcopt_k%nswrgr
+imligp = vcopt_k%imligr
+ircflp = vcopt_k%ircflu
+ischcp = vcopt_k%ischcv
+isstpp = vcopt_k%isstpc
 iescap = 0
 imucpp = 0
-idftnp = 1 ! no tensorial diffusivity
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
-thetap = thetav(ivar)
+idftnp = ISOTROPIC_DIFFUSION
+iswdyp = vcopt_k%iswdyn
+iwarnp = vcopt_k%iwarni
+blencp = vcopt_k%blencv
+epsilp = vcopt_k%epsilo
+epsrsp = vcopt_k%epsrsm
+epsrgp = vcopt_k%epsrgr
+climgp = vcopt_k%climgr
+extrap = vcopt_k%extrag
+relaxp = vcopt_k%relaxv
+thetap = vcopt_k%thetav
 ! all boundary convective flux with upwind
 icvflb = 0
+init   = 1
 
 call codits &
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
    iwarnp ,                                                       &
@@ -971,12 +1054,12 @@ call codits &
 ivar = iomg
 
 ! Face viscosity
-if (idiff(ivar).ge. 1) then
+if (vcopt_w%idiff.ge. 1) then
   do iel = 1, ncel
     xxf1 = xf1(iel)
     sigma = xxf1*ckwsw1 + (1.d0-xxf1)*ckwsw2
     w1(iel) = viscl(iel)                                          &
-                        + idifft(ivar)*cvisct(iel)/sigma
+                        + vcopt_w%idifft*cvisct(iel)/sigma
   enddo
   call viscfa &
  ( imvisf ,                                                       &
@@ -995,33 +1078,34 @@ else
 endif
 
 ! Solving omega
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
+iconvp = vcopt_w%iconv
+idiffp = vcopt_w%idiff
 ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+nswrsp = vcopt_w%nswrsm
+nswrgp = vcopt_w%nswrgr
+imligp = vcopt_w%imligr
+ircflp = vcopt_w%ircflu
+ischcp = vcopt_w%ischcv
+isstpp = vcopt_w%isstpc
 iescap = 0
 imucpp = 0
-idftnp = 1 ! no tensorial diffusivity
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
-thetap = thetav(ivar)
+idftnp = ISOTROPIC_DIFFUSION
+iswdyp = vcopt_w%iswdyn
+iwarnp = vcopt_w%iwarni
+blencp = vcopt_w%blencv
+epsilp = vcopt_w%epsilo
+epsrsp = vcopt_w%epsrsm
+epsrgp = vcopt_w%epsrgr
+climgp = vcopt_w%climgr
+extrap = vcopt_w%extrag
+relaxp = vcopt_w%relaxv
+thetap = vcopt_w%thetav
 ! all boundary convective flux with upwind
 icvflb = 0
+init   = 1
 
 call codits &
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
    iwarnp ,                                                       &
@@ -1085,24 +1169,26 @@ call log_iteration_clipping_field(ivarfl(ik), iclipk(1), 0,    &
                                   vrmin(1:1), vrmax(1:1),iclipk(1), iclpkmx(1))
 call log_iteration_clipping_field(ivarfl(iomg), iclipw, 0,  &
                                   vrmin(2:2), vrmax(2:2),iclipk(1), iclpkmx(1))
+
 !===============================================================================
 ! 16. Advanced reinit
 !===============================================================================
 
 ! Automatic reinitialization at the end of the first iteration:
-! wall distance y^+ is computed with -C log(1-alpha), where C=CL*Ceta*L*kappa, then y
-! so we have an idea of the wall distance in complexe geometries.
-! Then U is initialized with a Reichard lay
+! wall distance y^+ is computed with -C log(1-alpha), where C=CL*Ceta*L*kappa,
+! then y so we have an idea of the wall distance in complex geometries.
+! Then U is initialized with a Reichard layer,
 ! Epsilon by 1/(kappa y), clipped next to the wall at its value for y^+=15
 ! k is given by a blending between eps/(2 nu)*y^2 and utau/sqrt(Cmu)
 ! The blending function is chosen so that the asymptotic behavior
-! and give the correct pic of k (not the same blending than for the EBRSM
-! because k profile is not the same for k-omega)
-! For omega, far from the wall we take eps/Cmu/k, but next to the wall, omega solution is enforced
+! and give the correct peak of k (not the same blending than for the EBRSM
+! because k profile is not the same for k-omega).
+! For omega, far from the wall we take eps/Cmu/k, but next to the wall,
+! omega solution is enforced.
+
 !TODO FIXME: why not just before? Are the BC uncompatible?
 if (ntcabs.eq.1.and.reinit_turb.eq.1) then
-
-  call field_get_val_prev_v(ivarfl(iu), vel)
+  call field_get_val_v(ivarfl(iu), vel)
 
   utaurf = 0.05d0*uref
   nu0 = viscl0 / ro0
@@ -1112,7 +1198,7 @@ if (ntcabs.eq.1.and.reinit_turb.eq.1) then
     xunorm = vel(1,iel)**2 + vel(2,iel)**2 + vel(3,iel)**2
     xunorm = sqrt(xunorm)
 
-    ya = dispar(iel)
+    ya = w_dist(iel)
     ypa = ya*utaurf/nu0
     ! Velocity magnitude is imposed (limitted only), the direction is
     ! conserved
@@ -1139,9 +1225,7 @@ if (ntcabs.eq.1.and.reinit_turb.eq.1) then
     cvar_omg(iel) = ut2**3/(xkappa*15.d0*nu0/ut2)/(ut2**2/sqrt(cmu))/cmu
 
   enddo
-end if
-
-
+endif
 
 ! Free memory
 deallocate(viscf, viscb)
@@ -1150,6 +1234,8 @@ deallocate(tinstk, tinstw, xf1)
 deallocate(w1, w2, usimpk, usimpw)
 deallocate(dpvar)
 deallocate(prodk, prodw)
+if (allocated(gradv)) deallocate(gradv)
+if (allocated(s2pw2)) deallocate(s2pw2)
 
 if (allocated(rotfct))  deallocate(rotfct)
 
