@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -28,6 +28,11 @@
 !>
 !> \brief Solving the \f$ R_{ij} - \epsilon \f$ for incompressible flows or
 !>  slightly compressible flows for one time step.
+!>
+!> Please refer to the
+!> <a href="../../theory.pdf#rijeps"><b>\f$ R_{ij} - \epsilon \f$ model</b></a>
+!> section of the theory guide for more informations, as well as the
+!> <a href="../../theory.pdf#turrij"><b>turrij</b></a> section.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -41,7 +46,8 @@
 !> \param[in]     ncesmp        number of cells with mass source term
 !> \param[in]     icepdc        index of the ncepdp cells with head loss
 !> \param[in]     icetsm        index of cells with mass source term
-!> \param[in]     itypsm        mass source type for the variables (cf. cs_user_mass_source_terms)
+!> \param[in]     itypsm        mass source type for the variables
+!>                              (cf. cs_user_mass_source_terms)
 !> \param[in]     dt            time step (per cell)
 !> \param[in]     tslagr        coupling term of the lagangian module
 !> \param[in]     ckupdc        work array for the head loss
@@ -64,13 +70,12 @@ subroutine turrij &
 !===============================================================================
 
 use paramx
-use lagran, only: ntersl
+use lagran
 use numvar
 use entsor
 use cstphy
 use cstnum
 use optcal
-use lagran
 use pointe, only: rvoid1
 use ppincl
 use mesh
@@ -106,7 +111,6 @@ integer          inc   , iccocg
 integer          iwarnp, iclip
 integer          nswrgp, imligp
 integer          f_id0
-integer          iitsla
 integer          iprev
 double precision epsrgp, climgp, extrap
 double precision rhothe
@@ -122,7 +126,7 @@ double precision, allocatable, dimension(:,:) :: grad
 double precision, allocatable, dimension(:,:) :: smbrts
 double precision, allocatable, dimension(:,:,:) ::rovsdtts
 
-double precision, pointer, dimension(:) :: tslage, tslagi
+double precision, pointer, dimension(:) :: tslagi
 double precision, dimension(:,:), pointer :: coefau
 double precision, dimension(:,:,:), pointer :: coefbu
 double precision, dimension(:), pointer :: brom, crom
@@ -133,6 +137,9 @@ double precision, dimension(:), pointer :: cvara_r12, cvara_r13, cvara_r23
 double precision, dimension(:), pointer :: cvara_scalt
 double precision, dimension(:), pointer :: cvar_ep, cvar_al
 double precision, dimension(:,:), pointer :: cvara_rij, cvar_rij, vel
+double precision, dimension(:,:), pointer :: lagr_st_rij
+
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 
@@ -140,8 +147,7 @@ double precision, dimension(:,:), pointer :: cvara_rij, cvar_rij, vel
 ! 1. Initialization
 !===============================================================================
 
-tslage => rvoid1
-tslagi => rvoid1
+tslagi  => rvoid1
 
 call field_get_coefa_v(ivarfl(iu), coefau)
 call field_get_coefb_v(ivarfl(iu), coefbu)
@@ -176,7 +182,9 @@ if (iturb.eq.30) then
   endif
 endif
 
-if(iwarni(iep).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(iep), vcopt)
+
+if(vcopt%iwarni.ge.1) then
   if (iturb.eq.30) then
     write(nfecra,1000)
   elseif (iturb.eq.31) then
@@ -189,14 +197,15 @@ endif
 !===============================================================================
 ! 1.1 Advanced init for EBRSM
 !===============================================================================
+
 ! Automatic reinitialization at the end of the first iteration:
-! wall distance y^+ is computed with -C log(1-alpha), where C=CL*Ceta*L*kappa, then y
-! so we have an idea of the wall distance in complexe geometries.
-! Then U is initialized with a Reichard lay
+! wall distance y^+ is computed with -C log(1-alpha), where C=CL*Ceta*L*kappa,
+! then y so we have an idea of the wall distance in complex geometries.
+! Then U is initialized with a Reichard layer,
 ! Epsilon by 1/(kappa y), clipped next to the wall at its value for y^+=15
-! k is given by a blending between eps/(2 nu)*y^2 and utau/sqrt(Cmu)
+! k is given by a blending between eps/(2 nu)*y^2 and utau/sqrt(Cmu).
 ! The blending function is chosen so that the asymptotic behavior
-! and give the correct pic of k
+! and give the correct peak of k.
 
 !TODO FIXME Are the BC uncompatible?
 if (ntcabs.eq.1.and.reinit_turb.eq.1.and.iturb.eq.32) then
@@ -234,8 +243,7 @@ if (ntcabs.eq.1.and.reinit_turb.eq.1.and.iturb.eq.32) then
 
   call field_get_val_s(ivarfl(iep), cvar_ep)
   call field_get_val_s(ivarfl(ial), cvar_al)
-  call field_get_val_prev_v(ivarfl(iu), vel)
-
+  call field_get_val_v(ivarfl(iu), vel)
 
   nu0 = viscl0/ro0
 
@@ -244,15 +252,15 @@ if (ntcabs.eq.1.and.reinit_turb.eq.1.and.iturb.eq.32) then
     xunorm = vel(1,iel)**2 + vel(2,iel)**2 + vel(3,iel)**2
     xunorm = sqrt(xunorm)
 
-    ! y+ is bounded by 400, because in the Reichard profile, it corresponds to saturation (u>uref)
-    cvar_al(iel) = max(min(cvar_al(iel),(1.d0-exp(-400.d0/50.d0))) &
-    ,0.d0)
+    ! y+ is bounded by 400, because in the Reichard profile,
+    ! it corresponds to saturation (u>uref)
+    cvar_al(iel) = max(min(cvar_al(iel),(1.d0-exp(-400.d0/50.d0))),0.d0)
     ! Compute the magnitude of the Alpha gradient
     xnoral = ( grad(1,iel)*grad(1,iel)          &
            +   grad(2,iel)*grad(2,iel)          &
            +   grad(3,iel)*grad(3,iel) )
     xnoral = sqrt(xnoral)
-   ! Compute the unitary vector of Alpha !FIXME not homogeneous
+   ! Compute the unitary vector of Alpha
     if (xnoral.le.epzero/cell_f_vol(iel)**(1.d0/3.d0)) then
       xnal(1) = 1.d0/sqrt(3.d0)
       xnal(2) = 1.d0/sqrt(3.d0)
@@ -265,7 +273,6 @@ if (ntcabs.eq.1.and.reinit_turb.eq.1.and.iturb.eq.32) then
 
     alpha = cvar_al(iel)
 
-
     ! Compute YA, therefore alpha is given by 1-exp(-YA/(50 nu/utau))
     ! NB: y^+ = 50 give the best compromise
     ya = -dlog(1.d0-cvar_al(iel))*50.d0*nu0/utaurf
@@ -275,10 +282,10 @@ if (ntcabs.eq.1.and.reinit_turb.eq.1.and.iturb.eq.32) then
     if (xunorm.le.1.d-12*uref) then
       limiter = 1.d0
     else
-      limiter = min(utaurf/xunorm*(2.5d0*dlog(1.d0+0.4d0*ypa)            &
-      +7.8d0*(1.d0-dexp(-ypa/11.d0)          &
-      -(ypa/11.d0)*dexp(-0.33d0*ypa))),      &
-      1.d0)
+      limiter = min( utaurf/xunorm*(2.5d0*dlog(1.d0+0.4d0*ypa) &
+                    +7.8d0*(1.d0-dexp(-ypa/11.d0)              &
+                    -(ypa/11.d0)*dexp(-0.33d0*ypa))),          &
+                    1.d0)
     endif
 
     vel(1,iel) = limiter*vel(1,iel)
@@ -287,10 +294,10 @@ if (ntcabs.eq.1.and.reinit_turb.eq.1.and.iturb.eq.32) then
 
     ut2 = 0.05d0*uref
     cvar_ep(iel) = utaurf**3*min(1.d0/(xkappa*15.d0*nu0/utaurf), &
-    1.d0/(xkappa*ya))
-    tke = cvar_ep(iel)/2.d0/nu0*ya**2             &
-    * exp(-ypa/25.d0)**2                         &
-    + ut2**2/0.3d0*(1.d0-exp(-ypa/25.d0))**2
+                                 1.d0/(xkappa*ya))
+    tke =  cvar_ep(iel)/2.d0/nu0*ya**2                  &
+           * exp(-ypa/25.d0)**2                         &
+          + ut2**2/0.3d0*(1.d0-exp(-ypa/25.d0))**2
 
     if (irijco.eq.0) then
       cvar_r11(iel) = alpha**3       *2.d0/3.d0        *tke &
@@ -315,26 +322,20 @@ if (ntcabs.eq.1.and.reinit_turb.eq.1.and.iturb.eq.32) then
     end if
   enddo
 
-  do iel = 1, ncel
-    if (irijco.eq.0) then
-      cvara_r11(iel) = cvar_r11(iel)
-      cvara_r22(iel) = cvar_r22(iel)
-      cvara_r33(iel) = cvar_r33(iel)
-      cvara_r12(iel) = cvar_r12(iel)
-      cvara_r23(iel) = cvar_r23(iel)
-      cvara_r13(iel) = cvar_r13(iel)
-    else
-      cvara_rij(1,iel) = cvar_rij(1,iel)
-      cvara_rij(2,iel) = cvar_rij(2,iel)
-      cvara_rij(3,iel) = cvar_rij(3,iel)
-      cvara_rij(4,iel) = cvar_rij(4,iel)
-      cvara_rij(5,iel) = cvar_rij(5,iel)
-      cvara_rij(6,iel) = cvar_rij(6,iel)
-    end if
-  enddo
+  call field_current_to_previous(ivarfl(iu))
+  if (irijco.eq.0) then
+    call field_current_to_previous(ivarfl(ir11))
+    call field_current_to_previous(ivarfl(ir22))
+    call field_current_to_previous(ivarfl(ir33))
+    call field_current_to_previous(ivarfl(ir12))
+    call field_current_to_previous(ivarfl(ir23))
+    call field_current_to_previous(ivarfl(ir13))
+  else
+    call field_current_to_previous(ivarfl(irij))
+  endif
 
   deallocate(grad)
-end if
+endif
 
 
 !===============================================================================
@@ -497,12 +498,13 @@ if (igrari.eq.1 .and. ippmod(iatmos).ge.1) then
                              iccocg,                               &
                              gradro)
 
-  ! gradro stores: rho grad(theta)/theta
+  ! gradro stores: - rho grad(theta)/theta
+  ! grad(ro) and grad(teta) have opposite sign
   do iel = 1, ncel
     rhothe = cromo(iel)/cvara_scalt(iel)
-    gradro(1, iel) = rhothe*gradro(1, iel)
-    gradro(2, iel) = rhothe*gradro(2, iel)
-    gradro(3, iel) = rhothe*gradro(3, iel)
+    gradro(1, iel) = -rhothe*gradro(1, iel)
+    gradro(2, iel) = -rhothe*gradro(2, iel)
+    gradro(3, iel) = -rhothe*gradro(3, iel)
   enddo
 
 else if (igrari.eq.1) then
@@ -518,15 +520,17 @@ else if (igrari.eq.1) then
   enddo
 
 ! The choice below has the advantage to be simple
+  call field_get_key_struct_var_cal_opt(ivarfl(ir11), vcopt)
 
-  nswrgp = nswrgr(ir11)
-  imligp = imligr(ir11)
-  iwarnp = iwarni(ir11)
-  epsrgp = epsrgr(ir11)
-  climgp = climgr(ir11)
-  extrap = extrag(ir11)
+  nswrgp = vcopt%nswrgr
+  imligp = vcopt%imligr
+  iwarnp = vcopt%iwarni
+  epsrgp = vcopt%epsrgr
+  climgp = vcopt%climgr
+  extrap = vcopt%extrag
 
   f_id0 = -1
+  iccocg = 1
 
   ! If we extrapolate the source terms and rho, we use cpdt rho^n
   if(isto2t.gt.0.and.iroext.gt.0) then
@@ -552,36 +556,44 @@ endif
 !     We solve the equation in a routine similar to covofi.f90
 !===============================================================================
 
+if (iilagr.eq.2) then
+  call field_get_val_v_by_name('rij_st_lagr', lagr_st_rij)
+else
+  lagr_st_rij => null()
+endif
+
 if (irijco.eq.1) then
   ivar = irij
 
+  if (iilagr.eq.2) then
+    tslagi  => tslagr(1:ncelet,itsli)
+  endif
+
   ! Rij-epsilon standard (LRR)
   if (iturb.eq.30) then !TODO
     call resrij2 &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivar   ,                                                       &
    icepdc , icetsm , itypsm ,                                     &
    dt     ,                                                       &
    produc , gradro ,                                              &
    ckupdc , smacel ,                                              &
    viscf  , viscb  ,                                              &
-   tslage , tslagi ,                                              &
+   tslagi ,                                                       &
    smbrts   , rovsdtts )
 
   ! Rij-epsilon SSG or EBRSM
   elseif (iturb.eq.31.or.iturb.eq.32) then
 
-      call resssg2 &
-      !==========
-    ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-      ivar   ,                                                       &
-      icepdc , icetsm , itypsm ,                                     &
-      dt     ,                                                       &
-      gradv  , gradro ,                                              &
-      ckupdc , smacel ,                                              &
-      viscf  , viscb  ,                                              &
-      tslage , tslagi ,                                              &
-      smbrts   , rovsdtts )
+    call resssg2 &
+  ( nvar    , nscal  , ncepdp , ncesmp ,                            &
+    ivar    ,                                                       &
+    icepdc  , icetsm , itypsm ,                                     &
+    dt      ,                                                       &
+    gradv   , gradro ,                                              &
+    ckupdc  , smacel ,                                              &
+    viscf   , viscb  ,                                              &
+    tslagi ,                                                        &
+    smbrts  , rovsdtts )
   endif
 else
   do isou = 1, 6
@@ -600,15 +612,12 @@ else
     endif
 
     if (iilagr.eq.2) then
-      iitsla = itsr11 + (isou-1)
-      tslage => tslagr(1:ncelet,iitsla)
       tslagi => tslagr(1:ncelet,itsli)
     endif
 
     ! Rij-epsilon standard (LRR)
     if (iturb.eq.30) then
       call resrij &
-      !==========
    ( nvar   , nscal  , ncepdp , ncesmp ,                            &
      ivar   , isou   ,                                              &
      icepdc , icetsm , itypsm ,                                     &
@@ -616,13 +625,12 @@ else
      produc , gradro ,                                              &
      ckupdc , smacel ,                                              &
      viscf  , viscb  ,                                              &
-     tslage , tslagi ,                                              &
+     tslagi ,                                                       &
      smbr   , rovsdt )
 
     ! Rij-epsilon SSG or EBRSM
     elseif (iturb.eq.31.or.iturb.eq.32) then
-        call resssg &
-        !==========
+      call resssg &
       ( nvar   , nscal  , ncepdp , ncesmp ,                            &
         ivar   , isou   ,                                              &
         icepdc , icetsm , itypsm ,                                     &
@@ -630,12 +638,13 @@ else
         gradv  , gradro ,                                              &
         ckupdc , smacel ,                                              &
         viscf  , viscb  ,                                              &
-        tslage , tslagi ,                                              &
+        tslagi ,                                                       &
         smbr   , rovsdt )
     endif
 
   enddo
 endif
+
 !===============================================================================
 ! 5. Solve Epsilon
 !===============================================================================
@@ -647,7 +656,6 @@ call reseps &
    gradv  , produc , gradro ,                                     &
    ckupdc , smacel ,                                              &
    viscf  , viscb  ,                                              &
-   tslagr ,                                                       &
    smbr   , rovsdt )
 
 !===============================================================================
