@@ -4,7 +4,7 @@
 !                      --------------------------
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -76,6 +76,7 @@ use cstnum
 use parall
 use period
 use mesh
+use cs_c_bindings
 
 !===============================================================================
 
@@ -110,6 +111,9 @@ double precision flucel
 double precision vtot  , gamma
 
 integer, allocatable, dimension(:) :: lstelt
+
+type(var_cal_opt) :: vcopt
+
 !< [loc_var]
 
 !===============================================================================
@@ -258,10 +262,8 @@ elseif (iappel.eq.3) then
   xkent  = epzero
   xeent  = epzero
 
-  call keendb &
-  !==========
-( wind2, dh, ro0, viscl0, cmu, xkappa,        &
-  ustar2, xkent, xeent )
+  call turbulence_bc_ke_hyd_diam(wind2, dh, ro0, viscl0,  &
+                                 ustar2, xkent, xeent )
 
   flucel = 0.d0
   do ieltsm = 1, ncesmp
@@ -316,7 +318,9 @@ elseif (iappel.eq.3) then
     call parsom (flucel)
   endif
 
-  if (iwarni(ipr).ge.1) then
+  call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+
+  if (vcopt%iwarni.ge.1) then
     write(nfecra,1000) flucel
   endif
 !< [inlet_cal]
@@ -365,7 +369,9 @@ elseif (iappel.eq.3) then
     call parsom (flucel)
   endif
 
-  if (iwarni(ipr).ge.1) then
+  call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+
+  if (vcopt%iwarni.ge.1) then
     write(nfecra,2000) flucel, vtot
   endif
 !< [mass_suction]
@@ -395,4 +401,4 @@ deallocate(lstelt)
 
 return
 end subroutine cs_user_mass_source_terms
-!< [deallocate]
\ No newline at end of file
+!< [deallocate]
